@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
  * Copyright (c) 2015, 2020, Red Hat Inc. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
@@ -131,7 +131,7 @@ void VM_Version::initialize() {
   // Enable vendor specific features
 
   // Ampere eMAG
-  if (_cpu == CPU_AMCC && (_model == CPU_MODEL_EMAG) && (_variant == 0x3)) {
+  if (_cpu == CPU_AMCC && (_model == 0) && (_variant == 0x3)) {
     if (FLAG_IS_DEFAULT(AvoidUnalignedAccesses)) {
       FLAG_SET_DEFAULT(AvoidUnalignedAccesses, true);
     }
@@ -143,13 +143,6 @@ void VM_Version::initialize() {
     }
   }
 
-  // Ampere CPUs: Ampere-1 and Ampere-1A
-  if (_cpu == CPU_AMPERE && ((_model == CPU_MODEL_AMPERE_1) || (_model == CPU_MODEL_AMPERE_1A))) {
-    if (FLAG_IS_DEFAULT(UseSIMDForMemoryOps)) {
-      FLAG_SET_DEFAULT(UseSIMDForMemoryOps, true);
-    }
-  }
-
   // ThunderX
   if (_cpu == CPU_CAVIUM && (_model == 0xA1)) {
     guarantee(_variant != 0, "Pre-release hardware no longer supported.");
@@ -204,10 +197,8 @@ void VM_Version::initialize() {
     }
   }
 
-  // Neoverse N1, N2 and V1
-  if (_cpu == CPU_ARM && ((_model == 0xd0c || _model2 == 0xd0c)
-                          || (_model == 0xd49 || _model2 == 0xd49)
-                          || (_model == 0xd40 || _model2 == 0xd40))) {
+  // Neoverse N1
+  if (_cpu == CPU_ARM && (_model == 0xd0c || _model2 == 0xd0c)) {
     if (FLAG_IS_DEFAULT(UseSIMDForMemoryOps)) {
       FLAG_SET_DEFAULT(UseSIMDForMemoryOps, true);
     }
@@ -292,6 +283,7 @@ void VM_Version::initialize() {
     }
   }
 
+
   if (FLAG_IS_DEFAULT(UseCRC32Intrinsics)) {
     UseCRC32Intrinsics = true;
   }
@@ -477,6 +469,14 @@ void VM_Version::initialize() {
     }
   }
 
+  int inline_size = (UseSVE > 0 && MaxVectorSize >= 16) ? MaxVectorSize : 0;
+  if (FLAG_IS_DEFAULT(ArrayOperationPartialInlineSize)) {
+    FLAG_SET_DEFAULT(ArrayOperationPartialInlineSize, inline_size);
+  } else if (ArrayOperationPartialInlineSize != 0 && ArrayOperationPartialInlineSize != inline_size) {
+    warning("Setting ArrayOperationPartialInlineSize to %d", inline_size);
+    ArrayOperationPartialInlineSize = inline_size;
+  }
+
   if (FLAG_IS_DEFAULT(OptoScheduling)) {
     OptoScheduling = true;
   }
@@ -487,58 +487,4 @@ void VM_Version::initialize() {
 #endif
 
   _spin_wait = get_spin_wait_desc();
-
-  check_virtualizations();
-
-  UNSUPPORTED_OPTION(CriticalJNINatives);
-}
-
-#if defined(LINUX)
-static bool check_info_file(const char* fpath,
-                            const char* virt1, VirtualizationType vt1,
-                            const char* virt2, VirtualizationType vt2) {
-  char line[500];
-  FILE* fp = os::fopen(fpath, "r");
-  if (fp == nullptr) {
-    return false;
-  }
-  while (fgets(line, sizeof(line), fp) != nullptr) {
-    if (strcasestr(line, virt1) != 0) {
-      Abstract_VM_Version::_detected_virtualization = vt1;
-      fclose(fp);
-      return true;
-    }
-    if (virt2 != NULL && strcasestr(line, virt2) != 0) {
-      Abstract_VM_Version::_detected_virtualization = vt2;
-      fclose(fp);
-      return true;
-    }
-  }
-  fclose(fp);
-  return false;
-}
-#endif
-
-void VM_Version::check_virtualizations() {
-#if defined(LINUX)
-  const char* pname_file = "/sys/devices/virtual/dmi/id/product_name";
-  const char* tname_file = "/sys/hypervisor/type";
-  if (check_info_file(pname_file, "KVM", KVM, "VMWare", VMWare)) {
-    return;
-  }
-  check_info_file(tname_file, "Xen", XenPVHVM, NULL, NoDetectedVirtualization);
-#endif
-}
-
-void VM_Version::print_platform_virtualization_info(outputStream* st) {
-#if defined(LINUX)
-  VirtualizationType vrt = VM_Version::get_detected_virtualization();
-  if (vrt == KVM) {
-    st->print_cr("KVM virtualization detected");
-  } else if (vrt == VMWare) {
-    st->print_cr("VMWare virtualization detected");
-  } else if (vrt == XenPVHVM) {
-    st->print_cr("Xen virtualization detected");
-  }
-#endif
 }
