@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2021, Oracle and/or its affiliates. All rights reserved.
  * Copyright (c) 2012, 2021 SAP SE. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
@@ -31,6 +31,7 @@
 #include "memory/resourceArea.hpp"
 #include "oops/oop.inline.hpp"
 #include "os_posix.inline.hpp"
+#include "runtime/globals_extension.hpp"
 #include "runtime/handles.inline.hpp"
 #include "runtime/os.hpp"
 #include "runtime/perfMemory.hpp"
@@ -47,10 +48,6 @@
 # include <signal.h>
 # include <pwd.h>
 
-#if defined(LINUX)
-# include <sys/file.h>
-#endif
-
 static char* backing_store_file_name = NULL;  // name of the backing store
                                               // file, if successfully created.
 
@@ -79,6 +76,18 @@ static char* create_standard_memory(size_t size) {
   return mapAddress;
 }
 
+// delete the PerfData memory region
+//
+static void delete_standard_memory(char* addr, size_t size) {
+
+  // there are no persistent external resources to cleanup for standard
+  // memory. since DestroyJavaVM does not support unloading of the JVM,
+  // cleanup of the memory resource is not performed. The memory will be
+  // reclaimed by the OS upon termination of the process.
+  //
+  return;
+}
+
 // save the specified memory region to the given file
 //
 // Note: this function might be called from signal handler (by os::abort()),
@@ -89,25 +98,38 @@ static void save_memory_to_file(char* addr, size_t size) {
   const char* destfile = PerfMemory::get_perfdata_file_path();
   assert(destfile[0] != '\0', "invalid PerfData file path");
 
-  int fd;
+  int result;
 
-  RESTARTABLE(os::open(destfile, O_CREAT|O_WRONLY|O_TRUNC, S_IRUSR|S_IWUSR), fd);
-  if (fd == OS_ERR) {
-    warning("Could not create Perfdata save file: %s: %s\n",
-            destfile, os::strerror(errno));
-  } else {
-    ssize_t result;
-
-    bool successful_write = os::write(fd, addr, size);
-    if (!successful_write) {
-      warning("Could not write Perfdata save file: %s: %s\n",
+  RESTARTABLE(os::open(destfile, O_CREAT|O_WRONLY|O_TRUNC, S_IRUSR|S_IWUSR),
+              result);
+  if (result == OS_ERR) {
+    if (PrintMiscellaneous && Verbose) {
+      warning("Could not create Perfdata save file: %s: %s\n",
               destfile, os::strerror(errno));
     }
+  } else {
+    int fd = result;
 
+    for (size_t remaining = size; remaining > 0;) {
+
+      RESTARTABLE(::write(fd, addr, remaining), result);
+      if (result == OS_ERR) {
+        if (PrintMiscellaneous && Verbose) {
+          warning("Could not write Perfdata save file: %s: %s\n",
+                  destfile, os::strerror(errno));
+        }
+        break;
+      }
+
+      remaining -= (size_t)result;
+      addr += result;
+    }
 
     result = ::close(fd);
-    if (result == OS_ERR) {
-      warning("Could not close %s: %s\n", destfile, os::strerror(errno));
+    if (PrintMiscellaneous && Verbose) {
+      if (result == OS_ERR) {
+        warning("Could not close %s: %s\n", destfile, os::strerror(errno));
+      }
     }
   }
   FREE_C_HEAP_ARRAY(char, destfile);
@@ -686,17 +708,17 @@ static void remove_file(const char* path) {
   }
 }
 
-// cleanup stale shared memory files
+
+// cleanup stale shared memory resources
 //
 // This method attempts to remove all stale shared memory files in
 // the named user temporary directory. It scans the named directory
-// for files matching the pattern ^$[0-9]*$.
+// for files matching the pattern ^$[0-9]*$. For each file found, the
+// process id is extracted from the file name and a test is run to
+// determine if the process is alive. If the process is not alive,
+// any stale file resources are removed.
 //
-// This directory should be used only by JVM processes owned by the
-// current user to store PerfMemory files. Any other files found
-// in this directory may be removed.
-//
-static void cleanup_sharedmem_files(const char* dirname) {
+static void cleanup_sharedmem_resources(const char* dirname) {
 
   int saved_cwd_fd;
   // open the directory and set the current working directory to it
@@ -706,95 +728,48 @@ static void cleanup_sharedmem_files(const char* dirname) {
     return;
   }
 
-  // For each entry in the directory that matches the expected file
-  // name pattern, remove the file if it's determine to be stale
-  // Note, instrumented HotSpot processes for this user may start and/or
-  // terminate during this search and remove or create new files in this
-  // directory. The behavior of this loop under these conditions is dependent
-  // upon the implementation of opendir/readdir.
+  // for each entry in the directory that matches the expected file
+  // name pattern, determine if the file resources are stale and if
+  // so, remove the file resources. Note, instrumented HotSpot processes
+  // for this user may start and/or terminate during this search and
+  // remove or create new files in this directory. The behavior of this
+  // loop under these conditions is dependent upon the implementation of
+  // opendir/readdir.
   //
   struct dirent* entry;
   errno = 0;
   while ((entry = os::readdir(dirp)) != NULL) {
-    const char* filename = entry->d_name;
-    pid_t pid = filename_to_pid(filename);
+
+    pid_t pid = filename_to_pid(entry->d_name);
 
     if (pid == 0) {
-      if (strcmp(filename, ".") != 0 && strcmp(filename, "..") != 0) {
+
+      if (strcmp(entry->d_name, ".") != 0 && strcmp(entry->d_name, "..") != 0) {
         // attempt to remove all unexpected files, except "." and ".."
-        unlink(filename);
+        unlink(entry->d_name);
       }
 
       errno = 0;
       continue;
     }
 
-#if defined(LINUX)
-    // Special case on Linux, if multiple containers share the
-    // same /tmp directory:
+    // we now have a file name that converts to a valid integer
+    // that could represent a process id . if this process id
+    // matches the current process id or the process is not running,
+    // then remove the stale file resources.
     //
-    // - All the JVMs must have the JDK-8286030 fix, or the behavior
-    //   is undefined.
-    // - We cannot rely on the values of the pid, because it could
-    //   be a process in a different namespace. We must use the flock
-    //   protocol to determine if a live process is using this file.
-    //   See create_sharedmem_file().
-    int fd;
-    RESTARTABLE(os::open(filename, O_RDONLY, 0), fd);
-    if (fd == OS_ERR) {
-      // Something wrong happened. Ignore the error and don't try to remove the
-      // file.
-      log_debug(perf, memops)("os::open() for stale file check failed for %s/%s", dirname, filename);
-      errno = 0;
-      continue;
+    // process liveness is detected by sending signal number 0 to
+    // the process id (see kill(2)). if kill determines that the
+    // process does not exist, then the file resources are removed.
+    // if kill determines that that we don't have permission to
+    // signal the process, then the file resources are assumed to
+    // be stale and are removed because the resources for such a
+    // process should be in a different user specific directory.
+    //
+    if ((pid == os::current_process_id()) ||
+        (kill(pid, 0) == OS_ERR && (errno == ESRCH || errno == EPERM))) {
+        unlink(entry->d_name);
     }
-
-    int n;
-    RESTARTABLE(::flock(fd, LOCK_EX|LOCK_NB), n);
-    if (n != 0) {
-      // Either another process holds the exclusive lock on this file, or
-      // something wrong happened. Ignore the error and don't try to remove the
-      // file.
-      log_debug(perf, memops)("flock for stale file check failed for %s/%s", dirname, filename);
-      ::close(fd);
-      errno = 0;
-      continue;
-    }
-    // We are able to lock the file, but this file might have been created
-    // by an older JVM that doesn't use the flock prototol, so we must do
-    // the folowing checks (which are also done by older JVMs).
-#endif
-
-    // The following code assumes that pid must be in the same
-    // namespace as the current process.
-    bool stale = false;
-
-    if (pid == os::current_process_id()) {
-      // The file was created by a terminated process that happened
-      // to have the same pid as the current process.
-      stale = true;
-    } else if (kill(pid, 0) == OS_ERR) {
-      if (errno == ESRCH) {
-        // The target process does not exist.
-        stale = true;
-      } else if (errno == EPERM) {
-        // The file was created by a terminated process that happened
-        // to have the same pid as a process not owned by the current user.
-        stale = true;
-      }
-    }
-
-    if (stale) {
-      log_info(perf, memops)("Remove stale file %s/%s", dirname, filename);
-      unlink(filename);
-    }
-
-#if defined(LINUX)
-    // Hold the lock until here to prevent other JVMs from using this file
-    // while we were in the middle of deleting it.
-    ::close(fd);
-#endif
-
     errno = 0;
   }
 
@@ -840,13 +815,13 @@ static bool make_user_tmp_dir(const char* dirname) {
   return true;
 }
 
-// create the shared memory file
+// create the shared memory file resources
 //
 // This method creates the shared memory file with the given size
 // This method also creates the user specific temporary directory, if
 // it does not yet exist.
 //
-static int create_sharedmem_file(const char* dirname, const char* filename, size_t size) {
+static int create_sharedmem_resources(const char* dirname, const char* filename, size_t size) {
 
   // make the user temporary directory
   if (!make_user_tmp_dir(dirname)) {
@@ -867,9 +842,9 @@ static int create_sharedmem_file(const char* dirname, const char* filename, size
   // Open the filename in the current directory.
   // Cannot use O_TRUNC here; truncation of an existing file has to happen
   // after the is_file_secure() check below.
-  int fd;
-  RESTARTABLE(os::open(filename, O_RDWR|O_CREAT|O_NOFOLLOW, S_IRUSR|S_IWUSR), fd);
-  if (fd == OS_ERR) {
+  int result;
+  RESTARTABLE(os::open(filename, O_RDWR|O_CREAT|O_NOFOLLOW, S_IRUSR|S_IWUSR), result);
+  if (result == OS_ERR) {
     if (PrintMiscellaneous && Verbose) {
       if (errno == ELOOP) {
         warning("file %s is a symlink and is not secure\n", filename);
@@ -885,40 +860,15 @@ static int create_sharedmem_file(const char* dirname, const char* filename, size
   // close the directory and reset the current working directory
   close_directory_secure_cwd(dirp, saved_cwd_fd);
 
+  // save the file descriptor
+  int fd = result;
+
   // check to see if the file is secure
   if (!is_file_secure(fd, filename)) {
     ::close(fd);
     return -1;
   }
 
-#if defined(LINUX)
-  // On Linux, different containerized processes that share the same /tmp
-  // directory (e.g., with "docker --volume ...") may have the same pid and
-  // try to use the same file. To avoid conflicts among such
-  // processes, we allow only one of them (the winner of the flock() call)
-  // to write to the file. All the other processes will give up and will
-  // have perfdata disabled.
-  //
-  // Note that the flock will be automatically given up when the winner
-  // process exits.
-  //
-  // The locking protocol works only with other JVMs that have the JDK-8286030
-  // fix. If you are sharing the /tmp difrectory among different containers,
-  // do not use older JVMs that don't have this fix, or the behavior is undefined.
-  int n;
-  RESTARTABLE(::flock(fd, LOCK_EX|LOCK_NB), n);
-  if (n != 0) {
-    log_warning(perf, memops)("Cannot use file %s/%s because %s (errno = %d)", dirname, filename,
-                              (errno == EWOULDBLOCK) ?
-                              "it is locked by another process" :
-                              "flock() failed", errno);
-    ::close(fd);
-    return -1;
-  }
-#endif
-
- ssize_t result;
-
   // truncate the file to get rid of any existing data
   RESTARTABLE(::ftruncate(fd, (off_t)0), result);
   if (result == OS_ERR) {
@@ -945,11 +895,11 @@ static int create_sharedmem_file(const char* dirname, const char* filename, size
     int zero_int = 0;
     result = (int)os::seek_to_file_offset(fd, (jlong)(seekpos));
     if (result == -1 ) break;
-    if (!os::write(fd, &zero_int, 1)) {
+    RESTARTABLE(::write(fd, &zero_int, 1), result);
+    if (result != 1) {
       if (errno == ENOSPC) {
         warning("Insufficient space for shared memory file:\n   %s\nTry using the -Djava.io.tmpdir= option to select an alternate temp location.\n", filename);
       }
-      result = OS_ERR;
       break;
     }
   }
@@ -957,7 +907,7 @@ static int create_sharedmem_file(const char* dirname, const char* filename, size
   if (result != -1) {
     return fd;
   } else {
-    os::close(fd);
+    ::close(fd);
     return -1;
   }
 }
@@ -1033,13 +983,12 @@ static char* mmap_create_shared(size_t size) {
   }
 
   // cleanup any stale shared memory files
-  cleanup_sharedmem_files(dirname);
+  cleanup_sharedmem_resources(dirname);
 
   assert(((size > 0) && (size % os::vm_page_size() == 0)),
          "unexpected PerfMemory region size");
 
-  log_info(perf, memops)("Trying to open %s/%s", dirname, short_filename);
-  fd = create_sharedmem_file(dirname, short_filename, size);
+  fd = create_sharedmem_resources(dirname, short_filename, size);
 
   FREE_C_HEAP_ARRAY(char, user_name);
   FREE_C_HEAP_ARRAY(char, dirname);
@@ -1072,23 +1021,26 @@ static char* mmap_create_shared(size_t size) {
   // it does not go through os api, the operation has to record from here
   MemTracker::record_virtual_memory_reserve_and_commit((address)mapAddress, size, CURRENT_PC, mtInternal);
 
-  log_info(perf, memops)("Successfully opened");
-
   return mapAddress;
 }
 
-// release a named shared memory region
+// release a named shared memory region that was mmap-ed.
 //
 static void unmap_shared(char* addr, size_t bytes) {
-#if defined(_AIX)
-  // Do not rely on os::reserve_memory/os::release_memory to use mmap.
-  // Use os::reserve_memory/os::release_memory for PerfDisableSharedMem=1, mmap/munmap for PerfDisableSharedMem=0
-  if (::munmap(addr, bytes) == -1) {
-    warning("perfmemory: munmap failed (%d)\n", errno);
+  int res;
+  if (MemTracker::tracking_level() > NMT_minimal) {
+    // Note: Tracker contains a ThreadCritical.
+    Tracker tkr(Tracker::release);
+    res = ::munmap(addr, bytes);
+    if (res == 0) {
+      tkr.record((address)addr, bytes);
+    }
+  } else {
+    res = ::munmap(addr, bytes);
+  }
+  if (res != 0) {
+    log_info(os)("os::release_memory failed (" PTR_FORMAT ", " SIZE_FORMAT ")", p2i(addr), bytes);
   }
-#else
-  os::release_memory(addr, bytes);
-#endif
 }
 
 // create the PerfData memory region in shared memory.
@@ -1103,10 +1055,10 @@ static char* create_shared_memory(size_t size) {
 //
 static void delete_shared_memory(char* addr, size_t size) {
 
-  // Remove the shared memory file. Since DestroyJavaVM does
-  // not support unloading of the JVM, unmapping of the memory region is
+  // cleanup the persistent shared memory resources. since DestroyJavaVM does
+  // not support unloading of the JVM, unmapping of the memory resource is
   // not performed. The memory will be reclaimed by the OS upon termination of
-  // the process.
+  // the process. The backing store file is deleted from the file system.
 
   assert(!PerfDisableSharedMem, "shouldn't be here");
 
@@ -1288,7 +1240,7 @@ void PerfMemory::create_memory_region(size_t size) {
       if (PrintMiscellaneous && Verbose) {
         warning("Reverting to non-shared PerfMemory region.\n");
       }
-      PerfDisableSharedMem = true;
+      FLAG_SET_ERGO(PerfDisableSharedMem, true);
       _start = create_standard_memory(size);
     }
   }
@@ -1315,7 +1267,10 @@ void PerfMemory::delete_memory_region() {
     save_memory_to_file(start(), capacity());
   }
 
-  if (!PerfDisableSharedMem) {
+  if (PerfDisableSharedMem) {
+    delete_standard_memory(start(), capacity());
+  }
+  else {
     delete_shared_memory(start(), capacity());
   }
 }
