@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -33,19 +33,20 @@
 #include "runtime/sharedRuntime.hpp"
 #include "runtime/vm_version.hpp"
 
-Register LIR_OprDesc::as_register() const {
+Register LIR_Opr::as_register() const {
   return FrameMap::cpu_rnr2reg(cpu_regnr());
 }
 
-Register LIR_OprDesc::as_register_lo() const {
+Register LIR_Opr::as_register_lo() const {
   return FrameMap::cpu_rnr2reg(cpu_regnrLo());
 }
 
-Register LIR_OprDesc::as_register_hi() const {
+Register LIR_Opr::as_register_hi() const {
   return FrameMap::cpu_rnr2reg(cpu_regnrHi());
 }
 
 LIR_Opr LIR_OprFact::illegalOpr = LIR_OprFact::illegal();
+LIR_Opr LIR_OprFact::nullOpr = LIR_Opr();
 
 LIR_Opr LIR_OprFact::value_type(ValueType* type) {
   ValueTag tag = type->tag();
@@ -92,7 +93,7 @@ LIR_Address::Scale LIR_Address::scale(BasicType type) {
 
 //---------------------------------------------------
 
-char LIR_OprDesc::type_char(BasicType t) {
+char LIR_Opr::type_char(BasicType t) {
   switch (t) {
     case T_ARRAY:
       t = T_OBJECT;
@@ -120,7 +121,7 @@ char LIR_OprDesc::type_char(BasicType t) {
 }
 
 #ifndef PRODUCT
-void LIR_OprDesc::validate_type() const {
+void LIR_Opr::validate_type() const {
 
 #ifdef ASSERT
   if (!is_pointer() && !is_illegal()) {
@@ -131,17 +132,15 @@ void LIR_OprDesc::validate_type() const {
              size_field() == double_size, "must match");
       break;
     case T_FLOAT:
-      // FP return values can be also in CPU registers on ARM and PPC32 (softfp ABI)
+      // FP return values can be also in CPU registers on ARM (softfp ABI)
       assert((kindfield == fpu_register || kindfield == stack_value
-             ARM_ONLY(|| kindfield == cpu_register)
-             PPC32_ONLY(|| kindfield == cpu_register) ) &&
+             ARM_ONLY(|| kindfield == cpu_register) ) &&
              size_field() == single_size, "must match");
       break;
     case T_DOUBLE:
-      // FP return values can be also in CPU registers on ARM and PPC32 (softfp ABI)
+      // FP return values can be also in CPU registers on ARM (softfp ABI)
       assert((kindfield == fpu_register || kindfield == stack_value
-             ARM_ONLY(|| kindfield == cpu_register)
-             PPC32_ONLY(|| kindfield == cpu_register) ) &&
+             ARM_ONLY(|| kindfield == cpu_register) ) &&
              size_field() == double_size, "must match");
       break;
     case T_BOOLEAN:
@@ -172,7 +171,7 @@ void LIR_OprDesc::validate_type() const {
 #endif // PRODUCT
 
 
-bool LIR_OprDesc::is_oop() const {
+bool LIR_Opr::is_oop() const {
   if (is_pointer()) {
     return pointer()->is_oop_pointer();
   } else {
@@ -188,9 +187,6 @@ void LIR_Op2::verify() const {
 #ifdef ASSERT
   switch (code()) {
     case lir_cmove:
-#ifdef RISCV
-      assert(false, "lir_cmove is LIR_Op4 on RISCV");
-#endif
     case lir_xchg:
       break;
 
@@ -241,12 +237,8 @@ void LIR_Op2::verify() const {
 
 
 LIR_OpBranch::LIR_OpBranch(LIR_Condition cond, BlockBegin* block)
-#ifdef RISCV
-  : LIR_Op2(lir_branch, cond, LIR_OprFact::illegalOpr, LIR_OprFact::illegalOpr, (CodeEmitInfo*)NULL)
-#else
   : LIR_Op(lir_branch, LIR_OprFact::illegalOpr, (CodeEmitInfo*)NULL)
   , _cond(cond)
-#endif
   , _label(block->label())
   , _block(block)
   , _ublock(NULL)
@@ -254,12 +246,8 @@ LIR_OpBranch::LIR_OpBranch(LIR_Condition cond, BlockBegin* block)
 }
 
 LIR_OpBranch::LIR_OpBranch(LIR_Condition cond, CodeStub* stub) :
-#ifdef RISCV
-  LIR_Op2(lir_branch, cond, LIR_OprFact::illegalOpr, LIR_OprFact::illegalOpr, (CodeEmitInfo*)NULL)
-#else
   LIR_Op(lir_branch, LIR_OprFact::illegalOpr, (CodeEmitInfo*)NULL)
   , _cond(cond)
-#endif
   , _label(stub->entry())
   , _block(NULL)
   , _ublock(NULL)
@@ -267,12 +255,8 @@ LIR_OpBranch::LIR_OpBranch(LIR_Condition cond, CodeStub* stub) :
 }
 
 LIR_OpBranch::LIR_OpBranch(LIR_Condition cond, BlockBegin* block, BlockBegin* ublock)
-#ifdef RISCV
-  : LIR_Op2(lir_cond_float_branch, cond, LIR_OprFact::illegalOpr, LIR_OprFact::illegalOpr, (CodeEmitInfo*)NULL)
-#else
   : LIR_Op(lir_cond_float_branch, LIR_OprFact::illegalOpr, (CodeEmitInfo*)NULL)
   , _cond(cond)
-#endif
   , _label(block->label())
   , _block(block)
   , _ublock(ublock)
@@ -294,13 +278,13 @@ void LIR_OpBranch::change_ublock(BlockBegin* b) {
 }
 
 void LIR_OpBranch::negate_cond() {
-  switch (cond()) {
-    case lir_cond_equal:        set_cond(lir_cond_notEqual);     break;
-    case lir_cond_notEqual:     set_cond(lir_cond_equal);        break;
-    case lir_cond_less:         set_cond(lir_cond_greaterEqual); break;
-    case lir_cond_lessEqual:    set_cond(lir_cond_greater);      break;
-    case lir_cond_greaterEqual: set_cond(lir_cond_less);         break;
-    case lir_cond_greater:      set_cond(lir_cond_lessEqual);    break;
+  switch (_cond) {
+    case lir_cond_equal:        _cond = lir_cond_notEqual;     break;
+    case lir_cond_notEqual:     _cond = lir_cond_equal;        break;
+    case lir_cond_less:         _cond = lir_cond_greaterEqual; break;
+    case lir_cond_lessEqual:    _cond = lir_cond_greater;      break;
+    case lir_cond_greaterEqual: _cond = lir_cond_less;         break;
+    case lir_cond_greater:      _cond = lir_cond_lessEqual;    break;
     default: ShouldNotReachHere();
   }
 }
@@ -418,7 +402,6 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
   switch (op->code()) {
 
 // LIR_Op0
-    case lir_backwardbranch_target:    // result and info always invalid
     case lir_fpop_raw:                 // result and info always invalid
     case lir_breakpoint:               // result and info always invalid
     case lir_membar:                   // result and info always invalid
@@ -512,10 +495,6 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
       assert(opConvert->_info == NULL, "must be");
       if (opConvert->_opr->is_valid())       do_input(opConvert->_opr);
       if (opConvert->_result->is_valid())    do_output(opConvert->_result);
-#ifdef PPC32
-      if (opConvert->_tmp1->is_valid())      do_temp(opConvert->_tmp1);
-      if (opConvert->_tmp2->is_valid())      do_temp(opConvert->_tmp2);
-#endif
       do_stub(opConvert->_stub);
 
       break;
@@ -528,15 +507,6 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
       assert(op->as_OpBranch() != NULL, "must be");
       LIR_OpBranch* opBranch = (LIR_OpBranch*)op;
 
-#ifdef RISCV
-      assert(opBranch->_tmp1->is_illegal() && opBranch->_tmp2->is_illegal() &&
-             opBranch->_tmp3->is_illegal() && opBranch->_tmp4->is_illegal() &&
-             opBranch->_tmp5->is_illegal(), "not used");
-
-      if (opBranch->_opr1->is_valid()) do_input(opBranch->_opr1);
-      if (opBranch->_opr2->is_valid()) do_input(opBranch->_opr2);
-#endif
-
       if (opBranch->_info != NULL)     do_info(opBranch->_info);
       assert(opBranch->_result->is_illegal(), "not used");
       if (opBranch->_stub != NULL)     opBranch->stub()->visit(this);
@@ -625,21 +595,6 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
     // to the result operand, otherwise the backend fails
     case lir_cmove:
     {
-#ifdef RISCV
-      assert(op->as_Op4() != NULL, "must be");
-      LIR_Op4* op4 = (LIR_Op4*)op;
-
-      assert(op4->_info == NULL && op4->_tmp1->is_illegal() && op4->_tmp2->is_illegal() &&
-             op4->_tmp3->is_illegal() && op4->_tmp4->is_illegal() && op4->_tmp5->is_illegal(), "not used");
-      assert(op4->_opr1->is_valid() && op4->_opr2->is_valid() && op4->_result->is_valid(), "used");
-
-      do_input(op4->_opr1);
-      do_input(op4->_opr2);
-      if (op4->_opr3->is_valid()) do_input(op4->_opr3);
-      if (op4->_opr4->is_valid()) do_input(op4->_opr4);
-      do_temp(op4->_opr2);
-      do_output(op4->_result);
-#else
       assert(op->as_Op2() != NULL, "must be");
       LIR_Op2* op2 = (LIR_Op2*)op;
 
@@ -651,7 +606,6 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
       do_input(op2->_opr2);
       do_temp(op2->_opr2);
       do_output(op2->_result);
-#endif
 
       break;
     }
@@ -1095,12 +1049,6 @@ void LIR_Op3::emit_code(LIR_Assembler* masm) {
   masm->emit_op3(this);
 }
 
-#ifdef RISCV
-void LIR_Op4::emit_code(LIR_Assembler* masm) {
-  masm->emit_op4(this);
-}
-#endif
-
 void LIR_OpLock::emit_code(LIR_Assembler* masm) {
   masm->emit_lock(this);
   if (stub()) {
@@ -1141,10 +1089,6 @@ LIR_List::LIR_List(Compilation* compilation, BlockBegin* block)
   , _file(NULL)
   , _line(0)
 #endif
-#ifdef RISCV
-  , _cmp_opr1(LIR_OprFact::illegalOpr)
-  , _cmp_opr2(LIR_OprFact::illegalOpr)
-#endif
 { }
 
 
@@ -1162,38 +1106,6 @@ void LIR_List::set_file_and_line(const char * file, int line) {
 }
 #endif
 
-#ifdef RISCV
-void LIR_List::set_cmp_oprs(LIR_Op* op) {
-  switch (op->code()) {
-    case lir_cmp:
-      _cmp_opr1 = op->as_Op2()->in_opr1();
-      _cmp_opr2 = op->as_Op2()->in_opr2();
-      break;
-    case lir_branch: // fall through
-    case lir_cond_float_branch:
-      assert(op->as_OpBranch()->cond() == lir_cond_always ||
-            (_cmp_opr1 != LIR_OprFact::illegalOpr && _cmp_opr2 != LIR_OprFact::illegalOpr),
-            "conditional branches must have legal operands");
-      if (op->as_OpBranch()->cond() != lir_cond_always) {
-        op->as_Op2()->set_in_opr1(_cmp_opr1);
-        op->as_Op2()->set_in_opr2(_cmp_opr2);
-      }
-      break;
-    case lir_cmove:
-      op->as_Op4()->set_in_opr3(_cmp_opr1);
-      op->as_Op4()->set_in_opr4(_cmp_opr2);
-      break;
-#if INCLUDE_ZGC
-    case lir_zloadbarrier_test:
-      _cmp_opr1 = FrameMap::as_opr(t1);
-      _cmp_opr2 = LIR_OprFact::intConst(0);
-      break;
-#endif
-    default:
-      break;
-  }
-}
-#endif
 
 void LIR_List::append(LIR_InsertionBuffer* buffer) {
   assert(this == buffer->lir_list(), "wrong lir list");
@@ -1472,7 +1384,7 @@ void LIR_List::unlock_object(LIR_Opr hdr, LIR_Opr obj, LIR_Opr lock, LIR_Opr scr
 
 void check_LIR() {
   // cannot do the proper checking as PRODUCT and other modes return different results
-  // guarantee(sizeof(LIR_OprDesc) == wordSize, "may not have a v-table");
+  // guarantee(sizeof(LIR_Opr) == wordSize, "may not have a v-table");
 }
 
 
@@ -1547,12 +1459,12 @@ void print_LIR(BlockList* blocks) {
 }
 
 #else
-// LIR_OprDesc
-void LIR_OprDesc::print() const {
+// LIR_Opr
+void LIR_Opr::print() const {
   print(tty);
 }
 
-void LIR_OprDesc::print(outputStream* out) const {
+void LIR_Opr::print(outputStream* out) const {
   if (is_illegal()) {
     return;
   }
@@ -1668,7 +1580,7 @@ static void print_block(BlockBegin* x) {
     }
   }
 
-  if (x->number_of_sux() > 0) {
+  if (end != NULL && x->number_of_sux() > 0) {
     tty->print("sux: ");
     for (int i = 0; i < x->number_of_sux(); i ++) {
       tty->print("B%d ", x->sux_at(i)->block_id());
@@ -1736,7 +1648,6 @@ const char * LIR_Op::name() const {
      case lir_label:                 s = "label";         break;
      case lir_nop:                   s = "nop";           break;
      case lir_on_spin_wait:          s = "on_spin_wait";  break;
-     case lir_backwardbranch_target: s = "backbranch";    break;
      case lir_std_entry:             s = "std_entry";     break;
      case lir_osr_entry:             s = "osr_entry";     break;
      case lir_fpop_raw:              s = "fpop_raw";      break;
@@ -1880,8 +1791,6 @@ const char * LIR_Op1::name() const {
     switch (move_kind()) {
     case lir_move_normal:
       return "move";
-    case lir_move_unaligned:
-      return "unaligned move";
     case lir_move_volatile:
       return "volatile_move";
     case lir_move_wide:
@@ -1924,10 +1833,6 @@ void LIR_Op1::print_patch_code(outputStream* out, LIR_PatchCode code) {
 // LIR_OpBranch
 void LIR_OpBranch::print_instr(outputStream* out) const {
   print_condition(out, cond());             out->print(" ");
-#ifdef RISCV
-  in_opr1()->print(out); out->print(" ");
-  in_opr2()->print(out); out->print(" ");
-#endif
   if (block() != NULL) {
     out->print("[B%d] ", block()->block_id());
   } else if (stub() != NULL) {
@@ -1963,12 +1868,6 @@ void LIR_OpConvert::print_instr(outputStream* out) const {
   print_bytecode(out, bytecode());
   in_opr()->print(out);                  out->print(" ");
   result_opr()->print(out);              out->print(" ");
-#ifdef PPC32
-  if(tmp1()->is_valid()) {
-    tmp1()->print(out); out->print(" ");
-    tmp2()->print(out); out->print(" ");
-  }
-#endif
 }
 
 void LIR_OpConvert::print_bytecode(outputStream* out, Bytecodes::Code code) {
@@ -2014,11 +1913,7 @@ void LIR_OpRoundFP::print_instr(outputStream* out) const {
 
 // LIR_Op2
 void LIR_Op2::print_instr(outputStream* out) const {
-#ifdef RISCV
-  if (code() == lir_cmp || code() == lir_branch || code() == lir_cond_float_branch) {
-#else
   if (code() == lir_cmove || code() == lir_cmp) {
-#endif
     print_condition(out, condition());         out->print(" ");
   }
   in_opr1()->print(out);    out->print(" ");
@@ -2069,17 +1964,6 @@ void LIR_Op3::print_instr(outputStream* out) const {
   result_opr()->print(out);
 }
 
-#ifdef RISCV
-// LIR_Op4
-void LIR_Op4::print_instr(outputStream* out) const {
-  print_condition(out, condition()); out->print(" ");
-  in_opr1()->print(out);             out->print(" ");
-  in_opr2()->print(out);             out->print(" ");
-  in_opr3()->print(out);             out->print(" ");
-  in_opr4()->print(out);             out->print(" ");
-  result_opr()->print(out);
-}
-#endif
 
 void LIR_OpLock::print_instr(outputStream* out) const {
   hdr_opr()->print(out);   out->print(" ");
