@@ -38,6 +38,7 @@
 #include "classfile/systemDictionaryShared.hpp"
 #include "classfile/vmClasses.hpp"
 #include "classfile/vmSymbols.hpp"
+#include "gc/shared/collectedHeap.hpp"
 #include "gc/shared/gcLocker.hpp"
 #include "gc/shared/gcVMOperations.hpp"
 #include "logging/log.hpp"
@@ -67,13 +68,25 @@
 
 #if INCLUDE_CDS_JAVA_HEAP
 
-bool HeapShared::_closed_archive_heap_region_mapped = false;
-bool HeapShared::_open_archive_heap_region_mapped = false;
-bool HeapShared::_archive_heap_region_fixed = false;
+bool HeapShared::_closed_regions_mapped = false;
+bool HeapShared::_open_regions_mapped = false;
+bool HeapShared::_is_loaded = false;
 address   HeapShared::_narrow_oop_base;
 int       HeapShared::_narrow_oop_shift;
 DumpedInternedStrings *HeapShared::_dumped_interned_strings = NULL;
 
+uintptr_t HeapShared::_loaded_heap_bottom = 0;
+uintptr_t HeapShared::_loaded_heap_top = 0;
+uintptr_t HeapShared::_dumptime_base_0 = UINTPTR_MAX;
+uintptr_t HeapShared::_dumptime_base_1 = UINTPTR_MAX;
+uintptr_t HeapShared::_dumptime_base_2 = UINTPTR_MAX;
+uintptr_t HeapShared::_dumptime_base_3 = UINTPTR_MAX;
+uintptr_t HeapShared::_dumptime_top    = 0;
+intx HeapShared::_runtime_offset_0 = 0;
+intx HeapShared::_runtime_offset_1 = 0;
+intx HeapShared::_runtime_offset_2 = 0;
+intx HeapShared::_runtime_offset_3 = 0;
+bool HeapShared::_loading_failed = false;
 //
 // If you add new entries to the following tables, you should know what you're doing!
 //
@@ -117,16 +130,27 @@ GrowableArrayCHeap<oop, mtClassShared>* HeapShared::_pending_roots = NULL;
 narrowOop HeapShared::_roots_narrow;
 OopHandle HeapShared::_roots;
 
+#ifdef ASSERT
+bool HeapShared::is_archived_object_during_dumptime(oop p) {
+  assert(HeapShared::can_write(), "must be");
+  assert(DumpSharedSpaces, "this function is only used with -Xshare:dump");
+  return Universe::heap()->is_archived_object(p);
+}
+#endif
+
 ////////////////////////////////////////////////////////////////
 //
 // Java heap object archiving support
 //
 ////////////////////////////////////////////////////////////////
-void HeapShared::fixup_mapped_heap_regions() {
-  FileMapInfo *mapinfo = FileMapInfo::current_info();
-  mapinfo->fixup_mapped_heap_regions();
-  set_archive_heap_region_fixed();
+void HeapShared::fixup_regions() {
+  FileMapInfo* mapinfo = FileMapInfo::current_info();
   if (is_mapped()) {
+    mapinfo->fixup_mapped_heap_regions();
+  } else if (_loading_failed) {
+    fill_failed_loaded_region();
+  }
+  if (is_fully_available()) {
     _roots = OopHandle(Universe::vm_global(), decode_from_archive(_roots_narrow));
     if (!MetaspaceShared::use_full_module_graph()) {
       // Need to remove all the archived java.lang.Module objects from HeapShared::roots().
@@ -137,8 +161,6 @@ void HeapShared::fixup_mapped_heap_regions() {
 }
 
 unsigned HeapShared::oop_hash(oop const& p) {
-  assert(!p->mark().has_bias_pattern(),
-         "this object should never have been locked");  // so identity_hash won't safepoin
   unsigned hash = (unsigned)p->identity_hash();
   return hash;
 }
@@ -201,7 +223,7 @@ int HeapShared::append_root(oop obj) {
 objArrayOop HeapShared::roots() {
   if (DumpSharedSpaces) {
     assert(Thread::current() == (Thread*)VMThread::vm_thread(), "should be in vm thread");
-    if (!is_heap_object_archiving_allowed()) {
+    if (!HeapShared::can_write()) {
       return NULL;
     }
   } else {
@@ -215,7 +237,7 @@ objArrayOop HeapShared::roots() {
 
 void HeapShared::set_roots(narrowOop roots) {
   assert(UseSharedSpaces, "runtime only");
-  assert(open_archive_heap_region_mapped(), "must be");
+  assert(is_fully_available(), "must be");
   _roots_narrow = roots;
 }
 
@@ -240,7 +262,7 @@ oop HeapShared::get_root(int index, bool clear) {
 void HeapShared::clear_root(int index) {
   assert(index >= 0, "sanity");
   assert(UseSharedSpaces, "must be");
-  if (open_archive_heap_region_mapped()) {
+  if (is_fully_available()) {
     if (log_is_enabled(Debug, cds, heap)) {
       oop old = roots()->obj_at(index);
       log_debug(cds, heap)("Clearing root %d: was " PTR_FORMAT, index, p2i(old));
@@ -249,7 +271,7 @@ void HeapShared::clear_root(int index) {
   }
 }
 
-oop HeapShared::archive_heap_object(oop obj) {
+oop HeapShared::archive_object(oop obj) {
   assert(DumpSharedSpaces, "dump-time only");
 
   oop ao = find_archived_heap_object(obj);
@@ -317,7 +339,7 @@ void HeapShared::archive_klass_objects() {
 }
 
 void HeapShared::run_full_gc_in_vm_thread() {
-  if (is_heap_object_archiving_allowed()) {
+  if (HeapShared::can_write()) {
     // Avoid fragmentation while archiving heap objects.
     // We do this inside a safepoint, so that no further allocation can happen after GC
     // has finished.
@@ -335,8 +357,8 @@ void HeapShared::run_full_gc_in_vm_thread() {
   }
 }
 
-void HeapShared::archive_java_heap_objects(GrowableArray<MemRegion>* closed,
-                                           GrowableArray<MemRegion>* open) {
+void HeapShared::archive_objects(GrowableArray<MemRegion>* closed_regions,
+                                 GrowableArray<MemRegion>* open_regions) {
 
   G1HeapVerifier::verify_ready_for_archiving();
 
@@ -349,10 +371,10 @@ void HeapShared::archive_java_heap_objects(GrowableArray<MemRegion>* closed,
     log_info(cds)("Heap range = [" PTR_FORMAT " - "  PTR_FORMAT "]",
                   p2i(CompressedOops::begin()), p2i(CompressedOops::end()));
     log_info(cds)("Dumping objects to closed archive heap region ...");
-    copy_closed_archive_heap_objects(closed);
+    copy_closed_objects(closed_regions);
 
     log_info(cds)("Dumping objects to open archive heap region ...");
-    copy_open_archive_heap_objects(open);
+    copy_open_objects(open_regions);
 
     destroy_archived_object_cache();
   }
@@ -360,9 +382,8 @@ void HeapShared::archive_java_heap_objects(GrowableArray<MemRegion>* closed,
   G1HeapVerifier::verify_archive_regions();
 }
 
-void HeapShared::copy_closed_archive_heap_objects(
-                                    GrowableArray<MemRegion> * closed_archive) {
-  assert(is_heap_object_archiving_allowed(), "Cannot archive java heap objects");
+void HeapShared::copy_closed_objects(GrowableArray<MemRegion>* closed_regions) {
+  assert(HeapShared::can_write(), "must be");
 
   G1CollectedHeap::heap()->begin_archive_alloc_range();
 
@@ -374,13 +395,12 @@ void HeapShared::copy_closed_archive_heap_objects(
                            true /* is_closed_archive */,
                            false /* is_full_module_graph */);
 
-  G1CollectedHeap::heap()->end_archive_alloc_range(closed_archive,
+  G1CollectedHeap::heap()->end_archive_alloc_range(closed_regions,
                                                    os::vm_allocation_granularity());
 }
 
-void HeapShared::copy_open_archive_heap_objects(
-                                    GrowableArray<MemRegion> * open_archive) {
-  assert(is_heap_object_archiving_allowed(), "Cannot archive java heap objects");
+void HeapShared::copy_open_objects(GrowableArray<MemRegion>* open_regions) {
+  assert(HeapShared::can_write(), "must be");
 
   G1CollectedHeap::heap()->begin_archive_alloc_range(true /* open */);
 
@@ -402,7 +422,7 @@ void HeapShared::copy_open_archive_heap_objects(
 
   copy_roots();
 
-  G1CollectedHeap::heap()->end_archive_alloc_range(open_archive,
+  G1CollectedHeap::heap()->end_archive_alloc_range(open_regions,
                                                    os::vm_allocation_granularity());
 }
 
@@ -416,11 +436,7 @@ void HeapShared::copy_roots() {
   memset(mem, 0, size * BytesPerWord);
   {
     // This is copied from MemAllocator::finish
-    if (UseBiasedLocking) {
-      oopDesc::set_mark(mem, k->prototype_header());
-    } else {
-      oopDesc::set_mark(mem, markWord::prototype());
-    }
+    oopDesc::set_mark(mem, markWord::prototype());
     oopDesc::release_set_klass(mem, k);
   }
   {
@@ -658,7 +674,7 @@ void HeapShared::serialize_subgraph_info_table_header(SerializeClosure* soc) {
 }
 
 static void verify_the_heap(Klass* k, const char* which) {
-  if (VerifyArchivedFields > 0) {
+  if (VerifyArchivedFields) {
     ResourceMark rm;
     log_info(cds, heap)("Verify heap %s initializing static field(s) in %s",
                         which, k->external_name());
@@ -666,20 +682,15 @@ static void verify_the_heap(Klass* k, const char* which) {
     VM_Verify verify_op;
     VMThread::execute(&verify_op);
 
-    if (VerifyArchivedFields > 1 && is_init_completed()) {
-      // At this time, the oop->klass() of some archived objects in the heap may not
-      // have been loaded into the system dictionary yet. Nevertheless, oop->klass() should
-      // have enough information (object size, oop maps, etc) so that a GC can be safely
-      // performed.
-      //
-      // -XX:VerifyArchivedFields=2 force a GC to happen in such an early stage
-      // to check for GC safety.
-      log_info(cds, heap)("Trigger GC %s initializing static field(s) in %s",
-                          which, k->external_name());
-      FlagSetting fs1(VerifyBeforeGC, true);
-      FlagSetting fs2(VerifyDuringGC, true);
-      FlagSetting fs3(VerifyAfterGC,  true);
-      Universe::heap()->collect(GCCause::_java_lang_system_gc);
+    if (!FLAG_IS_DEFAULT(VerifyArchivedFields)) {
+      // If VerifyArchivedFields has a non-default value (e.g., specified on the command-line), do
+      // more expensive checks.
+      if (is_init_completed()) {
+        FlagSetting fs1(VerifyBeforeGC, true);
+        FlagSetting fs2(VerifyDuringGC, true);
+        FlagSetting fs3(VerifyAfterGC,  true);
+        Universe::heap()->collect(GCCause::_java_lang_system_gc);
+      }
     }
   }
 }
@@ -691,7 +702,7 @@ static void verify_the_heap(Klass* k, const char* which) {
 // ClassFileLoadHook is enabled, it's possible for this class to be dynamically replaced. In
 // this case, we will not load the ArchivedKlassSubGraphInfoRecord and will clear its roots.
 void HeapShared::resolve_classes(JavaThread* THREAD) {
-  if (!is_mapped()) {
+  if (!is_fully_available()) {
     return; // nothing to do
   }
   resolve_classes_for_subgraphs(closed_archive_subgraph_entry_fields,
@@ -729,7 +740,7 @@ void HeapShared::resolve_classes_for_subgraph_of(Klass* k, JavaThread* THREAD) {
 }
 
 void HeapShared::initialize_from_archived_subgraph(Klass* k, JavaThread* THREAD) {
-  if (!is_mapped()) {
+  if (!is_fully_available()) {
     return; // nothing to do
   }
 
@@ -888,7 +899,7 @@ class WalkOopAndArchiveClosure: public BasicOopIterateClosure {
   template <class T> void do_oop_work(T *p) {
     oop obj = RawAccess<>::oop_load(p);
     if (!CompressedOops::is_null(obj)) {
-      assert(!HeapShared::is_archived_object(obj),
+      assert(!HeapShared::is_archived_object_during_dumptime(obj),
              "original objects must not point to archived objects");
 
       size_t field_delta = pointer_delta(p, _orig_referencing_obj, sizeof(char));
@@ -907,7 +918,7 @@ class WalkOopAndArchiveClosure: public BasicOopIterateClosure {
       oop archived = HeapShared::archive_reachable_objects_from(
           _level + 1, _subgraph_info, obj, _is_closed_archive);
       assert(archived != NULL, "VM should have exited with unarchivable objects for _level > 1");
-      assert(HeapShared::is_archived_object(archived), "must be");
+      assert(HeapShared::is_archived_object_during_dumptime(archived), "must be");
 
       if (!_record_klasses_only) {
         // Update the reference in the archived copy of the referencing object.
@@ -919,7 +930,7 @@ class WalkOopAndArchiveClosure: public BasicOopIterateClosure {
   }
 };
 
-void HeapShared::check_closed_archive_heap_region_object(InstanceKlass* k) {
+void HeapShared::check_closed_region_object(InstanceKlass* k) {
   // Check fields in the object
   for (JavaFieldStream fs(k); !fs.done(); fs.next()) {
     if (!fs.access_flags().is_static()) {
@@ -963,7 +974,7 @@ oop HeapShared::archive_reachable_objects_from(int level,
                                                oop orig_obj,
                                                bool is_closed_archive) {
   assert(orig_obj != NULL, "must be");
-  assert(!is_archived_object(orig_obj), "sanity");
+  assert(!is_archived_object_during_dumptime(orig_obj), "sanity");
 
   if (!JavaClasses::is_supported_for_archiving(orig_obj)) {
     // This object has injected fields that cannot be supported easily, so we disallow them for now.
@@ -1001,7 +1012,7 @@ oop HeapShared::archive_reachable_objects_from(int level,
   bool record_klasses_only = (archived_obj != NULL);
   if (archived_obj == NULL) {
     ++_num_new_archived_objs;
-    archived_obj = archive_heap_object(orig_obj);
+    archived_obj = archive_object(orig_obj);
     if (archived_obj == NULL) {
       // Skip archiving the sub-graph referenced from the current entry field.
       ResourceMark rm;
@@ -1029,7 +1040,7 @@ oop HeapShared::archive_reachable_objects_from(int level,
       // class_data will be restored explicitly at run time.
       guarantee(orig_obj == SystemDictionary::java_platform_loader() ||
                 orig_obj == SystemDictionary::java_system_loader() ||
-                java_lang_ClassLoader::loader_data_raw(orig_obj) == NULL, "must be");
+                java_lang_ClassLoader::loader_data(orig_obj) == NULL, "must be");
       java_lang_ClassLoader::release_set_loader_data(archived_obj, NULL);
     }
   }
@@ -1042,7 +1053,7 @@ oop HeapShared::archive_reachable_objects_from(int level,
                                   subgraph_info, orig_obj, archived_obj);
   orig_obj->oop_iterate(&walker);
   if (is_closed_archive && orig_k->is_instance_klass()) {
-    check_closed_archive_heap_region_object(InstanceKlass::cast(orig_k));
+    check_closed_region_object(InstanceKlass::cast(orig_k));
   }
   return archived_obj;
 }
@@ -1181,10 +1192,10 @@ void HeapShared::verify_reachable_objects_from(oop obj, bool is_archived) {
     set_has_been_seen_during_subgraph_recording(obj);
 
     if (is_archived) {
-      assert(is_archived_object(obj), "must be");
+      assert(is_archived_object_during_dumptime(obj), "must be");
       assert(find_archived_heap_object(obj) == NULL, "must be");
     } else {
-      assert(!is_archived_object(obj), "must be");
+      assert(!is_archived_object_during_dumptime(obj), "must be");
       assert(find_archived_heap_object(obj) != NULL, "must be");
     }
 
@@ -1284,7 +1295,7 @@ void HeapShared::init_subgraph_entry_fields(ArchivableStaticFieldInfo fields[],
 }
 
 void HeapShared::init_subgraph_entry_fields(TRAPS) {
-  assert(is_heap_object_archiving_allowed(), "Sanity check");
+  assert(HeapShared::can_write(), "must be");
   _dump_time_subgraph_info_table = new (ResourceObj::C_HEAP, mtClass)DumpTimeKlassSubGraphInfoTable();
   init_subgraph_entry_fields(closed_archive_subgraph_entry_fields,
                              num_closed_archive_subgraph_entry_fields,
@@ -1300,7 +1311,7 @@ void HeapShared::init_subgraph_entry_fields(TRAPS) {
 }
 
 void HeapShared::init_for_dumping(TRAPS) {
-  if (is_heap_object_archiving_allowed()) {
+  if (HeapShared::can_write()) {
     _dumped_interned_strings = new (ResourceObj::C_HEAP, mtClass)DumpedInternedStrings();
     init_subgraph_entry_fields(CHECK);
   }
@@ -1444,8 +1455,10 @@ class PatchEmbeddedPointers: public BitMapClosure {
   }
 };
 
-void HeapShared::patch_archived_heap_embedded_pointers(MemRegion region, address oopmap,
-                                                       size_t oopmap_size_in_bits) {
+// Patch all the non-null pointers that are embedded in the archived heap objects
+// in this region
+void HeapShared::patch_embedded_pointers(MemRegion region, address oopmap,
+                                         size_t oopmap_size_in_bits) {
   BitMapView bm((BitMap::bm_word_t*)oopmap, oopmap_size_in_bits);
 
 #ifndef PRODUCT
@@ -1458,4 +1471,270 @@ void HeapShared::patch_archived_heap_embedded_pointers(MemRegion region, address
   bm.iterate(&patcher);
 }
 
+// The CDS archive remembers each heap object by its address at dump time, but
+// the heap object may be loaded at a different address at run time. This structure is used
+// to translate the dump time addresses for all objects in FileMapInfo::space_at(region_index)
+// to their runtime addresses.
+struct LoadedArchiveHeapRegion {
+  int       _region_index;   // index for FileMapInfo::space_at(index)
+  size_t    _region_size;    // number of bytes in this region
+  uintptr_t _dumptime_base;  // The dump-time (decoded) address of the first object in this region
+  intx      _runtime_offset; // If an object's dump time address P is within in this region, its
+                             // runtime address is P + _runtime_offset
+
+  static int comparator(const void* a, const void* b) {
+    LoadedArchiveHeapRegion* reg_a = (LoadedArchiveHeapRegion*)a;
+    LoadedArchiveHeapRegion* reg_b = (LoadedArchiveHeapRegion*)b;
+    if (reg_a->_dumptime_base < reg_b->_dumptime_base) {
+      return -1;
+    } else if (reg_a->_dumptime_base == reg_b->_dumptime_base) {
+      return 0;
+    } else {
+      return 1;
+    }
+  }
+
+  uintptr_t top() {
+    return _dumptime_base + _region_size;
+  }
+};
+
+void HeapShared::init_loaded_heap_relocation(LoadedArchiveHeapRegion* loaded_regions,
+                                             int num_loaded_regions) {
+  _dumptime_base_0 = loaded_regions[0]._dumptime_base;
+  _dumptime_base_1 = loaded_regions[1]._dumptime_base;
+  _dumptime_base_2 = loaded_regions[2]._dumptime_base;
+  _dumptime_base_3 = loaded_regions[3]._dumptime_base;
+  _dumptime_top = loaded_regions[num_loaded_regions-1].top();
+
+  _runtime_offset_0 = loaded_regions[0]._runtime_offset;
+  _runtime_offset_1 = loaded_regions[1]._runtime_offset;
+  _runtime_offset_2 = loaded_regions[2]._runtime_offset;
+  _runtime_offset_3 = loaded_regions[3]._runtime_offset;
+
+  assert(2 <= num_loaded_regions && num_loaded_regions <= 4, "must be");
+  if (num_loaded_regions < 4) {
+    _dumptime_base_3 = UINTPTR_MAX;
+  }
+  if (num_loaded_regions < 3) {
+    _dumptime_base_2 = UINTPTR_MAX;
+  }
+}
+
+bool HeapShared::can_load() {
+  return Universe::heap()->can_load_archived_objects();
+}
+
+template <int NUM_LOADED_REGIONS>
+class PatchLoadedRegionPointers: public BitMapClosure {
+  narrowOop* _start;
+  intx _offset_0;
+  intx _offset_1;
+  intx _offset_2;
+  intx _offset_3;
+  uintptr_t _base_0;
+  uintptr_t _base_1;
+  uintptr_t _base_2;
+  uintptr_t _base_3;
+  uintptr_t _top;
+
+  static_assert(MetaspaceShared::max_num_heap_regions == 4, "can't handle more than 4 regions");
+  static_assert(NUM_LOADED_REGIONS >= 2, "we have at least 2 loaded regions");
+  static_assert(NUM_LOADED_REGIONS <= 4, "we have at most 4 loaded regions");
+
+ public:
+  PatchLoadedRegionPointers(narrowOop* start, LoadedArchiveHeapRegion* loaded_regions)
+    : _start(start),
+      _offset_0(loaded_regions[0]._runtime_offset),
+      _offset_1(loaded_regions[1]._runtime_offset),
+      _offset_2(loaded_regions[2]._runtime_offset),
+      _offset_3(loaded_regions[3]._runtime_offset),
+      _base_0(loaded_regions[0]._dumptime_base),
+      _base_1(loaded_regions[1]._dumptime_base),
+      _base_2(loaded_regions[2]._dumptime_base),
+      _base_3(loaded_regions[3]._dumptime_base) {
+    _top = loaded_regions[NUM_LOADED_REGIONS-1].top();
+  }
+
+  bool do_bit(size_t offset) {
+    narrowOop* p = _start + offset;
+    narrowOop v = *p;
+    assert(!CompressedOops::is_null(v), "null oops should have been filtered out at dump time");
+    uintptr_t o = cast_from_oop<uintptr_t>(HeapShared::decode_from_archive(v));
+    assert(_base_0 <= o && o < _top, "must be");
+
+
+    // We usually have only 2 regions for the default archive. Use template to avoid unnecessary comparisons.
+    if (NUM_LOADED_REGIONS > 3 && o >= _base_3) {
+      o += _offset_3;
+    } else if (NUM_LOADED_REGIONS > 2 && o >= _base_2) {
+      o += _offset_2;
+    } else if (o >= _base_1) {
+      o += _offset_1;
+    } else {
+      o += _offset_0;
+    }
+    HeapShared::assert_in_loaded_heap(o);
+    RawAccess<IS_NOT_NULL>::oop_store(p, cast_to_oop(o));
+    return true;
+  }
+};
+
+int HeapShared::init_loaded_regions(FileMapInfo* mapinfo, LoadedArchiveHeapRegion* loaded_regions,
+                                    uintptr_t* buffer_ret) {
+  size_t total_bytes = 0;
+  int num_loaded_regions = 0;
+  for (int i = MetaspaceShared::first_archive_heap_region;
+       i <= MetaspaceShared::last_archive_heap_region; i++) {
+    FileMapRegion* r = mapinfo->space_at(i);
+    r->assert_is_heap_region();
+    if (r->used() > 0) {
+      assert(is_aligned(r->used(), HeapWordSize), "must be");
+      total_bytes += r->used();
+      LoadedArchiveHeapRegion* ri = &loaded_regions[num_loaded_regions++];
+      ri->_region_index = i;
+      ri->_region_size = r->used();
+      ri->_dumptime_base = (uintptr_t)mapinfo->start_address_as_decoded_from_archive(r);
+    }
+  }
+
+  assert(is_aligned(total_bytes, HeapWordSize), "must be");
+  uintptr_t buffer = (uintptr_t)
+    Universe::heap()->allocate_loaded_archive_space(total_bytes / HeapWordSize);
+  _loaded_heap_bottom = buffer;
+  _loaded_heap_top    = buffer + total_bytes;
+
+  *buffer_ret = buffer;
+  return num_loaded_regions;
+}
+
+void HeapShared::sort_loaded_regions(LoadedArchiveHeapRegion* loaded_regions, int num_loaded_regions,
+                                     uintptr_t buffer) {
+  // Find the relocation offset of the pointers in each region
+  qsort(loaded_regions, num_loaded_regions, sizeof(LoadedArchiveHeapRegion),
+        LoadedArchiveHeapRegion::comparator);
+
+  uintptr_t p = buffer;
+  for (int i = 0; i < num_loaded_regions; i++) {
+    // This region will be loaded at p, so all objects inside this
+    // region will be shifted by ri->offset
+    LoadedArchiveHeapRegion* ri = &loaded_regions[i];
+    ri->_runtime_offset = p - ri->_dumptime_base;
+    p += ri->_region_size;
+  }
+  assert(p == _loaded_heap_top, "must be");
+}
+
+bool HeapShared::load_regions(FileMapInfo* mapinfo, LoadedArchiveHeapRegion* loaded_regions,
+                              int num_loaded_regions, uintptr_t buffer) {
+  uintptr_t bitmap_base = (uintptr_t)mapinfo->map_bitmap_region();
+  uintptr_t load_address = buffer;
+  for (int i = 0; i < num_loaded_regions; i++) {
+    LoadedArchiveHeapRegion* ri = &loaded_regions[i];
+    FileMapRegion* r = mapinfo->space_at(ri->_region_index);
+
+    if (!mapinfo->read_region(ri->_region_index, (char*)load_address, r->used())) {
+      // There's no easy way to free the buffer, so we will fill it with zero later
+      // in fill_failed_loaded_region(), and it will eventually be GC'ed.
+      log_warning(cds)("Loading of heap region %d has failed. Archived objects are disabled", i);
+      _loading_failed = true;
+      return false;
+    }
+    log_info(cds)("Loaded heap    region #%d at base " INTPTR_FORMAT " size = " SIZE_FORMAT_W(8) " bytes, delta = " INTX_FORMAT,
+                  ri->_region_index, load_address, ri->_region_size, ri->_runtime_offset);
+
+    uintptr_t oopmap = bitmap_base + r->oopmap_offset();
+    BitMapView bm((BitMap::bm_word_t*)oopmap, r->oopmap_size_in_bits());
+
+    if (num_loaded_regions == 4) {
+      PatchLoadedRegionPointers<4> patcher((narrowOop*)load_address, loaded_regions);
+      bm.iterate(&patcher);
+    } else if (num_loaded_regions == 3) {
+      PatchLoadedRegionPointers<3> patcher((narrowOop*)load_address, loaded_regions);
+      bm.iterate(&patcher);
+    } else {
+      assert(num_loaded_regions == 2, "must be");
+      PatchLoadedRegionPointers<2> patcher((narrowOop*)load_address, loaded_regions);
+      bm.iterate(&patcher);
+    }
+
+    load_address += r->used();
+  }
+
+  return true;
+}
+
+bool HeapShared::load_heap_regions(FileMapInfo* mapinfo) {
+  init_narrow_oop_decoding(mapinfo->narrow_oop_base(), mapinfo->narrow_oop_shift());
+
+  LoadedArchiveHeapRegion loaded_regions[MetaspaceShared::max_num_heap_regions];
+  memset(loaded_regions, 0, sizeof(loaded_regions));
+
+  uintptr_t buffer;
+  int num_loaded_regions = init_loaded_regions(mapinfo, loaded_regions, &buffer);
+  sort_loaded_regions(loaded_regions, num_loaded_regions, buffer);
+  if (!load_regions(mapinfo, loaded_regions, num_loaded_regions, buffer)) {
+    return false;
+  }
+
+  init_loaded_heap_relocation(loaded_regions, num_loaded_regions);
+  _is_loaded = true;
+  set_roots(mapinfo->heap_obj_roots());
+
+  return true;
+}
+
+class VerifyLoadedHeapEmbeddedPointers: public BasicOopIterateClosure {
+  ResourceHashtable<uintptr_t, bool>* _table;
+
+ public:
+  VerifyLoadedHeapEmbeddedPointers(ResourceHashtable<uintptr_t, bool>* table) : _table(table) {}
+
+  virtual void do_oop(narrowOop* p) {
+    // This should be called before the loaded regions are modified, so all the embedded pointers
+    // must be NULL, or must point to a valid object in the loaded regions.
+    narrowOop v = *p;
+    if (!CompressedOops::is_null(v)) {
+      oop o = CompressedOops::decode_not_null(v);
+      uintptr_t u = cast_from_oop<uintptr_t>(o);
+      HeapShared::assert_in_loaded_heap(u);
+      guarantee(_table->contains(u), "must point to beginning of object in loaded archived regions");
+    }
+  }
+  virtual void do_oop(oop* p) {
+    ShouldNotReachHere();
+  }
+};
+
+void HeapShared::verify_loaded_heap() {
+  if (!VerifyArchivedFields || !is_loaded()) {
+    return;
+  }
+
+  ResourceMark rm;
+  ResourceHashtable<uintptr_t, bool> table;
+  VerifyLoadedHeapEmbeddedPointers verifier(&table);
+  HeapWord* bottom = (HeapWord*)_loaded_heap_bottom;
+  HeapWord* top    = (HeapWord*)_loaded_heap_top;
+
+  for (HeapWord* p = bottom; p < top; ) {
+    oop o = cast_to_oop(p);
+    table.put(cast_from_oop<uintptr_t>(o), true);
+    p += o->size();
+  }
+
+  for (HeapWord* p = bottom; p < top; ) {
+    oop o = cast_to_oop(p);
+    o->oop_iterate(&verifier);
+    p += o->size();
+  }
+}
+
+void HeapShared::fill_failed_loaded_region() {
+  assert(_loading_failed, "must be");
+  HeapWord* bottom = (HeapWord*)_loaded_heap_bottom;
+  HeapWord* top = (HeapWord*)_loaded_heap_top;
+  Universe::heap()->fill_with_objects(bottom, top - bottom);
+}
+
 #endif // INCLUDE_CDS_JAVA_HEAP
