@@ -23,10 +23,9 @@
  */
 
 #include "precompiled.hpp"
+#include "ci/ciCallSite.hpp"
 #include "ci/ciConstant.hpp"
 #include "ci/ciField.hpp"
-#include "ci/ciKlass.hpp"
-#include "ci/ciObjArrayKlass.hpp"
 #include "ci/ciStreams.hpp"
 #include "ci/ciSymbols.hpp"
 #include "ci/ciUtilities.inline.hpp"
@@ -192,21 +191,6 @@ ciKlass* ciBytecodeStream::get_klass(bool& will_link) {
   return CURRENT_ENV->get_klass_by_index(cpool, get_klass_index(), will_link, _holder);
 }
 
-// ciBytecodeStream::get_klass
-//
-// If this bytecode is a new, newarray, multianewarray, instanceof,
-// or checkcast, get the referenced klass. Retuns an unloaded ciKlass
-// if the referenced klass is not accessible.
-ciKlass* ciBytecodeStream::get_klass() {
-  bool will_link;
-  ciKlass* klass = get_klass(will_link);
-  if (!will_link && klass->is_loaded()) { // klass not accessible
-    VM_ENTRY_MARK;
-    klass = CURRENT_ENV->get_unloaded_klass(_holder, klass->name());
-  }
-  return klass;
-}
-
 // ------------------------------------------------------------------
 // ciBytecodeStream::get_constant_raw_index
 //
@@ -267,14 +251,6 @@ constantTag ciBytecodeStream::get_constant_pool_tag(int index) const {
   return _method->get_Method()->constants()->constant_tag_at(index);
 }
 
-// ------------------------------------------------------------------
-// ciBytecodeStream::get_basic_type_for_constant_at
-//
-BasicType ciBytecodeStream::get_basic_type_for_constant_at(int index) const {
-  VM_ENTRY_MARK;
-  return _method->get_Method()->constants()->basic_type_for_constant_at(index);
-}
-
 // ------------------------------------------------------------------
 // ciBytecodeStream::get_field_index
 //
