@@ -197,8 +197,6 @@ PackageEntryTable::~PackageEntryTable() {
 typedef ResourceHashtable<
   const PackageEntry*,
   PackageEntry*,
-  primitive_hash<const PackageEntry*>,
-  primitive_equals<const PackageEntry*>,
   557, // prime number
   ResourceObj::C_HEAP> ArchivedPackageEntries;
 static ArchivedPackageEntries* _archived_packages_entries = NULL;
