@@ -148,7 +148,6 @@
                                                                             \
   product(uint, ConcGCThreads, 0,                                           \
           "Number of threads concurrent gc will use")                       \
-          constraint(ConcGCThreadsConstraintFunc,AfterErgo)                 \
                                                                             \
   product(bool, AlwaysTenure, false,                                        \
           "Always tenure objects in eden (ParallelGC only)")                \
@@ -524,12 +523,8 @@
   product(bool, VerifyDuringGC, false, DIAGNOSTIC,                          \
           "Verify memory system during GC (between phases)")                \
                                                                             \
-  product(int, VerifyArchivedFields, 0, DIAGNOSTIC,                         \
-          "Verify memory when archived oop fields are loaded from CDS; "    \
-          "0: No check; "                                                   \
-          "1: Basic verification with VM_Verify (no side effects); "        \
-          "2: Detailed verification by forcing a GC (with side effects)")   \
-          range(0, 2)                                                       \
+  product(bool, VerifyArchivedFields, trueInDebug, DIAGNOSTIC,              \
+          "Verify memory when archived oop fields are loaded from CDS)")    \
                                                                             \
   product(ccstrlist, VerifyGCType, "", DIAGNOSTIC,                          \
              "GC type(s) to verify when Verify*GC is enabled."              \
