@@ -62,7 +62,6 @@
 #include "memory/resourceArea.hpp"
 #include "memory/universe.hpp"
 #include "oops/oop.inline.hpp"
-#include "runtime/biasedLocking.hpp"
 #include "runtime/handles.hpp"
 #include "runtime/handles.inline.hpp"
 #include "runtime/java.hpp"
@@ -439,12 +438,11 @@ bool GenCollectedHeap::must_clear_all_soft_refs() {
 }
 
 void GenCollectedHeap::collect_generation(Generation* gen, bool full, size_t size,
-                                          bool is_tlab, bool run_verification, bool clear_soft_refs,
-                                          bool restore_marks_for_biased_locking) {
+                                          bool is_tlab, bool run_verification, bool clear_soft_refs) {
   FormatBuffer<> title("Collect gen: %s", gen->short_name());
   GCTraceTime(Trace, gc, phases) t1(title);
   TraceCollectorStats tcs(gen->counters());
-  TraceMemoryManagerStats tmms(gen->gc_manager(), gc_cause(), heap()->is_young_gen(gen) ? "end of minor GC" : "end of major GC");
+  TraceMemoryManagerStats tmms(gen->gc_manager(), gc_cause());
 
   gen->stat_record()->invocations++;
   gen->stat_record()->accumulated_time.start();
@@ -461,14 +459,6 @@ void GenCollectedHeap::collect_generation(Generation* gen, bool full, size_t siz
   }
   COMPILER2_OR_JVMCI_PRESENT(DerivedPointerTable::clear());
 
-  if (restore_marks_for_biased_locking) {
-    // We perform this mark word preservation work lazily
-    // because it's only at this point that we know whether we
-    // absolutely have to do it; we want to avoid doing it for
-    // scavenge-only collections where it's unnecessary
-    BiasedLocking::preserve_marks();
-  }
-
   // Do collection work
   {
     // Note on ref discovery: For what appear to be historical reasons,
@@ -488,21 +478,11 @@ void GenCollectedHeap::collect_generation(Generation* gen, bool full, size_t siz
     // enqueue_discovered_references if the generation returns
     // without doing any work.
     ReferenceProcessor* rp = gen->ref_processor();
-    // If the discovery of ("weak") refs in this generation is
-    // atomic wrt other collectors in this configuration, we
-    // are guaranteed to have empty discovered ref lists.
-    if (rp->discovery_is_atomic()) {
-      rp->enable_discovery();
-      rp->setup_policy(clear_soft_refs);
-    } else {
-      // collect() below will enable discovery as appropriate
-    }
+    rp->start_discovery(clear_soft_refs);
+
     gen->collect(full, clear_soft_refs, size, is_tlab);
-    if (!rp->enqueuing_is_done()) {
-      rp->disable_discovery();
-    } else {
-      rp->set_enqueuing_is_done(false);
-    }
+
+    rp->disable_discovery();
     rp->verify_no_references_recorded();
   }
 
@@ -572,8 +552,7 @@ void GenCollectedHeap::do_collection(bool           full,
                        size,
                        is_tlab,
                        run_verification && VerifyGCLevel <= 0,
-                       do_clear_all_soft_refs,
-                       false);
+                       do_clear_all_soft_refs);
 
     if (size > 0 && (!is_tlab || _young_gen->supports_tlab_allocation()) &&
         size * HeapWordSize <= _young_gen->unsafe_max_alloc_nogc()) {
@@ -632,8 +611,7 @@ void GenCollectedHeap::do_collection(bool           full,
                        size,
                        is_tlab,
                        run_verification && VerifyGCLevel <= 1,
-                       do_clear_all_soft_refs,
-                       true);
+                       do_clear_all_soft_refs);
 
     // Adjust generation sizes.
     _old_gen->compute_new_size();
@@ -655,8 +633,6 @@ void GenCollectedHeap::do_collection(bool           full,
     // the initial value for "complete" flag.
     gc_epilogue(true);
 
-    BiasedLocking::restore_marks();
-
     print_heap_after_gc();
   }
 }
