@@ -243,12 +243,25 @@ inline bool ShenandoahHeap::cancelled_gc() const {
 }
 
 inline bool ShenandoahHeap::check_cancelled_gc_and_yield(bool sts_active) {
-  if (sts_active && ShenandoahSuspendibleWorkers && !cancelled_gc()) {
+  if (! (sts_active && ShenandoahSuspendibleWorkers)) {
+    return cancelled_gc();
+  }
+
+  jbyte prev = _cancelled_gc.cmpxchg(NOT_CANCELLED, CANCELLABLE);
+  if (prev == CANCELLABLE || prev == NOT_CANCELLED) {
     if (SuspendibleThreadSet::should_yield()) {
       SuspendibleThreadSet::yield();
     }
+
+    // Back to CANCELLABLE. The thread that poked NOT_CANCELLED first gets
+    // to restore to CANCELLABLE.
+    if (prev == CANCELLABLE) {
+      _cancelled_gc.set(CANCELLABLE);
+    }
+    return false;
+  } else {
+    return true;
   }
-  return cancelled_gc();
 }
 
 inline void ShenandoahHeap::clear_cancelled_gc() {
