@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -41,7 +41,6 @@
 #include "runtime/handles.inline.hpp"
 #include "runtime/fieldDescriptor.inline.hpp"
 #include "runtime/java.hpp"
-#include "runtime/javaCalls.hpp"
 #include "runtime/jniHandles.inline.hpp"
 #include "runtime/semaphore.inline.hpp"
 #include "runtime/synchronizer.hpp"
@@ -550,16 +549,14 @@ void JfrJavaSupport::throw_runtime_exception(const char* message, TRAPS) {
 
 void JfrJavaSupport::abort(jstring errorMsg, JavaThread* t) {
   DEBUG_ONLY(check_java_thread_in_vm(t));
-  ResourceMark rm(t);
-  abort(c_str(errorMsg, t));
-}
 
-void JfrJavaSupport::abort(const char* error_msg, bool dump_core /* true */) {
-  if (error_msg != nullptr) {
-    log_error(jfr, system)("%s", error_msg);
+  ResourceMark rm(t);
+  const char* const error_msg = c_str(errorMsg, t);
+  if (error_msg != NULL) {
+    log_error(jfr, system)("%s",error_msg);
   }
   log_error(jfr, system)("%s", "An irrecoverable error in Jfr. Shutting down VM...");
-  vm_abort(dump_core);
+  vm_abort();
 }
 
 JfrJavaSupport::CAUSE JfrJavaSupport::_cause = JfrJavaSupport::VM_ERROR;
@@ -608,23 +605,6 @@ JfrJavaSupport::CAUSE JfrJavaSupport::cause() {
 const char* const JDK_JFR_MODULE_NAME = "jdk.jfr";
 const char* const JDK_JFR_PACKAGE_NAME = "jdk/jfr";
 
-
-
-void JfrJavaSupport::load_jdk_jfr_module(TRAPS) {
-  DEBUG_ONLY(JfrJavaSupport::check_java_thread_in_vm(THREAD));
-  ResourceMark rm(THREAD);
-  HandleMark hm(THREAD);
-  Handle h_module_name = java_lang_String::create_from_str(JDK_JFR_MODULE_NAME, CHECK);
-  JavaValue result(T_OBJECT);
-  JavaCalls::call_static(&result,
-    vmClasses::module_Modules_klass(),
-    vmSymbols::loadModule_name(),
-    vmSymbols::loadModule_signature(),
-    h_module_name,
-    CHECK
-  );
-}
-
 static bool is_jdk_jfr_module_in_readability_graph() {
   // take one of the packages in the module to be located and query for its definition.
   TempNewSymbol pkg_sym = SymbolTable::new_symbol(JDK_JFR_PACKAGE_NAME);
@@ -744,21 +724,20 @@ static bool check_exclusion_state_on_thread_start(JavaThread* jt) {
   return true;
 }
 
-static JavaThread* get_native(ThreadsListHandle& tlh, jobject thread) {
+static JavaThread* get_native(jobject thread) {
+  ThreadsListHandle tlh;
   JavaThread* native_thread = NULL;
   (void)tlh.cv_internal_thread_to_JavaThread(thread, &native_thread, NULL);
   return native_thread;
 }
 
 jlong JfrJavaSupport::jfr_thread_id(jobject thread) {
-  ThreadsListHandle tlh;
-  JavaThread* native_thread = get_native(tlh, thread);
+  JavaThread* native_thread = get_native(thread);
   return native_thread != NULL ? JFR_THREAD_ID(native_thread) : 0;
 }
 
 void JfrJavaSupport::exclude(jobject thread) {
-  ThreadsListHandle tlh;
-  JavaThread* native_thread = get_native(tlh, thread);
+  JavaThread* native_thread = get_native(thread);
   if (native_thread != NULL) {
     JfrThreadLocal::exclude(native_thread);
   } else {
@@ -768,8 +747,7 @@ void JfrJavaSupport::exclude(jobject thread) {
 }
 
 void JfrJavaSupport::include(jobject thread) {
-  ThreadsListHandle tlh;
-  JavaThread* native_thread = get_native(tlh, thread);
+  JavaThread* native_thread = get_native(thread);
   if (native_thread != NULL) {
     JfrThreadLocal::include(native_thread);
   } else {
@@ -779,8 +757,7 @@ void JfrJavaSupport::include(jobject thread) {
 }
 
 bool JfrJavaSupport::is_excluded(jobject thread) {
-  ThreadsListHandle tlh;
-  JavaThread* native_thread = get_native(tlh, thread);
+  JavaThread* native_thread = get_native(thread);
   return native_thread != NULL ? native_thread->jfr_thread_local()->is_excluded() : is_thread_excluded(thread);
 }
 
@@ -844,9 +821,9 @@ void JfrJavaSupport::on_thread_start(Thread* t) {
   if (!t->is_Java_thread()) {
     return;
   }
-  DEBUG_ONLY(check_new_unstarted_java_thread(t->as_Java_thread());)
+  DEBUG_ONLY(check_new_unstarted_java_thread(JavaThread::cast(t));)
   HandleMark hm(t);
-  if (check_exclusion_state_on_thread_start(t->as_Java_thread())) {
+  if (check_exclusion_state_on_thread_start(JavaThread::cast(t))) {
     JfrThreadLocal::exclude(t);
   }
 }
