@@ -38,7 +38,6 @@
 #include "jfr/utilities/jfrIterator.hpp"
 #include "jfr/utilities/jfrLinkedList.inline.hpp"
 #include "jfr/utilities/jfrTime.hpp"
-#include "jfr/utilities/jfrTryLock.hpp"
 #include "jfr/writers/jfrNativeEventWriter.hpp"
 #include "logging/log.hpp"
 #include "runtime/mutexLocker.hpp"
@@ -329,8 +328,7 @@ static void log_discard(size_t pre_full_count, size_t post_full_count, size_t am
 }
 
 void JfrStorage::discard_oldest(Thread* thread) {
-  JfrMutexTryLock mutex(JfrBuffer_lock);
-  if (mutex.acquired()) {
+  if (JfrBuffer_lock->try_lock()) {
     if (!control().should_discard()) {
       // another thread handled it
       return;
@@ -352,6 +350,7 @@ void JfrStorage::discard_oldest(Thread* thread) {
       oldest->release(); // publish
       break;
     }
+    JfrBuffer_lock->unlock();
     log_discard(num_full_pre_discard, control().full_count(), discarded_size);
   }
 }
