@@ -254,7 +254,8 @@ class InstanceKlass: public Klass {
     _misc_is_shared_platform_class            = 1 << 11, // defining class loader is platform class loader
     _misc_is_shared_app_class                 = 1 << 12, // defining class loader is app class loader
     _misc_has_resolved_methods                = 1 << 13, // resolved methods table entries added for this class
-    _misc_has_contended_annotations           = 1 << 14  // has @Contended annotation
+    _misc_is_being_redefined                  = 1 << 14, // used for locking redefinition
+    _misc_has_contended_annotations           = 1 << 15  // has @Contended annotation
   };
   u2 shared_loader_type_bits() const {
     return _misc_is_shared_boot_class|_misc_is_shared_platform_class|_misc_is_shared_app_class;
@@ -737,16 +738,14 @@ public:
 
 #if INCLUDE_JVMTI
   // Redefinition locking.  Class can only be redefined by one thread at a time.
-  // The flag is in access_flags so that it can be set and reset using atomic
-  // operations, and not be reset by other misc_flag settings.
   bool is_being_redefined() const          {
-    return _access_flags.is_being_redefined();
+    return ((_misc_flags & _misc_is_being_redefined) != 0);
   }
   void set_is_being_redefined(bool value)  {
     if (value) {
-      _access_flags.set_is_being_redefined();
+      _misc_flags |= _misc_is_being_redefined;
     } else {
-      _access_flags.clear_is_being_redefined();
+      _misc_flags &= ~_misc_is_being_redefined;
     }
   }
 
@@ -815,7 +814,7 @@ public:
   }
 
  private:
-  static bool  _should_clean_previous_versions;
+  static bool  _has_previous_versions;
  public:
   static void purge_previous_versions(InstanceKlass* ik) {
     if (ik->has_been_redefined()) {
@@ -823,8 +822,8 @@ public:
     }
   }
 
-  static bool should_clean_previous_versions_and_reset();
-  static bool should_clean_previous_versions() { return _should_clean_previous_versions; }
+  static bool has_previous_versions_and_reset();
+  static bool has_previous_versions() { return _has_previous_versions; }
 
   // JVMTI: Support for caching a class file before it is modified by an agent that can do retransformation
   void set_cached_class_file(JvmtiCachedClassFileData *data) {
@@ -844,7 +843,7 @@ public:
 #else // INCLUDE_JVMTI
 
   static void purge_previous_versions(InstanceKlass* ik) { return; };
-  static bool should_clean_previous_versions_and_reset() { return false; }
+  static bool has_previous_versions_and_reset() { return false; }
 
   void set_cached_class_file(JvmtiCachedClassFileData *data) {
     assert(data == NULL, "unexpected call with JVMTI disabled");
@@ -1008,6 +1007,7 @@ public:
   void do_local_static_fields(FieldClosure* cl);
   void do_nonstatic_fields(FieldClosure* cl); // including inherited fields
   void do_local_static_fields(void f(fieldDescriptor*, Handle, TRAPS), Handle, TRAPS);
+  void print_nonstatic_fields(FieldClosure* cl); // including inherited and injected fields
 
   void methods_do(void f(Method* method));
   void array_klasses_do(void f(Klass* k));
@@ -1108,7 +1108,7 @@ public:
   // callbacks for actions during class unloading
   static void unload_class(InstanceKlass* ik);
 
-  virtual void release_C_heap_structures(bool release_constant_pool = true);
+  virtual void release_C_heap_structures();
 
   // Naming
   const char* signature_name() const;
@@ -1179,8 +1179,6 @@ private:
   bool idnum_can_increment() const      { return has_been_redefined(); }
   inline jmethodID* methods_jmethod_ids_acquire() const;
   inline void release_set_methods_jmethod_ids(jmethodID* jmeths);
-  // This nulls out jmethodIDs for all methods in 'klass'
-  static void clear_jmethod_ids(InstanceKlass* klass);
 
   // Lock during initialization
 public:
@@ -1207,6 +1205,8 @@ private:
   void initialize_impl                           (TRAPS);
   void initialize_super_interfaces               (TRAPS);
   void eager_initialize_impl                     ();
+  /* jni_id_for_impl for jfieldID only */
+  JNIid* jni_id_for_impl                         (int offset);
 
   void add_initialization_error(JavaThread* current, Handle exception);
   oop get_initialization_error(JavaThread* current);
@@ -1225,6 +1225,9 @@ private:
                                   StaticLookupMode static_mode,
                                   PrivateLookupMode private_mode);
 
+  // Free CHeap allocated fields.
+  void release_C_heap_structures_internal();
+
 #if INCLUDE_JVMTI
   // RedefineClasses support
   void link_previous_versions(InstanceKlass* pv) { _previous_versions = pv; }
