@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -24,9 +24,9 @@
 
 #include "precompiled.hpp"
 #include "ci/ciReplay.hpp"
-#include "classfile/systemDictionary.hpp"
 #include "classfile/vmSymbols.hpp"
 #include "compiler/compileBroker.hpp"
+#include "compiler/compilerEvent.hpp"
 #include "compiler/compileLog.hpp"
 #include "interpreter/linkResolver.hpp"
 #include "jfr/jfrEvents.hpp"
@@ -41,14 +41,13 @@
 InlineTree::InlineTree(Compile* c,
                        const InlineTree *caller_tree, ciMethod* callee,
                        JVMState* caller_jvms, int caller_bci,
-                       float site_invoke_ratio, int max_inline_level) :
+                       int max_inline_level) :
   C(c),
   _caller_jvms(caller_jvms),
-  _caller_tree((InlineTree*) caller_tree),
   _method(callee),
-  _site_invoke_ratio(site_invoke_ratio),
-  _max_inline_level(max_inline_level),
+  _caller_tree((InlineTree*) caller_tree),
   _count_inline_bcs(method()->code_size_for_inlining()),
+  _max_inline_level(max_inline_level),
   _subtrees(c->comp_arena(), 2, 0, NULL),
   _msg(NULL)
 {
@@ -113,24 +112,18 @@ static bool is_unboxing_method(ciMethod* callee_method, Compile* C) {
 
 // positive filter: should callee be inlined?
 bool InlineTree::should_inline(ciMethod* callee_method, ciMethod* caller_method,
-                               int caller_bci, ciCallProfile& profile,
-                               WarmCallInfo* wci_result) {
+                               int caller_bci, ciCallProfile& profile) {
   // Allows targeted inlining
   if (C->directive()->should_inline(callee_method)) {
-    *wci_result = *(WarmCallInfo::always_hot());
-    if (C->print_inlining() && Verbose) {
-      CompileTask::print_inline_indent(inline_level());
-      tty->print_cr("Inlined method is hot: ");
-    }
     set_msg("force inline by CompileCommand");
     _forced_inline = true;
     return true;
   }
 
   if (callee_method->force_inline()) {
-      set_msg("force inline by annotation");
-      _forced_inline = true;
-      return true;
+    set_msg("force inline by annotation");
+    _forced_inline = true;
+    return true;
   }
 
 #ifndef PRODUCT
@@ -147,7 +140,6 @@ bool InlineTree::should_inline(ciMethod* callee_method, ciMethod* caller_method,
   // Check for too many throws (and not too huge)
   if(callee_method->interpreter_throwout_count() > InlineThrowCount &&
      size < InlineThrowMaxSize ) {
-    wci_result->set_profit(wci_result->profit() * 100);
     if (C->print_inlining() && Verbose) {
       CompileTask::print_inline_indent(inline_level());
       tty->print_cr("Inlined method with many throws (throws=%d):", callee_method->interpreter_throwout_count());
@@ -164,18 +156,17 @@ bool InlineTree::should_inline(ciMethod* callee_method, ciMethod* caller_method,
   int invoke_count     = method()->interpreter_invocation_count();
 
   assert(invoke_count != 0, "require invocation count greater than zero");
-  int freq = call_site_count / invoke_count;
+  double freq = (double)call_site_count / (double)invoke_count;
 
   // bump the max size if the call is frequent
   if ((freq >= InlineFrequencyRatio) ||
-      (call_site_count >= InlineFrequencyCount) ||
       is_unboxing_method(callee_method, C) ||
       is_init_with_ea(callee_method, caller_method, C)) {
 
     max_inline_size = C->freq_inline_size();
     if (size <= max_inline_size && TraceFrequencyInlining) {
       CompileTask::print_inline_indent(inline_level());
-      tty->print_cr("Inlined frequent method (freq=%d count=%d):", freq, call_site_count);
+      tty->print_cr("Inlined frequent method (freq=%lf):", freq);
       CompileTask::print_inline_indent(inline_level());
       callee_method->print();
       tty->cr();
@@ -203,21 +194,20 @@ bool InlineTree::should_inline(ciMethod* callee_method, ciMethod* caller_method,
 // negative filter: should callee NOT be inlined?
 bool InlineTree::should_not_inline(ciMethod *callee_method,
                                    ciMethod* caller_method,
-                                   JVMState* jvms,
-                                   WarmCallInfo* wci_result) {
+                                   JVMState* jvms) {
 
   const char* fail_msg = NULL;
 
   // First check all inlining restrictions which are required for correctness
-  if ( callee_method->is_abstract()) {
+  if (callee_method->is_abstract()) {
     fail_msg = "abstract method"; // // note: we allow ik->is_abstract()
   } else if (!callee_method->holder()->is_initialized() &&
              // access allowed in the context of static initializer
-             !C->is_compiling_clinit_for(callee_method->holder())) {
+             C->needs_clinit_barrier(callee_method->holder(), caller_method)) {
     fail_msg = "method holder not initialized";
-  } else if ( callee_method->is_native()) {
+  } else if (callee_method->is_native()) {
     fail_msg = "native method";
-  } else if ( callee_method->dont_inline()) {
+  } else if (callee_method->dont_inline()) {
     fail_msg = "don't inline by annotation";
   }
 
@@ -297,8 +287,8 @@ bool InlineTree::should_not_inline(ciMethod *callee_method,
     return false;
   }
 
-  // don't use counts with -Xcomp or CTW
-  if (UseInterpreter && !CompileTheWorld) {
+  // don't use counts with -Xcomp
+  if (UseInterpreter) {
 
     if (!callee_method->has_compiled_code() &&
         !callee_method->was_executed_more_than(0)) {
@@ -328,12 +318,41 @@ bool InlineTree::should_not_inline(ciMethod *callee_method,
   return false;
 }
 
+bool InlineTree::is_not_reached(ciMethod* callee_method, ciMethod* caller_method, int caller_bci, ciCallProfile& profile) {
+  if (!UseInterpreter) {
+    return false; // -Xcomp
+  }
+  if (profile.count() > 0) {
+    return false; // reachable according to profile
+  }
+  if (!callee_method->was_executed_more_than(0)) {
+    return true; // callee was never executed
+  }
+  if (caller_method->is_not_reached(caller_bci)) {
+    return true; // call site not resolved
+  }
+  if (profile.count() == -1) {
+    return false; // immature profile; optimistically treat as reached
+  }
+  assert(profile.count() == 0, "sanity");
+
+  // Profile info is scarce.
+  // Try to guess: check if the call site belongs to a start block.
+  // Call sites in a start block should be reachable if no exception is thrown earlier.
+  ciMethodBlocks* caller_blocks = caller_method->get_method_blocks();
+  bool is_start_block = caller_blocks->block_containing(caller_bci)->start_bci() == 0;
+  if (is_start_block) {
+    return false; // treat the call reached as part of start block
+  }
+  return true; // give up and treat the call site as not reached
+}
+
 //-----------------------------try_to_inline-----------------------------------
 // return true if ok
 // Relocated from "InliningClosure::try_to_inline"
 bool InlineTree::try_to_inline(ciMethod* callee_method, ciMethod* caller_method,
                                int caller_bci, JVMState* jvms, ciCallProfile& profile,
-                               WarmCallInfo* wci_result, bool& should_delay) {
+                               bool& should_delay) {
 
   if (ClipInlining && (int)count_inline_bcs() >= DesiredMethodLimit) {
     if (!callee_method->force_inline() || !IncrementalInline) {
@@ -345,11 +364,10 @@ bool InlineTree::try_to_inline(ciMethod* callee_method, ciMethod* caller_method,
   }
 
   _forced_inline = false; // Reset
-  if (!should_inline(callee_method, caller_method, caller_bci, profile,
-                     wci_result)) {
+  if (!should_inline(callee_method, caller_method, caller_bci, profile)) {
     return false;
   }
-  if (should_not_inline(callee_method, caller_method, jvms, wci_result)) {
+  if (should_not_inline(callee_method, caller_method, jvms)) {
     return false;
   }
 
@@ -373,13 +391,13 @@ bool InlineTree::try_to_inline(ciMethod* callee_method, ciMethod* caller_method,
       }
     }
 
-    if ((!UseInterpreter || CompileTheWorld) &&
+    if (!UseInterpreter &&
         is_init_with_ea(callee_method, caller_method, C)) {
-      // Escape Analysis stress testing when running Xcomp or CTW:
+      // Escape Analysis stress testing when running Xcomp:
       // inline constructors even if they are not reached.
     } else if (forced_inline()) {
       // Inlining was forced by CompilerOracle, ciReplay or annotation
-    } else if (profile.count() == 0) {
+    } else if (is_not_reached(callee_method, caller_method, caller_bci, profile)) {
       // don't inline unreached call sites
        set_msg("call site not reached");
        return false;
@@ -456,17 +474,21 @@ bool InlineTree::try_to_inline(ciMethod* callee_method, ciMethod* caller_method,
 
 //------------------------------pass_initial_checks----------------------------
 bool InlineTree::pass_initial_checks(ciMethod* caller_method, int caller_bci, ciMethod* callee_method) {
-  ciInstanceKlass *callee_holder = callee_method ? callee_method->holder() : NULL;
   // Check if a callee_method was suggested
-  if( callee_method == NULL )            return false;
-  // Check if klass of callee_method is loaded
-  if( !callee_holder->is_loaded() )      return false;
-  if( !callee_holder->is_initialized() &&
-      // access allowed in the context of static initializer
-      !C->is_compiling_clinit_for(callee_holder)) {
+  if (callee_method == NULL) {
     return false;
   }
-  if( !UseInterpreter || CompileTheWorld /* running Xcomp or CTW */ ) {
+  ciInstanceKlass *callee_holder = callee_method->holder();
+  // Check if klass of callee_method is loaded
+  if (!callee_holder->is_loaded()) {
+    return false;
+  }
+  if (!callee_holder->is_initialized() &&
+      // access allowed in the context of static initializer
+      C->needs_clinit_barrier(callee_holder, caller_method)) {
+    return false;
+  }
+  if( !UseInterpreter ) /* running Xcomp */ {
     // Checks that constant pool's call site has been visited
     // stricter than callee_holder->is_initialized()
     ciBytecodeStream iter(caller_method);
@@ -498,25 +520,6 @@ const char* InlineTree::check_can_parse(ciMethod* callee) {
   return NULL;
 }
 
-static void post_inlining_event(int compile_id,const char* msg, bool success, int bci, ciMethod* caller, ciMethod* callee) {
-  assert(caller != NULL, "invariant");
-  assert(callee != NULL, "invariant");
-  EventCompilerInlining event;
-  if (event.should_commit()) {
-    JfrStructCalleeMethod callee_struct;
-    callee_struct.set_type(callee->holder()->name()->as_utf8());
-    callee_struct.set_name(callee->name()->as_utf8());
-    callee_struct.set_descriptor(callee->signature()->as_symbol()->as_utf8());
-    event.set_compileId(compile_id);
-    event.set_message(msg);
-    event.set_succeeded(success);
-    event.set_bci(bci);
-    event.set_caller(caller->get_Method());
-    event.set_callee(callee_struct);
-    event.commit();
-  }
-}
-
 //------------------------------print_inlining---------------------------------
 void InlineTree::print_inlining(ciMethod* callee_method, int caller_bci,
                                 ciMethod* caller_method, bool success) const {
@@ -533,18 +536,22 @@ void InlineTree::print_inlining(ciMethod* callee_method, int caller_bci,
                                                caller_bci, inline_msg);
   if (C->print_inlining()) {
     C->print_inlining(callee_method, inline_level(), caller_bci, inline_msg);
-    guarantee(callee_method != NULL, "would crash in post_inlining_event");
+    guarantee(callee_method != NULL, "would crash in CompilerEvent::InlineEvent::post");
     if (Verbose) {
       const InlineTree *top = this;
       while (top->caller_tree() != NULL) { top = top->caller_tree(); }
       //tty->print("  bcs: %d+%d  invoked: %d", top->count_inline_bcs(), callee_method->code_size(), callee_method->interpreter_invocation_count());
     }
   }
-  post_inlining_event(C->compile_id(), inline_msg, success, caller_bci, caller_method, callee_method);
+  EventCompilerInlining event;
+  if (event.should_commit()) {
+    CompilerEvent::InlineEvent::post(event, C->compile_id(), caller_method->get_Method(), callee_method, success, inline_msg, caller_bci);
+  }
 }
 
 //------------------------------ok_to_inline-----------------------------------
-WarmCallInfo* InlineTree::ok_to_inline(ciMethod* callee_method, JVMState* jvms, ciCallProfile& profile, WarmCallInfo* initial_wci, bool& should_delay) {
+bool InlineTree::ok_to_inline(ciMethod* callee_method, JVMState* jvms, ciCallProfile& profile,
+                              bool& should_delay) {
   assert(callee_method != NULL, "caller checks for optimized virtual!");
   assert(!should_delay, "should be initialized to false");
 #ifdef ASSERT
@@ -564,85 +571,39 @@ WarmCallInfo* InlineTree::ok_to_inline(ciMethod* callee_method, JVMState* jvms,
   if (!pass_initial_checks(caller_method, caller_bci, callee_method)) {
     set_msg("failed initial checks");
     print_inlining(callee_method, caller_bci, caller_method, false /* !success */);
-    return NULL;
+    return false;
   }
 
   // Do some parse checks.
   set_msg(check_can_parse(callee_method));
   if (msg() != NULL) {
     print_inlining(callee_method, caller_bci, caller_method, false /* !success */);
-    return NULL;
+    return false;
   }
 
   // Check if inlining policy says no.
-  WarmCallInfo wci = *(initial_wci);
-  bool success = try_to_inline(callee_method, caller_method, caller_bci,
-                               jvms, profile, &wci, should_delay);
-
-#ifndef PRODUCT
-  if (InlineWarmCalls && (PrintOpto || C->print_inlining())) {
-    bool cold = wci.is_cold();
-    bool hot  = !cold && wci.is_hot();
-    bool old_cold = !success;
-    if (old_cold != cold || (Verbose || WizardMode)) {
-      if (msg() == NULL) {
-        set_msg("OK");
-      }
-      tty->print("   OldInlining= %4s : %s\n           WCI=",
-                 old_cold ? "cold" : "hot", msg());
-      wci.print();
-    }
-  }
-#endif
+  bool success = try_to_inline(callee_method, caller_method, caller_bci, jvms, profile,
+                               should_delay); // out
   if (success) {
-    wci = *(WarmCallInfo::always_hot());
-  } else {
-    wci = *(WarmCallInfo::always_cold());
-  }
-
-  if (!InlineWarmCalls) {
-    if (!wci.is_cold() && !wci.is_hot()) {
-      // Do not inline the warm calls.
-      wci = *(WarmCallInfo::always_cold());
-    }
-  }
-
-  if (!wci.is_cold()) {
     // Inline!
     if (msg() == NULL) {
       set_msg("inline (hot)");
     }
     print_inlining(callee_method, caller_bci, caller_method, true /* success */);
     build_inline_tree_for_callee(callee_method, jvms, caller_bci);
-    if (InlineWarmCalls && !wci.is_hot()) {
-      return new (C) WarmCallInfo(wci);  // copy to heap
+    return true;
+  } else {
+    // Do not inline
+    if (msg() == NULL) {
+      set_msg("too cold to inline");
     }
-    return WarmCallInfo::always_hot();
+    print_inlining(callee_method, caller_bci, caller_method, false /* !success */ );
+    return false;
   }
-
-  // Do not inline
-  if (msg() == NULL) {
-    set_msg("too cold to inline");
-  }
-  print_inlining(callee_method, caller_bci, caller_method, false /* !success */ );
-  return NULL;
-}
-
-//------------------------------compute_callee_frequency-----------------------
-float InlineTree::compute_callee_frequency( int caller_bci ) const {
-  int count  = method()->interpreter_call_site_count(caller_bci);
-  int invcnt = method()->interpreter_invocation_count();
-  float freq = (float)count/(float)invcnt;
-  // Call-site count / interpreter invocation count, scaled recursively.
-  // Always between 0.0 and 1.0.  Represents the percentage of the method's
-  // total execution time used at this call site.
-
-  return freq;
 }
 
 //------------------------------build_inline_tree_for_callee-------------------
 InlineTree *InlineTree::build_inline_tree_for_callee( ciMethod* callee_method, JVMState* caller_jvms, int caller_bci) {
-  float recur_frequency = _site_invoke_ratio * compute_callee_frequency(caller_bci);
   // Attempt inlining.
   InlineTree* old_ilt = callee_at(caller_bci, callee_method);
   if (old_ilt != NULL) {
@@ -667,7 +628,7 @@ InlineTree *InlineTree::build_inline_tree_for_callee( ciMethod* callee_method, J
     }
   }
   // Allocate in the comp_arena to make sure the InlineTree is live when dumping a replay compilation file
-  InlineTree* ilt = new (C->comp_arena()) InlineTree(C, this, callee_method, caller_jvms, caller_bci, recur_frequency, _max_inline_level + max_inline_level_adjust);
+  InlineTree* ilt = new (C->comp_arena()) InlineTree(C, this, callee_method, caller_jvms, caller_bci, _max_inline_level + max_inline_level_adjust);
   _subtrees.append(ilt);
 
   NOT_PRODUCT( _count_inlines += 1; )
@@ -693,7 +654,7 @@ InlineTree *InlineTree::build_inline_tree_root() {
   Compile* C = Compile::current();
 
   // Root of inline tree
-  InlineTree* ilt = new InlineTree(C, NULL, C->method(), NULL, -1, 1.0F, MaxInlineLevel);
+  InlineTree* ilt = new InlineTree(C, NULL, C->method(), NULL, -1, MaxInlineLevel);
 
   return ilt;
 }
