@@ -67,7 +67,7 @@ CallGenerator* Compile::call_generator(ciMethod* callee, int vtable_index, bool
                                        JVMState* jvms, bool allow_inline,
                                        float prof_factor, ciKlass* speculative_receiver_type,
                                        bool allow_intrinsics) {
-  assert(callee != nullptr, "failed method resolution");
+  assert(callee != NULL, "failed method resolution");
 
   ciMethod*       caller      = jvms->method();
   int             bci         = jvms->bci();
@@ -99,7 +99,7 @@ CallGenerator* Compile::call_generator(ciMethod* callee, int vtable_index, bool
   }
 
   CompileLog* log = this->log();
-  if (log != nullptr) {
+  if (log != NULL) {
     int rid = (receiver_count >= 0)? log->identify(profile.receiver(0)): -1;
     int r2id = (rid != -1 && profile.has_receiver(1))? log->identify(profile.receiver(1)):-1;
     log->begin_elem("call method='%d' count='%d' prof_factor='%f'",
@@ -121,15 +121,15 @@ CallGenerator* Compile::call_generator(ciMethod* callee, int vtable_index, bool
   // Special case the handling of certain common, profitable library
   // methods.  If these methods are replaced with specialized code,
   // then we return it as the inlined version of the call.
-  CallGenerator* cg_intrinsic = nullptr;
+  CallGenerator* cg_intrinsic = NULL;
   if (allow_inline && allow_intrinsics) {
     CallGenerator* cg = find_intrinsic(callee, call_does_dispatch);
-    if (cg != nullptr) {
+    if (cg != NULL) {
       if (cg->is_predicated()) {
         // Code without intrinsic but, hopefully, inlined.
         CallGenerator* inline_cg = this->call_generator(callee,
               vtable_index, call_does_dispatch, jvms, allow_inline, prof_factor, speculative_receiver_type, false);
-        if (inline_cg != nullptr) {
+        if (inline_cg != NULL) {
           cg = CallGenerator::for_predicated_intrinsic(cg, inline_cg);
         }
       }
@@ -139,7 +139,7 @@ CallGenerator* Compile::call_generator(ciMethod* callee, int vtable_index, bool
       // We will retry the intrinsic if nothing had claimed it afterwards.
       if (cg->does_virtual_dispatch()) {
         cg_intrinsic = cg;
-        cg = nullptr;
+        cg = NULL;
       } else if (IncrementalInline && should_delay_vector_inlining(callee, jvms)) {
         return CallGenerator::for_late_inline(callee, cg);
       } else {
@@ -168,7 +168,7 @@ CallGenerator* Compile::call_generator(ciMethod* callee, int vtable_index, bool
     // Try inlining a bytecoded method:
     if (!call_does_dispatch) {
       InlineTree* ilt = InlineTree::find_subtree_from_root(this->ilt(), jvms->caller(), jvms->method());
-      bool should_delay = false;
+      bool should_delay = AlwaysIncrementalInline;
       if (ilt->ok_to_inline(callee, jvms, profile, should_delay)) {
         CallGenerator* cg = CallGenerator::for_inline(callee, expected_uses);
         // For optimized virtual calls assert at runtime that receiver object
@@ -177,13 +177,13 @@ CallGenerator* Compile::call_generator(ciMethod* callee, int vtable_index, bool
         // sometimes has a broader type. Similar scenario is possible with
         // default methods when type system loses information about implemented
         // interfaces.
-        if (cg != nullptr && is_virtual_or_interface && !callee->is_static()) {
+        if (cg != NULL && is_virtual_or_interface && !callee->is_static()) {
           CallGenerator* trap_cg = CallGenerator::for_uncommon_trap(callee,
               Deoptimization::Reason_receiver_constraint, Deoptimization::Action_none);
 
           cg = CallGenerator::for_guarded_call(callee->holder(), trap_cg, cg);
         }
-        if (cg != nullptr) {
+        if (cg != NULL) {
           // Delay the inlining of this method to give us the
           // opportunity to perform some high level optimizations
           // first.
@@ -193,7 +193,7 @@ CallGenerator* Compile::call_generator(ciMethod* callee, int vtable_index, bool
             return CallGenerator::for_boxing_late_inline(callee, cg);
           } else if (should_delay_vector_reboxing_inlining(callee, jvms)) {
             return CallGenerator::for_vector_reboxing_late_inline(callee, cg);
-          } else if ((should_delay || AlwaysIncrementalInline)) {
+          } else if (should_delay) {
             return CallGenerator::for_late_inline(callee, cg);
           } else {
             return cg;
@@ -206,10 +206,10 @@ CallGenerator* Compile::call_generator(ciMethod* callee, int vtable_index, bool
     if (call_does_dispatch && site_count > 0 && UseTypeProfile) {
       // The major receiver's count >= TypeProfileMajorReceiverPercent of site_count.
       bool have_major_receiver = profile.has_receiver(0) && (100.*profile.receiver_prob(0) >= (float)TypeProfileMajorReceiverPercent);
-      ciMethod* receiver_method = nullptr;
+      ciMethod* receiver_method = NULL;
 
       int morphism = profile.morphism();
-      if (speculative_receiver_type != nullptr) {
+      if (speculative_receiver_type != NULL) {
         if (!too_many_traps_or_recompiles(caller, bci, Deoptimization::Reason_speculate_class_check)) {
           // We have a speculative type, we should be able to resolve
           // the call. We do that before looking at the profiling at
@@ -217,18 +217,18 @@ CallGenerator* Compile::call_generator(ciMethod* callee, int vtable_index, bool
           // a speculative type should help us avoid.
           receiver_method = callee->resolve_invoke(jvms->method()->holder(),
                                                    speculative_receiver_type);
-          if (receiver_method == nullptr) {
-            speculative_receiver_type = nullptr;
+          if (receiver_method == NULL) {
+            speculative_receiver_type = NULL;
           } else {
             morphism = 1;
           }
         } else {
           // speculation failed before. Use profiling at the call
           // (could allow bimorphic inlining for instance).
-          speculative_receiver_type = nullptr;
+          speculative_receiver_type = NULL;
         }
       }
-      if (receiver_method == nullptr &&
+      if (receiver_method == NULL &&
           (have_major_receiver || morphism == 1 ||
            (morphism == 2 && UseBimorphicInlining))) {
         // receiver_method = profile.method();
@@ -236,33 +236,33 @@ CallGenerator* Compile::call_generator(ciMethod* callee, int vtable_index, bool
         receiver_method = callee->resolve_invoke(jvms->method()->holder(),
                                                       profile.receiver(0));
       }
-      if (receiver_method != nullptr) {
+      if (receiver_method != NULL) {
         // The single majority receiver sufficiently outweighs the minority.
         CallGenerator* hit_cg = this->call_generator(receiver_method,
               vtable_index, !call_does_dispatch, jvms, allow_inline, prof_factor);
-        if (hit_cg != nullptr) {
+        if (hit_cg != NULL) {
           // Look up second receiver.
-          CallGenerator* next_hit_cg = nullptr;
-          ciMethod* next_receiver_method = nullptr;
+          CallGenerator* next_hit_cg = NULL;
+          ciMethod* next_receiver_method = NULL;
           if (morphism == 2 && UseBimorphicInlining) {
             next_receiver_method = callee->resolve_invoke(jvms->method()->holder(),
                                                                profile.receiver(1));
-            if (next_receiver_method != nullptr) {
+            if (next_receiver_method != NULL) {
               next_hit_cg = this->call_generator(next_receiver_method,
                                   vtable_index, !call_does_dispatch, jvms,
                                   allow_inline, prof_factor);
-              if (next_hit_cg != nullptr && !next_hit_cg->is_inline() &&
+              if (next_hit_cg != NULL && !next_hit_cg->is_inline() &&
                   have_major_receiver && UseOnlyInlinedBimorphic) {
                   // Skip if we can't inline second receiver's method
-                  next_hit_cg = nullptr;
+                  next_hit_cg = NULL;
               }
             }
           }
           CallGenerator* miss_cg;
           Deoptimization::DeoptReason reason = (morphism == 2
                                                ? Deoptimization::Reason_bimorphic
-                                               : Deoptimization::reason_class_check(speculative_receiver_type != nullptr));
-          if ((morphism == 1 || (morphism == 2 && next_hit_cg != nullptr)) &&
+                                               : Deoptimization::reason_class_check(speculative_receiver_type != NULL));
+          if ((morphism == 1 || (morphism == 2 && next_hit_cg != NULL)) &&
               !too_many_traps_or_recompiles(caller, bci, reason)
              ) {
             // Generate uncommon trap for class check failure path
@@ -275,20 +275,20 @@ CallGenerator* Compile::call_generator(ciMethod* callee, int vtable_index, bool
             miss_cg = (IncrementalInlineVirtual ? CallGenerator::for_late_inline_virtual(callee, vtable_index, prof_factor)
                                                 : CallGenerator::for_virtual_call(callee, vtable_index));
           }
-          if (miss_cg != nullptr) {
-            if (next_hit_cg != nullptr) {
-              assert(speculative_receiver_type == nullptr, "shouldn't end up here if we used speculation");
+          if (miss_cg != NULL) {
+            if (next_hit_cg != NULL) {
+              assert(speculative_receiver_type == NULL, "shouldn't end up here if we used speculation");
               trace_type_profile(C, jvms->method(), jvms->depth() - 1, jvms->bci(), next_receiver_method, profile.receiver(1), site_count, profile.receiver_count(1));
               // We don't need to record dependency on a receiver here and below.
               // Whenever we inline, the dependency is added by Parse::Parse().
               miss_cg = CallGenerator::for_predicted_call(profile.receiver(1), miss_cg, next_hit_cg, PROB_MAX);
             }
-            if (miss_cg != nullptr) {
-              ciKlass* k = speculative_receiver_type != nullptr ? speculative_receiver_type : profile.receiver(0);
+            if (miss_cg != NULL) {
+              ciKlass* k = speculative_receiver_type != NULL ? speculative_receiver_type : profile.receiver(0);
               trace_type_profile(C, jvms->method(), jvms->depth() - 1, jvms->bci(), receiver_method, k, site_count, receiver_count);
-              float hit_prob = speculative_receiver_type != nullptr ? 1.0 : profile.receiver_prob(0);
+              float hit_prob = speculative_receiver_type != NULL ? 1.0 : profile.receiver_prob(0);
               CallGenerator* cg = CallGenerator::for_predicted_call(k, miss_cg, hit_cg, hit_prob);
-              if (cg != nullptr)  return cg;
+              if (cg != NULL)  return cg;
             }
           }
         }
@@ -314,13 +314,13 @@ CallGenerator* Compile::call_generator(ciMethod* callee, int vtable_index, bool
           caller->get_declared_method_holder_at_bci(bci)->as_instance_klass();
       ciInstanceKlass* singleton = declared_interface->unique_implementor();
 
-      if (singleton != nullptr) {
+      if (singleton != NULL) {
         assert(singleton != declared_interface, "not a unique implementor");
 
         ciMethod* cha_monomorphic_target =
             callee->find_monomorphic_target(caller->holder(), declared_interface, singleton);
 
-        if (cha_monomorphic_target != nullptr &&
+        if (cha_monomorphic_target != NULL &&
             cha_monomorphic_target->holder() != env()->Object_klass()) { // subtype check against Object is useless
           ciKlass* holder = cha_monomorphic_target->holder();
 
@@ -334,7 +334,7 @@ CallGenerator* Compile::call_generator(ciMethod* callee, int vtable_index, bool
 
           ciKlass* constraint = (holder->is_subclass_of(singleton) ? holder : singleton); // avoid upcasts
           CallGenerator* cg = CallGenerator::for_guarded_call(constraint, miss_cg, hit_cg);
-          if (hit_cg != nullptr && cg != nullptr) {
+          if (hit_cg != NULL && cg != NULL) {
             dependencies()->assert_unique_implementor(declared_interface, singleton);
             dependencies()->assert_unique_concrete_method(declared_interface, cha_monomorphic_target, declared_interface, callee);
             return cg;
@@ -345,7 +345,7 @@ CallGenerator* Compile::call_generator(ciMethod* callee, int vtable_index, bool
 
     // Nothing claimed the intrinsic, we go with straight-forward inlining
     // for already discovered intrinsic.
-    if (allow_intrinsics && cg_intrinsic != nullptr) {
+    if (allow_intrinsics && cg_intrinsic != NULL) {
       assert(cg_intrinsic->does_virtual_dispatch(), "sanity");
       return cg_intrinsic;
     }
@@ -369,7 +369,7 @@ CallGenerator* Compile::call_generator(ciMethod* callee, int vtable_index, bool
     CallGenerator* cg = CallGenerator::for_direct_call(callee, should_delay_inlining(callee, jvms));
     // For optimized virtual calls assert at runtime that receiver object
     // is a subtype of the method holder.
-    if (cg != nullptr && is_virtual_or_interface && !callee->is_static()) {
+    if (cg != NULL && is_virtual_or_interface && !callee->is_static()) {
       CallGenerator* trap_cg = CallGenerator::for_uncommon_trap(callee,
           Deoptimization::Reason_receiver_constraint, Deoptimization::Action_none);
       cg = CallGenerator::for_guarded_call(callee->holder(), trap_cg, cg);
@@ -415,7 +415,7 @@ bool Compile::should_delay_string_inlining(ciMethod* call_method, JVMState* jvms
           if (receiver->is_Proj() && receiver->in(0)->is_CallStaticJava()) {
             CallStaticJavaNode* csj = receiver->in(0)->as_CallStaticJava();
             ciMethod* m = csj->method();
-            if (m != nullptr &&
+            if (m != NULL &&
                 (m->intrinsic_id() == vmIntrinsics::_StringBuffer_toString ||
                  m->intrinsic_id() == vmIntrinsics::_StringBuilder_toString))
               // Delay String.<init>(new SB())
@@ -505,12 +505,12 @@ void Parse::do_call() {
 
   // Find target being called
   bool             will_link;
-  ciSignature*     declared_signature = nullptr;
+  ciSignature*     declared_signature = NULL;
   ciMethod*        orig_callee  = iter().get_method(will_link, &declared_signature);  // callee in the bytecode
   ciInstanceKlass* holder_klass = orig_callee->holder();
   ciKlass*         holder       = iter().get_declared_method_holder();
   ciInstanceKlass* klass = ciEnv::get_instance_klass_for_declared_method_holder(holder);
-  assert(declared_signature != nullptr, "cannot be null");
+  assert(declared_signature != NULL, "cannot be null");
 
   // Bump max node limit for JSR292 users
   if (bc() == Bytecodes::_invokedynamic || orig_callee->is_method_handle_intrinsic()) {
@@ -530,7 +530,7 @@ void Parse::do_call() {
   //assert((bc_callee->is_static() || is_invokedynamic) == !has_receiver , "must match bc");  // XXX invokehandle (cur_bc_raw)
   // Note: this takes into account invokeinterface of methods declared in java/lang/Object,
   // which should be invokevirtuals but according to the VM spec may be invokeinterfaces
-  assert(holder_klass->is_interface() || holder_klass->super() == nullptr || (bc() != Bytecodes::_invokeinterface), "must match bc");
+  assert(holder_klass->is_interface() || holder_klass->super() == NULL || (bc() != Bytecodes::_invokeinterface), "must match bc");
   // Note:  In the absence of miranda methods, an abstract class K can perform
   // an invokevirtual directly on an interface method I.m if K implements I.
 
@@ -560,7 +560,7 @@ void Parse::do_call() {
   bool      call_does_dispatch = false;
 
   // Speculative type of the receiver if any
-  ciKlass* speculative_receiver_type = nullptr;
+  ciKlass* speculative_receiver_type = NULL;
   if (is_virtual_or_interface) {
     Node* receiver_node             = stack(sp() - nargs);
     const TypeOopPtr* receiver_type = _gvn.type(receiver_node)->isa_oopptr();
@@ -574,11 +574,11 @@ void Parse::do_call() {
     callee = C->optimize_virtual_call(method(), klass, holder, orig_callee,
                                       receiver_type, is_virtual,
                                       call_does_dispatch, vtable_index);  // out-parameters
-    speculative_receiver_type = receiver_type != nullptr ? receiver_type->speculative_type() : nullptr;
+    speculative_receiver_type = receiver_type != NULL ? receiver_type->speculative_type() : NULL;
   }
 
   // Additional receiver subtype checks for interface calls via invokespecial or invokeinterface.
-  ciKlass* receiver_constraint = nullptr;
+  ciKlass* receiver_constraint = NULL;
   if (iter().cur_bc_raw() == Bytecodes::_invokespecial && !orig_callee->is_object_initializer()) {
     ciInstanceKlass* calling_klass = method()->holder();
     ciInstanceKlass* sender_klass = calling_klass;
@@ -590,12 +590,12 @@ void Parse::do_call() {
     receiver_constraint = holder;
   }
 
-  if (receiver_constraint != nullptr) {
+  if (receiver_constraint != NULL) {
     Node* receiver_node = stack(sp() - nargs);
     Node* cls_node = makecon(TypeKlassPtr::make(receiver_constraint));
-    Node* bad_type_ctrl = nullptr;
+    Node* bad_type_ctrl = NULL;
     Node* casted_receiver = gen_checkcast(receiver_node, cls_node, &bad_type_ctrl);
-    if (bad_type_ctrl != nullptr) {
+    if (bad_type_ctrl != NULL) {
       PreserveJVMState pjvms(this);
       set_control(bad_type_ctrl);
       uncommon_trap(Deoptimization::Reason_class_check,
@@ -623,7 +623,7 @@ void Parse::do_call() {
   CallGenerator* cg = C->call_generator(callee, vtable_index, call_does_dispatch, jvms, try_inline, prof_factor(), speculative_receiver_type);
 
   // NOTE:  Don't use orig_callee and callee after this point!  Use cg->method() instead.
-  orig_callee = callee = nullptr;
+  orig_callee = callee = NULL;
 
   // ---------------------
   // Round double arguments before call
@@ -645,17 +645,17 @@ void Parse::do_call() {
   assert(jvms_in_sync(),       "jvms must carry full info into CG");
 
   // save across call, for a subsequent cast_not_null.
-  Node* receiver = has_receiver ? argument(0) : nullptr;
+  Node* receiver = has_receiver ? argument(0) : NULL;
 
   // The extra CheckCastPPs for speculative types mess with PhaseStringOpts
-  if (receiver != nullptr && !call_does_dispatch && !cg->is_string_late_inline()) {
+  if (receiver != NULL && !call_does_dispatch && !cg->is_string_late_inline()) {
     // Feed profiling data for a single receiver to the type system so
     // it can propagate it as a speculative type
     receiver = record_profiled_receiver_for_speculation(receiver);
   }
 
   JVMState* new_jvms = cg->generate(jvms);
-  if (new_jvms == nullptr) {
+  if (new_jvms == NULL) {
     // When inlining attempt fails (e.g., too many arguments),
     // it may contaminate the current compile state, making it
     // impossible to pull back and try again.  Once we call
@@ -669,7 +669,7 @@ void Parse::do_call() {
     // get a normal java call that may inline in that case
     cg = C->call_generator(cg->method(), vtable_index, call_does_dispatch, jvms, try_inline, prof_factor(), speculative_receiver_type, /* allow_intrinsics= */ false);
     new_jvms = cg->generate(jvms);
-    if (new_jvms == nullptr) {
+    if (new_jvms == NULL) {
       guarantee(failing(), "call failed to generate:  calls should work");
       return;
     }
@@ -695,7 +695,7 @@ void Parse::do_call() {
   if (!stopped()) {
     // This was some sort of virtual call, which did a null check for us.
     // Now we can assert receiver-not-null, on the normal return path.
-    if (receiver != nullptr && cg->is_virtual()) {
+    if (receiver != NULL && cg->is_virtual()) {
       Node* cast = cast_not_null(receiver);
       // %%% assert(receiver == cast, "should already have cast the receiver");
     }
@@ -721,7 +721,7 @@ void Parse::do_call() {
           if (ctype->is_loaded()) {
             const TypeOopPtr* arg_type = TypeOopPtr::make_from_klass(rtype->as_klass());
             const Type*       sig_type = TypeOopPtr::make_from_klass(ctype->as_klass());
-            if (arg_type != nullptr && !arg_type->higher_equal(sig_type)) {
+            if (arg_type != NULL && !arg_type->higher_equal(sig_type)) {
               Node* retnode = pop();
               Node* cast_obj = _gvn.transform(new CheckCastPPNode(control(), retnode, sig_type));
               push(cast_obj);
@@ -754,7 +754,7 @@ void Parse::do_call() {
         method()->print_name(); tty->print_cr(" asserting nullness of result at bci: %d", bci());
         cg->method()->print_name(); tty->cr();
       }
-      if (C->log() != nullptr) {
+      if (C->log() != NULL) {
         C->log()->elem("assert_null reason='return' klass='%d'",
                        C->log()->identify(rtype));
       }
@@ -786,7 +786,7 @@ void Parse::catch_call_exceptions(ciExceptionHandlerStream& handlers) {
 
   // Add a CatchNode.
   GrowableArray<int>* bcis = new (C->node_arena()) GrowableArray<int>(C->node_arena(), 8, 0, -1);
-  GrowableArray<const Type*>* extypes = new (C->node_arena()) GrowableArray<const Type*>(C->node_arena(), 8, 0, nullptr);
+  GrowableArray<const Type*>* extypes = new (C->node_arena()) GrowableArray<const Type*>(C->node_arena(), 8, 0, NULL);
   GrowableArray<int>* saw_unloaded = new (C->node_arena()) GrowableArray<int>(C->node_arena(), 8, 0, 0);
 
   bool default_handler = false;
@@ -819,9 +819,7 @@ void Parse::catch_call_exceptions(ciExceptionHandlerStream& handlers) {
 
   if (!default_handler) {
     bcis->append(-1);
-    const Type* extype = TypeOopPtr::make_from_klass(env()->Throwable_klass())->is_instptr();
-    extype = extype->join(TypeInstPtr::NOTNULL);
-    extypes->append(extype);
+    extypes->append(TypeOopPtr::make_from_klass(env()->Throwable_klass())->is_instptr());
   }
 
   int len = bcis->length();
@@ -898,7 +896,7 @@ void Parse::catch_call_exceptions(ciExceptionHandlerStream& handlers) {
 // So we insert a RethrowCall and all the logic that goes with it.
 void Parse::catch_inline_exceptions(SafePointNode* ex_map) {
   // Caller is responsible for saving away the map for normal control flow!
-  assert(stopped(), "call set_map(nullptr) first");
+  assert(stopped(), "call set_map(NULL) first");
   assert(method()->has_exception_handlers(), "don't come here w/o work to do");
 
   Node* ex_node = saved_ex_oop(ex_map);
@@ -907,8 +905,8 @@ void Parse::catch_inline_exceptions(SafePointNode* ex_map) {
     return;
   }
   const TypeInstPtr* ex_type = _gvn.type(ex_node)->isa_instptr();
-  NOT_PRODUCT(if (ex_type==nullptr) tty->print_cr("*** Exception not InstPtr"));
-  if (ex_type == nullptr)
+  NOT_PRODUCT(if (ex_type==NULL) tty->print_cr("*** Exception not InstPtr"));
+  if (ex_type == NULL)
     ex_type = TypeOopPtr::make_from_klass(env()->Throwable_klass())->is_instptr();
 
   // determine potential exception handlers
@@ -921,10 +919,10 @@ void Parse::catch_inline_exceptions(SafePointNode* ex_map) {
   ex_node = use_exception_state(ex_map);
 
   // Get the exception oop klass from its header
-  Node* ex_klass_node = nullptr;
+  Node* ex_klass_node = NULL;
   if (has_ex_handler() && !ex_type->klass_is_exact()) {
     Node* p = basic_plus_adr( ex_node, ex_node, oopDesc::klass_offset_in_bytes());
-    ex_klass_node = _gvn.transform(LoadKlassNode::make(_gvn, nullptr, immutable_memory(), p, TypeInstPtr::KLASS, TypeKlassPtr::OBJECT));
+    ex_klass_node = _gvn.transform(LoadKlassNode::make(_gvn, NULL, immutable_memory(), p, TypeInstPtr::KLASS, TypeInstKlassPtr::OBJECT));
 
     // Compute the exception klass a little more cleverly.
     // Obvious solution is to simple do a LoadKlass from the 'ex_node'.
@@ -933,19 +931,19 @@ void Parse::catch_inline_exceptions(SafePointNode* ex_map) {
     // I'm loading the class from, I can replace the LoadKlass with the
     // klass constant for the exception oop.
     if (ex_node->is_Phi()) {
-      ex_klass_node = new PhiNode(ex_node->in(0), TypeKlassPtr::OBJECT);
+      ex_klass_node = new PhiNode(ex_node->in(0), TypeInstKlassPtr::OBJECT);
       for (uint i = 1; i < ex_node->req(); i++) {
         Node* ex_in = ex_node->in(i);
-        if (ex_in == top() || ex_in == nullptr) {
+        if (ex_in == top() || ex_in == NULL) {
           // This path was not taken.
           ex_klass_node->init_req(i, top());
           continue;
         }
         Node* p = basic_plus_adr(ex_in, ex_in, oopDesc::klass_offset_in_bytes());
-        Node* k = _gvn.transform( LoadKlassNode::make(_gvn, nullptr, immutable_memory(), p, TypeInstPtr::KLASS, TypeKlassPtr::OBJECT));
+        Node* k = _gvn.transform( LoadKlassNode::make(_gvn, NULL, immutable_memory(), p, TypeInstPtr::KLASS, TypeInstKlassPtr::OBJECT));
         ex_klass_node->init_req( i, k );
       }
-      _gvn.set_type(ex_klass_node, TypeKlassPtr::OBJECT);
+      _gvn.set_type(ex_klass_node, TypeInstKlassPtr::OBJECT);
 
     }
   }
@@ -979,8 +977,6 @@ void Parse::catch_inline_exceptions(SafePointNode* ex_map) {
       if (PrintOpto && WizardMode) {
         tty->print_cr("  Catching every inline exception bci:%d -> handler_bci:%d", bci(), handler_bci);
       }
-      // If this is a backwards branch in the bytecodes, add safepoint
-      maybe_add_safepoint(handler_bci);
       merge_exception(handler_bci); // jump to handler
       return;                   // No more handling to be done here!
     }
@@ -1012,8 +1008,6 @@ void Parse::catch_inline_exceptions(SafePointNode* ex_map) {
         klass->print_name();
         tty->cr();
       }
-      // If this is a backwards branch in the bytecodes, add safepoint
-      maybe_add_safepoint(handler_bci);
       merge_exception(handler_bci);
     }
     set_control(not_subtype_ctrl);
@@ -1032,7 +1026,7 @@ void Parse::catch_inline_exceptions(SafePointNode* ex_map) {
   make_runtime_call(RC_NO_LEAF | RC_MUST_THROW,
                     OptoRuntime::rethrow_Type(),
                     OptoRuntime::rethrow_stub(),
-                    nullptr, nullptr,
+                    NULL, NULL,
                     ex_node);
 
   // Rethrow is a pure call, no side effects, only a result.
@@ -1095,7 +1089,7 @@ ciMethod* Compile::optimize_virtual_call(ciMethod* caller, ciInstanceKlass* klas
                                                          receiver_type, check_access);
 
   // Have the call been sufficiently improved such that it is no longer a virtual?
-  if (optimized_virtual_method != nullptr) {
+  if (optimized_virtual_method != NULL) {
     callee             = optimized_virtual_method;
     call_does_dispatch = false;
   } else if (!UseInlineCaches && is_virtual && callee->is_loaded()) {
@@ -1119,8 +1113,8 @@ ciMethod* Compile::optimize_inlining(ciMethod* caller, ciInstanceKlass* klass, c
     return callee;
   }
 
-  if (receiver_type == nullptr) {
-    return nullptr; // no receiver type info
+  if (receiver_type == NULL) {
+    return NULL; // no receiver type info
   }
 
   // Attempt to improve the receiver
@@ -1136,7 +1130,7 @@ ciMethod* Compile::optimize_inlining(ciMethod* caller, ciInstanceKlass* klass, c
 
   // All other interesting cases are instance klasses.
   if (!receiver_type->isa_instptr()) {
-    return nullptr;
+    return NULL;
   }
 
   ciInstanceKlass* receiver_klass = receiver_type->klass()->as_instance_klass();
@@ -1152,7 +1146,7 @@ ciMethod* Compile::optimize_inlining(ciMethod* caller, ciInstanceKlass* klass, c
   ciInstanceKlass*   calling_klass = caller->holder();
   ciMethod* cha_monomorphic_target = callee->find_monomorphic_target(calling_klass, klass, actual_receiver, check_access);
 
-  if (cha_monomorphic_target != nullptr) {
+  if (cha_monomorphic_target != NULL) {
     // Hardwiring a virtual.
     assert(!callee->can_be_statically_bound(), "should have been handled earlier");
     assert(!cha_monomorphic_target->is_abstract(), "");
@@ -1173,10 +1167,10 @@ ciMethod* Compile::optimize_inlining(ciMethod* caller, ciInstanceKlass* klass, c
     // In case of evolution, there is a dependence on every inlined method, since each
     // such method can be changed when its class is redefined.
     ciMethod* exact_method = callee->resolve_invoke(calling_klass, actual_receiver);
-    if (exact_method != nullptr) {
+    if (exact_method != NULL) {
       return exact_method;
     }
   }
 
-  return nullptr;
+  return NULL;
 }
