@@ -72,7 +72,7 @@ class GraphKit : public Phase {
 
  private:
   SafePointNode*     map_not_null() const {
-    assert(_map != nullptr, "must call stopped() to test for reset compiler map");
+    assert(_map != NULL, "must call stopped() to test for reset compiler map");
     return _map;
   }
 
@@ -82,20 +82,18 @@ class GraphKit : public Phase {
 
 #ifdef ASSERT
   ~GraphKit() {
-    assert(failing() || !has_exceptions(),
-           "unless compilation failed, user must call transfer_exceptions_into_jvms");
+    assert(!has_exceptions(), "user must call transfer_exceptions_into_jvms");
   }
 #endif
 
-  virtual Parse*          is_Parse()          const { return nullptr; }
-  virtual LibraryCallKit* is_LibraryCallKit() const { return nullptr; }
+  virtual Parse*          is_Parse()          const { return NULL; }
+  virtual LibraryCallKit* is_LibraryCallKit() const { return NULL; }
 
   ciEnv*        env()               const { return _env; }
   PhaseGVN&     gvn()               const { return _gvn; }
   void*         barrier_set_state() const { return C->barrier_set_state(); }
 
   void record_for_igvn(Node* n) const { C->record_for_igvn(n); }  // delegate to Compile
-  void remove_for_igvn(Node* n) const { C->remove_for_igvn(n); }
 
   // Handy well-known nodes:
   Node*         null()          const { return zerocon(T_OBJECT); }
@@ -133,7 +131,7 @@ class GraphKit : public Phase {
   // See layout accessors in class JVMState.
 
   SafePointNode*     map()      const { return _map; }
-  bool               has_exceptions() const { return _exceptions != nullptr; }
+  bool               has_exceptions() const { return _exceptions != NULL; }
   JVMState*          jvms()     const { return map_not_null()->_jvms; }
   int                sp()       const { return _sp; }
   int                bci()      const { return _bci; }
@@ -144,7 +142,7 @@ class GraphKit : public Phase {
                                         assert(jvms == this->jvms(), "sanity");
                                         _sp = jvms->sp();
                                         _bci = jvms->bci();
-                                        _method = jvms->has_method() ? jvms->method() : nullptr; }
+                                        _method = jvms->has_method() ? jvms->method() : NULL; }
   void set_map(SafePointNode* m)      { _map = m; debug_only(verify_map()); }
   void set_sp(int sp)                 { assert(sp >= 0, "sp must be non-negative: %d", sp); _sp = sp; }
   void clean_stack(int from_sp); // clear garbage beyond from_sp to top
@@ -172,25 +170,20 @@ class GraphKit : public Phase {
   // Clone the existing map state.  (Implements PreserveJVMState.)
   SafePointNode* clone_map();
 
-  // Reverses the work done by clone_map(). Should only be used when the node returned by
-  // clone_map() is ultimately not used. Calling Node::destruct directly in the previously
-  // mentioned circumstance instead of this method may result in use-after-free.
-  void destruct_map_clone(SafePointNode* sfp);
-
   // Set the map to a clone of the given one.
   void set_map_clone(SafePointNode* m);
 
   // Tell if the compilation is failing.
   bool failing() const { return C->failing(); }
 
-  // Set _map to null, signalling a stop to further bytecode execution.
+  // Set _map to NULL, signalling a stop to further bytecode execution.
   // Preserve the map intact for future use, and return it back to the caller.
-  SafePointNode* stop() { SafePointNode* m = map(); set_map(nullptr); return m; }
+  SafePointNode* stop() { SafePointNode* m = map(); set_map(NULL); return m; }
 
-  // Stop, but first smash the map's inputs to null, to mark it dead.
+  // Stop, but first smash the map's inputs to NULL, to mark it dead.
   void stop_and_kill_map();
 
-  // Tell if _map is null, or control is top.
+  // Tell if _map is NULL, or control is top.
   bool stopped();
 
   // Tell if this method or any caller method has exception handlers.
@@ -222,9 +215,9 @@ class GraphKit : public Phase {
   // Detach and return an exception state.
   SafePointNode* pop_exception_state() {
     SafePointNode* ex_map = _exceptions;
-    if (ex_map != nullptr) {
+    if (ex_map != NULL) {
       _exceptions = ex_map->next_exception();
-      ex_map->set_next_exception(nullptr);
+      ex_map->set_next_exception(NULL);
       debug_only(verify_exception_state(ex_map));
     }
     return ex_map;
@@ -247,10 +240,10 @@ class GraphKit : public Phase {
 
   // Combine all exceptions of any sort whatever into a single master state.
   SafePointNode* combine_and_pop_all_exception_states() {
-    if (_exceptions == nullptr)  return nullptr;
+    if (_exceptions == NULL)  return NULL;
     SafePointNode* phi_map = pop_exception_state();
     SafePointNode* ex_map;
-    while ((ex_map = pop_exception_state()) != nullptr) {
+    while ((ex_map = pop_exception_state()) != NULL) {
       combine_exception_states(ex_map, phi_map);
     }
     return phi_map;
@@ -358,16 +351,16 @@ class GraphKit : public Phase {
                            bool replace_length_in_map);
 
 
-  // Helper function to do a null pointer check or ZERO check based on type.
+  // Helper function to do a NULL pointer check or ZERO check based on type.
   // Throw an exception if a given value is null.
   // Return the value cast to not-null.
   // Be clever about equivalent dominating null checks.
   Node* null_check_common(Node* value, BasicType type,
                           bool assert_null = false,
-                          Node* *null_control = nullptr,
+                          Node* *null_control = NULL,
                           bool speculative = false);
   Node* null_check(Node* value, BasicType type = T_OBJECT) {
-    return null_check_common(value, type, false, nullptr, !_gvn.type(value)->speculative_maybe_null());
+    return null_check_common(value, type, false, NULL, !_gvn.type(value)->speculative_maybe_null());
   }
   Node* null_check_receiver() {
     assert(argument(0)->bottom_type()->isa_ptr(), "must be");
@@ -386,7 +379,7 @@ class GraphKit : public Phase {
   // Throw an uncommon trap if a given value is __not__ null.
   // Return the value cast to null, and be clever about dominating checks.
   Node* null_assert(Node* value, BasicType type = T_OBJECT) {
-    return null_check_common(value, type, true, nullptr, _gvn.type(value)->speculative_always_null());
+    return null_check_common(value, type, true, NULL, _gvn.type(value)->speculative_always_null());
   }
 
   // Check if value is null and abort if it is
@@ -419,7 +412,7 @@ class GraphKit : public Phase {
         profile.morphism() == 1) {
       return profile.receiver(0);
     }
-    return nullptr;
+    return NULL;
   }
 
   // record type from profiling with the type system
@@ -445,8 +438,6 @@ class GraphKit : public Phase {
   // Replace all occurrences of one node by another.
   void replace_in_map(Node* old, Node* neww);
 
-  Node* maybe_narrow_object_type(Node* obj, ciKlass* type);
-
   void  push(Node* n)     { map_not_null();        _map->set_stack(_map->_jvms,   _sp++        , n); }
   Node* pop()             { map_not_null(); return _map->stack(    _map->_jvms, --_sp             ); }
   Node* peek(int off = 0) { map_not_null(); return _map->stack(    _map->_jvms,   _sp - off - 1   ); }
@@ -486,7 +477,7 @@ class GraphKit : public Phase {
     int n_size = type2size[n_type];
     if      (n_size == 1)  return pop();
     else if (n_size == 2)  return pop_pair();
-    else                   return nullptr;
+    else                   return NULL;
   }
 
   Node* control()               const { return map_not_null()->control(); }
@@ -556,7 +547,7 @@ class GraphKit : public Phase {
                   bool require_atomic_access = false, bool unaligned = false,
                   bool mismatched = false, bool unsafe = false, uint8_t barrier_data = 0) {
     // This version computes alias_index from an address type
-    assert(adr_type != nullptr, "use other make_load factory");
+    assert(adr_type != NULL, "use other make_load factory");
     return make_load(ctl, adr, t, bt, C->get_alias_index(adr_type),
                      mo, control_dependency, require_atomic_access,
                      unaligned, mismatched, unsafe, barrier_data);
@@ -583,7 +574,7 @@ class GraphKit : public Phase {
                         bool mismatched = false,
                         bool unsafe = false) {
     // This version computes alias_index from an address type
-    assert(adr_type != nullptr, "use other store_to_memory factory");
+    assert(adr_type != NULL, "use other store_to_memory factory");
     return store_to_memory(ctl, adr, val, bt,
                            C->get_alias_index(adr_type),
                            mo, require_atomic_access,
@@ -664,12 +655,12 @@ class GraphKit : public Phase {
   // Return addressing for an array element.
   Node* array_element_address(Node* ary, Node* idx, BasicType elembt,
                               // Optional constraint on the array size:
-                              const TypeInt* sizetype = nullptr,
+                              const TypeInt* sizetype = NULL,
                               // Optional control dependency (for example, on range check)
-                              Node* ctrl = nullptr);
+                              Node* ctrl = NULL);
 
   // Return a load of array element at idx.
-  Node* load_array_element(Node* ary, Node* idx, const TypeAryPtr* arytype, bool set_ctrl);
+  Node* load_array_element(Node* ctl, Node* ary, Node* idx, const TypeAryPtr* arytype);
 
   //---------------- Dtrace support --------------------
   void make_dtrace_method_entry_exit(ciMethod* method, bool is_entry);
@@ -721,12 +712,12 @@ class GraphKit : public Phase {
 
   // Similar to set_edges_for_java_call, but simplified for runtime calls.
   void  set_predefined_output_for_runtime_call(Node* call) {
-    set_predefined_output_for_runtime_call(call, nullptr, nullptr);
+    set_predefined_output_for_runtime_call(call, NULL, NULL);
   }
   void  set_predefined_output_for_runtime_call(Node* call,
                                                Node* keep_mem,
                                                const TypePtr* hook_mem);
-  Node* set_predefined_input_for_runtime_call(SafePointNode* call, Node* narrow_mem = nullptr);
+  Node* set_predefined_input_for_runtime_call(SafePointNode* call, Node* narrow_mem = NULL);
 
   // Replace the call with the current state of the kit.  Requires
   // that the call was generated with separate io_projs so that
@@ -742,13 +733,13 @@ class GraphKit : public Phase {
   // The optional reason is debug information written to the compile log.
   // Optional must_throw is the same as with add_safepoint_edges.
   void uncommon_trap(int trap_request,
-                     ciKlass* klass = nullptr, const char* reason_string = nullptr,
+                     ciKlass* klass = NULL, const char* reason_string = NULL,
                      bool must_throw = false, bool keep_exact_action = false);
 
   // Shorthand, to avoid saying "Deoptimization::" so many times.
   void uncommon_trap(Deoptimization::DeoptReason reason,
                      Deoptimization::DeoptAction action,
-                     ciKlass* klass = nullptr, const char* reason_string = nullptr,
+                     ciKlass* klass = NULL, const char* reason_string = NULL,
                      bool must_throw = false, bool keep_exact_action = false) {
     uncommon_trap(Deoptimization::make_trap_request(reason, action),
                   klass, reason_string, must_throw, keep_exact_action);
@@ -757,7 +748,7 @@ class GraphKit : public Phase {
   // Bail out to the interpreter and keep exact action (avoid switching to Action_none).
   void uncommon_trap_exact(Deoptimization::DeoptReason reason,
                            Deoptimization::DeoptAction action,
-                           ciKlass* klass = nullptr, const char* reason_string = nullptr,
+                           ciKlass* klass = NULL, const char* reason_string = NULL,
                            bool must_throw = false) {
     uncommon_trap(Deoptimization::make_trap_request(reason, action),
                   klass, reason_string, must_throw, /*keep_exact_action=*/true);
@@ -807,11 +798,11 @@ class GraphKit : public Phase {
   Node* make_runtime_call(int flags,
                           const TypeFunc* call_type, address call_addr,
                           const char* call_name,
-                          const TypePtr* adr_type, // null if no memory effects
-                          Node* parm0 = nullptr, Node* parm1 = nullptr,
-                          Node* parm2 = nullptr, Node* parm3 = nullptr,
-                          Node* parm4 = nullptr, Node* parm5 = nullptr,
-                          Node* parm6 = nullptr, Node* parm7 = nullptr);
+                          const TypePtr* adr_type, // NULL if no memory effects
+                          Node* parm0 = NULL, Node* parm1 = NULL,
+                          Node* parm2 = NULL, Node* parm3 = NULL,
+                          Node* parm4 = NULL, Node* parm5 = NULL,
+                          Node* parm6 = NULL, Node* parm7 = NULL);
 
   Node* sign_extend_byte(Node* in);
   Node* sign_extend_short(Node* in);
@@ -835,8 +826,8 @@ class GraphKit : public Phase {
 
   // Helper functions to build synchronizations
   int next_monitor();
-  Node* insert_mem_bar(int opcode, Node* precedent = nullptr);
-  Node* insert_mem_bar_volatile(int opcode, int alias_idx, Node* precedent = nullptr);
+  Node* insert_mem_bar(int opcode, Node* precedent = NULL);
+  Node* insert_mem_bar_volatile(int opcode, int alias_idx, Node* precedent = NULL);
   // Optional 'precedent' is appended as an extra edge, to force ordering.
   FastLockNode* shared_lock(Node* obj);
   void shared_unlock(Node* box, Node* obj);
@@ -851,7 +842,7 @@ class GraphKit : public Phase {
   // Generate a check-cast idiom.  Used by both the check-cast bytecode
   // and the array-store bytecode
   Node* gen_checkcast( Node *subobj, Node* superkls,
-                       Node* *failure_control = nullptr );
+                       Node* *failure_control = NULL );
 
   Node* gen_subtype_check(Node* obj, Node* superklass);
 
@@ -871,11 +862,11 @@ class GraphKit : public Phase {
                                   bool deoptimize_on_exception=false);
   Node* get_layout_helper(Node* klass_node, jint& constant_value);
   Node* new_instance(Node* klass_node,
-                     Node* slow_test = nullptr,
-                     Node* *return_size_val = nullptr,
+                     Node* slow_test = NULL,
+                     Node* *return_size_val = NULL,
                      bool deoptimize_on_exception = false);
   Node* new_array(Node* klass_node, Node* count_val, int nargs,
-                  Node* *return_size_val = nullptr,
+                  Node* *return_size_val = NULL,
                   bool deoptimize_on_exception = false);
 
   // java.lang.String helpers
