@@ -48,6 +48,8 @@ const char *IdealGraphPrinter::NODE_ELEMENT = "node";
 const char *IdealGraphPrinter::NODES_ELEMENT = "nodes";
 const char *IdealGraphPrinter::REMOVE_EDGE_ELEMENT = "removeEdge";
 const char *IdealGraphPrinter::REMOVE_NODE_ELEMENT = "removeNode";
+const char *IdealGraphPrinter::COMPILATION_ID_PROPERTY = "compilationId";
+const char *IdealGraphPrinter::COMPILATION_OSR_PROPERTY = "osr";
 const char *IdealGraphPrinter::METHOD_NAME_PROPERTY = "name";
 const char *IdealGraphPrinter::METHOD_IS_PUBLIC_PROPERTY = "public";
 const char *IdealGraphPrinter::METHOD_IS_STATIC_PROPERTY = "static";
@@ -89,21 +91,47 @@ IdealGraphPrinter *IdealGraphPrinter::printer() {
 }
 
 void IdealGraphPrinter::clean_up() {
-  for (JavaThreadIteratorWithHandle jtiwh; JavaThread *p = jtiwh.next(); ) {
+  for (JavaThreadIteratorWithHandle jtiwh; JavaThread* p = jtiwh.next(); ) {
     if (p->is_Compiler_thread()) {
-      CompilerThread *c = (CompilerThread *)p;
-      IdealGraphPrinter *printer = c->ideal_graph_printer();
+      CompilerThread* c = (CompilerThread*)p;
+      IdealGraphPrinter* printer = c->ideal_graph_printer();
       if (printer) {
         delete printer;
       }
       c->set_ideal_graph_printer(NULL);
     }
   }
+  IdealGraphPrinter* debug_file_printer = Compile::debug_file_printer();
+  if (debug_file_printer != NULL) {
+    delete debug_file_printer;
+  }
+  IdealGraphPrinter* debug_network_printer = Compile::debug_network_printer();
+  if (debug_network_printer != NULL) {
+    delete debug_network_printer;
+  }
 }
 
-// Constructor, either file or network output
+// Either print methods to file specified with PrintIdealGraphFile or otherwise over the network to the IGV
 IdealGraphPrinter::IdealGraphPrinter() {
+  init(PrintIdealGraphFile, true, false);
+}
 
+// Either print methods to the specified file 'file_name' or if NULL over the network to the IGV. If 'append'
+// is set, the next phase is directly appended to the specified file 'file_name'. This is useful when doing
+// replay compilation with a tool like rr that cannot alter the current program state but only the file.
+IdealGraphPrinter::IdealGraphPrinter(Compile* compile, const char* file_name, bool append) {
+  assert(!append || (append && file_name != NULL), "can only use append flag when printing to file");
+  init(file_name, false, append);
+  C = compile;
+  if (append) {
+    // When directly appending the next graph, we only need to set _current_method and not set up a new method
+    _current_method = C->method();
+  } else {
+    begin_method();
+  }
+}
+
+void IdealGraphPrinter::init(const char* file_name, bool use_multiple_files, bool append) {
   // By default dump both ins and outs since dead or unreachable code
   // needs to appear in the graph.  There are also some special cases
   // in the mach where kill projections have no users but should
@@ -114,60 +142,21 @@ IdealGraphPrinter::IdealGraphPrinter() {
   buffer[0] = 0;
   _depth = 0;
   _current_method = NULL;
-  assert(!_current_method, "current method must be initialized to NULL");
-  _stream = NULL;
+  _network_stream = NULL;
 
-  if (PrintIdealGraphFile != NULL) {
-    ThreadCritical tc;
-    // User wants all output to go to files
-    if (_file_count != 0) {
-      ResourceMark rm;
-      stringStream st;
-      const char* dot = strrchr(PrintIdealGraphFile, '.');
-      if (dot) {
-        st.write(PrintIdealGraphFile, dot - PrintIdealGraphFile);
-        st.print("%d%s", _file_count, dot);
-      } else {
-        st.print("%s%d", PrintIdealGraphFile, _file_count);
-      }
-      fileStream *stream = new (ResourceObj::C_HEAP, mtCompiler) fileStream(st.as_string());
-      _output = stream;
-    } else {
-      fileStream *stream = new (ResourceObj::C_HEAP, mtCompiler) fileStream(PrintIdealGraphFile);
-      _output = stream;
-    }
-    _file_count++;
+  if (file_name != NULL) {
+    init_file_stream(file_name, use_multiple_files, append);
   } else {
-    _stream = new (ResourceObj::C_HEAP, mtCompiler) networkStream();
-
-    // Try to connect to visualizer
-    if (_stream->connect(PrintIdealGraphAddress, PrintIdealGraphPort)) {
-      char c = 0;
-      _stream->read(&c, 1);
-      if (c != 'y') {
-        tty->print_cr("Client available, but does not want to receive data!");
-        _stream->close();
-        delete _stream;
-        _stream = NULL;
-        return;
-      }
-      _output = _stream;
-    } else {
-      // It would be nice if we could shut down cleanly but it should
-      // be an error if we can't connect to the visualizer.
-      fatal("Couldn't connect to visualizer at %s:" INTX_FORMAT,
-            PrintIdealGraphAddress, PrintIdealGraphPort);
-    }
+    init_network_stream();
   }
-
   _xml = new (ResourceObj::C_HEAP, mtCompiler) xmlStream(_output);
-
-  head(TOP_ELEMENT);
+  if (!append) {
+    head(TOP_ELEMENT);
+  }
 }
 
 // Destructor, close file or network stream
 IdealGraphPrinter::~IdealGraphPrinter() {
-
   tail(TOP_ELEMENT);
 
   // tty->print_cr("Walk time: %d", (int)_walk_time.milliseconds());
@@ -179,12 +168,12 @@ IdealGraphPrinter::~IdealGraphPrinter() {
     _xml = NULL;
   }
 
-  if (_stream) {
-    delete _stream;
-    if (_stream == _output) {
+  if (_network_stream) {
+    delete _network_stream;
+    if (_network_stream == _output) {
       _output = NULL;
     }
-    _stream = NULL;
+    _network_stream = NULL;
   }
 
   if (_output) {
@@ -282,12 +271,9 @@ void IdealGraphPrinter::print_method(ciMethod *method, int bci, InlineTree *tree
 }
 
 void IdealGraphPrinter::print_inline_tree(InlineTree *tree) {
-
-  if (tree == NULL) return;
-
-  ciMethod *method = tree->method();
-  print_method(tree->method(), tree->caller_bci(), tree);
-
+  if (tree != NULL) {
+    print_method(tree->method(), tree->caller_bci(), tree);
+  }
 }
 
 void IdealGraphPrinter::print_inlining() {
@@ -329,6 +315,12 @@ void IdealGraphPrinter::begin_method() {
     print_prop(METHOD_IS_STATIC_PROPERTY, TRUE_VALUE);
   }
 
+  if (C->is_osr_compilation()) {
+      print_prop(COMPILATION_OSR_PROPERTY, TRUE_VALUE);
+  }
+
+  print_prop(COMPILATION_ID_PROPERTY, C->compile_id());
+
   tail(PROPERTIES_ELEMENT);
 
   _should_send_method = true;
@@ -339,22 +331,11 @@ void IdealGraphPrinter::begin_method() {
 
 // Has to be called whenever a method has finished compilation
 void IdealGraphPrinter::end_method() {
-
-  nmethod* method = (nmethod*)this->_current_method->code();
-
   tail(GROUP_ELEMENT);
   _current_method = NULL;
   _xml->flush();
 }
 
-// Print indent
-void IdealGraphPrinter::print_indent() {
-  tty->print_cr("printing indent %d", _depth);
-  for (int i = 0; i < _depth; i++) {
-    _xml->print("%s", INDENT);
-  }
-}
-
 bool IdealGraphPrinter::traverse_outs() {
   return _traverse_outs;
 }
@@ -367,14 +348,12 @@ void IdealGraphPrinter::visit_node(Node *n, bool edges, VectorSet* temp_set) {
 
   if (edges) {
 
-    // Output edge
-    node_idx_t dest_id = n->_idx;
-    for ( uint i = 0; i < n->len(); i++ ) {
-      if ( n->in(i) ) {
+    for (uint i = 0; i < n->len(); i++) {
+      if (n->in(i)) {
         Node *source = n->in(i);
         begin_elem(EDGE_ELEMENT);
-        print_attr(FROM_PROPERTY, source->_idx);
-        print_attr(TO_PROPERTY, dest_id);
+        print_attr(FROM_PROPERTY, source->_igv_idx);
+        print_attr(TO_PROPERTY, n->_igv_idx);
         print_attr(INDEX_PROPERTY, i);
         end_elem();
       }
@@ -384,7 +363,7 @@ void IdealGraphPrinter::visit_node(Node *n, bool edges, VectorSet* temp_set) {
 
     // Output node
     begin_head(NODE_ELEMENT);
-    print_attr(NODE_ID_PROPERTY, n->_idx);
+    print_attr(NODE_ID_PROPERTY, n->_igv_idx);
     end_head();
 
     head(PROPERTIES_ELEMENT);
@@ -419,22 +398,22 @@ void IdealGraphPrinter::visit_node(Node *n, bool edges, VectorSet* temp_set) {
     }
 
     switch (t->category()) {
-      case Type::CatData:
+      case Type::Category::Data:
         print_prop("category", "data");
         break;
-      case Type::CatMemory:
+      case Type::Category::Memory:
         print_prop("category", "memory");
         break;
-      case Type::CatMixed:
+      case Type::Category::Mixed:
         print_prop("category", "mixed");
         break;
-      case Type::CatControl:
+      case Type::Category::Control:
         print_prop("category", "control");
         break;
-      case Type::CatOther:
+      case Type::Category::Other:
         print_prop("category", "other");
         break;
-      case Type::CatUndef:
+      case Type::Category::Undef:
         print_prop("category", "undef");
         break;
     }
@@ -479,13 +458,10 @@ void IdealGraphPrinter::visit_node(Node *n, bool edges, VectorSet* temp_set) {
       } else {
         print_prop("is_dontcare", "false");
       }
-
-#ifdef ASSERT
       Node* old = C->matcher()->find_old_node(node);
       if (old != NULL) {
         print_prop("old_node_idx", old->_idx);
       }
-#endif
     }
 
     if (node->is_Proj()) {
@@ -569,7 +545,7 @@ void IdealGraphPrinter::visit_node(Node *n, bool edges, VectorSet* temp_set) {
         } else {
           print_prop(short_name, "L");
         }
-      } else if (t->base() == Type::KlassPtr) {
+      } else if (t->base() == Type::KlassPtr || t->base() == Type::InstKlassPtr || t->base() == Type::AryKlassPtr) {
         const TypeKlassPtr *typeKlass = t->is_klassptr();
         print_prop(short_name, "CP");
       } else if (t->base() == Type::Control) {
@@ -619,12 +595,8 @@ void IdealGraphPrinter::visit_node(Node *n, bool edges, VectorSet* temp_set) {
 
 #ifdef ASSERT
     if (node->debug_orig() != NULL) {
-      temp_set->Clear();
       stringStream dorigStream;
-      Node* dorig = node->debug_orig();
-      while (dorig && temp_set->test_set(dorig->_idx)) {
-        dorigStream.print("%d ", dorig->_idx);
-      }
+      node->dump_orig(&dorigStream, false);
       print_prop("debug_orig", dorigStream.as_string());
     }
 #endif
@@ -651,7 +623,7 @@ void IdealGraphPrinter::visit_node(Node *n, bool edges, VectorSet* temp_set) {
 void IdealGraphPrinter::walk_nodes(Node *start, bool edges, VectorSet* temp_set) {
 
 
-  VectorSet visited(Thread::current()->resource_area());
+  VectorSet visited;
   GrowableArray<Node *> nodeStack(Thread::current()->resource_area(), 0, 0, NULL);
   nodeStack.push(start);
   visited.test_set(start->_idx);
@@ -690,14 +662,16 @@ void IdealGraphPrinter::walk_nodes(Node *start, bool edges, VectorSet* temp_set)
   }
 }
 
-void IdealGraphPrinter::print_method(const char *name, int level, bool clear_nodes) {
-  print(name, (Node *)C->root(), level, clear_nodes);
+void IdealGraphPrinter::print_method(const char *name, int level) {
+  if (C->should_print(level)) {
+    print(name, (Node *) C->root());
+  }
 }
 
 // Print current ideal graph
-void IdealGraphPrinter::print(const char *name, Node *node, int level, bool clear_nodes) {
+void IdealGraphPrinter::print(const char *name, Node *node) {
 
-  if (!_current_method || !_should_send_method || !C->should_print(level)) return;
+  if (!_current_method || !_should_send_method || node == NULL) return;
 
   // Warning, unsafe cast?
   _chaitin = (PhaseChaitin *)C->regalloc();
@@ -706,7 +680,7 @@ void IdealGraphPrinter::print(const char *name, Node *node, int level, bool clea
   print_attr(GRAPH_NAME_PROPERTY, (const char *)name);
   end_head();
 
-  VectorSet temp_set(Thread::current()->resource_area());
+  VectorSet temp_set;
 
   head(NODES_ELEMENT);
   if (C->cfg() != NULL) {
@@ -744,7 +718,7 @@ void IdealGraphPrinter::print(const char *name, Node *node, int level, bool clea
       head(NODES_ELEMENT);
       for (uint s = 0; s < block->number_of_nodes(); s++) {
         begin_elem(NODE_ELEMENT);
-        print_attr(NODE_ID_PROPERTY, block->get_node(s)->_idx);
+        print_attr(NODE_ID_PROPERTY, block->get_node(s)->_igv_idx);
         end_elem();
       }
       tail(NODES_ELEMENT);
@@ -757,6 +731,61 @@ void IdealGraphPrinter::print(const char *name, Node *node, int level, bool clea
   _xml->flush();
 }
 
+void IdealGraphPrinter::init_file_stream(const char* file_name, bool use_multiple_files, bool append) {
+  ThreadCritical tc;
+  if (use_multiple_files && _file_count != 0) {
+    assert(!append, "append should only be used for debugging with a single file");
+    ResourceMark rm;
+    stringStream st;
+    const char* dot = strrchr(file_name, '.');
+    if (dot) {
+      st.write(file_name, dot - file_name);
+      st.print("%d%s", _file_count, dot);
+    } else {
+      st.print("%s%d", file_name, _file_count);
+    }
+    _output = new (ResourceObj::C_HEAP, mtCompiler) fileStream(st.as_string(), "w");
+  } else {
+    _output = new (ResourceObj::C_HEAP, mtCompiler) fileStream(file_name, append ? "a" : "w");
+  }
+  if (use_multiple_files) {
+    assert(!append, "append should only be used for debugging with a single file");
+    _file_count++;
+  }
+}
+
+void IdealGraphPrinter::init_network_stream() {
+  _network_stream = new (ResourceObj::C_HEAP, mtCompiler) networkStream();
+  // Try to connect to visualizer
+  if (_network_stream->connect(PrintIdealGraphAddress, PrintIdealGraphPort)) {
+    char c = 0;
+    _network_stream->read(&c, 1);
+    if (c != 'y') {
+      tty->print_cr("Client available, but does not want to receive data!");
+      _network_stream->close();
+      delete _network_stream;
+      _network_stream = NULL;
+      return;
+    }
+    _output = _network_stream;
+  } else {
+    // It would be nice if we could shut down cleanly but it should
+    // be an error if we can't connect to the visualizer.
+    fatal("Couldn't connect to visualizer at %s:" INTX_FORMAT,
+          PrintIdealGraphAddress, PrintIdealGraphPort);
+  }
+}
+
+void IdealGraphPrinter::update_compiled_method(ciMethod* current_method) {
+  assert(C != NULL, "must already be set");
+  if (current_method != _current_method) {
+    // If a different method, end the old and begin with the new one.
+    end_method();
+    _current_method = NULL;
+    begin_method();
+  }
+}
+
 extern const char *NodeClassNames[];
 
 #endif
