@@ -78,7 +78,7 @@ void Parse::print_statistics() {
   tty->print_cr("Blocks parsed: %d  Blocks seen: %d", blocks_parsed, blocks_seen);
 
   if (explicit_null_checks_inserted) {
-    tty->print_cr("%d original null checks - %d elided (%2d%%); optimizer leaves %d,",
+    tty->print_cr("%d original NULL checks - %d elided (%2d%%); optimizer leaves %d,",
                   explicit_null_checks_inserted, explicit_null_checks_elided,
                   (100*explicit_null_checks_elided)/explicit_null_checks_inserted,
                   all_null_checks_found);
@@ -112,7 +112,7 @@ Node *Parse::fetch_interpreter_state(int index,
 
   // Very similar to LoadNode::make, except we handle un-aligned longs and
   // doubles on Sparc.  Intel can handle them just fine directly.
-  Node *l = nullptr;
+  Node *l = NULL;
   switch (bt) {                // Signature is flattened
   case T_INT:     l = new LoadINode(ctl, mem, adr, TypeRawPtr::BOTTOM, TypeInt::INT,        MemNode::unordered); break;
   case T_FLOAT:   l = new LoadFNode(ctl, mem, adr, TypeRawPtr::BOTTOM, Type::FLOAT,         MemNode::unordered); break;
@@ -152,7 +152,7 @@ Node* Parse::check_interpreter_type(Node* l, const Type* type,
 
   // TypeFlow may assert null-ness if a type appears unloaded.
   if (type == TypePtr::NULL_PTR ||
-      (tp != nullptr && !tp->klass()->is_loaded())) {
+      (tp != NULL && !tp->klass()->is_loaded())) {
     // Value must be null, not a real oop.
     Node* chk = _gvn.transform( new CmpPNode(l, null()) );
     Node* tst = _gvn.transform( new BoolNode(chk, BoolTest::eq) );
@@ -168,9 +168,9 @@ Node* Parse::check_interpreter_type(Node* l, const Type* type,
   // When paths are cut off, values at later merge points can rise
   // toward more specific classes.  Make sure these specific classes
   // are still in effect.
-  if (tp != nullptr && tp->klass() != C->env()->Object_klass()) {
+  if (tp != NULL && tp->klass() != C->env()->Object_klass()) {
     // TypeFlow asserted a specific object type.  Value must have that type.
-    Node* bad_type_ctrl = nullptr;
+    Node* bad_type_ctrl = NULL;
     l = gen_checkcast(l, makecon(TypeKlassPtr::make(tp->klass())), &bad_type_ctrl);
     bad_type_exit->control()->add_req(bad_type_ctrl);
   }
@@ -272,7 +272,7 @@ void Parse::load_interpreter_state(Node* osr_buf) {
 
     const Type *type = osr_block->local_type_at(index);
 
-    if (type->isa_oopptr() != nullptr) {
+    if (type->isa_oopptr() != NULL) {
 
       // 6403625: Verify that the interpreter oopMap thinks that the oop is live
       // else we might load a stale oop if the MethodLiveness disagrees with the
@@ -281,7 +281,7 @@ void Parse::load_interpreter_state(Node* osr_buf) {
       //
 
       if (!live_oops.at(index)) {
-        if (C->log() != nullptr) {
+        if (C->log() != NULL) {
           C->log()->elem("OSR_mismatch local_index='%d'",index);
         }
         set_local(index, null());
@@ -303,7 +303,7 @@ void Parse::load_interpreter_state(Node* osr_buf) {
     // Construct code to access the appropriate local.
     BasicType bt = type->basic_type();
     if (type == TypePtr::NULL_PTR) {
-      // Ptr types are mixed together with T_ADDRESS but null is
+      // Ptr types are mixed together with T_ADDRESS but NULL is
       // really for T_OBJECT types so correct it.
       bt = T_OBJECT;
     }
@@ -338,7 +338,7 @@ void Parse::load_interpreter_state(Node* osr_buf) {
     Node* l = local(index);
     if (l->is_top())  continue;  // nothing here
     const Type *type = osr_block->local_type_at(index);
-    if (type->isa_oopptr() != nullptr) {
+    if (type->isa_oopptr() != NULL) {
       if (!live_oops.at(index)) {
         // skip type check for dead oops
         continue;
@@ -400,10 +400,10 @@ Parse::Parse(JVMState* caller, ciMethod* parse_method, float expected_uses)
   _wrote_volatile = false;
   _wrote_stable = false;
   _wrote_fields = false;
-  _alloc_with_final = nullptr;
+  _alloc_with_final = NULL;
   _entry_bci = InvocationEntryBci;
-  _tf = nullptr;
-  _block = nullptr;
+  _tf = NULL;
+  _block = NULL;
   _first_return = true;
   _replaced_nodes_for_exceptions = false;
   _new_idx = C->unique();
@@ -449,7 +449,7 @@ Parse::Parse(JVMState* caller, ciMethod* parse_method, float expected_uses)
   }
 
   CompileLog* log = C->log();
-  if (log != nullptr) {
+  if (log != NULL) {
     log->begin_head("parse method='%d' uses='%f'",
                     log->identify(parse_method), expected_uses);
     if (depth() == 1 && C->is_osr_compilation()) {
@@ -474,7 +474,7 @@ Parse::Parse(JVMState* caller, ciMethod* parse_method, float expected_uses)
       if (total_count < old_count || total_count < md_count)
         total_count = (uint)-1;
       C->set_trap_count(reason, total_count);
-      if (log != nullptr)
+      if (log != NULL)
         log->elem("observe trap='%s' count='%d' total='%d'",
                   Deoptimization::trap_reason_name(reason),
                   md_count, total_count);
@@ -483,11 +483,11 @@ Parse::Parse(JVMState* caller, ciMethod* parse_method, float expected_uses)
   // Accumulate total sum of decompilations, also.
   C->set_decompile_count(C->decompile_count() + md->decompile_count());
 
-  if (log != nullptr && method()->has_exception_handlers()) {
+  if (log != NULL && method()->has_exception_handlers()) {
     log->elem("observe that='has_exception_handlers'");
   }
 
-  assert(InlineTree::check_can_parse(method()) == nullptr, "Can not parse this method, cutout earlier");
+  assert(InlineTree::check_can_parse(method()) == NULL, "Can not parse this method, cutout earlier");
   assert(method()->has_balanced_monitors(), "Can not parse unbalanced monitors, cutout earlier");
 
   // Always register dependence if JVMTI is enabled, because
@@ -554,7 +554,7 @@ Parse::Parse(JVMState* caller, ciMethod* parse_method, float expected_uses)
   SafePointNode* entry_map = create_entry_map();
 
   // Check for bailouts during map initialization
-  if (failing() || entry_map == nullptr) {
+  if (failing() || entry_map == NULL) {
     if (log)  log->done("parse");
     return;
   }
@@ -613,6 +613,8 @@ Parse::Parse(JVMState* caller, ciMethod* parse_method, float expected_uses)
   // Parse all the basic blocks.
   do_all_blocks();
 
+  C->set_default_node_notes(caller_nn);
+
   // Check for bailouts during conversion to graph
   if (failing()) {
     if (log)  log->done("parse");
@@ -623,10 +625,6 @@ Parse::Parse(JVMState* caller, ciMethod* parse_method, float expected_uses)
   set_map(entry_map);
   do_exits();
 
-  // Only reset this now, to make sure that debug information emitted
-  // for exiting control flow still refers to the inlined method.
-  C->set_default_node_notes(caller_nn);
-
   if (log)  log->done("parse nodes='%d' live='%d' memory='" SIZE_FORMAT "'",
                       C->unique(), C->live_nodes(), C->node_arena()->used());
 }
@@ -812,7 +810,7 @@ void Parse::build_exits() {
 
 //----------------------------build_start_state-------------------------------
 // Construct a state which contains only the incoming arguments from an
-// unknown caller.  The method & bci will be null & InvocationEntryBci.
+// unknown caller.  The method & bci will be NULL & InvocationEntryBci.
 JVMState* Compile::build_start_state(StartNode* start, const TypeFunc* tf) {
   int        arg_size = tf->domain()->cnt();
   int        max_size = MAX2(arg_size, (int)tf->range()->cnt());
@@ -821,7 +819,7 @@ JVMState* Compile::build_start_state(StartNode* start, const TypeFunc* tf) {
   record_for_igvn(map);
   assert(arg_size == TypeFunc::Parms + (is_osr_compilation() ? 1 : method()->arg_size()), "correct arg_size");
   Node_Notes* old_nn = default_node_notes();
-  if (old_nn != nullptr && has_method()) {
+  if (old_nn != NULL && has_method()) {
     Node_Notes* entry_nn = old_nn->clone(this);
     JVMState* entry_jvms = new(this) JVMState(method(), old_nn->jvms());
     entry_jvms->set_offsets(0);
@@ -847,7 +845,7 @@ JVMState* Compile::build_start_state(StartNode* start, const TypeFunc* tf) {
 
 //-----------------------------make_node_notes---------------------------------
 Node_Notes* Parse::make_node_notes(Node_Notes* caller_nn) {
-  if (caller_nn == nullptr)  return nullptr;
+  if (caller_nn == NULL)  return NULL;
   Node_Notes* nn = caller_nn->clone(C);
   JVMState* caller_jvms = nn->jvms();
   JVMState* jvms = new (C) JVMState(method(), caller_jvms);
@@ -909,14 +907,14 @@ void Parse::do_exceptions() {
 
   if (failing()) {
     // Pop them all off and throw them away.
-    while (pop_exception_state() != nullptr) ;
+    while (pop_exception_state() != NULL) ;
     return;
   }
 
   PreserveJVMState pjvms(this, false);
 
   SafePointNode* ex_map;
-  while ((ex_map = pop_exception_state()) != nullptr) {
+  while ((ex_map = pop_exception_state()) != NULL) {
     if (!method()->has_exception_handlers()) {
       // Common case:  Transfer control outward.
       // Doing it this early allows the exceptions to common up
@@ -1069,7 +1067,7 @@ void Parse::do_exits() {
     SafePointNode* normal_map = kit.map();  // keep this guy safe
     // Now re-collect the exceptions into _exits:
     SafePointNode* ex_map;
-    while ((ex_map = kit.pop_exception_state()) != nullptr) {
+    while ((ex_map = kit.pop_exception_state()) != NULL) {
       Node* ex_oop = kit.use_exception_state(ex_map);
       // Force the exiting JVM state to have this method at InvocationEntryBci.
       // The exiting JVM state is otherwise a copy of the calling JVMS.
@@ -1104,7 +1102,7 @@ void Parse::do_exits() {
     // Capture very early exceptions (receiver null checks) from caller JVMS
     GraphKit caller(_caller);
     SafePointNode* ex_map;
-    while ((ex_map = caller.pop_exception_state()) != nullptr) {
+    while ((ex_map = caller.pop_exception_state()) != NULL) {
       _exits.add_exception_state(ex_map);
     }
   }
@@ -1120,7 +1118,7 @@ SafePointNode* Parse::create_entry_map() {
   uint len = TypeFunc::Parms + method()->max_locals() + method()->max_stack();
   if (len >= 32760) {
     C->record_method_not_compilable("too many local variables");
-    return nullptr;
+    return NULL;
   }
 
   // clear current replaced nodes that are of no use from here on (map was cloned in build_exits).
@@ -1134,21 +1132,21 @@ SafePointNode* Parse::create_entry_map() {
     if (kit.stopped()) {
       _exits.add_exception_states_from(_caller);
       _exits.set_jvms(_caller);
-      return nullptr;
+      return NULL;
     }
   }
 
-  assert(method() != nullptr, "parser must have a method");
+  assert(method() != NULL, "parser must have a method");
 
   // Create an initial safepoint to hold JVM state during parsing
-  JVMState* jvms = new (C) JVMState(method(), _caller->has_method() ? _caller : nullptr);
+  JVMState* jvms = new (C) JVMState(method(), _caller->has_method() ? _caller : NULL);
   set_map(new SafePointNode(len, jvms));
   jvms->set_map(map());
   record_for_igvn(map());
   assert(jvms->endoff() == len, "correct jvms sizing");
 
   SafePointNode* inmap = _caller->map();
-  assert(inmap != nullptr, "must have inmap");
+  assert(inmap != NULL, "must have inmap");
   // In case of null check on receiver above
   map()->transfer_replaced_nodes_from(inmap, _new_idx);
 
@@ -1204,7 +1202,7 @@ void Parse::do_method_entry() {
     Node* receiver_obj = local(0);
     const TypeInstPtr* receiver_type = _gvn.type(receiver_obj)->isa_instptr();
 
-    if (receiver_type != nullptr && !receiver_type->higher_equal(holder_type)) {
+    if (receiver_type != NULL && !receiver_type->higher_equal(holder_type)) {
       // Receiver should always be a subtype of callee holder.
       // But, since C2 type system doesn't properly track interfaces,
       // the invariant can't be expressed in the type system for default methods.
@@ -1234,7 +1232,7 @@ void Parse::do_method_entry() {
     // FastLockNode becomes the new control parent to pin it to the start.
 
     // Setup Object Pointer
-    Node *lock_obj = nullptr;
+    Node *lock_obj = NULL;
     if (method()->is_static()) {
       ciInstance* mirror = _method->holder()->java_mirror();
       const TypeInstPtr *t_lock = TypeInstPtr::make(mirror);
@@ -1282,11 +1280,11 @@ Parse::Block::Block(Parse* outer, int rpo) : _live_locals() {
   _is_parsed = false;
   _is_handler = false;
   _has_merged_backedge = false;
-  _start_map = nullptr;
+  _start_map = NULL;
   _has_predicates = false;
   _num_successors = 0;
   _all_successors = 0;
-  _successors = nullptr;
+  _successors = NULL;
   assert(pred_count() == 0 && preds_parsed() == 0, "sanity");
   assert(!(is_merged() || is_parsed() || is_handler() || has_merged_backedge()), "sanity");
   assert(_live_locals.size() == 0, "sanity");
@@ -1305,7 +1303,7 @@ void Parse::Block::init_graph(Parse* outer) {
   int ne = tfe->length();
   _num_successors = ns;
   _all_successors = ns+ne;
-  _successors = (ns+ne == 0) ? nullptr : NEW_RESOURCE_ARRAY(Block*, ns+ne);
+  _successors = (ns+ne == 0) ? NULL : NEW_RESOURCE_ARRAY(Block*, ns+ne);
   int p = 0;
   for (int i = 0; i < ns+ne; i++) {
     ciTypeFlow::Block* tf2 = (i < ns) ? tfs->at(i) : tfe->at(i-ns);
@@ -1348,7 +1346,7 @@ Parse::Block* Parse::Block::successor_for_bci(int bci) {
   // of bytecodes.  For example, "obj.field = null" is executable even
   // if the field's type is an unloaded class; the flow pass used to
   // make a trap for such code.
-  return nullptr;
+  return NULL;
 }
 
 
@@ -1528,7 +1526,7 @@ void Parse::do_one_block() {
     }
     assert(bci() < block()->limit(), "bci still in block");
 
-    if (log != nullptr) {
+    if (log != NULL) {
       // Output an optional context marker, to help place actions
       // that occur during parsing of this BC.  If there is no log
       // output until the next context string, this context string
@@ -1563,7 +1561,7 @@ void Parse::do_one_block() {
 
     NOT_PRODUCT( parse_histogram()->record_change(); );
 
-    if (log != nullptr)
+    if (log != NULL)
       log->clear_context();  // skip marker if nothing was printed
 
     // Fall into next bytecode.  Each bytecode normally has 1 sequential
@@ -1578,7 +1576,7 @@ void Parse::do_one_block() {
 void Parse::set_parse_bci(int bci) {
   set_bci(bci);
   Node_Notes* nn = C->default_node_notes();
-  if (nn == nullptr)  return;
+  if (nn == NULL)  return;
 
   // Collect debug info for inlined calls unless -XX:-DebugInlinedCalls.
   if (!DebugInlinedCalls && depth() > 1) {
@@ -1587,7 +1585,7 @@ void Parse::set_parse_bci(int bci) {
 
   // Update the JVMS annotation, if present.
   JVMState* jvms = nn->jvms();
-  if (jvms != nullptr && jvms->bci() != bci) {
+  if (jvms != NULL && jvms->bci() != bci) {
     // Update the JVMS.
     jvms = jvms->clone_shallow(C);
     jvms->set_bci(bci);
@@ -1599,7 +1597,7 @@ void Parse::set_parse_bci(int bci) {
 // Merge the current mapping into the basic block starting at bci
 void Parse::merge(int target_bci) {
   Block* target = successor_for_bci(target_bci);
-  if (target == nullptr) { handle_missing_successor(target_bci); return; }
+  if (target == NULL) { handle_missing_successor(target_bci); return; }
   assert(!target->is_ready(), "our arrival must be expected");
   int pnum = target->next_path_num();
   merge_common(target, pnum);
@@ -1609,7 +1607,7 @@ void Parse::merge(int target_bci) {
 // Merge the current mapping into the basic block, using a new path
 void Parse::merge_new_path(int target_bci) {
   Block* target = successor_for_bci(target_bci);
-  if (target == nullptr) { handle_missing_successor(target_bci); return; }
+  if (target == NULL) { handle_missing_successor(target_bci); return; }
   assert(!target->is_ready(), "new path into frozen graph");
   int pnum = target->add_new_path();
   merge_common(target, pnum);
@@ -1626,7 +1624,7 @@ void Parse::merge_exception(int target_bci) {
 #endif
   assert(sp() == 1, "must have only the throw exception on the stack");
   Block* target = successor_for_bci(target_bci);
-  if (target == nullptr) { handle_missing_successor(target_bci); return; }
+  if (target == NULL) { handle_missing_successor(target_bci); return; }
   assert(target->is_handler(), "exceptions are handled by special blocks");
   int pnum = target->add_new_path();
   merge_common(target, pnum);
@@ -1687,8 +1685,8 @@ void Parse::merge_common(Parse::Block* target, int pnum) {
       RegionNode *r = new RegionNode(edges+1);
       gvn().set_type(r, Type::CONTROL);
       record_for_igvn(r);
-      // zap all inputs to null for debugging (done in Node(uint) constructor)
-      // for (int j = 1; j < edges+1; j++) { r->init_req(j, nullptr); }
+      // zap all inputs to NULL for debugging (done in Node(uint) constructor)
+      // for (int j = 1; j < edges+1; j++) { r->init_req(j, NULL); }
       r->init_req(pnum, control());
       set_control(r);
       set_parse_bci(current_bci); // Restore bci
@@ -1747,7 +1745,7 @@ void Parse::merge_common(Parse::Block* target, int pnum) {
       if (m->is_Phi() && m->as_Phi()->region() == r)
         phi = m->as_Phi();
       else
-        phi = nullptr;
+        phi = NULL;
       if (m != n) {             // Different; must merge
         switch (j) {
         // Frame pointer and Return Address never changes
@@ -1755,11 +1753,11 @@ void Parse::merge_common(Parse::Block* target, int pnum) {
         case TypeFunc::ReturnAdr:
           break;
         case TypeFunc::Memory:  // Merge inputs to the MergeMem node
-          assert(phi == nullptr, "the merge contains phis, not vice versa");
+          assert(phi == NULL, "the merge contains phis, not vice versa");
           merge_memory_edges(n->as_MergeMem(), pnum, nophi);
           continue;
         default:                // All normal stuff
-          if (phi == nullptr) {
+          if (phi == NULL) {
             const JVMState* jvms = map()->jvms();
             if (EliminateNestedLocks &&
                 jvms->is_mon(j) && jvms->is_monitor_box(j)) {
@@ -1781,7 +1779,7 @@ void Parse::merge_common(Parse::Block* target, int pnum) {
       //  - the corresponding control edges is top (a dead incoming path)
       // It is a bug if we create a phi which sees a garbage value on a live path.
 
-      if (phi != nullptr) {
+      if (phi != NULL) {
         assert(n != top() || r->in(pnum) == top(), "live value must not be garbage");
         assert(phi->region() == r, "");
         phi->set_req(pnum, n);  // Then add 'n' to the merge
@@ -1827,15 +1825,15 @@ void Parse::merge_common(Parse::Block* target, int pnum) {
 //--------------------------merge_memory_edges---------------------------------
 void Parse::merge_memory_edges(MergeMemNode* n, int pnum, bool nophi) {
   // (nophi means we must not create phis, because we already parsed here)
-  assert(n != nullptr, "");
+  assert(n != NULL, "");
   // Merge the inputs to the MergeMems
   MergeMemNode* m = merged_memory();
 
   assert(control()->is_Region(), "must be merging to a region");
   RegionNode* r = control()->as_Region();
 
-  PhiNode* base = nullptr;
-  MergeMemNode* remerge = nullptr;
+  PhiNode* base = NULL;
+  MergeMemNode* remerge = NULL;
   for (MergeMemStream mms(m, n); mms.next_non_empty2(); ) {
     Node *p = mms.force_memory();
     Node *q = mms.memory2();
@@ -1843,9 +1841,9 @@ void Parse::merge_memory_edges(MergeMemNode* n, int pnum, bool nophi) {
       // Trouble:  No new splits allowed after a loop body is parsed.
       // Instead, wire the new split into a MergeMem on the backedge.
       // The optimizer will sort it out, slicing the phi.
-      if (remerge == nullptr) {
-        guarantee(base != nullptr, "");
-        assert(base->in(0) != nullptr, "should not be xformed away");
+      if (remerge == NULL) {
+        guarantee(base != NULL, "");
+        assert(base->in(0) != NULL, "should not be xformed away");
         remerge = MergeMemNode::make(base->in(pnum));
         gvn().set_type(remerge, Type::MEMORY);
         base->set_req(pnum, remerge);
@@ -1861,10 +1859,10 @@ void Parse::merge_memory_edges(MergeMemNode* n, int pnum, bool nophi) {
       if (p->is_Phi() && p->as_Phi()->region() == r)
         phi = p->as_Phi();
       else
-        phi = nullptr;
+        phi = NULL;
     }
     // Insert q into local phi
-    if (phi != nullptr) {
+    if (phi != NULL) {
       assert(phi->region() == r, "");
       p = phi;
       phi->set_req(pnum, q);
@@ -1878,7 +1876,7 @@ void Parse::merge_memory_edges(MergeMemNode* n, int pnum, bool nophi) {
     }
   }
   // Transform base last, in case we must fiddle with remerging.
-  if (base != nullptr && pnum == 1) {
+  if (base != NULL && pnum == 1) {
     record_for_igvn(base);
     m->set_base_memory( _gvn.transform_no_reclaim(base) );
   }
@@ -1937,7 +1935,7 @@ int Parse::Block::add_new_path() {
 
   // Add new path to the region.
   uint pnum = r->req();
-  r->add_req(nullptr);
+  r->add_req(NULL);
 
   for (uint i = 1; i < map->req(); i++) {
     Node* n = map->in(i);
@@ -1947,13 +1945,13 @@ int Parse::Block::add_new_path() {
         Node* phi = mms.memory();
         if (phi->is_Phi() && phi->as_Phi()->region() == r) {
           assert(phi->req() == pnum, "must be same size as region");
-          phi->add_req(nullptr);
+          phi->add_req(NULL);
         }
       }
     } else {
       if (n->is_Phi() && n->as_Phi()->region() == r) {
         assert(n->req() == pnum, "must be same size as region");
-        n->add_req(nullptr);
+        n->add_req(NULL);
       }
     }
   }
@@ -1969,9 +1967,9 @@ PhiNode *Parse::ensure_phi(int idx, bool nocreate) {
   assert(region->is_Region(), "");
 
   Node* o = map->in(idx);
-  assert(o != nullptr, "");
+  assert(o != NULL, "");
 
-  if (o == top())  return nullptr; // TOP always merges into TOP
+  if (o == top())  return NULL; // TOP always merges into TOP
 
   if (o->is_Phi() && o->as_Phi()->region() == region) {
     return o->as_Phi();
@@ -1980,7 +1978,7 @@ PhiNode *Parse::ensure_phi(int idx, bool nocreate) {
   // Now use a Phi here for merging
   assert(!nocreate, "Cannot build a phi for a block already parsed.");
   const JVMState* jvms = map->jvms();
-  const Type* t = nullptr;
+  const Type* t = NULL;
   if (jvms->is_loc(idx)) {
     t = block()->local_type_at(idx - jvms->locoff());
   } else if (jvms->is_stk(idx)) {
@@ -1999,14 +1997,14 @@ PhiNode *Parse::ensure_phi(int idx, bool nocreate) {
   // makes it go dead.
   if (t == Type::BOTTOM) {
     map->set_req(idx, top());
-    return nullptr;
+    return NULL;
   }
 
   // Do not create phis for top either.
   // A top on a non-null control flow must be an unused even after the.phi.
   if (t == Type::TOP || t == Type::HALF) {
     map->set_req(idx, top());
-    return nullptr;
+    return NULL;
   }
 
   PhiNode* phi = PhiNode::make(region, o, t);
@@ -2024,7 +2022,7 @@ PhiNode *Parse::ensure_memory_phi(int idx, bool nocreate) {
   assert(region->is_Region(), "");
 
   Node *o = (idx == Compile::AliasIdxBot)? mem->base_memory(): mem->memory_at(idx);
-  assert(o != nullptr && o != top(), "");
+  assert(o != NULL && o != top(), "");
 
   PhiNode* phi;
   if (o->is_Phi() && o->as_Phi()->region() == region) {
@@ -2058,11 +2056,11 @@ PhiNode *Parse::ensure_memory_phi(int idx, bool nocreate) {
 // class need finalization.
 void Parse::call_register_finalizer() {
   Node* receiver = local(0);
-  assert(receiver != nullptr && receiver->bottom_type()->isa_instptr() != nullptr,
+  assert(receiver != NULL && receiver->bottom_type()->isa_instptr() != NULL,
          "must have non-null instance type");
 
   const TypeInstPtr *tinst = receiver->bottom_type()->isa_instptr();
-  if (tinst != nullptr && tinst->klass()->is_loaded() && !tinst->klass_is_exact()) {
+  if (tinst != NULL && tinst->klass()->is_loaded() && !tinst->klass_is_exact()) {
     // The type isn't known exactly so see if CHA tells us anything.
     ciInstanceKlass* ik = tinst->klass()->as_instance_klass();
     if (!Dependencies::has_finalizable_subclass(ik)) {
@@ -2076,10 +2074,10 @@ void Parse::call_register_finalizer() {
   // finalization.  In general this will fold up since the concrete
   // class is often visible so the access flags are constant.
   Node* klass_addr = basic_plus_adr( receiver, receiver, oopDesc::klass_offset_in_bytes() );
-  Node* klass = _gvn.transform(LoadKlassNode::make(_gvn, nullptr, immutable_memory(), klass_addr, TypeInstPtr::KLASS));
+  Node* klass = _gvn.transform(LoadKlassNode::make(_gvn, NULL, immutable_memory(), klass_addr, TypeInstPtr::KLASS));
 
   Node* access_flags_addr = basic_plus_adr(klass, klass, in_bytes(Klass::access_flags_offset()));
-  Node* access_flags = make_load(nullptr, access_flags_addr, TypeInt::INT, T_INT, MemNode::unordered);
+  Node* access_flags = make_load(NULL, access_flags_addr, TypeInt::INT, T_INT, MemNode::unordered);
 
   Node* mask  = _gvn.transform(new AndINode(access_flags, intcon(JVM_ACC_HAS_FINALIZER)));
   Node* check = _gvn.transform(new CmpINode(mask, intcon(0)));
@@ -2102,7 +2100,7 @@ void Parse::call_register_finalizer() {
     Node *call = make_runtime_call(RC_NO_LEAF,
                                    OptoRuntime::register_finalizer_Type(),
                                    OptoRuntime::register_finalizer_Java(),
-                                   nullptr, TypePtr::BOTTOM,
+                                   NULL, TypePtr::BOTTOM,
                                    receiver);
     make_slow_call_ex(call, env()->Throwable_klass(), true);
 
@@ -2231,7 +2229,7 @@ void Parse::return_current(Node* value) {
   }
 
   // frame pointer is always same, already captured
-  if (value != nullptr) {
+  if (value != NULL) {
     // If returning oops to an interface-return, there is a silent free
     // cast from oop to interface allowed by the Verifier.  Make it explicit
     // here.
@@ -2279,7 +2277,7 @@ void Parse::add_safepoint() {
   kill_dead_locals();
 
   // Clone the JVM State
-  SafePointNode *sfpnt = new SafePointNode(parms, nullptr);
+  SafePointNode *sfpnt = new SafePointNode(parms, NULL);
 
   // Capture memory state BEFORE a SafePoint.  Since we can block at a
   // SafePoint we need our GC state to be safe; i.e. we need all our current
@@ -2321,7 +2319,7 @@ void Parse::add_safepoint() {
   // Provide an edge from root to safepoint.  This makes the safepoint
   // appear useful until the parse has completed.
   if (transformed_sfpnt->is_SafePoint()) {
-    assert(C->root() != nullptr, "Expect parse is still valid");
+    assert(C->root() != NULL, "Expect parse is still valid");
     C->root()->add_prec(transformed_sfpnt);
   }
 }
@@ -2329,8 +2327,8 @@ void Parse::add_safepoint() {
 #ifndef PRODUCT
 //------------------------show_parse_info--------------------------------------
 void Parse::show_parse_info() {
-  InlineTree* ilt = nullptr;
-  if (C->ilt() != nullptr) {
+  InlineTree* ilt = NULL;
+  if (C->ilt() != NULL) {
     JVMState* caller_jvms = is_osr_parse() ? caller()->caller() : caller();
     ilt = InlineTree::find_subtree_from_root(C->ilt(), caller_jvms, method());
   }
@@ -2395,7 +2393,7 @@ void Parse::show_parse_info() {
 //------------------------------dump-------------------------------------------
 // Dump information associated with the bytecodes of current _method
 void Parse::dump() {
-  if( method() != nullptr ) {
+  if( method() != NULL ) {
     // Iterate over bytecodes
     ciBytecodeStream iter(method());
     for( Bytecodes::Code bc = iter.next(); bc != ciBytecodeStream::EOBC() ; bc = iter.next() ) {
