@@ -56,7 +56,7 @@ NodeHash::NodeHash(uint est_max_size) :
 #endif
 {
   // _sentinel must be in the current node space
-  _sentinel = new ProjNode(nullptr, TypeFunc::Control);
+  _sentinel = new ProjNode(NULL, TypeFunc::Control);
   memset(_table,0,sizeof(Node*)*_max);
 }
 
@@ -73,7 +73,7 @@ NodeHash::NodeHash(Arena *arena, uint est_max_size) :
 #endif
 {
   // _sentinel must be in the current node space
-  _sentinel = new ProjNode(nullptr, TypeFunc::Control);
+  _sentinel = new ProjNode(NULL, TypeFunc::Control);
   memset(_table,0,sizeof(Node*)*_max);
 }
 
@@ -99,7 +99,7 @@ Node *NodeHash::hash_find( const Node *n ) {
   uint hash = n->hash();
   if (hash == Node::NO_HASH) {
     NOT_PRODUCT( _lookup_misses++ );
-    return nullptr;
+    return NULL;
   }
   uint key = hash & (_max-1);
   uint stride = key | 0x01;
@@ -107,7 +107,7 @@ Node *NodeHash::hash_find( const Node *n ) {
   Node *k = _table[key];        // Get hashed value
   if( !k ) {                    // ?Miss?
     NOT_PRODUCT( _lookup_misses++ );
-    return nullptr;             // Miss!
+    return NULL;                // Miss!
   }
 
   int op = n->Opcode();
@@ -129,11 +129,11 @@ Node *NodeHash::hash_find( const Node *n ) {
     k = _table[key];            // Get hashed value
     if( !k ) {                  // ?Miss?
       NOT_PRODUCT( _lookup_misses++ );
-      return nullptr;           // Miss!
+      return NULL;              // Miss!
     }
   }
   ShouldNotReachHere();
-  return nullptr;
+  return NULL;
 }
 
 //------------------------------hash_find_insert-------------------------------
@@ -144,7 +144,7 @@ Node *NodeHash::hash_find_insert( Node *n ) {
   uint hash = n->hash();
   if (hash == Node::NO_HASH) {
     NOT_PRODUCT( _lookup_misses++ );
-    return nullptr;
+    return NULL;
   }
   uint key = hash & (_max-1);
   uint stride = key | 0x01;     // stride must be relatively prime to table siz
@@ -156,7 +156,7 @@ Node *NodeHash::hash_find_insert( Node *n ) {
     _table[key] = n;            // Insert into table!
     debug_only(n->enter_hash_lock()); // Lock down the node while in the table.
     check_grow();               // Grow table if insert hit limit
-    return nullptr;             // Miss!
+    return NULL;                // Miss!
   }
   else if( k == _sentinel ) {
     first_sentinel = key;      // Can insert here
@@ -185,7 +185,7 @@ Node *NodeHash::hash_find_insert( Node *n ) {
       _table[key] = n;          // Insert into table!
       debug_only(n->enter_hash_lock()); // Lock down the node while in the table.
       check_grow();             // Grow table if insert hit limit
-      return nullptr;           // Miss!
+      return NULL;              // Miss!
     }
     else if( first_sentinel == 0 && k == _sentinel ) {
       first_sentinel = key;    // Can insert here
@@ -193,7 +193,7 @@ Node *NodeHash::hash_find_insert( Node *n ) {
 
   }
   ShouldNotReachHere();
-  return nullptr;
+  return NULL;
 }
 
 //------------------------------hash_insert------------------------------------
@@ -235,7 +235,7 @@ bool NodeHash::hash_delete( const Node *n ) {
   uint key = hash & (_max-1);
   uint stride = key | 0x01;
   debug_only( uint counter = 0; );
-  for( ; /* (k != nullptr) && (k != _sentinel) */; ) {
+  for( ; /* (k != NULL) && (k != _sentinel) */; ) {
     debug_only( counter++ );
     NOT_PRODUCT( _delete_probes++ );
     k = _table[key];            // Get hashed value
@@ -294,7 +294,7 @@ void  NodeHash::grow() {
 }
 
 //------------------------------clear------------------------------------------
-// Clear all entries in _table to null but keep storage
+// Clear all entries in _table to NULL but keep storage
 void  NodeHash::clear() {
 #ifdef ASSERT
   // Unlock all nodes upon removal from table.
@@ -319,7 +319,7 @@ void NodeHash::remove_useless_nodes(VectorSet &useful) {
   Node *sentinel_node = sentinel();
   for( uint i = 0; i < max; ++i ) {
     Node *n = at(i);
-    if(n != nullptr && n != sentinel_node && !useful.test(n->_idx)) {
+    if(n != NULL && n != sentinel_node && !useful.test(n->_idx)) {
       debug_only(n->exit_hash_lock()); // Unlock the node when removed
       _table[i] = sentinel_node;       // Replace with placeholder
     }
@@ -335,7 +335,7 @@ void NodeHash::check_no_speculative_types() {
   Node *sentinel_node = sentinel();
   for (uint i = 0; i < max; ++i) {
     Node *n = at(i);
-    if (n != nullptr &&
+    if (n != NULL &&
         n != sentinel_node &&
         n->is_Type() &&
         live_nodes.member(n)) {
@@ -379,7 +379,7 @@ Node *NodeHash::find_index(uint idx) { // For debugging
     if( !m || m == _sentinel ) continue;
     if( m->_idx == (uint)idx ) return m;
   }
-  return nullptr;
+  return NULL;
 }
 #endif
 
@@ -423,7 +423,7 @@ PhaseRemoveUseless::PhaseRemoveUseless(PhaseGVN* gvn, Unique_Node_List* worklist
   worklist->remove_useless_nodes(_useful.member_set());
 
   // Disconnect 'useless' nodes that are adjacent to useful nodes
-  C->disconnect_useless_nodes(_useful, worklist);
+  C->remove_useless_nodes(_useful);
 }
 
 //=============================================================================
@@ -469,14 +469,6 @@ PhaseRenumberLive::PhaseRenumberLive(PhaseGVN* gvn,
 
   uint worklist_size = worklist->size();
 
-  GrowableArray<Node_Notes*>* old_node_note_array = C->node_note_array();
-  if (old_node_note_array != nullptr) {
-    int new_size = (_useful.size() >> 8) + 1; // The node note array uses blocks, see C->_log2_node_notes_block_size
-    new_size = MAX2(8, new_size);
-    C->set_node_note_array(new (C->comp_arena()) GrowableArray<Node_Notes*> (C->comp_arena(), new_size, 0, nullptr));
-    C->grow_node_notes(C->node_note_array(), new_size);
-  }
-
   // Iterate over the set of live nodes.
   for (uint current_idx = 0; current_idx < _useful.size(); current_idx++) {
     Node* n = _useful.at(current_idx);
@@ -492,11 +484,6 @@ PhaseRenumberLive::PhaseRenumberLive(PhaseGVN* gvn,
     assert(_old2new_map.at(n->_idx) == -1, "already seen");
     _old2new_map.at_put(n->_idx, current_idx);
 
-    if (old_node_note_array != nullptr) {
-      Node_Notes* nn = C->locate_node_notes(old_node_note_array, n->_idx);
-      C->set_node_notes_at(current_idx, nn);
-    }
-
     n->set_idx(current_idx); // Update node ID.
 
     if (in_worklist) {
@@ -567,7 +554,7 @@ int PhaseRenumberLive::update_embedded_ids(Node* n) {
   }
 
   const Type* type = _new_type_array.fast_lookup(n->_idx);
-  if (type != nullptr && type->isa_oopptr() && type->is_oopptr()->is_known_instance()) {
+  if (type != NULL && type->isa_oopptr() && type->is_oopptr()->is_known_instance()) {
     if (!_is_pass_finished) {
         return -1; // delay
     }
@@ -595,7 +582,7 @@ PhaseTransform::PhaseTransform( PhaseNumber pnum ) : Phase(pnum),
   set_allow_progress(true);
 #endif
   // Force allocation for currently existing nodes
-  _types.map(C->unique(), nullptr);
+  _types.map(C->unique(), NULL);
 }
 
 //------------------------------PhaseTransform---------------------------------
@@ -611,7 +598,7 @@ PhaseTransform::PhaseTransform( Arena *arena, PhaseNumber pnum ) : Phase(pnum),
   set_allow_progress(true);
 #endif
   // Force allocation for currently existing nodes
-  _types.map(C->unique(), nullptr);
+  _types.map(C->unique(), NULL);
 }
 
 //------------------------------PhaseTransform---------------------------------
@@ -638,22 +625,22 @@ void PhaseTransform::init_con_caches() {
 
 //--------------------------------find_int_type--------------------------------
 const TypeInt* PhaseTransform::find_int_type(Node* n) {
-  if (n == nullptr)  return nullptr;
+  if (n == NULL)  return NULL;
   // Call type_or_null(n) to determine node's type since we might be in
   // parse phase and call n->Value() may return wrong type.
   // (For example, a phi node at the beginning of loop parsing is not ready.)
   const Type* t = type_or_null(n);
-  if (t == nullptr)  return nullptr;
+  if (t == NULL)  return NULL;
   return t->isa_int();
 }
 
 
 //-------------------------------find_long_type--------------------------------
 const TypeLong* PhaseTransform::find_long_type(Node* n) {
-  if (n == nullptr)  return nullptr;
+  if (n == NULL)  return NULL;
   // (See comment above on type_or_null.)
   const Type* t = type_or_null(n);
-  if (t == nullptr)  return nullptr;
+  if (t == NULL)  return NULL;
   return t->isa_long();
 }
 
@@ -695,7 +682,7 @@ void PhaseTransform::dump_nodes_and_types_recur( const Node *n, uint depth, bool
     dump_nodes_and_types_recur( n->in(i), depth-1, only_ctrl, visited );
   }
   n->dump();
-  if (type_or_null(n) != nullptr) {
+  if (type_or_null(n) != NULL) {
     tty->print("      "); type(n)->dump(); tty->cr();
   }
 }
@@ -758,10 +745,10 @@ ConNode* PhaseValues::uncached_makecon(const Type *t) {
   assert(t->singleton(), "must be a constant");
   ConNode* x = ConNode::make(t);
   ConNode* k = (ConNode*)hash_find_insert(x); // Value numbering
-  if (k == nullptr) {
+  if (k == NULL) {
     set_type(x, t);             // Missed, provide type mapping
     GrowableArray<Node_Notes*>* nna = C->node_note_array();
-    if (nna != nullptr) {
+    if (nna != NULL) {
       Node_Notes* loc = C->locate_node_notes(nna, x->_idx, true);
       loc->clear(); // do not put debug info on constants
     }
@@ -778,7 +765,7 @@ ConINode* PhaseTransform::intcon(jint i) {
   // Small integer?  Check cache! Check that cached node is not dead
   if (i >= _icon_min && i <= _icon_max) {
     ConINode* icon = _icons[i-_icon_min];
-    if (icon != nullptr && icon->in(TypeFunc::Control) != nullptr)
+    if (icon != NULL && icon->in(TypeFunc::Control) != NULL)
       return icon;
   }
   ConINode* icon = (ConINode*) uncached_makecon(TypeInt::make(i));
@@ -794,7 +781,7 @@ ConLNode* PhaseTransform::longcon(jlong l) {
   // Small integer?  Check cache! Check that cached node is not dead
   if (l >= _lcon_min && l <= _lcon_max) {
     ConLNode* lcon = _lcons[l-_lcon_min];
-    if (lcon != nullptr && lcon->in(TypeFunc::Control) != nullptr)
+    if (lcon != NULL && lcon->in(TypeFunc::Control) != NULL)
       return lcon;
   }
   ConLNode* lcon = (ConLNode*) uncached_makecon(TypeLong::make(l));
@@ -805,9 +792,7 @@ ConLNode* PhaseTransform::longcon(jlong l) {
 }
 ConNode* PhaseTransform::integercon(jlong l, BasicType bt) {
   if (bt == T_INT) {
-    jint int_con = (jint)l;
-    assert(((long)int_con) == l, "not an int");
-    return intcon(int_con);
+    return intcon(checked_cast<jint>(l));
   }
   assert(bt == T_LONG, "not an integer");
   return longcon(l);
@@ -819,7 +804,7 @@ ConNode* PhaseTransform::integercon(jlong l, BasicType bt) {
 ConNode* PhaseTransform::zerocon(BasicType bt) {
   assert((uint)bt <= _zcon_max, "domain check");
   ConNode* zcon = _zcons[bt];
-  if (zcon != nullptr && zcon->in(TypeFunc::Control) != nullptr)
+  if (zcon != NULL && zcon->in(TypeFunc::Control) != NULL)
     return zcon;
   zcon = (ConNode*) uncached_makecon(Type::get_zero_type(bt));
   _zcons[bt] = zcon;
@@ -831,7 +816,7 @@ ConNode* PhaseTransform::zerocon(BasicType bt) {
 //=============================================================================
 Node* PhaseGVN::apply_ideal(Node* k, bool can_reshape) {
   Node* i = BarrierSet::barrier_set()->barrier_set_c2()->ideal_node(this, k, can_reshape);
-  if (i == nullptr) {
+  if (i == NULL) {
     i = k->Ideal(this, can_reshape);
   }
   return i;
@@ -854,7 +839,7 @@ Node *PhaseGVN::transform_no_reclaim(Node *n) {
   Node* k = n;
   Node* i = apply_ideal(k, /*can_reshape=*/false);
   NOT_PRODUCT(uint loop_count = 1;)
-  while (i != nullptr) {
+  while (i != NULL) {
     assert(i->_idx >= k->_idx, "Idealize should return new nodes, use Identity to return old nodes" );
     k = i;
 #ifdef ASSERT
@@ -875,11 +860,11 @@ Node *PhaseGVN::transform_no_reclaim(Node *n) {
   // cache Value.  Later requests for the local phase->type of this Node can
   // use the cached Value instead of suffering with 'bottom_type'.
   const Type* t = k->Value(this); // Get runtime Value set
-  assert(t != nullptr, "value sanity");
+  assert(t != NULL, "value sanity");
   if (type_or_null(k) != t) {
 #ifndef PRODUCT
     // Do not count initial visit to node as a transformation
-    if (type_or_null(k) == nullptr) {
+    if (type_or_null(k) == NULL) {
       inc_new_values();
       set_progress();
     }
@@ -925,7 +910,7 @@ bool PhaseGVN::is_dominator_helper(Node *d, Node *n, bool linear_only) {
   while (d != n) {
     n = IfNode::up_one_dom(n, linear_only);
     i++;
-    if (n == nullptr || i >= 100) {
+    if (n == NULL || i >= 100) {
       return false;
     }
   }
@@ -938,7 +923,7 @@ bool PhaseGVN::is_dominator_helper(Node *d, Node *n, bool linear_only) {
 // or through an other data node excluding cons and phis.
 void PhaseGVN::dead_loop_check( Node *n ) {
   // Phi may reference itself in a loop
-  if (n != nullptr && !n->is_dead_loop_safe() && !n->is_CFG()) {
+  if (n != NULL && !n->is_dead_loop_safe() && !n->is_CFG()) {
     // Do 2 levels check and only data inputs.
     bool no_dead_loop = true;
     uint cnt = n->req();
@@ -946,7 +931,7 @@ void PhaseGVN::dead_loop_check( Node *n ) {
       Node *in = n->in(i);
       if (in == n) {
         no_dead_loop = false;
-      } else if (in != nullptr && !in->is_dead_loop_safe()) {
+      } else if (in != NULL && !in->is_dead_loop_safe()) {
         uint icnt = in->req();
         for (uint j = 1; j < icnt && no_dead_loop; j++) {
           if (in->in(j) == n || in->in(j) == in)
@@ -1001,7 +986,7 @@ PhaseIterGVN::PhaseIterGVN(PhaseGVN* gvn) : PhaseGVN(gvn),
   max = _table.size();
   for( uint i = 0; i < max; ++i ) {
     Node *n = _table.at(i);
-    if(n != nullptr && n != _table.sentinel() && n->outcnt() == 0) {
+    if(n != NULL && n != _table.sentinel() && n->outcnt() == 0) {
       if( n->is_top() ) continue;
       // If remove_useless_nodes() has run, we expect no such nodes left.
       assert(false, "remove_useless_nodes missed this node");
@@ -1048,7 +1033,7 @@ void PhaseIterGVN::verify_step(Node* n) {
     }
     for (int i = 0; i < _verify_window_size; i++) {
       Node* n = _verify_window[i];
-      if (n == nullptr) {
+      if (n == NULL) {
         continue;
       }
       if (n->in(0) == NodeSentinel) { // xform_idom
@@ -1072,7 +1057,7 @@ void PhaseIterGVN::trace_PhaseIterGVN(Node* n, Node* nn, const Type* oldtype) {
     if (nn != n) {
       // print old node
       tty->print("< ");
-      if (oldtype != newtype && oldtype != nullptr) {
+      if (oldtype != newtype && oldtype != NULL) {
         oldtype->dump();
       }
       do { tty->print("\t"); } while (tty->position() < 16);
@@ -1081,14 +1066,14 @@ void PhaseIterGVN::trace_PhaseIterGVN(Node* n, Node* nn, const Type* oldtype) {
     }
     if (oldtype != newtype || nn != n) {
       // print new node and/or new type
-      if (oldtype == nullptr) {
+      if (oldtype == NULL) {
         tty->print("* ");
       } else if (nn != n) {
         tty->print("> ");
       } else {
         tty->print("= ");
       }
-      if (newtype == nullptr) {
+      if (newtype == NULL) {
         tty->print("null");
       } else {
         newtype->dump();
@@ -1106,7 +1091,7 @@ void PhaseIterGVN::trace_PhaseIterGVN(Node* n, Node* nn, const Type* oldtype) {
     }
     if (nn != n) {
       // ignore n, it might be subsumed
-      verify_step((Node*) nullptr);
+      verify_step((Node*) NULL);
     }
   }
 }
@@ -1115,12 +1100,12 @@ void PhaseIterGVN::init_verifyPhaseIterGVN() {
   _verify_counter = 0;
   _verify_full_passes = 0;
   for (int i = 0; i < _verify_window_size; i++) {
-    _verify_window[i] = nullptr;
+    _verify_window[i] = NULL;
   }
 #ifdef ASSERT
   // Verify that all modified nodes are on _worklist
   Unique_Node_List* modified_list = C->modified_nodes();
-  while (modified_list != nullptr && modified_list->size()) {
+  while (modified_list != NULL && modified_list->size()) {
     Node* n = modified_list->pop();
     if (!n->is_Con() && !_worklist.member(n)) {
       n->dump();
@@ -1134,7 +1119,7 @@ void PhaseIterGVN::verify_PhaseIterGVN() {
 #ifdef ASSERT
   // Verify nodes with changed inputs.
   Unique_Node_List* modified_list = C->modified_nodes();
-  while (modified_list != nullptr && modified_list->size()) {
+  while (modified_list != NULL && modified_list->size()) {
     Node* n = modified_list->pop();
     if (!n->is_Con()) { // skip Con nodes
       n->dump();
@@ -1155,7 +1140,7 @@ void PhaseIterGVN::verify_PhaseIterGVN() {
   }
 
 #ifdef ASSERT
-  if (modified_list != nullptr) {
+  if (modified_list != NULL) {
     while (modified_list->size() > 0) {
       Node* n = modified_list->pop();
       n->dump();
@@ -1233,7 +1218,7 @@ void PhaseIterGVN::optimize() {
 Node* PhaseIterGVN::register_new_node_with_optimizer(Node* n, Node* orig) {
   set_type_bottom(n);
   _worklist.push(n);
-  if (orig != nullptr)  C->copy_node_notes_to(n, orig);
+  if (orig != NULL)  C->copy_node_notes_to(n, orig);
   return n;
 }
 
@@ -1248,7 +1233,7 @@ Node *PhaseIterGVN::transform( Node *n ) {
 
   // If brand new node, make space in type array, and give it a type.
   ensure_type_or_null(n);
-  if (type_or_null(n) == nullptr) {
+  if (type_or_null(n) == NULL) {
     set_type_bottom(n);
   }
 
@@ -1275,7 +1260,7 @@ Node *PhaseIterGVN::transform_old(Node* n) {
 #endif
 
   DEBUG_ONLY(uint loop_count = 1;)
-  while (i != nullptr) {
+  while (i != NULL) {
 #ifdef ASSERT
     if (loop_count >= K + C->live_nodes()) {
       dump_infinite_loop_info(i, "PhaseIterGVN::transform_old");
@@ -1307,7 +1292,7 @@ Node *PhaseIterGVN::transform_old(Node* n) {
 
   // See what kind of values 'k' takes on at runtime
   const Type* t = k->Value(this);
-  assert(t != nullptr, "value sanity");
+  assert(t != NULL, "value sanity");
 
   // Since I just called 'Value' to compute the set of run-time values
   // for this Node, and 'Value' is non-local (and therefore expensive) I'll
@@ -1392,8 +1377,8 @@ void PhaseIterGVN::remove_globally_dead_node( Node *dead ) {
         // Smash all inputs to 'dead', isolating him completely
         for (uint i = 0; i < dead->req(); i++) {
           Node *in = dead->in(i);
-          if (in != nullptr && in != C->top()) {  // Points to something?
-            int nrep = dead->replace_edge(in, nullptr, this);  // Kill edges
+          if (in != NULL && in != C->top()) {  // Points to something?
+            int nrep = dead->replace_edge(in, NULL, this);  // Kill edges
             assert((nrep > 0), "sanity");
             if (in->outcnt() == 0) { // Made input go dead?
               _stack.push(in, PROCESS_INPUTS); // Recursively remove
@@ -1418,7 +1403,7 @@ void PhaseIterGVN::remove_globally_dead_node( Node *dead ) {
               BarrierSet::barrier_set()->barrier_set_c2()->enqueue_useful_gc_barrier(this, in);
             }
             if (ReduceFieldZeroing && dead->is_Load() && i == MemNode::Memory &&
-                in->is_Proj() && in->in(0) != nullptr && in->in(0)->is_Initialize()) {
+                in->is_Proj() && in->in(0) != NULL && in->in(0)->is_Initialize()) {
               // A Load that directly follows an InitializeNode is
               // going away. The Stores that follow are candidates
               // again to be captured by the InitializeNode.
@@ -1429,7 +1414,7 @@ void PhaseIterGVN::remove_globally_dead_node( Node *dead ) {
                 }
               }
             }
-          } // if (in != nullptr && in != C->top())
+          } // if (in != NULL && in != C->top())
         } // for (uint i = 0; i < dead->req(); i++)
         if (recurse) {
           continue;
@@ -1485,11 +1470,11 @@ void PhaseIterGVN::subsume_node( Node *old, Node *nn ) {
 
   // Search for instance field data PhiNodes in the same region pointing to the old
   // memory PhiNode and update their instance memory ids to point to the new node.
-  if (old->is_Phi() && old->as_Phi()->type()->has_memory() && old->in(0) != nullptr) {
+  if (old->is_Phi() && old->as_Phi()->type()->has_memory() && old->in(0) != NULL) {
     Node* region = old->in(0);
     for (DUIterator_Fast imax, i = region->fast_outs(imax); i < imax; i++) {
       PhiNode* phi = region->fast_out(i)->isa_Phi();
-      if (phi != nullptr && phi->inst_mem_id() == (int)old->_idx) {
+      if (phi != NULL && phi->inst_mem_id() == (int)old->_idx) {
         phi->set_inst_mem_id((int)nn->_idx);
       }
     }
@@ -1500,7 +1485,7 @@ void PhaseIterGVN::subsume_node( Node *old, Node *nn ) {
   temp->init_req(0,nn);     // Add a use to nn to prevent him from dying
   remove_dead_node( old );
   temp->del_req(0);         // Yank bogus edge
-  if (nn != nullptr && nn->outcnt() == 0) {
+  if (nn != NULL && nn->outcnt() == 0) {
     _worklist.push(nn);
   }
 #ifndef PRODUCT
@@ -1532,14 +1517,14 @@ static PhiNode* countedloop_phi_from_cmp(CmpNode* cmp, Node* n) {
         BaseCountedLoopEndNode* cle = iff->as_BaseCountedLoopEnd();
         if (cle->limit() == n) {
           PhiNode* phi = cle->phi();
-          if (phi != nullptr) {
+          if (phi != NULL) {
             return phi;
           }
         }
       }
     }
   }
-  return nullptr;
+  return NULL;
 }
 
 void PhaseIterGVN::add_users_to_worklist( Node *n ) {
@@ -1554,12 +1539,12 @@ void PhaseIterGVN::add_users_to_worklist( Node *n ) {
       add_users_to_worklist0(use);
 
     // If we changed the receiver type to a call, we need to revisit
-    // the Catch following the call.  It's looking for a non-null
+    // the Catch following the call.  It's looking for a non-NULL
     // receiver to know when to enable the regular fall-through path
     // in addition to the NullPtrException path.
     if (use->is_CallDynamicJava() && n == use->in(TypeFunc::Parms)) {
       Node* p = use->as_CallDynamicJava()->proj_out_or_null(TypeFunc::Control);
-      if (p != nullptr) {
+      if (p != NULL) {
         add_users_to_worklist0(p);
       }
     }
@@ -1587,7 +1572,7 @@ void PhaseIterGVN::add_users_to_worklist( Node *n ) {
       }
       if (use_op == Op_CmpI) {
         Node* phi = countedloop_phi_from_cmp((CmpINode*)use, n);
-        if (phi != nullptr) {
+        if (phi != NULL) {
           // If an opaque node feeds into the limit condition of a
           // CountedLoop, we need to process the Phi node for the
           // induction variable when the opaque node is removed:
@@ -1650,24 +1635,14 @@ void PhaseIterGVN::add_users_to_worklist( Node *n ) {
     // If changed initialization activity, check dependent Stores
     if (use_op == Op_Allocate || use_op == Op_AllocateArray) {
       InitializeNode* init = use->as_Allocate()->initialization();
-      if (init != nullptr) {
+      if (init != NULL) {
         Node* imem = init->proj_out_or_null(TypeFunc::Memory);
-        if (imem != nullptr)  add_users_to_worklist0(imem);
+        if (imem != NULL)  add_users_to_worklist0(imem);
       }
     }
-    // If the ValidLengthTest input changes then the fallthrough path out of the AllocateArray may have become dead.
-    // CatchNode::Value() is responsible for killing that path. The CatchNode has to be explicitly enqueued for igvn
-    // to guarantee the change is not missed.
-    if (use_op == Op_AllocateArray && n == use->in(AllocateNode::ValidLengthTest)) {
-      Node* p = use->as_AllocateArray()->proj_out_or_null(TypeFunc::Control);
-      if (p != nullptr) {
-        add_users_to_worklist0(p);
-      }
-    }
-
     if (use_op == Op_Initialize) {
       Node* imem = use->as_Initialize()->proj_out_or_null(TypeFunc::Memory);
-      if (imem != nullptr)  add_users_to_worklist0(imem);
+      if (imem != NULL)  add_users_to_worklist0(imem);
     }
     // Loading the java mirror from a Klass requires two loads and the type
     // of the mirror load depends on the type of 'n'. See LoadNode::Value().
@@ -1703,7 +1678,7 @@ void PhaseIterGVN::remove_speculative_types()  {
   assert(UseTypeSpeculation, "speculation is off");
   for (uint i = 0; i < _types.Size(); i++)  {
     const Type* t = _types.fast_lookup(i);
-    if (t != nullptr) {
+    if (t != NULL) {
       _types.map(i, t->remove_speculative());
     }
   }
@@ -1716,7 +1691,7 @@ bool PhaseIterGVN::no_dependent_zero_check(Node* n) const {
     case Op_DivI:
     case Op_ModI: {
       // Type of divisor includes 0?
-      if (type(n->in(2)) == Type::TOP) {
+      if (n->in(2)->is_top()) {
         // 'n' is dead. Treat as if zero check is still there to avoid any further optimizations.
         return false;
       }
@@ -1726,7 +1701,7 @@ bool PhaseIterGVN::no_dependent_zero_check(Node* n) const {
     case Op_DivL:
     case Op_ModL: {
       // Type of divisor includes 0?
-      if (type(n->in(2)) == Type::TOP) {
+      if (n->in(2)->is_top()) {
         // 'n' is dead. Treat as if zero check is still there to avoid any further optimizations.
         return false;
       }
@@ -1763,7 +1738,7 @@ PhaseCCP::~PhaseCCP() {
 
 #ifdef ASSERT
 static bool ccp_type_widens(const Type* t, const Type* t0) {
-  assert(t->meet(t0) == t->remove_speculative(), "Not monotonic");
+  assert(t->meet(t0) == t, "Not monotonic");
   switch (t->base() == t0->base() ? t->base() : Type::Top) {
   case Type::Int:
     assert(t0->isa_int()->_widen <= t->isa_int()->_widen, "widen increases");
@@ -1789,9 +1764,6 @@ void PhaseCCP::analyze() {
   Unique_Node_List worklist;
   worklist.push(C->root());
 
-  assert(_root_and_safepoints.size() == 0, "must be empty (unused)");
-  _root_and_safepoints.push(C->root());
-
   // Pull from worklist; compute new value; push changes out.
   // This loop is the meat of CCP.
   while( worklist.size() ) {
@@ -1801,11 +1773,6 @@ void PhaseCCP::analyze() {
     } else {
       n = worklist.pop();
     }
-    if (n->is_SafePoint()) {
-      // Make sure safepoints are processed by PhaseCCP::transform even if they are
-      // not reachable from the bottom. Otherwise, infinite loops would be removed.
-      _root_and_safepoints.push(n);
-    }
     const Type *t = n->Value(this);
     if (t != type(n)) {
       assert(ccp_type_widens(t, type(n)), "ccp type must widen");
@@ -1828,16 +1795,15 @@ void PhaseCCP::analyze() {
           }
         }
         // If we changed the receiver type to a call, we need to revisit
-        // the Catch following the call.  It's looking for a non-nullptr
+        // the Catch following the call.  It's looking for a non-NULL
         // receiver to know when to enable the regular fall-through path
         // in addition to the NullPtrException path
-        // Same is true if the type of a ValidLengthTest input to an AllocateArrayNode changes.
         if (m->is_Call()) {
           for (DUIterator_Fast i2max, i2 = m->fast_outs(i2max); i2 < i2max; i2++) {
             Node* p = m->fast_out(i2);  // Propagate changes to uses
             if (p->is_Proj() && p->as_Proj()->_con == TypeFunc::Control) {
               Node* catch_node = p->find_out_with(Op_Catch);
-              if (catch_node != nullptr) {
+              if (catch_node != NULL) {
                 worklist.push(catch_node);
               }
             }
@@ -1868,7 +1834,7 @@ void PhaseCCP::analyze() {
         // PhiNode::Value().
         if (m_op == Op_CmpI || m_op == Op_CmpL) {
           PhiNode* phi = countedloop_phi_from_cmp(m->as_Cmp(), n);
-          if (phi != nullptr) {
+          if (phi != NULL) {
             worklist.push(phi);
           }
         }
@@ -1895,30 +1861,6 @@ void PhaseCCP::analyze() {
             }
           }
         }
-        push_cast_ii(worklist, n, m);
-      }
-    }
-  }
-}
-
-void PhaseCCP::push_if_not_bottom_type(Unique_Node_List& worklist, Node* n) const {
-  if (n->bottom_type() != type(n)) {
-    worklist.push(n);
-  }
-}
-
-// CastII::Value() optimizes CmpI/If patterns if the right input of the CmpI has a constant type. If the CastII input is
-// the same node as the left input into the CmpI node, the type of the CastII node can be improved accordingly. Add the
-// CastII node back to the worklist to re-apply Value() to either not miss this optimization or to undo it because it
-// cannot be applied anymore. We could have optimized the type of the CastII before but now the type of the right input
-// of the CmpI (i.e. 'parent') is no longer constant. The type of the CastII must be widened in this case.
-void PhaseCCP::push_cast_ii(Unique_Node_List& worklist, const Node* parent, const Node* use) const {
-  if (use->Opcode() == Op_CmpI && use->in(2) == parent) {
-    Node* other_cmp_input = use->in(1);
-    for (DUIterator_Fast imax, i = other_cmp_input->fast_outs(imax); i < imax; i++) {
-      Node* cast_ii = other_cmp_input->fast_out(i);
-      if (cast_ii->is_CastII()) {
-        push_if_not_bottom_type(worklist, cast_ii);
       }
     }
   }
@@ -1938,73 +1880,35 @@ void PhaseCCP::do_transform() {
 // Convert any of his old-space children into new-space children.
 Node *PhaseCCP::transform( Node *n ) {
   Node *new_node = _nodes[n->_idx]; // Check for transformed node
-  if( new_node != nullptr )
+  if( new_node != NULL )
     return new_node;                // Been there, done that, return old answer
-
-  assert(n->is_Root(), "traversal must start at root");
-  assert(_root_and_safepoints.member(n), "root (n) must be in list");
+  new_node = transform_once(n);     // Check for constant
+  _nodes.map( n->_idx, new_node );  // Flag as having been cloned
 
   // Allocate stack of size _nodes.Size()/2 to avoid frequent realloc
-  GrowableArray <Node *> transform_stack(C->live_nodes() >> 1);
-  Unique_Node_List useful; // track all visited nodes, so that we can remove the complement
+  GrowableArray <Node *> trstack(C->live_nodes() >> 1);
 
-  // Initialize the traversal.
-  // This CCP pass may prove that no exit test for a loop ever succeeds (i.e. the loop is infinite). In that case,
-  // the logic below doesn't follow any path from Root to the loop body: there's at least one such path but it's proven
-  // never taken (its type is TOP). As a consequence the node on the exit path that's input to Root (let's call it n) is
-  // replaced by the top node and the inputs of that node n are not enqueued for further processing. If CCP only works
-  // through the graph from Root, this causes the loop body to never be processed here even when it's not dead (that
-  // is reachable from Root following its uses). To prevent that issue, transform() starts walking the graph from Root
-  // and all safepoints.
-  for (uint i = 0; i < _root_and_safepoints.size(); ++i) {
-    Node* nn = _root_and_safepoints.at(i);
-    Node* new_node = _nodes[nn->_idx];
-    assert(new_node == nullptr, "");
-    new_node = transform_once(nn);  // Check for constant
-    _nodes.map(nn->_idx, new_node); // Flag as having been cloned
-    transform_stack.push(new_node); // Process children of cloned node
-    useful.push(new_node);
-  }
-
-  while (transform_stack.is_nonempty()) {
-    Node* clone = transform_stack.pop();
+  trstack.push(new_node);           // Process children of cloned node
+  while ( trstack.is_nonempty() ) {
+    Node *clone = trstack.pop();
     uint cnt = clone->req();
     for( uint i = 0; i < cnt; i++ ) {          // For all inputs do
       Node *input = clone->in(i);
-      if( input != nullptr ) {                 // Ignore nulls
+      if( input != NULL ) {                    // Ignore NULLs
         Node *new_input = _nodes[input->_idx]; // Check for cloned input node
-        if( new_input == nullptr ) {
+        if( new_input == NULL ) {
           new_input = transform_once(input);   // Check for constant
           _nodes.map( input->_idx, new_input );// Flag as having been cloned
-          transform_stack.push(new_input);     // Process children of cloned node
-          useful.push(new_input);
+          trstack.push(new_input);
         }
         assert( new_input == clone->in(i), "insanity check");
       }
     }
   }
-
-  // The above transformation might lead to subgraphs becoming unreachable from the
-  // bottom while still being reachable from the top. As a result, nodes in that
-  // subgraph are not transformed and their bottom types are not updated, leading to
-  // an inconsistency between bottom_type() and type(). In rare cases, LoadNodes in
-  // such a subgraph, might be re-enqueued for IGVN indefinitely by MemNode::Ideal_common
-  // because their address type is inconsistent. Therefore, we aggressively remove
-  // all useless nodes here even before PhaseIdealLoop::build_loop_late gets a chance
-  // to remove them anyway.
-  if (C->cached_top_node()) {
-    useful.push(C->cached_top_node());
-  }
-  C->update_dead_node_list(useful);
-  remove_useless_nodes(useful.member_set());
-  _worklist.remove_useless_nodes(useful.member_set());
-  C->disconnect_useless_nodes(useful, &_worklist);
-
-  Node* new_root = _nodes[n->_idx];
-  assert(new_root->is_Root(), "transformed root node must be a root node");
-  return new_root;
+  return new_node;
 }
 
+
 //------------------------------transform_once---------------------------------
 // For PhaseCCP, transformation is IDENTITY unless Node computed a constant.
 Node *PhaseCCP::transform_once( Node *n ) {
@@ -2014,7 +1918,7 @@ Node *PhaseCCP::transform_once( Node *n ) {
     Node *nn = n;               // Default is to return the original constant
     if( t == Type::TOP ) {
       // cache my top node on the Compile instance
-      if( C->cached_top_node() == nullptr || C->cached_top_node()->in(0) == nullptr ) {
+      if( C->cached_top_node() == NULL || C->cached_top_node()->in(0) == NULL ) {
         C->set_cached_top_node(ConNode::make(Type::TOP));
         set_type(C->top(), Type::TOP);
       }
@@ -2026,7 +1930,7 @@ Node *PhaseCCP::transform_once( Node *n ) {
         NOT_PRODUCT( inc_constants(); )
       } else if( n->is_Region() ) { // Unreachable region
         // Note: nn == C->top()
-        n->set_req(0, nullptr);     // Cut selfreference
+        n->set_req(0, NULL);        // Cut selfreference
         bool progress = true;
         uint max = n->outcnt();
         DUIterator i;
@@ -2059,9 +1963,11 @@ Node *PhaseCCP::transform_once( Node *n ) {
     _worklist.push(n);          // n re-enters the hash table via the worklist
   }
 
-  // TEMPORARY fix to ensure that 2nd GVN pass eliminates null checks
+  // TEMPORARY fix to ensure that 2nd GVN pass eliminates NULL checks
   switch( n->Opcode() ) {
-  case Op_FastLock:      // Revisit FastLocks for lock coarsening
+  case Op_CallStaticJava:  // Give post-parse call devirtualization a chance
+  case Op_CallDynamicJava:
+  case Op_FastLock:        // Revisit FastLocks for lock coarsening
   case Op_If:
   case Op_CountedLoopEnd:
   case Op_Region:
@@ -2119,7 +2025,7 @@ PhasePeephole::~PhasePeephole() {
 //------------------------------transform--------------------------------------
 Node *PhasePeephole::transform( Node *n ) {
   ShouldNotCallThis();
-  return nullptr;
+  return NULL;
 }
 
 //------------------------------do_transform-----------------------------------
@@ -2234,7 +2140,7 @@ void Node::set_req_X( uint i, Node *n, PhaseIterGVN *igvn ) {
 
 void Node::set_req_X(uint i, Node *n, PhaseGVN *gvn) {
   PhaseIterGVN* igvn = gvn->is_IterGVN();
-  if (igvn == nullptr) {
+  if (igvn == NULL) {
     set_req(i, n);
     return;
   }
@@ -2267,7 +2173,7 @@ void Type_Array::grow( uint i ) {
   if( !_max ) {
     _max = 1;
     _types = (const Type**)_a->Amalloc( _max * sizeof(Type*) );
-    _types[0] = nullptr;
+    _types[0] = NULL;
   }
   uint old = _max;
   _max = next_power_of_2(i);
@@ -2280,7 +2186,7 @@ void Type_Array::grow( uint i ) {
 void Type_Array::dump() const {
   uint max = Size();
   for( uint i = 0; i < max; i++ ) {
-    if( _types[i] != nullptr ) {
+    if( _types[i] != NULL ) {
       tty->print("  %d\t== ", i); _types[i]->dump(); tty->cr();
     }
   }
