@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -46,6 +46,9 @@
 #include "utilities/formatBuffer.hpp"
 #include "utilities/utf8.hpp"
 
+// Complain every extra number of unplanned local refs
+#define CHECK_JNI_LOCAL_REF_CAP_WARN_THRESHOLD 32
+
 // Heap objects are allowed to be directly referenced only in VM code,
 // not in native code.
 
@@ -96,7 +99,7 @@ extern "C" {                                                             \
       tty->print_cr("%s", fatal_using_jnienv_in_nonjava);                \
       os::abort(true);                                                   \
     }                                                                    \
-    JavaThread* thr = cur->as_Java_thread();                             \
+    JavaThread* thr = JavaThread::cast(cur);                             \
     JNIEnv* xenv = thr->jni_environment();                               \
     if (env != xenv) {                                                   \
       NativeReportJNIFatalError(thr, warn_wrong_jnienv);                 \
@@ -142,7 +145,7 @@ static const char * fatal_non_utf8_class_name2 = "\"";
 // When in VM state:
 static void ReportJNIWarning(JavaThread* thr, const char *msg) {
   tty->print_cr("WARNING in native method: %s", msg);
-  thr->print_jni_stack();
+  thr->print_stack();
 }
 
 // When in NATIVE state:
@@ -193,12 +196,23 @@ check_pending_exception(JavaThread* thr) {
     IN_VM(
       tty->print_cr("WARNING in native method: JNI call made without checking exceptions when required to from %s",
         thr->get_pending_jni_exception_check());
-      thr->print_jni_stack();
+      thr->print_stack();
     )
     thr->clear_pending_jni_exception_check(); // Just complain once
   }
 }
 
+/**
+ * Add to the planned number of handles. I.e. plus current live & warning threshold
+ */
+static inline void
+add_planned_handle_capacity(JNIHandleBlock* handles, size_t capacity) {
+  handles->set_planned_capacity(capacity +
+                                handles->get_number_of_live_handles() +
+                                CHECK_JNI_LOCAL_REF_CAP_WARN_THRESHOLD);
+}
+
+
 static inline void
 functionEnterCritical(JavaThread* thr)
 {
@@ -230,7 +244,18 @@ functionEnterExceptionAllowed(JavaThread* thr)
 static inline void
 functionExit(JavaThread* thr)
 {
-  // No checks at this time
+  JNIHandleBlock* handles = thr->active_handles();
+  size_t planned_capacity = handles->get_planned_capacity();
+  size_t live_handles = handles->get_number_of_live_handles();
+  if (live_handles > planned_capacity) {
+    IN_VM(
+      tty->print_cr("WARNING: JNI local refs: " SIZE_FORMAT ", exceeds capacity: " SIZE_FORMAT,
+                    live_handles, planned_capacity);
+      thr->print_stack();
+    )
+    // Complain just the once, reset to current + warn threshold
+    add_planned_handle_capacity(handles, 0);
+  }
 }
 
 static inline void
@@ -351,33 +376,24 @@ check_is_obj_array(JavaThread* thr, jarray jArray) {
   }
 }
 
-// Arbitrary (but well-known) tag for GetStringChars
-const void* STRING_TAG = (void*)0x47114711;
-
-// Arbitrary (but well-known) tag for GetStringUTFChars
-const void* STRING_UTF_TAG = (void*) 0x48124812;
-
-// Arbitrary (but well-known) tag for GetPrimitiveArrayCritical
-const void* CRITICAL_TAG = (void*)0x49134913;
-
 /*
  * Copy and wrap array elements for bounds checking.
  * Remember the original elements (GuardedMemory::get_tag())
  */
 static void* check_jni_wrap_copy_array(JavaThread* thr, jarray array,
-                                       void* orig_elements, jboolean is_critical = JNI_FALSE) {
+    void* orig_elements) {
   void* result;
   IN_VM(
     oop a = JNIHandles::resolve_non_null(array);
     size_t len = arrayOop(a)->length() <<
         TypeArrayKlass::cast(a->klass())->log2_element_size();
-    result = GuardedMemory::wrap_copy(orig_elements, len, orig_elements, is_critical ? CRITICAL_TAG : nullptr);
+    result = GuardedMemory::wrap_copy(orig_elements, len, orig_elements);
   )
   return result;
 }
 
 static void* check_wrapped_array(JavaThread* thr, const char* fn_name,
-                                 void* obj, void* carray, size_t* rsz, jboolean is_critical) {
+    void* obj, void* carray, size_t* rsz) {
   if (carray == NULL) {
     tty->print_cr("%s: elements vector NULL" PTR_FORMAT, fn_name, p2i(obj));
     NativeReportJNIFatalError(thr, "Elements vector NULL");
@@ -396,29 +412,6 @@ static void* check_wrapped_array(JavaThread* thr, const char* fn_name,
     DEBUG_ONLY(guarded.print_on(tty);) // This may crash.
     NativeReportJNIFatalError(thr, err_msg("%s: unrecognized elements", fn_name));
   }
-  if (orig_result == STRING_TAG || orig_result == STRING_UTF_TAG) {
-    bool was_utf = orig_result == STRING_UTF_TAG;
-    tty->print_cr("%s: called on something allocated by %s",
-                  fn_name, was_utf ? "GetStringUTFChars" : "GetStringChars");
-    DEBUG_ONLY(guarded.print_on(tty);) // This may crash.
-    NativeReportJNIFatalError(thr, err_msg("%s called on something allocated by %s",
-                                           fn_name, was_utf ? "GetStringUTFChars" : "GetStringChars"));
-  }
-
-  if (is_critical && (guarded.get_tag2() != CRITICAL_TAG)) {
-    tty->print_cr("%s: called on something not allocated by GetPrimitiveArrayCritical", fn_name);
-    DEBUG_ONLY(guarded.print_on(tty);) // This may crash.
-    NativeReportJNIFatalError(thr, err_msg("%s called on something not allocated by GetPrimitiveArrayCritical",
-                                           fn_name));
-  }
-
-  if (!is_critical && (guarded.get_tag2() == CRITICAL_TAG)) {
-    tty->print_cr("%s: called on something allocated by GetPrimitiveArrayCritical", fn_name);
-    DEBUG_ONLY(guarded.print_on(tty);) // This may crash.
-    NativeReportJNIFatalError(thr, err_msg("%s called on something allocated by GetPrimitiveArrayCritical",
-                                           fn_name));
-  }
-
   if (rsz != NULL) {
     *rsz = guarded.get_user_size();
   }
@@ -428,7 +421,7 @@ static void* check_wrapped_array(JavaThread* thr, const char* fn_name,
 static void* check_wrapped_array_release(JavaThread* thr, const char* fn_name,
                                          void* obj, void* carray, jint mode, jboolean is_critical) {
   size_t sz;
-  void* orig_result = check_wrapped_array(thr, fn_name, obj, carray, &sz, is_critical);
+  void* orig_result = check_wrapped_array(thr, fn_name, obj, carray, &sz);
   switch (mode) {
   case 0:
     memcpy(orig_result, carray, sz);
@@ -753,6 +746,9 @@ JNI_ENTRY_CHECKED(jint,
     if (capacity < 0)
       NativeReportJNIFatalError(thr, "negative capacity");
     jint result = UNCHECKED()->PushLocalFrame(env, capacity);
+    if (result == JNI_OK) {
+      add_planned_handle_capacity(thr->active_handles(), capacity);
+    }
     functionExit(thr);
     return result;
 JNI_END
@@ -854,6 +850,12 @@ JNI_ENTRY_CHECKED(jint,
       NativeReportJNIFatalError(thr, "negative capacity");
     }
     jint result = UNCHECKED()->EnsureLocalCapacity(env, capacity);
+    if (result == JNI_OK) {
+      // increase local ref capacity if needed
+      if ((size_t)capacity > thr->active_handles()->get_planned_capacity()) {
+        add_planned_handle_capacity(thr->active_handles(), capacity);
+      }
+    }
     functionExit(thr);
     return result;
 JNI_END
@@ -1463,6 +1465,9 @@ JNI_ENTRY_CHECKED(jsize,
     return result;
 JNI_END
 
+// Arbitrary (but well-known) tag
+const void* STRING_TAG = (void*)0x47114711;
+
 JNI_ENTRY_CHECKED(const jchar *,
   checked_jni_GetStringChars(JNIEnv *env,
                              jstring str,
@@ -1544,6 +1549,9 @@ JNI_ENTRY_CHECKED(jsize,
     return result;
 JNI_END
 
+// Arbitrary (but well-known) tag - different than GetStringChars
+const void* STRING_UTF_TAG = (void*) 0x48124812;
+
 JNI_ENTRY_CHECKED(const char *,
   checked_jni_GetStringUTFChars(JNIEnv *env,
                                 jstring str,
@@ -1865,7 +1873,7 @@ JNI_ENTRY_CHECKED(void *,
     )
     void *result = UNCHECKED()->GetPrimitiveArrayCritical(env, array, isCopy);
     if (result != NULL) {
-      result = check_jni_wrap_copy_array(thr, array, result, JNI_TRUE);
+      result = check_jni_wrap_copy_array(thr, array, result);
     }
     functionExit(thr);
     return result;
@@ -2308,7 +2316,7 @@ struct JNINativeInterface_* jni_functions_check() {
   // make sure the last pointer in the checked table is not null, indicating
   // an addition to the JNINativeInterface_ structure without initializing
   // it in the checked table.
-  debug_only(intptr_t *lastPtr = (intptr_t *)((char *)&checked_jni_NativeInterface + \
+  debug_only(int *lastPtr = (int *)((char *)&checked_jni_NativeInterface + \
              sizeof(*unchecked_jni_NativeInterface) - sizeof(char *));)
   assert(*lastPtr != 0,
          "Mismatched JNINativeInterface tables, check for new entries");
