@@ -76,59 +76,48 @@ class InterfaceSupport: AllStatic {
 class ThreadStateTransition : public StackObj {
  protected:
   JavaThread* _thread;
+
  public:
-  ThreadStateTransition(JavaThread *thread) {
-    _thread = thread;
+  ThreadStateTransition(JavaThread *thread) : _thread(thread) {
     assert(thread != NULL, "must be active Java thread");
     assert(thread == Thread::current(), "must be current thread");
   }
 
-  // Change threadstate in a manner, so safepoint can detect changes.
-  // Time-critical: called on exit from every runtime routine
-  static inline void transition(JavaThread *thread, JavaThreadState from, JavaThreadState to) {
-    assert(from != _thread_in_Java, "use transition_from_java");
-    assert(from != _thread_in_native, "use transition_from_native");
-    assert((from & 1) == 0 && (to & 1) == 0, "odd numbers are transitions states");
-    assert(thread->thread_state() == from, "coming from wrong thread state");
-
-    // Check NoSafepointVerifier
-    // This also clears unhandled oops if CheckUnhandledOops is used.
-    thread->check_possible_safepoint();
-
-    // Change to transition state and ensure it is seen by the VM thread.
-    thread->set_thread_state_fence((JavaThreadState)(from + 1));
-
-    SafepointMechanism::process_if_requested(thread);
-    thread->set_thread_state(to);
-  }
-
-  // Same as above, but assumes from = _thread_in_Java. This is simpler, since we
-  // never block on entry to the VM. This will break the code, since e.g. preserve arguments
-  // have not been setup.
   static inline void transition_from_java(JavaThread *thread, JavaThreadState to) {
     assert(thread->thread_state() == _thread_in_Java, "coming from wrong thread state");
+    assert(to == _thread_in_vm || to == _thread_in_native, "invalid transition");
     thread->set_thread_state(to);
   }
 
-  static inline void transition_from_native(JavaThread *thread, JavaThreadState to) {
-    assert((to & 1) == 0, "odd numbers are transitions states");
+  // We never install asynchronous exceptions when coming (back) in to the runtime
+  // from native code because the runtime is not set up to handle exceptions floating
+  // around at arbitrary points.
+  static inline void transition_from_native(JavaThread *thread, JavaThreadState to, bool check_asyncs = true) {
     assert(thread->thread_state() == _thread_in_native, "coming from wrong thread state");
-    assert(!thread->has_last_Java_frame() || thread->frame_anchor()->walkable(), "Unwalkable stack in native->vm transition");
+    assert(to == _thread_in_vm || to == _thread_in_Java, "invalid transition");
+    assert(!thread->has_last_Java_frame() || thread->frame_anchor()->walkable(), "Unwalkable stack in native transition");
 
-    // Change to transition state and ensure it is seen by the VM thread.
-    thread->set_thread_state_fence(_thread_in_native_trans);
-
-    // We never install asynchronous exceptions when coming (back) in
-    // to the runtime from native code because the runtime is not set
-    // up to handle exceptions floating around at arbitrary points.
-    SafepointMechanism::process_if_requested_with_exit_check(thread, false /* check asyncs */);
+    thread->set_thread_state_fence(_thread_in_vm);
+    SafepointMechanism::process_if_requested_with_exit_check(thread, to != _thread_in_Java ? false : check_asyncs);
     thread->set_thread_state(to);
   }
 
- protected:
-   void trans(JavaThreadState from, JavaThreadState to)  { transition(_thread, from, to); }
-   void trans_from_java(JavaThreadState to)              { transition_from_java(_thread, to); }
-   void trans_from_native(JavaThreadState to)            { transition_from_native(_thread, to); }
+  static inline void transition_from_vm(JavaThread *thread, JavaThreadState to, bool check_asyncs = true) {
+    assert(thread->thread_state() == _thread_in_vm, "coming from wrong thread state");
+    if (to == _thread_in_Java) {
+      SafepointMechanism::process_if_requested_with_exit_check(thread, check_asyncs);
+      thread->set_thread_state(to);
+    } else {
+      assert(to == _thread_in_native || to == _thread_blocked, "invalid transition");
+      // Check NoSafepointVerifier. This also clears unhandled oops if CheckUnhandledOops is used.
+      thread->check_possible_safepoint();
+
+      // Once we are in native/blocked vm expects stack to be walkable
+      thread->frame_anchor()->make_walkable(thread);
+      OrderAccess::storestore(); // Keep thread_state change and make_walkable() separate.
+      thread->set_thread_state(to);
+    }
+  }
 };
 
 class ThreadInVMForHandshake : public ThreadStateTransition {
@@ -138,7 +127,7 @@ class ThreadInVMForHandshake : public ThreadStateTransition {
       _original_state(thread->thread_state()) {
 
     if (thread->has_last_Java_frame()) {
-      thread->frame_anchor()->make_walkable();
+      thread->frame_anchor()->make_walkable(thread);
     }
 
     thread->set_thread_state(_thread_in_vm);
@@ -158,16 +147,15 @@ class ThreadInVMfromJava : public ThreadStateTransition {
   bool _check_asyncs;
  public:
   ThreadInVMfromJava(JavaThread* thread, bool check_asyncs = true) : ThreadStateTransition(thread), _check_asyncs(check_asyncs) {
-    trans_from_java(_thread_in_vm);
+    transition_from_java(thread, _thread_in_vm);
   }
   ~ThreadInVMfromJava()  {
     if (_thread->stack_overflow_state()->stack_yellow_reserved_zone_disabled()) {
       _thread->stack_overflow_state()->enable_stack_yellow_reserved_zone();
     }
-    trans(_thread_in_vm, _thread_in_Java);
     // We prevent asynchronous exceptions from being installed on return to Java in situations
     // where we can't tolerate them. See bugs: 4324348, 4854693, 4998314, 5040492, 5050705.
-    if (_thread->has_special_runtime_exit_condition()) _thread->handle_special_runtime_exit_condition(_check_asyncs);
+    transition_from_vm(_thread, _thread_in_Java, _check_asyncs);
   }
 };
 
@@ -178,7 +166,7 @@ class ThreadInVMfromUnknown {
   ThreadInVMfromUnknown() : _thread(NULL) {
     Thread* t = Thread::current();
     if (t->is_Java_thread()) {
-      JavaThread* t2 = t->as_Java_thread();
+      JavaThread* t2 = JavaThread::cast(t);
       if (t2->thread_state() == _thread_in_native) {
         _thread = t2;
         ThreadStateTransition::transition_from_native(t2, _thread_in_vm);
@@ -191,7 +179,7 @@ class ThreadInVMfromUnknown {
   }
   ~ThreadInVMfromUnknown()  {
     if (_thread) {
-      ThreadStateTransition::transition(_thread, _thread_in_vm, _thread_in_native);
+      ThreadStateTransition::transition_from_vm(_thread, _thread_in_native);
     }
   }
 };
@@ -201,17 +189,12 @@ class ThreadInVMfromNative : public ThreadStateTransition {
   ResetNoHandleMark __rnhm;
  public:
   ThreadInVMfromNative(JavaThread* thread) : ThreadStateTransition(thread) {
-    trans_from_native(_thread_in_vm);
+    transition_from_native(thread, _thread_in_vm);
   }
   ~ThreadInVMfromNative() {
-    assert(_thread->thread_state() == _thread_in_vm, "coming from wrong thread state");
     // We cannot assert !_thread->owns_locks() since we have valid cases where
     // we call known native code using this wrapper holding locks.
-    _thread->check_possible_safepoint();
-    // Once we are in native vm expects stack to be walkable
-    _thread->frame_anchor()->make_walkable();
-    OrderAccess::storestore(); // Keep thread_state change and make_walkable() separate.
-    _thread->set_thread_state(_thread_in_native);
+    transition_from_vm(_thread, _thread_in_native);
   }
 };
 
@@ -219,17 +202,11 @@ class ThreadInVMfromNative : public ThreadStateTransition {
 class ThreadToNativeFromVM : public ThreadStateTransition {
  public:
   ThreadToNativeFromVM(JavaThread *thread) : ThreadStateTransition(thread) {
-    // We are leaving the VM at this point and going directly to native code.
-    // Block, if we are in the middle of a safepoint synchronization.
     assert(!thread->owns_locks(), "must release all locks when leaving VM");
-    thread->frame_anchor()->make_walkable();
-    trans(_thread_in_vm, _thread_in_native);
-    // Check for pending. async. exceptions or suspends.
-    if (_thread->has_special_runtime_exit_condition()) _thread->handle_special_runtime_exit_condition(false);
+    transition_from_vm(thread, _thread_in_native);
   }
-
   ~ThreadToNativeFromVM() {
-    trans_from_native(_thread_in_vm);
+    transition_from_native(_thread, _thread_in_vm);
     assert(!_thread->is_pending_jni_exception_check(), "Pending JNI Exception Check");
     // We don't need to clear_walkable because it will happen automagically when we return to java
   }
@@ -239,60 +216,32 @@ class ThreadToNativeFromVM : public ThreadStateTransition {
 // SafepointMechanism::process_if_requested when returning to the VM. This allows us
 // to perform an "undo" action if we might block processing a safepoint/handshake operation
 // (such as thread suspension).
-template <typename PRE_PROC>
+template <typename PRE_PROC = void(JavaThread*)>
 class ThreadBlockInVMPreprocess : public ThreadStateTransition {
  private:
   PRE_PROC& _pr;
   bool _allow_suspend;
  public:
-  ThreadBlockInVMPreprocess(JavaThread* thread, PRE_PROC& pr, bool allow_suspend = true)
+  ThreadBlockInVMPreprocess(JavaThread* thread, PRE_PROC& pr = emptyOp, bool allow_suspend = false)
     : ThreadStateTransition(thread), _pr(pr), _allow_suspend(allow_suspend) {
-    assert(thread->thread_state() == _thread_in_vm, "coming from wrong thread state");
-    thread->check_possible_safepoint();
-    // Once we are blocked vm expects stack to be walkable
-    thread->frame_anchor()->make_walkable();
-    OrderAccess::storestore(); // Keep thread_state change and make_walkable() separate.
-    thread->set_thread_state(_thread_blocked);
+    transition_from_vm(thread, _thread_blocked);
   }
   ~ThreadBlockInVMPreprocess() {
     assert(_thread->thread_state() == _thread_blocked, "coming from wrong thread state");
-    // Change to transition state and ensure it is seen by the VM thread.
-    _thread->set_thread_state_fence(_thread_blocked_trans);
+    // Change back to _thread_in_vm and ensure it is seen by the VM thread.
+    _thread->set_thread_state_fence(_thread_in_vm);
 
     if (SafepointMechanism::should_process(_thread, _allow_suspend)) {
       _pr(_thread);
       SafepointMechanism::process_if_requested(_thread, _allow_suspend);
     }
-
-    _thread->set_thread_state(_thread_in_vm);
   }
+
+  static void emptyOp(JavaThread* current) {}
 };
 
-class InFlightMutexRelease {
- private:
-  Mutex** _in_flight_mutex_addr;
- public:
-  InFlightMutexRelease(Mutex** in_flight_mutex_addr) : _in_flight_mutex_addr(in_flight_mutex_addr) {}
-  void operator()(JavaThread* current) {
-    if (_in_flight_mutex_addr != NULL && *_in_flight_mutex_addr != NULL) {
-      (*_in_flight_mutex_addr)->release_for_safepoint();
-      *_in_flight_mutex_addr = NULL;
-    }
-  }
-};
+typedef ThreadBlockInVMPreprocess<> ThreadBlockInVM;
 
-// Parameter in_flight_mutex_addr is only used by class Mutex to avoid certain deadlock
-// scenarios while making transitions that might block for a safepoint or handshake.
-// It's the address of a pointer to the mutex we are trying to acquire. This will be used to
-// access and release said mutex when transitioning back from blocked to vm (destructor) in
-// case we need to stop for a safepoint or handshake.
-class ThreadBlockInVM {
-  InFlightMutexRelease _ifmr;
-  ThreadBlockInVMPreprocess<InFlightMutexRelease> _tbivmpp;
- public:
-  ThreadBlockInVM(JavaThread* thread, Mutex** in_flight_mutex_addr = NULL)
-    : _ifmr(in_flight_mutex_addr), _tbivmpp(thread, _ifmr, /* allow_suspend= */ false) {}
-};
 
 // Debug class instantiated in JRT_ENTRY macro.
 // Can be used to verify properties on enter/exit of the VM.
@@ -325,6 +274,8 @@ class VMNativeEntryWrapper {
 
 #define VM_LEAF_BASE(result_type, header)                            \
   debug_only(NoHandleMark __hm;)                                     \
+  MACOS_AARCH64_ONLY(ThreadWXEnable __wx(WXWrite,                    \
+                                         JavaThread::current()));    \
   os::verify_stack_alignment();                                      \
   /* begin of body */
 
@@ -347,6 +298,7 @@ class VMNativeEntryWrapper {
 
 #define JRT_ENTRY(result_type, header)                               \
   result_type header {                                               \
+    assert(current == JavaThread::current(), "Must be");             \
     MACOS_AARCH64_ONLY(ThreadWXEnable __wx(WXWrite, current));       \
     ThreadInVMfromJava __tiv(current);                               \
     VM_ENTRY_BASE(result_type, header, current)                      \
@@ -374,6 +326,7 @@ class VMNativeEntryWrapper {
 
 #define JRT_ENTRY_NO_ASYNC(result_type, header)                      \
   result_type header {                                               \
+    assert(current == JavaThread::current(), "Must be");             \
     MACOS_AARCH64_ONLY(ThreadWXEnable __wx(WXWrite, current));       \
     ThreadInVMfromJava __tiv(current, false /* check asyncs */);     \
     VM_ENTRY_BASE(result_type, header, current)                      \
@@ -383,17 +336,20 @@ class VMNativeEntryWrapper {
 // to get back into Java from the VM
 #define JRT_BLOCK_ENTRY(result_type, header)                         \
   result_type header {                                               \
+    assert(current == JavaThread::current(), "Must be");             \
     MACOS_AARCH64_ONLY(ThreadWXEnable __wx(WXWrite, current));       \
     HandleMarkCleaner __hm(current);
 
 #define JRT_BLOCK                                                    \
     {                                                                \
+    assert(current == JavaThread::current(), "Must be");             \
     ThreadInVMfromJava __tiv(current);                               \
     JavaThread* THREAD = current; /* For exception macros. */        \
     debug_only(VMEntryWrapper __vew;)
 
 #define JRT_BLOCK_NO_ASYNC                                           \
     {                                                                \
+    assert(current == JavaThread::current(), "Must be");             \
     ThreadInVMfromJava __tiv(current, false /* check asyncs */);     \
     JavaThread* THREAD = current; /* For exception macros. */        \
     debug_only(VMEntryWrapper __vew;)
