@@ -38,6 +38,7 @@
 #include "logging/log.hpp"
 #include "logging/logStream.hpp"
 #include "memory/allocation.inline.hpp"
+#include "memory/guardedMemory.hpp"
 #include "memory/resourceArea.hpp"
 #include "memory/universe.hpp"
 #include "oops/compressedOops.inline.hpp"
@@ -54,7 +55,7 @@
 #include "runtime/mutexLocker.hpp"
 #include "runtime/os.inline.hpp"
 #include "runtime/osThread.hpp"
-#include "runtime/safefetch.hpp"
+#include "runtime/safefetch.inline.hpp"
 #include "runtime/sharedRuntime.hpp"
 #include "runtime/thread.inline.hpp"
 #include "runtime/threadSMR.hpp"
@@ -82,6 +83,13 @@ int               os::_processor_count    = 0;
 int               os::_initial_active_processor_count = 0;
 os::PageSizes     os::_page_sizes;
 
+#ifndef PRODUCT
+julong os::num_mallocs = 0;         // # of calls to malloc/realloc
+julong os::alloc_bytes = 0;         // # of bytes allocated
+julong os::num_frees = 0;           // # of calls to free
+julong os::free_bytes = 0;          // # of bytes freed
+#endif
+
 static size_t cur_malloc_words = 0;  // current size for MallocMaxTestWords
 
 DEBUG_ONLY(bool os::_mutex_init_done = false;)
@@ -392,10 +400,8 @@ static void signal_thread_entry(JavaThread* thread, TRAPS) {
         // Any SIGBREAK operations added here should make sure to flush
         // the output stream (e.g. tty->flush()) after output.  See 4803766.
         // Each module also prints an extra carriage return after its output.
-        VM_PrintThreads op;
+        VM_PrintThreads op(tty, PrintConcurrentLocks, false /* no extended info */, true /* print JNI handle info */);
         VMThread::execute(&op);
-        VM_PrintJNI jni_op;
-        VMThread::execute(&jni_op);
         VM_FindDeadlocks op1(tty);
         VMThread::execute(&op1);
         Universe::print_heap_at_SIGBREAK();
@@ -464,31 +470,16 @@ void os::init_before_ergo() {
 void os::initialize_jdk_signal_support(TRAPS) {
   if (!ReduceSignalUsage) {
     // Setup JavaThread for processing signals
-    const char thread_name[] = "Signal Dispatcher";
-    Handle string = java_lang_String::create_from_str(thread_name, CHECK);
-
-    // Initialize thread_oop to put it into the system threadGroup
-    Handle thread_group (THREAD, Universe::system_thread_group());
-    Handle thread_oop = JavaCalls::construct_new_instance(vmClasses::Thread_klass(),
-                           vmSymbols::threadgroup_string_void_signature(),
-                           thread_group,
-                           string,
-                           CHECK);
-
-    Klass* group = vmClasses::ThreadGroup_klass();
-    JavaValue result(T_VOID);
-    JavaCalls::call_special(&result,
-                            thread_group,
-                            group,
-                            vmSymbols::add_method_name(),
-                            vmSymbols::thread_void_signature(),
-                            thread_oop,
-                            CHECK);
+    const char* name = "Signal Dispatcher";
+    Handle thread_oop = JavaThread::create_system_thread_object(name, true /* visible */, CHECK);
 
     JavaThread* thread = new JavaThread(&signal_thread_entry);
     JavaThread::vm_exit_on_osthread_failure(thread);
 
     JavaThread::start_internal_daemon(THREAD, thread, thread_oop, NearMaxPriority);
+
+    // Handle ^BREAK
+    os::signal(SIGBREAK, os::user_handler());
   }
 }
 
@@ -612,11 +603,30 @@ char* os::strdup_check_oom(const char* str, MEMFLAGS flags) {
   return p;
 }
 
+
+#define paranoid                 0  /* only set to 1 if you suspect checking code has bug */
+
+#ifdef ASSERT
+
+static void verify_memory(void* ptr) {
+  GuardedMemory guarded(ptr);
+  if (!guarded.verify_guards()) {
+    LogTarget(Warning, malloc, free) lt;
+    ResourceMark rm;
+    LogStream ls(lt);
+    ls.print_cr("## nof_mallocs = " UINT64_FORMAT ", nof_frees = " UINT64_FORMAT, os::num_mallocs, os::num_frees);
+    ls.print_cr("## memory stomp:");
+    guarded.print_on(&ls);
+    fatal("memory stomping error");
+  }
+}
+
+#endif
+
 //
 // This function supports testing of the malloc out of memory
 // condition without really running the system out of memory.
 //
-
 static bool has_reached_max_malloc_test_peak(size_t alloc_size) {
   if (MallocMaxTestWords > 0) {
     size_t words = (alloc_size / BytesPerWord);
@@ -629,24 +639,13 @@ static bool has_reached_max_malloc_test_peak(size_t alloc_size) {
   return false;
 }
 
-#ifdef ASSERT
-static void check_crash_protection() {
-  assert(!os::ThreadCrashProtection::is_crash_protected(Thread::current_or_null()),
-         "not allowed when crash protection is set");
-}
-static void break_if_ptr_caught(void* ptr) {
-  if (p2i(ptr) == (intptr_t)MallocCatchPtr) {
-    log_warning(malloc, free)("ptr caught: " PTR_FORMAT, p2i(ptr));
-    breakpoint();
-  }
-}
-#endif // ASSERT
-
 void* os::malloc(size_t size, MEMFLAGS flags) {
   return os::malloc(size, flags, CALLER_PC);
 }
 
 void* os::malloc(size_t size, MEMFLAGS memflags, const NativeCallStack& stack) {
+  NOT_PRODUCT(inc_stat_counter(&num_mallocs, 1));
+  NOT_PRODUCT(inc_stat_counter(&alloc_bytes, size));
 
 #if INCLUDE_NMT
   {
@@ -657,40 +656,58 @@ void* os::malloc(size_t size, MEMFLAGS memflags, const NativeCallStack& stack) {
   }
 #endif
 
-  DEBUG_ONLY(check_crash_protection());
+  // Since os::malloc can be called when the libjvm.{dll,so} is
+  // first loaded and we don't have a thread yet we must accept NULL also here.
+  assert(!os::ThreadCrashProtection::is_crash_protected(Thread::current_or_null()),
+         "malloc() not allowed when crash protection is set");
 
-  // On malloc(0), implementators of malloc(3) have the choice to return either
-  // NULL or a unique non-NULL pointer. To unify libc behavior across our platforms
-  // we chose the latter.
-  size = MAX2((size_t)1, size);
+  if (size == 0) {
+    // return a valid pointer if size is zero
+    // if NULL is returned the calling functions assume out of memory.
+    size = 1;
+  }
+
+  // NMT support
+  NMT_TrackingLevel level = MemTracker::tracking_level();
+  const size_t nmt_overhead =
+      MemTracker::malloc_header_size(level) + MemTracker::malloc_footer_size(level);
+
+#ifndef ASSERT
+  const size_t alloc_size = size + nmt_overhead;
+#else
+  const size_t alloc_size = GuardedMemory::get_total_size(size + nmt_overhead);
+  if (size + nmt_overhead > alloc_size) { // Check for rollover.
+    return NULL;
+  }
+#endif
 
   // For the test flag -XX:MallocMaxTestWords
   if (has_reached_max_malloc_test_peak(size)) {
     return NULL;
   }
 
-  const NMT_TrackingLevel level = MemTracker::tracking_level();
-  const size_t nmt_overhead =
-      MemTracker::malloc_header_size(level) + MemTracker::malloc_footer_size(level);
+  u_char* ptr;
+  ptr = (u_char*)::malloc(alloc_size);
 
-  const size_t outer_size = size + nmt_overhead;
-
-  // Check for overflow.
-  if (outer_size < size) {
+#ifdef ASSERT
+  if (ptr == NULL) {
     return NULL;
   }
+  // Wrap memory with guard
+  GuardedMemory guarded(ptr, size + nmt_overhead);
+  ptr = guarded.get_user_ptr();
 
-  void* const outer_ptr = (u_char*)::malloc(outer_size);
-  if (outer_ptr == NULL) {
-    return NULL;
+  if ((intptr_t)ptr == (intptr_t)MallocCatchPtr) {
+    log_warning(malloc, free)("os::malloc caught, " SIZE_FORMAT " bytes --> " PTR_FORMAT, size, p2i(ptr));
+    breakpoint();
   }
+  if (paranoid) {
+    verify_memory(ptr);
+  }
+#endif
 
-  void* inner_ptr = MemTracker::record_malloc((address)outer_ptr, size, memflags, stack, level);
-
-  DEBUG_ONLY(::memset(inner_ptr, uninitBlockPad, size);)
-  DEBUG_ONLY(break_if_ptr_caught(inner_ptr);)
-
-  return inner_ptr;
+  // we do not track guard memory
+  return MemTracker::record_malloc((address)ptr, size, memflags, stack, level);
 }
 
 void* os::realloc(void *memblock, size_t size, MEMFLAGS flags) {
@@ -708,41 +725,59 @@ void* os::realloc(void *memblock, size_t size, MEMFLAGS memflags, const NativeCa
   }
 #endif
 
-  if (memblock == NULL) {
-    return os::malloc(size, memflags, stack);
-  }
-
-  DEBUG_ONLY(check_crash_protection());
-
-  // On realloc(p, 0), implementators of realloc(3) have the choice to return either
-  // NULL or a unique non-NULL pointer. To unify libc behavior across our platforms
-  // we chose the latter.
-  size = MAX2((size_t)1, size);
-
   // For the test flag -XX:MallocMaxTestWords
   if (has_reached_max_malloc_test_peak(size)) {
     return NULL;
   }
 
-  const NMT_TrackingLevel level = MemTracker::tracking_level();
+  if (size == 0) {
+    // return a valid pointer if size is zero
+    // if NULL is returned the calling functions assume out of memory.
+    size = 1;
+  }
+
+#ifndef ASSERT
+  NOT_PRODUCT(inc_stat_counter(&num_mallocs, 1));
+  NOT_PRODUCT(inc_stat_counter(&alloc_bytes, size));
+   // NMT support
+  NMT_TrackingLevel level = MemTracker::tracking_level();
+  void* membase = MemTracker::record_free(memblock, level);
   const size_t nmt_overhead =
       MemTracker::malloc_header_size(level) + MemTracker::malloc_footer_size(level);
-
-  const size_t new_outer_size = size + nmt_overhead;
-
-  // If NMT is enabled, this checks for heap overwrites, then de-accounts the old block.
-  void* const old_outer_ptr = MemTracker::record_free(memblock, level);
-
-  void* const new_outer_ptr = ::realloc(old_outer_ptr, new_outer_size);
-
-  // If NMT is enabled, this checks for heap overwrites, then de-accounts the old block.
-  void* const new_inner_ptr = MemTracker::record_malloc(new_outer_ptr, size, memflags, stack, level);
-
-  DEBUG_ONLY(break_if_ptr_caught(new_inner_ptr);)
-
-  return new_inner_ptr;
+  void* ptr = ::realloc(membase, size + nmt_overhead);
+  return MemTracker::record_malloc(ptr, size, memflags, stack, level);
+#else
+  if (memblock == NULL) {
+    return os::malloc(size, memflags, stack);
+  }
+  if ((intptr_t)memblock == (intptr_t)MallocCatchPtr) {
+    log_warning(malloc, free)("os::realloc caught " PTR_FORMAT, p2i(memblock));
+    breakpoint();
+  }
+  // NMT support
+  void* membase = MemTracker::malloc_base(memblock);
+  verify_memory(membase);
+  // always move the block
+  void* ptr = os::malloc(size, memflags, stack);
+  // Copy to new memory if malloc didn't fail
+  if (ptr != NULL ) {
+    GuardedMemory guarded(MemTracker::malloc_base(memblock));
+    // Guard's user data contains NMT header
+    NMT_TrackingLevel level = MemTracker::tracking_level();
+    const size_t nmt_overhead =
+        MemTracker::malloc_header_size(level) + MemTracker::malloc_footer_size(level);
+    size_t memblock_size = guarded.get_user_size() - nmt_overhead;
+    memcpy(ptr, memblock, MIN2(size, memblock_size));
+    if (paranoid) {
+      verify_memory(MemTracker::malloc_base(ptr));
+    }
+    os::free(memblock);
+  }
+  return ptr;
+#endif
 }
 
+// handles NULL pointers
 void  os::free(void *memblock) {
 
 #if INCLUDE_NMT
@@ -751,17 +786,25 @@ void  os::free(void *memblock) {
   }
 #endif
 
-  if (memblock == NULL) {
-    return;
+  NOT_PRODUCT(inc_stat_counter(&num_frees, 1));
+#ifdef ASSERT
+  if (memblock == NULL) return;
+  if ((intptr_t)memblock == (intptr_t)MallocCatchPtr) {
+    log_warning(malloc, free)("os::free caught " PTR_FORMAT, p2i(memblock));
+    breakpoint();
   }
+  void* membase = MemTracker::record_free(memblock, MemTracker::tracking_level());
+  verify_memory(membase);
 
-  DEBUG_ONLY(break_if_ptr_caught(memblock);)
-
-  const NMT_TrackingLevel level = MemTracker::tracking_level();
-
-  // If NMT is enabled, this checks for heap overwrites, then de-accounts the old block.
-  void* const old_outer_ptr = MemTracker::record_free(memblock, level);
-  ::free(old_outer_ptr);
+  GuardedMemory guarded(membase);
+  size_t size = guarded.get_user_size();
+  inc_stat_counter(&free_bytes, size);
+  membase = guarded.release_for_freeing();
+  ::free(membase);
+#else
+  void* membase = MemTracker::record_free(memblock, MemTracker::tracking_level());
+  ::free(membase);
+#endif
 }
 
 void os::init_random(unsigned int initval) {
@@ -977,7 +1020,9 @@ void os::print_environment_variables(outputStream* st, const char** env_list) {
       if (envvar != NULL) {
         st->print("%s", env_list[i]);
         st->print("=");
-        st->print_cr("%s", envvar);
+        st->print("%s", envvar);
+        // Use separate cr() printing to avoid unnecessary buffer operations that might cause truncation.
+        st->cr();
       }
     }
   }
@@ -1073,7 +1118,11 @@ void os::print_date_and_time(outputStream *st, char* buf, size_t buflen) {
 // Check if pointer can be read from (4-byte read access).
 // Helps to prove validity of a not-NULL pointer.
 // Returns true in very early stages of VM life when stub is not yet generated.
+#define SAFEFETCH_DEFAULT true
 bool os::is_readable_pointer(const void* p) {
+  if (!CanUseSafeFetch32()) {
+    return SAFEFETCH_DEFAULT;
+  }
   int* const aligned = (int*) align_down((intptr_t)p, 4);
   int cafebabe = 0xcafebabe;  // tester value 1
   int deadbeef = 0xdeadbeef;  // tester value 2
@@ -1125,13 +1174,6 @@ void os::print_location(outputStream* st, intptr_t x, bool verbose) {
       st->print_cr(INTPTR_FORMAT " is a weak global jni handle", p2i(addr));
       return;
     }
-#ifndef PRODUCT
-    // we don't keep the block list in product mode
-    if (JNIHandles::is_local_handle((jobject) addr)) {
-      st->print_cr(INTPTR_FORMAT " is a local jni handle", p2i(addr));
-      return;
-    }
-#endif
   }
 
   // Check if addr belongs to a Java thread.
@@ -1204,34 +1246,34 @@ void os::print_location(outputStream* st, intptr_t x, bool verbose) {
   st->print_cr(INTPTR_FORMAT " is an unknown value", p2i(addr));
 }
 
-bool is_pointer_bad(intptr_t* ptr) {
-  return !is_aligned(ptr, sizeof(uintptr_t)) || !os::is_readable_pointer(ptr);
-}
-
 // Looks like all platforms can use the same function to check if C
 // stack is walkable beyond current frame.
-// Returns true if this is not the case, i.e. the frame is possibly
-// the first C frame on the stack.
 bool os::is_first_C_frame(frame* fr) {
 
 #ifdef _WINDOWS
   return true; // native stack isn't walkable on windows this way.
 #endif
+
   // Load up sp, fp, sender sp and sender fp, check for reasonable values.
   // Check usp first, because if that's bad the other accessors may fault
   // on some architectures.  Ditto ufp second, etc.
+  uintptr_t fp_align_mask = (uintptr_t)(sizeof(address)-1);
+  // sp on amd can be 32 bit aligned.
+  uintptr_t sp_align_mask = (uintptr_t)(sizeof(int)-1);
 
-  if (is_pointer_bad(fr->sp())) return true;
+  uintptr_t usp    = (uintptr_t)fr->sp();
+  if ((usp & sp_align_mask) != 0) return true;
 
   uintptr_t ufp    = (uintptr_t)fr->fp();
-  if (is_pointer_bad(fr->fp())) return true;
+  if ((ufp & fp_align_mask) != 0) return true;
 
   uintptr_t old_sp = (uintptr_t)fr->sender_sp();
-  if ((uintptr_t)fr->sender_sp() == (uintptr_t)-1 || is_pointer_bad(fr->sender_sp())) return true;
+  if ((old_sp & sp_align_mask) != 0) return true;
+  if (old_sp == 0 || old_sp == (uintptr_t)-1) return true;
 
-  uintptr_t old_fp = (uintptr_t)fr->link_or_null();
-  if (old_fp == 0 || old_fp == (uintptr_t)-1 || old_fp == ufp ||
-    is_pointer_bad(fr->link_or_null())) return true;
+  uintptr_t old_fp = (uintptr_t)fr->link();
+  if ((old_fp & fp_align_mask) != 0) return true;
+  if (old_fp == 0 || old_fp == (uintptr_t)-1 || old_fp == ufp) return true;
 
   // stack grows downwards; if old_fp is below current fp or if the stack
   // frame is too large, either the stack is corrupted or fp is not saved
@@ -1243,6 +1285,7 @@ bool os::is_first_C_frame(frame* fr) {
   return false;
 }
 
+
 // Set up the boot classpath.
 
 char* os::format_boot_path(const char* format_string,
@@ -1350,6 +1393,14 @@ bool os::set_boot_path(char fileSep, char pathSep) {
   return false;
 }
 
+bool os::file_exists(const char* filename) {
+  struct stat statbuf;
+  if (filename == NULL || strlen(filename) == 0) {
+    return false;
+  }
+  return os::stat(filename, &statbuf) == 0;
+}
+
 // Splits a path, based on its separator, the number of
 // elements is returned back in "elements".
 // file_name_length is used as a modifier for each path's
@@ -1417,7 +1468,7 @@ bool os::stack_shadow_pages_available(Thread *thread, const methodHandle& method
   const int framesize_in_bytes =
     Interpreter::size_top_interpreter_activation(method()) * wordSize;
 
-  address limit = thread->as_Java_thread()->stack_end() +
+  address limit = JavaThread::cast(thread)->stack_end() +
                   (StackOverflow::stack_guard_zone_size() + StackOverflow::stack_shadow_zone_size());
 
   return sp > (limit + framesize_in_bytes);
