@@ -34,7 +34,6 @@
 #include "oops/markWord.hpp"
 #include "oops/oop.inline.hpp"
 #include "runtime/atomic.hpp"
-#include "runtime/biasedLocking.hpp"
 #include "runtime/handles.inline.hpp"
 #include "runtime/handshake.hpp"
 #include "runtime/interfaceSupport.inline.hpp"
@@ -127,7 +126,7 @@ size_t MonitorList::unlink_deflated(Thread* current, LogStream* ls,
 
     if (current->is_Java_thread()) {
       // A JavaThread must check for a safepoint/handshake and honor it.
-      ObjectSynchronizer::chk_for_block_req(current->as_Java_thread(), "unlinking",
+      ObjectSynchronizer::chk_for_block_req(JavaThread::cast(current), "unlinking",
                                             "unlinked_count", unlinked_count,
                                             ls, timer_p);
     }
@@ -213,9 +212,6 @@ void ObjectSynchronizer::initialize() {
   }
   // Start the ceiling with the estimate for one thread.
   set_in_use_list_ceiling(AvgMonitorsPerThreadEstimate);
-
-  // Start the timer for deflations, so it does not trigger immediately.
-  _last_async_deflation_time_ns = os::javaTimeNanos();
 }
 
 MonitorList ObjectSynchronizer::_in_use_list;
@@ -244,7 +240,6 @@ bool volatile ObjectSynchronizer::_is_async_deflation_requested = false;
 bool volatile ObjectSynchronizer::_is_final_audit = false;
 jlong ObjectSynchronizer::_last_async_deflation_time_ns = 0;
 static uintx _no_progress_cnt = 0;
-static bool _no_progress_skip_increment = false;
 
 // =====================> Quick functions
 
@@ -304,7 +299,7 @@ bool ObjectSynchronizer::quick_notify(oopDesc* obj, JavaThread* current, bool al
     return true;
   }
 
-  // biased locking and any other IMS exception states take the slow-path
+  // other IMS exception states take the slow-path
   return false;
 }
 
@@ -353,8 +348,7 @@ bool ObjectSynchronizer::quick_enter(oop obj, JavaThread* current,
     // being locked. We do this unconditionally so that this thread's
     // BasicLock cannot be mis-interpreted by any stack walkers. For
     // performance reasons, stack walkers generally first check for
-    // Biased Locking in the object's header, the second check is for
-    // stack-locking in the object's header, the third check is for
+    // stack-locking in the object's header, the second check is for
     // recursive stack-locking in the displaced header in the BasicLock,
     // and last are the inflated Java Monitor (ObjectMonitor) checks.
     lock->set_displaced_header(markWord::unused_mark());
@@ -368,7 +362,6 @@ bool ObjectSynchronizer::quick_enter(oop obj, JavaThread* current,
   // Note that we could inflate in quick_enter.
   // This is likely a useful optimization
   // Critically, in quick_enter() we must not:
-  // -- perform bias revocation, or
   // -- block indefinitely, or
   // -- reach a safepoint
 
@@ -435,13 +428,7 @@ void ObjectSynchronizer::enter(Handle obj, BasicLock* lock, JavaThread* current)
     handle_sync_on_value_based_class(obj, current);
   }
 
-  if (UseBiasedLocking) {
-    BiasedLocking::revoke(current, obj);
-  }
-
   markWord mark = obj->mark();
-  assert(!mark.has_bias_pattern(), "should not see bias pattern here");
-
   if (mark.is_neutral()) {
     // Anticipate successful CAS -- the ST of the displaced mark must
     // be visible <= the ST performed by the CAS.
@@ -476,9 +463,6 @@ void ObjectSynchronizer::enter(Handle obj, BasicLock* lock, JavaThread* current)
 
 void ObjectSynchronizer::exit(oop object, BasicLock* lock, JavaThread* current) {
   markWord mark = object->mark();
-  // We cannot check for Biased Locking if we are racing an inflation.
-  assert(mark == markWord::INFLATING() ||
-         !mark.has_bias_pattern(), "should not see bias pattern here");
 
   markWord dhw = lock->displaced_header();
   if (dhw.value() == 0) {
@@ -539,11 +523,6 @@ void ObjectSynchronizer::exit(oop object, BasicLock* lock, JavaThread* current)
 //  5) lock lock2
 // NOTE: must use heavy weight monitor to handle complete_exit/reenter()
 intx ObjectSynchronizer::complete_exit(Handle obj, JavaThread* current) {
-  if (UseBiasedLocking) {
-    BiasedLocking::revoke(current, obj);
-    assert(!obj->mark().has_bias_pattern(), "biases should be revoked by now");
-  }
-
   // The ObjectMonitor* can't be async deflated until ownership is
   // dropped inside exit() and the ObjectMonitor* must be !is_busy().
   ObjectMonitor* monitor = inflate(current, obj(), inflate_cause_vm_internal);
@@ -553,11 +532,6 @@ intx ObjectSynchronizer::complete_exit(Handle obj, JavaThread* current) {
 
 // NOTE: must use heavy weight monitor to handle complete_exit/reenter()
 void ObjectSynchronizer::reenter(Handle obj, intx recursions, JavaThread* current) {
-  if (UseBiasedLocking) {
-    BiasedLocking::revoke(current, obj);
-    assert(!obj->mark().has_bias_pattern(), "biases should be revoked by now");
-  }
-
   // An async deflation can race after the inflate() call and before
   // reenter() -> enter() can make the ObjectMonitor busy. reenter() ->
   // enter() returns false if we have lost the race to async deflation
@@ -579,10 +553,6 @@ void ObjectSynchronizer::jni_enter(Handle obj, JavaThread* current) {
   }
 
   // the current locking is from JNI instead of Java code
-  if (UseBiasedLocking) {
-    BiasedLocking::revoke(current, obj);
-    assert(!obj->mark().has_bias_pattern(), "biases should be revoked by now");
-  }
   current->set_current_pending_monitor_is_from_java(false);
   // An async deflation can race after the inflate() call and before
   // enter() can make the ObjectMonitor busy. enter() returns false if
@@ -599,12 +569,6 @@ void ObjectSynchronizer::jni_enter(Handle obj, JavaThread* current) {
 // NOTE: must use heavy weight monitor to handle jni monitor exit
 void ObjectSynchronizer::jni_exit(oop obj, TRAPS) {
   JavaThread* current = THREAD;
-  if (UseBiasedLocking) {
-    Handle h_obj(current, obj);
-    BiasedLocking::revoke(current, h_obj);
-    obj = h_obj();
-  }
-  assert(!obj->mark().has_bias_pattern(), "biases should be revoked by now");
 
   // The ObjectMonitor* can't be async deflated until ownership is
   // dropped inside exit() and the ObjectMonitor* must be !is_busy().
@@ -642,10 +606,6 @@ ObjectLocker::~ObjectLocker() {
 // NOTE: must use heavy weight monitor to handle wait()
 int ObjectSynchronizer::wait(Handle obj, jlong millis, TRAPS) {
   JavaThread* current = THREAD;
-  if (UseBiasedLocking) {
-    BiasedLocking::revoke(current, obj);
-    assert(!obj->mark().has_bias_pattern(), "biases should be revoked by now");
-  }
   if (millis < 0) {
     THROW_MSG_0(vmSymbols::java_lang_IllegalArgumentException(), "timeout value is negative");
   }
@@ -668,10 +628,6 @@ int ObjectSynchronizer::wait(Handle obj, jlong millis, TRAPS) {
 // No exception are possible in this case as we only use this internally when locking is
 // correct and we have to wait until notified - so no interrupts or timeouts.
 void ObjectSynchronizer::wait_uninterruptibly(Handle obj, JavaThread* current) {
-  if (UseBiasedLocking) {
-    BiasedLocking::revoke(current, obj);
-    assert(!obj->mark().has_bias_pattern(), "biases should be revoked by now");
-  }
   // The ObjectMonitor* can't be async deflated because the _waiters
   // field is incremented before ownership is dropped and decremented
   // after ownership is regained.
@@ -681,10 +637,6 @@ void ObjectSynchronizer::wait_uninterruptibly(Handle obj, JavaThread* current) {
 
 void ObjectSynchronizer::notify(Handle obj, TRAPS) {
   JavaThread* current = THREAD;
-  if (UseBiasedLocking) {
-    BiasedLocking::revoke(current, obj);
-    assert(!obj->mark().has_bias_pattern(), "biases should be revoked by now");
-  }
 
   markWord mark = obj->mark();
   if (mark.has_locker() && current->is_lock_owned((address)mark.locker())) {
@@ -700,10 +652,6 @@ void ObjectSynchronizer::notify(Handle obj, TRAPS) {
 // NOTE: see comment of notify()
 void ObjectSynchronizer::notifyall(Handle obj, TRAPS) {
   JavaThread* current = THREAD;
-  if (UseBiasedLocking) {
-    BiasedLocking::revoke(current, obj);
-    assert(!obj->mark().has_bias_pattern(), "biases should be revoked by now");
-  }
 
   markWord mark = obj->mark();
   if (mark.has_locker() && current->is_lock_owned((address)mark.locker())) {
@@ -850,25 +798,6 @@ static inline intptr_t get_next_hash(Thread* current, oop obj) {
 }
 
 intptr_t ObjectSynchronizer::FastHashCode(Thread* current, oop obj) {
-  if (UseBiasedLocking) {
-    // NOTE: many places throughout the JVM do not expect a safepoint
-    // to be taken here. However, we only ever bias Java instances and all
-    // of the call sites of identity_hash that might revoke biases have
-    // been checked to make sure they can handle a safepoint. The
-    // added check of the bias pattern is to avoid useless calls to
-    // thread-local storage.
-    if (obj->mark().has_bias_pattern()) {
-      // Handle for oop obj in case of STW safepoint
-      Handle hobj(current, obj);
-      if (SafepointSynchronize::is_at_safepoint()) {
-        BiasedLocking::revoke_at_safepoint(hobj);
-      } else {
-        BiasedLocking::revoke(current->as_Java_thread(), hobj);
-      }
-      obj = hobj();
-      assert(!obj->mark().has_bias_pattern(), "biases should be revoked by now");
-    }
-  }
 
   while (true) {
     ObjectMonitor* monitor = NULL;
@@ -876,9 +805,6 @@ intptr_t ObjectSynchronizer::FastHashCode(Thread* current, oop obj) {
     intptr_t hash;
     markWord mark = read_stable_mark(obj);
 
-    // object should remain ineligible for biased locking
-    assert(!mark.has_bias_pattern(), "invariant");
-
     if (mark.is_neutral()) {               // if this is a normal header
       hash = mark.hash();
       if (hash != 0) {                     // if it has a hash, just return it
@@ -989,11 +915,6 @@ intptr_t ObjectSynchronizer::identity_hash_value_for(Handle obj) {
 
 bool ObjectSynchronizer::current_thread_holds_lock(JavaThread* current,
                                                    Handle h_obj) {
-  if (UseBiasedLocking) {
-    BiasedLocking::revoke(current, h_obj);
-    assert(!h_obj->mark().has_bias_pattern(), "biases should be revoked by now");
-  }
-
   assert(current == JavaThread::current(), "Can only be called on current thread");
   oop obj = h_obj();
 
@@ -1017,15 +938,6 @@ bool ObjectSynchronizer::current_thread_holds_lock(JavaThread* current,
 
 // FIXME: jvmti should call this
 JavaThread* ObjectSynchronizer::get_lock_owner(ThreadsList * t_list, Handle h_obj) {
-  if (UseBiasedLocking) {
-    if (SafepointSynchronize::is_at_safepoint()) {
-      BiasedLocking::revoke_at_safepoint(h_obj);
-    } else {
-      BiasedLocking::revoke(JavaThread::current(), h_obj);
-    }
-    assert(!h_obj->mark().has_bias_pattern(), "biases should be revoked by now");
-  }
-
   oop obj = h_obj();
   address owner = NULL;
 
@@ -1110,14 +1022,7 @@ static bool monitors_used_above_threshold(MonitorList* list) {
 
   // Check if our monitor usage is above the threshold:
   size_t monitor_usage = (monitors_used * 100LL) / ceiling;
-  if (int(monitor_usage) > MonitorUsedDeflationThreshold) {
-    log_info(monitorinflation)("monitors_used=" SIZE_FORMAT ", ceiling=" SIZE_FORMAT
-                               ", monitor_usage=" SIZE_FORMAT ", threshold=" INTX_FORMAT,
-                               monitors_used, ceiling, monitor_usage, MonitorUsedDeflationThreshold);
-    return true;
-  }
-
-  return false;
+  return int(monitor_usage) > MonitorUsedDeflationThreshold;
 }
 
 size_t ObjectSynchronizer::in_use_list_ceiling() {
@@ -1139,49 +1044,17 @@ void ObjectSynchronizer::set_in_use_list_ceiling(size_t new_value) {
 bool ObjectSynchronizer::is_async_deflation_needed() {
   if (is_async_deflation_requested()) {
     // Async deflation request.
-    log_info(monitorinflation)("Async deflation needed: explicit request");
     return true;
   }
-
-  jlong time_since_last = time_since_last_async_deflation_ms();
-
   if (AsyncDeflationInterval > 0 &&
-      time_since_last > AsyncDeflationInterval &&
+      time_since_last_async_deflation_ms() > AsyncDeflationInterval &&
       monitors_used_above_threshold(&_in_use_list)) {
     // It's been longer than our specified deflate interval and there
     // are too many monitors in use. We don't deflate more frequently
     // than AsyncDeflationInterval (unless is_async_deflation_requested)
     // in order to not swamp the MonitorDeflationThread.
-    log_info(monitorinflation)("Async deflation needed: monitors used are above the threshold");
     return true;
   }
-
-  if (GuaranteedAsyncDeflationInterval > 0 &&
-      time_since_last > GuaranteedAsyncDeflationInterval) {
-    // It's been longer than our specified guaranteed deflate interval.
-    // We need to clean up the used monitors even if the threshold is
-    // not reached, to keep the memory utilization at bay when many threads
-    // touched many monitors.
-    log_info(monitorinflation)("Async deflation needed: guaranteed interval (" INTX_FORMAT " ms) "
-                               "is greater than time since last deflation (" JLONG_FORMAT " ms)",
-                               GuaranteedAsyncDeflationInterval, time_since_last);
-
-    // If this deflation has no progress, then it should not affect the no-progress
-    // tracking, otherwise threshold heuristics would think it was triggered, experienced
-    // no progress, and needs to backoff more aggressively. In this "no progress" case,
-    // the generic code would bump the no-progress counter, and we compensate for that
-    // by telling it to skip the update.
-    //
-    // If this deflation has progress, then it should let non-progress tracking
-    // know about this, otherwise the threshold heuristics would kick in, potentially
-    // experience no-progress due to aggressive cleanup by this deflation, and think
-    // it is still in no-progress stride. In this "progress" case, the generic code would
-    // zero the counter, and we allow it to happen.
-    _no_progress_skip_increment = true;
-
-    return true;
-  }
-
   return false;
 }
 
@@ -1248,14 +1121,12 @@ ObjectMonitor* ObjectSynchronizer::inflate(Thread* current, oop object,
 
   for (;;) {
     const markWord mark = object->mark_acquire();
-    assert(!mark.has_bias_pattern(), "invariant");
 
     // The mark can be in one of the following states:
     // *  Inflated     - just return
     // *  Stack-locked - coerce it to inflated
     // *  INFLATING    - busy wait for conversion to complete
     // *  Neutral      - aggressively inflate the object.
-    // *  BIASED       - Illegal.  We should never see this
 
     // CASE: inflated
     if (mark.has_monitor()) {
@@ -1463,7 +1334,7 @@ size_t ObjectSynchronizer::deflate_monitor_list(Thread* current, LogStream* ls,
 
     if (current->is_Java_thread()) {
       // A JavaThread must check for a safepoint/handshake and honor it.
-      chk_for_block_req(current->as_Java_thread(), "deflation", "deflated_count",
+      chk_for_block_req(JavaThread::cast(current), "deflation", "deflated_count",
                         deflated_count, ls, timer_p);
     }
   }
@@ -1553,7 +1424,7 @@ size_t ObjectSynchronizer::deflate_idle_monitors() {
 
       if (current->is_Java_thread()) {
         // A JavaThread must check for a safepoint/handshake and honor it.
-        chk_for_block_req(current->as_Java_thread(), "deletion", "deleted_count",
+        chk_for_block_req(JavaThread::cast(current), "deletion", "deleted_count",
                           deleted_count, ls, &timer);
       }
     }
@@ -1576,8 +1447,6 @@ size_t ObjectSynchronizer::deflate_idle_monitors() {
 
   if (deflated_count != 0) {
     _no_progress_cnt = 0;
-  } else if (_no_progress_skip_increment) {
-    _no_progress_skip_increment = false;
   } else {
     _no_progress_cnt++;
   }
@@ -1666,7 +1535,6 @@ void ObjectSynchronizer::do_final_audit_and_print_stats() {
     return;
   }
   set_is_final_audit();
-  log_info(monitorinflation)("Starting the final audit.");
 
   if (log_is_enabled(Info, monitorinflation)) {
     // Do a deflation in order to reduce the in-use monitor population
@@ -1676,7 +1544,7 @@ void ObjectSynchronizer::do_final_audit_and_print_stats() {
       ; // empty
     }
     // The other audit_and_print_stats() call is done at the Debug
-    // level at a safepoint in SafepointSynchronize::do_cleanup_tasks.
+    // level at a safepoint in ObjectSynchronizer::do_safepoint_work().
     ObjectSynchronizer::audit_and_print_stats(true /* on_exit */);
   }
 }
