@@ -35,7 +35,6 @@
 #include COMPILER_HEADER(utilities/globalDefinitions)
 
 #include <cstddef>
-#include <type_traits>
 
 class oopDesc;
 
@@ -94,12 +93,10 @@ class oopDesc;
 #define BOOL_TO_STR(_b_) ((_b_) ? "true" : "false")
 
 // Format 32-bit quantities.
-#define INT32_FORMAT             "%"          PRId32
-#define INT32_FORMAT_X_0         "0x%08"      PRIx32
-#define INT32_FORMAT_W(width)    "%"   #width PRId32
-#define UINT32_FORMAT            "%"          PRIu32
-#define UINT32_FORMAT_X_0        "0x%08"      PRIx32
-#define UINT32_FORMAT_W(width)   "%"   #width PRIu32
+#define INT32_FORMAT           "%" PRId32
+#define UINT32_FORMAT          "%" PRIu32
+#define INT32_FORMAT_W(width)  "%" #width PRId32
+#define UINT32_FORMAT_W(width) "%" #width PRIu32
 
 #define PTR32_FORMAT           "0x%08" PRIx32
 #define PTR32_FORMAT_W(width)  "0x%" #width PRIx32
@@ -143,7 +140,6 @@ class oopDesc;
 #define INTPTR_FORMAT_W(width)   "%" #width PRIxPTR
 
 #define SSIZE_FORMAT             "%"   PRIdPTR
-#define SSIZE_PLUS_FORMAT        "%+"  PRIdPTR
 #define SIZE_FORMAT              "%"   PRIuPTR
 #define SIZE_FORMAT_HEX          "0x%" PRIxPTR
 #define SSIZE_FORMAT_W(width)    "%"   #width PRIdPTR
@@ -327,9 +323,6 @@ inline T byte_size_in_proper_unit(T s) {
   }
 }
 
-#define PROPERFMT             SIZE_FORMAT "%s"
-#define PROPERFMTARGS(s)      byte_size_in_proper_unit(s), proper_unit_for_byte_size(s)
-
 inline const char* exact_unit_for_byte_size(size_t s) {
 #ifdef _LP64
   if (s >= G && (s % G) == 0) {
@@ -360,9 +353,6 @@ inline size_t byte_size_in_exact_unit(size_t s) {
   return s;
 }
 
-#define EXACTFMT            SIZE_FORMAT "%s"
-#define EXACTFMTARGS(s)     byte_size_in_exact_unit(s), exact_unit_for_byte_size(s)
-
 // Memory size transition formatting.
 
 #define HEAP_CHANGE_FORMAT "%s: " SIZE_FORMAT "K(" SIZE_FORMAT "K)->" SIZE_FORMAT "K(" SIZE_FORMAT "K)"
@@ -728,11 +718,10 @@ inline bool is_floating_point_type(BasicType t) {
 extern char type2char_tab[T_CONFLICT+1];     // Map a BasicType to a jchar
 inline char type2char(BasicType t) { return (uint)t < T_CONFLICT+1 ? type2char_tab[t] : 0; }
 extern int type2size[T_CONFLICT+1];         // Map BasicType to result stack elements
-extern const char* type2name_tab[T_CONFLICT+1];     // Map a BasicType to a char*
+extern const char* type2name_tab[T_CONFLICT+1];     // Map a BasicType to a jchar
+inline const char* type2name(BasicType t) { return (uint)t < T_CONFLICT+1 ? type2name_tab[t] : NULL; }
 extern BasicType name2type(const char* name);
 
-const char* type2name(BasicType t);
-
 inline jlong max_signed_integer(BasicType bt) {
   if (bt == T_INT) {
     return max_jint;
@@ -1064,7 +1053,7 @@ inline bool is_even(intx x) { return !is_odd(x); }
 
 // abs methods which cannot overflow and so are well-defined across
 // the entire domain of integer types.
-static inline unsigned int g_uabs(unsigned int n) {
+static inline unsigned int uabs(unsigned int n) {
   union {
     unsigned int result;
     int value;
@@ -1073,7 +1062,7 @@ static inline unsigned int g_uabs(unsigned int n) {
   if (value < 0) result = 0-result;
   return result;
 }
-static inline julong g_uabs(julong n) {
+static inline julong uabs(julong n) {
   union {
     julong result;
     jlong value;
@@ -1082,8 +1071,8 @@ static inline julong g_uabs(julong n) {
   if (value < 0) result = 0-result;
   return result;
 }
-static inline julong g_uabs(jlong n) { return g_uabs((julong)n); }
-static inline unsigned int g_uabs(int n) { return g_uabs((unsigned int)n); }
+static inline julong uabs(jlong n) { return uabs((julong)n); }
+static inline unsigned int uabs(int n) { return uabs((unsigned int)n); }
 
 // "to" should be greater than "from."
 inline intx byte_size(void* from, void* to) {
@@ -1209,7 +1198,7 @@ typedef const char* ccstr;
 typedef const char* ccstrlist;   // represents string arguments which accumulate
 
 //----------------------------------------------------------------------------------------------------
-// Default hash/equals functions used by ResourceHashtable and KVHashtable
+// Default hash/equals functions used by ResourceHashtable
 
 template<typename K> unsigned primitive_hash(const K& k) {
   unsigned hash = (unsigned)((uintptr_t)k);
@@ -1221,8 +1210,4 @@ template<typename K> bool primitive_equals(const K& k0, const K& k1) {
 }
 
 
-// Converts any type T to a reference type.
-template<typename T>
-std::add_rvalue_reference_t<T> declval() noexcept;
-
 #endif // SHARE_UTILITIES_GLOBALDEFINITIONS_HPP
