@@ -1,7 +1,6 @@
 /*
  * Copyright (c) 2003, 2021, Oracle and/or its affiliates. All rights reserved.
  * Copyright (c) 2017, 2020 SAP SE. All rights reserved.
- * Copyright (c) 2023, Red Hat, Inc. and/or its affiliates.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -45,7 +44,7 @@
 #include "runtime/init.hpp"
 #include "runtime/os.hpp"
 #include "runtime/osThread.hpp"
-#include "runtime/safefetch.hpp"
+#include "runtime/safefetch.inline.hpp"
 #include "runtime/safepointMechanism.hpp"
 #include "runtime/stackFrameStream.inline.hpp"
 #include "runtime/thread.inline.hpp"
@@ -64,6 +63,9 @@
 #if INCLUDE_JFR
 #include "jfr/jfr.hpp"
 #endif
+#if INCLUDE_JVMCI
+#include "jvmci/jvmci.hpp"
+#endif
 
 #ifndef PRODUCT
 #include <signal.h>
@@ -85,7 +87,6 @@ Thread*           VMError::_thread;
 address           VMError::_pc;
 void*             VMError::_siginfo;
 void*             VMError::_context;
-bool              VMError::_print_native_stack_used = false;
 const char*       VMError::_filename;
 int               VMError::_lineno;
 size_t            VMError::_size;
@@ -96,13 +97,10 @@ static const char* env_list[] = {
   "JAVA_HOME", "JAVA_TOOL_OPTIONS", "_JAVA_OPTIONS", "CLASSPATH",
   "PATH", "USERNAME",
 
-  "XDG_CACHE_HOME", "XDG_CONFIG_HOME", "FC_LANG", "FONTCONFIG_USE_MMAP",
-
   // Env variables that are defined on Linux/BSD
   "LD_LIBRARY_PATH", "LD_PRELOAD", "SHELL", "DISPLAY",
   "HOSTTYPE", "OSTYPE", "ARCH", "MACHTYPE",
-  "LANG", "LC_ALL", "LC_CTYPE", "LC_NUMERIC", "LC_TIME",
-  "TERM", "TMPDIR", "TZ",
+  "LANG", "LC_ALL", "LC_CTYPE", "TZ",
 
   // defined on AIX
   "LIBPATH", "LDR_PRELOAD", "LDR_PRELOAD64",
@@ -116,7 +114,7 @@ static const char* env_list[] = {
   "DYLD_INSERT_LIBRARIES",
 
   // defined on Windows
-  "OS", "PROCESSOR_IDENTIFIER", "_ALT_JAVA_HOME_DIR", "TMP", "TEMP",
+  "OS", "PROCESSOR_IDENTIFIER", "_ALT_JAVA_HOME_DIR",
 
   (const char *)0
 };
@@ -158,7 +156,7 @@ static void print_bug_submit_message(outputStream *out, Thread *thread) {
   // provider of that code.
   if (thread && thread->is_Java_thread() &&
       !thread->is_hidden_from_external_view()) {
-    if (thread->as_Java_thread()->thread_state() == _thread_in_native) {
+    if (JavaThread::cast(thread)->thread_state() == _thread_in_native) {
       out->print_cr("# The crash happened outside the Java Virtual Machine in native code.\n# See problematic frame for where to report the bug.");
     }
   }
@@ -243,129 +241,6 @@ void VMError::print_stack_trace(outputStream* st, JavaThread* jt,
 #endif // ZERO
 }
 
-/**
- * Adds `value` to `list` iff it's not already present and there is sufficient
- * capacity (i.e. length(list) < `list_capacity`). The length of the list
- * is the index of the first nullptr entry or `list_capacity` if there are
- * no nullptr entries.
- *
- * @ return true if the value was added, false otherwise
- */
-static bool add_if_absent(address value, address* list, int list_capacity) {
-  for (int i = 0; i < list_capacity; i++) {
-    if (list[i] == value) {
-      return false;
-    }
-    if (list[i] == nullptr) {
-      list[i] = value;
-      if (i + 1 < list_capacity) {
-        list[i + 1] = nullptr;
-      }
-      return true;
-    }
-  }
-  return false;
-}
-
-/**
- * Prints the VM generated code unit, if any, containing `pc` if it has not already
- * been printed. If the code unit is an InterpreterCodelet or StubCodeDesc, it is
- * only printed if `is_crash_pc` is true.
- *
- * @param printed array of code units that have already been printed (delimited by NULL entry)
- * @param printed_capacity the capacity of `printed`
- * @return true if the code unit was printed, false otherwise
- */
-static bool print_code(outputStream* st, Thread* thread, address pc, bool is_crash_pc,
-                       address* printed, int printed_capacity) {
-  if (Interpreter::contains(pc)) {
-    if (is_crash_pc) {
-      // The interpreter CodeBlob is very large so try to print the codelet instead.
-      InterpreterCodelet* codelet = Interpreter::codelet_containing(pc);
-      if (codelet != nullptr) {
-        if (add_if_absent((address) codelet, printed, printed_capacity)) {
-          codelet->print_on(st);
-          Disassembler::decode(codelet->code_begin(), codelet->code_end(), st);
-          return true;
-        }
-      }
-    }
-  } else {
-    StubCodeDesc* desc = StubCodeDesc::desc_for(pc);
-    if (desc != nullptr) {
-      if (is_crash_pc) {
-        if (add_if_absent((address) desc, printed, printed_capacity)) {
-          desc->print_on(st);
-          Disassembler::decode(desc->begin(), desc->end(), st);
-          return true;
-        }
-      }
-    } else if (thread != nullptr) {
-      CodeBlob* cb = CodeCache::find_blob(pc);
-      if (cb != nullptr && add_if_absent((address) cb, printed, printed_capacity)) {
-        // Disassembling nmethod will incur resource memory allocation,
-        // only do so when thread is valid.
-        ResourceMark rm(thread);
-        Disassembler::decode(cb, st);
-        st->cr();
-        return true;
-      }
-    }
-  }
-  return false;
-}
-
-// Like above, but only try to figure out a short name. Return nullptr if not found.
-static const char* find_code_name(address pc) {
-  if (Interpreter::contains(pc)) {
-    InterpreterCodelet* codelet = Interpreter::codelet_containing(pc);
-    if (codelet != nullptr) {
-      return codelet->description();
-    }
-  } else {
-    StubCodeDesc* desc = StubCodeDesc::desc_for(pc);
-    if (desc != nullptr) {
-      return desc->name();
-    } else {
-      CodeBlob* cb = CodeCache::find_blob(pc);
-      if (cb != nullptr) {
-        return cb->name();
-      }
-    }
-  }
-  return nullptr;
-}
-
-/**
- * Gets the caller frame of `fr`.
- *
- * @returns an invalid frame (i.e. fr.pc() === 0) if the caller cannot be obtained
- */
-static frame next_frame(frame fr, Thread* t) {
-  // Compiled code may use EBP register on x86 so it looks like
-  // non-walkable C frame. Use frame.sender() for java frames.
-  frame invalid;
-  if (t != nullptr && t->is_Java_thread()) {
-    // Catch very first native frame by using stack address.
-    // For JavaThread stack_base and stack_size should be set.
-    if (!t->is_in_full_stack((address)(fr.real_fp() + 1))) {
-      return invalid;
-    }
-    if (fr.is_java_frame() || fr.is_native_frame() || fr.is_runtime_frame()) {
-      RegisterMap map(t->as_Java_thread(), false); // No update
-      return fr.sender(&map);
-    } else {
-      // is_first_C_frame() does only simple checks for frame pointer,
-      // it will pass if java compiled code has a pointer in EBP.
-      if (os::is_first_C_frame(&fr)) return invalid;
-      return os::get_sender_for_C_frame(&fr);
-    }
-  } else {
-    if (os::is_first_C_frame(&fr)) return invalid;
-    return os::get_sender_for_C_frame(&fr);
-  }
-}
-
 void VMError::print_native_stack(outputStream* st, frame fr, Thread* t, char* buf, int buf_size) {
 
   // see if it's a valid frame
@@ -383,9 +258,26 @@ void VMError::print_native_stack(outputStream* st, frame fr, Thread* t, char* bu
         }
       }
       st->cr();
-      fr = next_frame(fr, t);
-      if (fr.pc() == nullptr) {
-        break;
+      // Compiled code may use EBP register on x86 so it looks like
+      // non-walkable C frame. Use frame.sender() for java frames.
+      if (t && t->is_Java_thread()) {
+        // Catch very first native frame by using stack address.
+        // For JavaThread stack_base and stack_size should be set.
+        if (!t->is_in_full_stack((address)(fr.real_fp() + 1))) {
+          break;
+        }
+        if (fr.is_java_frame() || fr.is_native_frame() || fr.is_runtime_frame()) {
+          RegisterMap map(JavaThread::cast(t), false); // No update
+          fr = fr.sender(&map);
+        } else {
+          // is_first_C_frame() does only simple checks for frame pointer,
+          // it will pass if java compiled code has a pointer in EBP.
+          if (os::is_first_C_frame(&fr)) break;
+          fr = os::get_sender_for_C_frame(&fr);
+        }
+      } else {
+        if (os::is_first_C_frame(&fr)) break;
+        fr = os::get_sender_for_C_frame(&fr);
       }
     }
 
@@ -551,10 +443,6 @@ void VMError::report(outputStream* st, bool _verbose) {
   // don't allocate large buffer on stack
   static char buf[O_BUFLEN];
 
-  // Native stack trace may get stuck. We try to handle the last pc if it
-  // belongs to VM generated code.
-  address lastpc = nullptr;
-
   BEGIN
 
   STEP("printing fatal error message")
@@ -612,14 +500,18 @@ void VMError::report(outputStream* st, bool _verbose) {
     // to test that resetting the signal handler works correctly.
     if (_verbose && TestSafeFetchInErrorHandler) {
       st->print_cr("Will test SafeFetch...");
-      int* const invalid_pointer = (int*)segfault_address;
-      const int x = 0x76543210;
-      int i1 = SafeFetch32(invalid_pointer, x);
-      int i2 = SafeFetch32(invalid_pointer, x);
-      if (i1 == x && i2 == x) {
-        st->print_cr("SafeFetch OK."); // Correctly deflected and returned default pattern
+      if (CanUseSafeFetch32()) {
+        int* const invalid_pointer = (int*)segfault_address;
+        const int x = 0x76543210;
+        int i1 = SafeFetch32(invalid_pointer, x);
+        int i2 = SafeFetch32(invalid_pointer, x);
+        if (i1 == x && i2 == x) {
+          st->print_cr("SafeFetch OK."); // Correctly deflected and returned default pattern
+        } else {
+          st->print_cr("??");
+        }
       } else {
-        st->print_cr("??");
+        st->print_cr("not possible; skipped.");
       }
     }
 #endif // ASSERT
@@ -847,29 +739,21 @@ void VMError::report(outputStream* st, bool _verbose) {
   STEP("printing native stack")
 
    if (_verbose) {
-     if (os::platform_print_native_stack(st, _context, buf, sizeof(buf), lastpc)) {
+     if (os::platform_print_native_stack(st, _context, buf, sizeof(buf))) {
        // We have printed the native stack in platform-specific code
        // Windows/x64 needs special handling.
-       // Stack walking may get stuck. Try to find the calling code.
-       if (lastpc != nullptr) {
-         const char* name = find_code_name(lastpc);
-         if (name != nullptr) {
-           st->print_cr("The last pc belongs to %s (printed below).", name);
-         }
-       }
      } else {
        frame fr = _context ? os::fetch_frame_from_context(_context)
                            : os::current_frame();
 
        print_native_stack(st, fr, _thread, buf, sizeof(buf));
-       _print_native_stack_used = true;
      }
    }
 
   STEP("printing Java stack")
 
      if (_verbose && _thread && _thread->is_Java_thread()) {
-       print_stack_trace(st, _thread->as_Java_thread(), buf, sizeof(buf));
+       print_stack_trace(st, JavaThread::cast(_thread), buf, sizeof(buf));
      }
 
   STEP("printing target Java thread stack")
@@ -878,7 +762,7 @@ void VMError::report(outputStream* st, bool _verbose) {
      if (_verbose && _thread && (_thread->is_Named_thread())) {
        Thread* thread = ((NamedThread *)_thread)->processed_thread();
        if (thread != NULL && thread->is_Java_thread()) {
-         JavaThread* jt = thread->as_Java_thread();
+         JavaThread* jt = JavaThread::cast(thread);
          st->print_cr("JavaThread " PTR_FORMAT " (nid = %d) was being processed", p2i(jt), jt->osthread()->thread_id());
          print_stack_trace(st, jt, buf, sizeof(buf), true);
        }
@@ -901,14 +785,6 @@ void VMError::report(outputStream* st, bool _verbose) {
        st->cr();
      }
 
-  STEP("printing registers")
-
-     // printing registers
-     if (_verbose && _context) {
-       os::print_context(st, _context);
-       st->cr();
-     }
-
   STEP("printing register info")
 
      // decode register contents if possible
@@ -918,11 +794,11 @@ void VMError::report(outputStream* st, bool _verbose) {
        st->cr();
      }
 
-  STEP("printing top of stack, instructions near pc")
+  STEP("printing registers, top of stack, instructions near pc")
 
-     // printing top of stack, instructions near pc
+     // registers, top of stack, instructions near pc
      if (_verbose && _context) {
-       os::print_tos_pc(st, _context);
+       os::print_context(st, _context);
        st->cr();
      }
 
@@ -945,54 +821,29 @@ void VMError::report(outputStream* st, bool _verbose) {
        st->cr();
      }
 
-  STEP("printing code blobs if possible")
+  STEP("printing code blob if possible")
 
-     if (_verbose) {
-       const int printed_capacity = max_error_log_print_code;
-       address printed[printed_capacity];
-       printed[0] = nullptr;
-       int printed_len = 0;
-       // Even though ErrorLogPrintCodeLimit is ranged checked
-       // during argument parsing, there's no way to prevent it
-       // subsequently (i.e., after parsing) being set to a
-       // value outside the range.
-       int limit = MIN2(ErrorLogPrintCodeLimit, printed_capacity);
-       if (limit > 0) {
-         // Check if a pc was found by native stack trace above.
-         if (lastpc != nullptr) {
-           if (print_code(st, _thread, lastpc, true, printed, printed_capacity)) {
-             printed_len++;
-           }
-         }
-
-         // Scan the native stack
-         if (!_print_native_stack_used) {
-           // Only try to print code of the crashing frame since
-           // the native stack cannot be walked with next_frame.
-           if (print_code(st, _thread, _pc, true, printed, printed_capacity)) {
-             printed_len++;
+     if (_verbose && _context) {
+       CodeBlob* cb = CodeCache::find_blob(_pc);
+       if (cb != NULL) {
+         if (Interpreter::contains(_pc)) {
+           // The interpreter CodeBlob is very large so try to print the codelet instead.
+           InterpreterCodelet* codelet = Interpreter::codelet_containing(_pc);
+           if (codelet != NULL) {
+             codelet->print_on(st);
+             Disassembler::decode(codelet->code_begin(), codelet->code_end(), st);
            }
          } else {
-           frame fr = _context ? os::fetch_frame_from_context(_context)
-                               : os::current_frame();
-           while (printed_len < limit && fr.pc() != nullptr) {
-             if (print_code(st, _thread, fr.pc(), fr.pc() == _pc, printed, printed_capacity)) {
-               printed_len++;
-             }
-             fr = next_frame(fr, _thread);
-           }
-         }
-
-         // Scan the Java stack
-         if (_thread != nullptr && _thread->is_Java_thread()) {
-           JavaThread* jt = _thread->as_Java_thread();
-           if (jt->has_last_Java_frame()) {
-             for (StackFrameStream sfs(jt, true /* update */, true /* process_frames */); printed_len < limit && !sfs.is_done(); sfs.next()) {
-               address pc = sfs.current()->pc();
-               if (print_code(st, _thread, pc, pc == _pc, printed, printed_capacity)) {
-                 printed_len++;
-               }
-             }
+           StubCodeDesc* desc = StubCodeDesc::desc_for(_pc);
+           if (desc != NULL) {
+             desc->print_on(st);
+             Disassembler::decode(desc->begin(), desc->end(), st);
+           } else if (_thread != NULL) {
+             // Disassembling nmethod will incur resource memory allocation,
+             // only do so when thread is valid.
+             ResourceMark rm(_thread);
+             Disassembler::decode(cb, st);
+             st->cr();
            }
          }
        }
@@ -1183,15 +1034,6 @@ void VMError::report(outputStream* st, bool _verbose) {
        st->cr();
      }
 
-#ifndef _WIN32
-  STEP("printing locale settings")
-
-     if (_verbose) {
-       os::Posix::print_active_locale(st);
-       st->cr();
-     }
-#endif
-
   STEP("printing signal handlers")
 
      if (_verbose) {
@@ -1373,12 +1215,6 @@ void VMError::print_vm_info(outputStream* st) {
   os::print_environment_variables(st, env_list);
   st->cr();
 
-  // STEP("printing locale settings")
-#ifndef _WIN32
-  os::Posix::print_active_locale(st);
-  st->cr();
-#endif
-
   // STEP("printing signal handlers")
 
   os::print_signal_handlers(st, buf, sizeof(buf));
@@ -1440,7 +1276,7 @@ static int expand_and_open(const char* pattern, bool overwrite_existing, char* b
  * Name and location depends on pattern, default_pattern params and access
  * permissions.
  */
-static int prepare_log_file(const char* pattern, const char* default_pattern, bool overwrite_existing, char* buf, size_t buflen) {
+int VMError::prepare_log_file(const char* pattern, const char* default_pattern, bool overwrite_existing, char* buf, size_t buflen) {
   int fd = -1;
 
   // If possible, use specified pattern to construct log file name
@@ -1751,6 +1587,13 @@ void VMError::report_and_die(int id, const char* message, const char* detail_fmt
     }
   }
 
+#if INCLUDE_JVMCI
+  if (JVMCI::fatal_log_filename() != NULL) {
+    out.print_raw("#\n# The JVMCI shared library error report file is saved as:\n# ");
+    out.print_raw_cr(JVMCI::fatal_log_filename());
+  }
+#endif
+
   static bool skip_bug_url = !should_submit_bug_report(_id);
   if (!skip_bug_url) {
     skip_bug_url = true;
@@ -1864,68 +1707,43 @@ void VMError::show_message_box(char *buf, int buflen) {
   } while (yes);
 }
 
-// Fatal error handling is subject to several timeouts:
-// - a global timeout (controlled via ErrorLogTimeout)
-// - local error reporting step timeouts.
-//
-// The latter aims to "give the JVM a kick" if it gets stuck in one particular place during
-// error reporting. This prevents one error reporting step from hogging all the time allotted
-// to error reporting under ErrorLogTimeout.
-//
-// VMError::check_timeout() is called from the watcher thread and checks for either global
-// or step timeout. If a timeout happened, we interrupt the reporting thread and set either
-// _reporting_did_timeout or _step_did_timeout to signal which timeout fired. Function returns
-// true if the *global* timeout fired, which will cause WatcherThread to shut down the JVM
-// immediately.
+// Timeout handling: check if a timeout happened (either a single step did
+// timeout or the whole of error reporting hit ErrorLogTimeout). Interrupt
+// the reporting thread if that is the case.
 bool VMError::check_timeout() {
 
-  // This function is supposed to be called from watcher thread during fatal error handling only.
-  assert(VMError::is_error_reported(), "Only call during error handling");
-  assert(Thread::current()->is_Watcher_thread(), "Only call from watcher thread");
-
   if (ErrorLogTimeout == 0) {
     return false;
   }
 
-  // There are three situations where we suppress the *global* error timeout:
-  // - if the JVM is embedded and the launcher has its abort hook installed.
-  //   That must be allowed to run.
-  // - if the user specified one or more OnError commands to run, and these
-  //   did not yet run. These must have finished.
-  // - if the user (typically developer) specified ShowMessageBoxOnError,
-  //   and the error box has not yet been shown
-  const bool ignore_global_timeout =
-      (ShowMessageBoxOnError
-            || (OnError != nullptr && OnError[0] != '\0')
-            || Arguments::abort_hook() != nullptr);
+  // Do not check for timeouts if we still have a message box to show to the
+  // user or if there are OnError handlers to be run.
+  if (ShowMessageBoxOnError
+      || (OnError != NULL && OnError[0] != '\0')
+      || Arguments::abort_hook() != NULL) {
+    return false;
+  }
 
+  const jlong reporting_start_time_l = get_reporting_start_time();
   const jlong now = get_current_timestamp();
-
-  // Global timeout hit?
-  if (!ignore_global_timeout) {
-    const jlong reporting_start_time = get_reporting_start_time();
-    // Timestamp is stored in nanos.
-    if (reporting_start_time > 0) {
-      const jlong end = reporting_start_time + (jlong)ErrorLogTimeout * TIMESTAMP_TO_SECONDS_FACTOR;
-      if (end <= now && !_reporting_did_timeout) {
-        // We hit ErrorLogTimeout and we haven't interrupted the reporting
-        // thread yet.
-        _reporting_did_timeout = true;
-        interrupt_reporting_thread();
-        return true; // global timeout
-      }
+  // Timestamp is stored in nanos.
+  if (reporting_start_time_l > 0) {
+    const jlong end = reporting_start_time_l + (jlong)ErrorLogTimeout * TIMESTAMP_TO_SECONDS_FACTOR;
+    if (end <= now && !_reporting_did_timeout) {
+      // We hit ErrorLogTimeout and we haven't interrupted the reporting
+      // thread yet.
+      _reporting_did_timeout = true;
+      interrupt_reporting_thread();
+      return true; // global timeout
     }
   }
 
-  // Reporting step timeout?
-  const jlong step_start_time = get_step_start_time();
-  if (step_start_time > 0) {
+  const jlong step_start_time_l = get_step_start_time();
+  if (step_start_time_l > 0) {
     // A step times out after a quarter of the total timeout. Steps are mostly fast unless they
     // hang for some reason, so this simple rule allows for three hanging step and still
     // hopefully leaves time enough for the rest of the steps to finish.
-    const int max_step_timeout_secs = 5;
-    const jlong timeout_duration = MAX2((jlong)max_step_timeout_secs, (jlong)ErrorLogTimeout * TIMESTAMP_TO_SECONDS_FACTOR / 4);
-    const jlong end = step_start_time + timeout_duration;
+    const jlong end = step_start_time_l + (jlong)ErrorLogTimeout * TIMESTAMP_TO_SECONDS_FACTOR / 4;
     if (end <= now && !_step_did_timeout) {
       // The step timed out and we haven't interrupted the reporting
       // thread yet.
