@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2025, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,17 +25,15 @@
 
 package sun.security.ssl;
 
-import static sun.security.ssl.SignatureScheme.CERTIFICATE_SCOPE;
-import static sun.security.ssl.SignatureScheme.HANDSHAKE_SCOPE;
-
 import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.security.PrivateKey;
 import java.security.cert.X509Certificate;
 import java.text.MessageFormat;
 import java.util.ArrayList;
-import java.util.Arrays;
+import java.util.Collection;
 import java.util.Collections;
+import java.util.HashSet;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Locale;
@@ -136,7 +134,7 @@ final class CertificateRequest {
             ArrayList<String> keyTypes = new ArrayList<>(3);
             for (byte id : ids) {
                 ClientCertificateType cct = ClientCertificateType.valueOf(id);
-                if (cct != null && cct.isAvailable) {
+                if (cct.isAvailable) {
                     cct.keyAlgorithm.forEach(key -> {
                         if (!keyTypes.contains(key)) {
                             keyTypes.add(key);
@@ -206,12 +204,9 @@ final class CertificateRequest {
             return  ClientCertificateType.getKeyTypes(types);
         }
 
-        // This method will throw IllegalArgumentException if the
-        // X500Principal cannot be parsed.
         X500Principal[] getAuthorities() {
             X500Principal[] principals = new X500Principal[authorities.size()];
             int i = 0;
-
             for (byte[] encoded : authorities) {
                 principals[i++] = new X500Principal(encoded);
             }
@@ -264,12 +259,8 @@ final class CertificateRequest {
 
             List<String> authorityNames = new ArrayList<>(authorities.size());
             for (byte[] encoded : authorities) {
-                try {
-                    X500Principal principal = new X500Principal(encoded);
-                    authorityNames.add(principal.toString());
-                } catch (IllegalArgumentException iae) {
-                    authorityNames.add("unparseable distinguished name: " + iae);
-                }
+                X500Principal principal = new X500Principal(encoded);
+                authorityNames.add(principal.toString());
             }
             Object[] messageFields = {
                 typeNames,
@@ -342,16 +333,6 @@ final class CertificateRequest {
 
             // clean up this consumer
             chc.handshakeConsumers.remove(SSLHandshake.CERTIFICATE_REQUEST.id);
-            chc.receivedCertReq = true;
-
-            // If we're processing this message and the server's certificate
-            // message consumer has not already run then this is a state
-            // machine violation.
-            if (chc.handshakeConsumers.containsKey(
-                    SSLHandshake.CERTIFICATE.id)) {
-                throw chc.conContext.fatal(Alert.UNEXPECTED_MESSAGE,
-                        "Unexpected CertificateRequest handshake message");
-            }
 
             SSLConsumer certStatCons = chc.handshakeConsumers.remove(
                     SSLHandshake.CERTIFICATE_STATUS.id);
@@ -384,25 +365,15 @@ final class CertificateRequest {
 
             X509ExtendedKeyManager km = chc.sslContext.getX509KeyManager();
             String clientAlias = null;
-
-            try {
-                if (chc.conContext.transport instanceof SSLSocketImpl) {
-                    clientAlias = km.chooseClientAlias(crm.getKeyTypes(),
-                        crm.getAuthorities(),
-                        (SSLSocket) chc.conContext.transport);
-                } else if (chc.conContext.transport instanceof SSLEngineImpl) {
-                    clientAlias =
-                        km.chooseEngineClientAlias(crm.getKeyTypes(),
-                            crm.getAuthorities(),
-                            (SSLEngine) chc.conContext.transport);
-                }
-            } catch (IllegalArgumentException iae) {
-                chc.conContext.fatal(Alert.DECODE_ERROR,
-                    "The distinguished names of the peer's "
-                    + "certificate authorities could not be parsed",
-                        iae);
+            if (chc.conContext.transport instanceof SSLSocketImpl) {
+                clientAlias = km.chooseClientAlias(crm.getKeyTypes(),
+                    crm.getAuthorities(), (SSLSocket)chc.conContext.transport);
+            } else if (chc.conContext.transport instanceof SSLEngineImpl) {
+                clientAlias = km.chooseEngineClientAlias(crm.getKeyTypes(),
+                    crm.getAuthorities(), (SSLEngine)chc.conContext.transport);
             }
 
+
             if (clientAlias == null) {
                 if (SSLLogger.isOn && SSLLogger.isOn("ssl,handshake")) {
                     SSLLogger.warning("No available client authentication");
@@ -536,12 +507,9 @@ final class CertificateRequest {
             return ClientCertificateType.getKeyTypes(types);
         }
 
-        // This method will throw IllegalArgumentException if the
-        // X500Principal cannot be parsed.
         X500Principal[] getAuthorities() {
             X500Principal[] principals = new X500Principal[authorities.size()];
             int i = 0;
-
             for (byte[] encoded : authorities) {
                 principals[i++] = new X500Principal(encoded);
             }
@@ -605,13 +573,8 @@ final class CertificateRequest {
 
             List<String> authorityNames = new ArrayList<>(authorities.size());
             for (byte[] encoded : authorities) {
-                try {
-                    X500Principal principal = new X500Principal(encoded);
-                    authorityNames.add(principal.toString());
-                } catch (IllegalArgumentException iae) {
-                    authorityNames.add("unparseable distinguished name: " +
-                        iae);
-                }
+                X500Principal principal = new X500Principal(encoded);
+                authorityNames.add(principal.toString());
             }
             Object[] messageFields = {
                 typeNames,
@@ -637,33 +600,16 @@ final class CertificateRequest {
         public byte[] produce(ConnectionContext context,
                 HandshakeMessage message) throws IOException {
             // The producing happens in server side only.
-            ServerHandshakeContext shc = (ServerHandshakeContext) context;
-
+            ServerHandshakeContext shc = (ServerHandshakeContext)context;
             if (shc.localSupportedSignAlgs == null) {
                 shc.localSupportedSignAlgs =
-                        SignatureScheme.getSupportedAlgorithms(
-                                shc.sslConfig,
-                                shc.algorithmConstraints, shc.activeProtocols,
-                                HANDSHAKE_SCOPE);
+                    SignatureScheme.getSupportedAlgorithms(
+                            shc.sslConfig,
+                            shc.algorithmConstraints, shc.activeProtocols);
             }
 
-            if (shc.localSupportedCertSignAlgs == null) {
-                shc.localSupportedCertSignAlgs =
-                        SignatureScheme.getSupportedAlgorithms(
-                                shc.sslConfig,
-                                shc.algorithmConstraints, shc.activeProtocols,
-                                CERTIFICATE_SCOPE);
-            }
-
-            // According to TLSv1.2 RFC, CertificateRequest message must
-            // contain signature schemes supported for both:
-            // handshake signatures and certificate signatures.
-            List<SignatureScheme> certReqSignAlgs =
-                    new ArrayList<>(shc.localSupportedSignAlgs);
-            certReqSignAlgs.retainAll(shc.localSupportedCertSignAlgs);
-
-            if (certReqSignAlgs == null ||
-                    certReqSignAlgs.isEmpty()) {
+            if (shc.localSupportedSignAlgs == null ||
+                    shc.localSupportedSignAlgs.isEmpty()) {
                 throw shc.conContext.fatal(Alert.HANDSHAKE_FAILURE,
                     "No supported signature algorithm");
             }
@@ -672,7 +618,7 @@ final class CertificateRequest {
                     shc.sslContext.getX509TrustManager().getAcceptedIssuers();
             T12CertificateRequestMessage crm = new T12CertificateRequestMessage(
                     shc, caCerts, shc.negotiatedCipherSuite.keyExchange,
-                    certReqSignAlgs);
+                    shc.localSupportedSignAlgs);
             if (SSLLogger.isOn && SSLLogger.isOn("ssl,handshake")) {
                 SSLLogger.fine(
                     "Produced CertificateRequest handshake message", crm);
@@ -713,16 +659,6 @@ final class CertificateRequest {
 
             // clean up this consumer
             chc.handshakeConsumers.remove(SSLHandshake.CERTIFICATE_REQUEST.id);
-            chc.receivedCertReq = true;
-
-            // If we're processing this message and the server's certificate
-            // message consumer has not already run then this is a state
-            // machine violation.
-            if (chc.handshakeConsumers.containsKey(
-                    SSLHandshake.CERTIFICATE.id)) {
-                throw chc.conContext.fatal(Alert.UNEXPECTED_MESSAGE,
-                        "Unexpected CertificateRequest handshake message");
-            }
 
             SSLConsumer certStatCons = chc.handshakeConsumers.remove(
                     SSLHandshake.CERTIFICATE_STATUS.id);
@@ -753,42 +689,27 @@ final class CertificateRequest {
             chc.handshakeProducers.put(SSLHandshake.CERTIFICATE.id,
                     SSLHandshake.CERTIFICATE);
 
-            List<SignatureScheme> signAlgs =
+            List<SignatureScheme> sss =
                     SignatureScheme.getSupportedAlgorithms(
                             chc.sslConfig,
                             chc.algorithmConstraints, chc.negotiatedProtocol,
-                            crm.algorithmIds,
-                            HANDSHAKE_SCOPE);
-
-            List<SignatureScheme> signCertAlgs =
-                    SignatureScheme.getSupportedAlgorithms(
-                            chc.sslConfig,
-                            chc.algorithmConstraints, chc.negotiatedProtocol,
-                            crm.algorithmIds,
-                            CERTIFICATE_SCOPE);
-
-            if (signAlgs == null || signAlgs.isEmpty() || signCertAlgs.isEmpty()) {
+                            crm.algorithmIds);
+            if (sss == null || sss.isEmpty()) {
                 throw chc.conContext.fatal(Alert.HANDSHAKE_FAILURE,
                         "No supported signature algorithm");
             }
 
-            chc.peerRequestedSignatureSchemes = signAlgs;
-            chc.peerRequestedCertSignSchemes = signCertAlgs;
-            chc.handshakeSession.setPeerSupportedSignatureAlgorithms(signCertAlgs);
+            chc.peerRequestedSignatureSchemes = sss;
+            chc.peerRequestedCertSignSchemes = sss;     // use the same schemes
+            chc.handshakeSession.setPeerSupportedSignatureAlgorithms(sss);
+            chc.peerSupportedAuthorities = crm.getAuthorities();
 
-            try {
-                chc.peerSupportedAuthorities = crm.getAuthorities();
-            } catch (IllegalArgumentException iae) {
-                chc.conContext.fatal(Alert.DECODE_ERROR, "The "
-                    + "distinguished names of the peer's certificate "
-                    + "authorities could not be parsed", iae);
-            }
             // For TLS 1.2, we no longer use the certificate_types field
             // from the CertificateRequest message to directly determine
             // the SSLPossession.  Instead, the choosePossession method
             // will use the accepted signature schemes in the message to
             // determine the set of acceptable certificate types to select from.
-            SSLPossession pos = choosePossession(chc, crm);
+            SSLPossession pos = choosePossession(chc);
             if (pos == null) {
                 return;
             }
@@ -798,8 +719,8 @@ final class CertificateRequest {
                     SSLHandshake.CERTIFICATE_VERIFY);
         }
 
-        private static SSLPossession choosePossession(HandshakeContext hc,
-                T12CertificateRequestMessage crm) throws IOException {
+        private static SSLPossession choosePossession(HandshakeContext hc)
+                throws IOException {
             if (hc.peerRequestedCertSignSchemes == null ||
                     hc.peerRequestedCertSignSchemes.isEmpty()) {
                 if (SSLLogger.isOn && SSLLogger.isOn("ssl,handshake")) {
@@ -809,37 +730,46 @@ final class CertificateRequest {
                 return null;
             }
 
-            // Put the CR key type into a more friendly format for searching
-            List<String> crKeyTypes = new ArrayList<>(
-                    Arrays.asList(crm.getKeyTypes()));
-            // For TLS 1.2 only if RSA is a requested key type then we
-            // should also allow RSASSA-PSS.
-            if (crKeyTypes.contains("RSA")) {
-                crKeyTypes.add("RSASSA-PSS");
+            Collection<String> checkedKeyTypes = new HashSet<>();
+            List<String> supportedKeyTypes = new ArrayList<>();
+            for (SignatureScheme ss : hc.peerRequestedCertSignSchemes) {
+                if (checkedKeyTypes.contains(ss.keyAlgorithm)) {
+                    if (SSLLogger.isOn && SSLLogger.isOn("ssl,handshake")) {
+                        SSLLogger.warning(
+                            "Unsupported authentication scheme: " + ss.name);
+                    }
+                    continue;
+                }
+                checkedKeyTypes.add(ss.keyAlgorithm);
+
+                // Don't select a signature scheme unless we will be able to
+                // produce a CertificateVerify message later
+                if (SignatureScheme.getPreferableAlgorithm(
+                        hc.algorithmConstraints,
+                        hc.peerRequestedSignatureSchemes,
+                        ss, hc.negotiatedProtocol) == null) {
+
+                    if (SSLLogger.isOn && SSLLogger.isOn("ssl,handshake")) {
+                        SSLLogger.warning(
+                            "Unable to produce CertificateVerify for " +
+                            "signature scheme: " + ss.name);
+                    }
+                    continue;
+                }
+
+                X509Authentication ka = X509Authentication.valueOf(ss);
+                if (ka == null) {
+                    if (SSLLogger.isOn && SSLLogger.isOn("ssl,handshake")) {
+                        SSLLogger.warning(
+                            "Unsupported authentication scheme: " + ss.name);
+                    }
+                    continue;
+                }
+                supportedKeyTypes.add(ss.keyAlgorithm);
             }
 
-            String[] supportedKeyTypes = hc.peerRequestedCertSignSchemes
-                    .stream()
-                    .map(ss -> ss.keyAlgorithm)
-                    .distinct()
-                    .filter(ka -> SignatureScheme.getPreferableAlgorithm(   // Don't select a signature scheme unless
-                            hc.algorithmConstraints,                        //  we will be able to produce
-                            hc.peerRequestedSignatureSchemes,               //  a CertificateVerify message later
-                            ka, hc.negotiatedProtocol) != null
-                            || SSLLogger.logWarning("ssl,handshake",
-                                    "Unable to produce CertificateVerify for key algorithm: " + ka))
-                    .filter(ka -> {
-                        var xa = X509Authentication.valueOfKeyAlgorithm(ka);
-                        // Any auth object will have a set of allowed key types.
-                        // This set should share at least one common algorithm with
-                        // the CR's allowed key types.
-                        return xa != null && !Collections.disjoint(crKeyTypes, Arrays.asList(xa.keyTypes))
-                                || SSLLogger.logWarning("ssl,handshake", "Unsupported key algorithm: " + ka);
-                    })
-                    .toArray(String[]::new);
-
             SSLPossession pos = X509Authentication
-                    .createPossession(hc, supportedKeyTypes);
+                    .createPossession(hc, supportedKeyTypes.toArray(String[]::new));
             if (pos == null) {
                 if (SSLLogger.isOn && SSLLogger.isOn("ssl,handshake")) {
                     SSLLogger.warning("No available authentication scheme");
@@ -990,15 +920,6 @@ final class CertificateRequest {
 
             // clean up this consumer
             chc.handshakeConsumers.remove(SSLHandshake.CERTIFICATE_REQUEST.id);
-            chc.receivedCertReq = true;
-
-            // Ensure that the CertificateRequest has not been sent prior
-            // to EncryptedExtensions
-            if (chc.handshakeConsumers.containsKey(
-                    SSLHandshake.ENCRYPTED_EXTENSIONS.id)) {
-                throw chc.conContext.fatal(Alert.UNEXPECTED_MESSAGE,
-                        "Unexpected CertificateRequest handshake message");
-            }
 
             T13CertificateRequestMessage crm =
                     new T13CertificateRequestMessage(chc, message);
