@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,10 +25,8 @@
 
 package sun.security.util;
 
-import sun.security.ssl.SSLScope;
 import sun.security.validator.Validator;
 
-import java.lang.ref.SoftReference;
 import java.security.AlgorithmParameters;
 import java.security.CryptoPrimitive;
 import java.security.Key;
@@ -41,14 +39,9 @@ import java.security.spec.InvalidParameterSpecException;
 import java.security.spec.MGF1ParameterSpec;
 import java.security.spec.NamedParameterSpec;
 import java.security.spec.PSSParameterSpec;
-import java.time.DateTimeException;
-import java.time.Instant;
-import java.time.ZoneId;
-import java.time.ZonedDateTime;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Collection;
-import java.util.Collections;
+import java.util.Calendar;
 import java.util.Date;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -56,10 +49,11 @@ import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
+import java.util.Collection;
 import java.util.StringTokenizer;
-import java.util.concurrent.ConcurrentHashMap;
-import java.util.regex.Matcher;
+import java.util.TimeZone;
 import java.util.regex.Pattern;
+import java.util.regex.Matcher;
 
 /**
  * Algorithm constraints for disabled algorithms property
@@ -104,10 +98,7 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
     }
 
     private final Set<String> disabledAlgorithms;
-    private final List<Pattern> disabledPatterns;
     private final Constraints algorithmConstraints;
-    private volatile SoftReference<Map<String, Boolean>> cacheRef =
-            new SoftReference<>(null);
 
     public static DisabledAlgorithmConstraints certPathConstraints() {
         return CertPathHolder.CONSTRAINTS;
@@ -140,13 +131,6 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
         super(decomposer);
         disabledAlgorithms = getAlgorithms(propertyName);
 
-        // Support patterns only for jdk.tls.disabledAlgorithms
-        if (PROPERTY_TLS_DISABLED_ALGS.equals(propertyName)) {
-            disabledPatterns = getDisabledPatterns();
-        } else {
-            disabledPatterns = null;
-        }
-
         // Check for alias
         for (String s : disabledAlgorithms) {
             Matcher matcher = INCLUDE_PATTERN.matcher(s);
@@ -171,11 +155,8 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
             throw new IllegalArgumentException("The primitives cannot be null" +
                     " or empty.");
         }
-        if (algorithm == null || algorithm.isEmpty()) {
-            throw new IllegalArgumentException("No algorithm name specified");
-        }
 
-        if (!cachedCheckAlgorithm(algorithm)) {
+        if (!checkAlgorithm(disabledAlgorithms, algorithm, decomposer)) {
             return false;
         }
 
@@ -186,12 +167,6 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
         return true;
     }
 
-    // Checks if algorithm is disabled for the given TLS scopes.
-    public boolean permits(String algorithm, Set<SSLScope> scopes) {
-        List<Constraint> list = algorithmConstraints.getConstraints(algorithm);
-        return list == null || list.stream().allMatch(c -> c.permits(scopes));
-    }
-
     /*
      * Checks if the key algorithm has been disabled or constraints have been
      * placed on the key.
@@ -226,7 +201,7 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
     }
 
     private void permits(AlgorithmParameters ap, ConstraintsParameters cp)
-            throws CertPathValidatorException {
+        throws CertPathValidatorException {
 
         switch (ap.getAlgorithm().toUpperCase(Locale.ENGLISH)) {
             case "RSASSA-PSS":
@@ -260,12 +235,11 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
 
     public final void permits(String algorithm, ConstraintsParameters cp,
             boolean checkKey) throws CertPathValidatorException {
-
         if (checkKey) {
             // Check if named curves in the key are disabled.
             for (Key key : cp.getKeys()) {
                 for (String curve : getNamedCurveFromKey(key)) {
-                    if (!cachedCheckAlgorithm(curve)) {
+                    if (!checkAlgorithm(disabledAlgorithms, curve, decomposer)) {
                         throw new CertPathValidatorException(
                             "Algorithm constraints check failed on disabled " +
                                     "algorithm: " + curve,
@@ -274,7 +248,6 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
                 }
             }
         }
-
         algorithmConstraints.permits(algorithm, cp, checkKey);
     }
 
@@ -331,10 +304,6 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
     /**
      * Key and Certificate Constraints
      *
-     * The complete disabling of an algorithm is not handled by Constraints or
-     * Constraint classes.  That is addressed with
-     *   permit(Set<CryptoPrimitive>, String, AlgorithmParameters)
-     *
      * When passing a Key to permit(), the boolean return values follow the
      * same as the interface class AlgorithmConstraints.permit().  This is to
      * maintain compatibility:
@@ -345,7 +314,6 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
      * will be thrown on a failure to better identify why the operation was
      * disallowed.
      */
-
     private static class Constraints {
         private Map<String, List<Constraint>> constraintsMap = new HashMap<>();
 
@@ -368,9 +336,9 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
                 // Check if constraint is a complete disabling of an
                 // algorithm or has conditions.
                 int space = constraintEntry.indexOf(' ');
-                String algorithm = AlgorithmDecomposer.hashName(
-                        ((space > 0 ? constraintEntry.substring(0, space) :
-                                constraintEntry)));
+                String algorithm = AlgorithmDecomposer.decomposeDigestName(
+                        space > 0 ? constraintEntry.substring(0, space) :
+                                constraintEntry);
                 List<Constraint> constraintList =
                         constraintsMap.getOrDefault(
                                 algorithm.toUpperCase(Locale.ENGLISH),
@@ -448,7 +416,7 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
                         denyAfterLimit = true;
                     } else if (entry.startsWith("usage")) {
                         String s[] = (entry.substring(5)).trim().split(" ");
-                        c = new UsageConstraint(algorithm, s, propertyName);
+                        c = new UsageConstraint(algorithm, s);
                         if (debug != null) {
                             debug.println("Constraints usage length is " + s.length);
                         }
@@ -524,7 +492,7 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
             // Get all signature algorithms to check for constraints
             Set<String> algorithms = new HashSet<>();
             if (algorithm != null) {
-                algorithms.addAll(AlgorithmDecomposer.decomposeOneHash(algorithm));
+                algorithms.addAll(AlgorithmDecomposer.decomposeName(algorithm));
                 algorithms.add(algorithm);
             }
 
@@ -619,17 +587,6 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
             return true;
         }
 
-        /**
-         * Check if the algorithm constraint permits the given TLS scopes.
-         *
-         * @param scopes TLS scopes
-         * @return 'true' if TLS scopes are allowed,
-         *         'false' otherwise.
-         */
-        public boolean permits(Set<SSLScope> scopes) {
-            return true;
-        }
-
         /**
          * Check if an algorithm constraint is permitted with a given
          * ConstraintsParameters.
@@ -729,30 +686,41 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
      * timezone.
      */
     private static class DenyAfterConstraint extends Constraint {
-        private ZonedDateTime zdt;
-        private Instant denyAfterDate;
+        private Date denyAfterDate;
 
         DenyAfterConstraint(String algo, int year, int month, int day) {
+            Calendar c;
 
             algorithm = algo;
 
             if (debug != null) {
-                debug.println("DenyAfterConstraint read in as: year " +
+                debug.println("DenyAfterConstraint read in as:  year " +
                         year + ", month = " + month + ", day = " + day);
             }
 
-            try {
-                zdt = ZonedDateTime
-                    .of(year, month, day, 0, 0, 0, 0, ZoneId.of("GMT"));
-                denyAfterDate = zdt.toInstant();
-            } catch (DateTimeException dte) {
+            c = new Calendar.Builder().setTimeZone(TimeZone.getTimeZone("GMT"))
+                    .setDate(year, month - 1, day).build();
+
+            if (year > c.getActualMaximum(Calendar.YEAR) ||
+                    year < c.getActualMinimum(Calendar.YEAR)) {
                 throw new IllegalArgumentException(
-                    "Invalid denyAfter date", dte);
+                        "Invalid year given in constraint: " + year);
+            }
+            if ((month - 1) > c.getActualMaximum(Calendar.MONTH) ||
+                    (month - 1) < c.getActualMinimum(Calendar.MONTH)) {
+                throw new IllegalArgumentException(
+                        "Invalid month given in constraint: " + month);
+            }
+            if (day > c.getActualMaximum(Calendar.DAY_OF_MONTH) ||
+                    day < c.getActualMinimum(Calendar.DAY_OF_MONTH)) {
+                throw new IllegalArgumentException(
+                        "Invalid Day of Month given in constraint: " + day);
             }
 
+            denyAfterDate = c.getTime();
             if (debug != null) {
                 debug.println("DenyAfterConstraint date set to: " +
-                        zdt.toLocalDate());
+                        denyAfterDate);
             }
         }
 
@@ -767,22 +735,23 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
         @Override
         public void permits(ConstraintsParameters cp)
                 throws CertPathValidatorException {
-            Instant currentDate;
+            Date currentDate;
+            String errmsg;
 
             if (cp.getDate() != null) {
-                currentDate = cp.getDate().toInstant();
+                currentDate = cp.getDate();
             } else {
-                currentDate = Instant.now();
+                currentDate = new Date();
             }
 
-            if (!denyAfterDate.isAfter(currentDate)) {
+            if (!denyAfterDate.after(currentDate)) {
                 if (next(cp)) {
                     return;
                 }
                 throw new CertPathValidatorException(
                         "denyAfter constraint check failed: " + algorithm +
                         " used with Constraint date: " +
-                        zdt.toLocalDate() + "; params date: " +
+                        denyAfterDate + "; params date: " +
                         currentDate + cp.extendedExceptionMsg(),
                         null, null, -1, BasicReason.ALGORITHM_CONSTRAINED);
             }
@@ -801,55 +770,20 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
                 debug.println("DenyAfterConstraints.permits(): " + algorithm);
             }
 
-            return denyAfterDate.isAfter(Instant.now());
+            return denyAfterDate.after(new Date());
         }
     }
 
     /*
      * The usage constraint is for the "usage" keyword.  It checks against the
-     * variant value in ConstraintsParameters and against TLS scopes.
+     * variant value in ConstraintsParameters.
      */
     private static class UsageConstraint extends Constraint {
         String[] usages;
-        Set<SSLScope> scopes;
 
-        UsageConstraint(
-                String algorithm, String[] usages, String propertyName) {
+        UsageConstraint(String algorithm, String[] usages) {
             this.algorithm = algorithm;
-
-            // Support TLS scopes only for jdk.tls.disabledAlgorithms property.
-            if (PROPERTY_TLS_DISABLED_ALGS.equals(propertyName)) {
-                for (String usage : usages) {
-                    SSLScope scope = SSLScope.nameOf(usage);
-
-                    if (scope != null) {
-                        if (this.scopes == null) {
-                            this.scopes = new HashSet<>(usages.length);
-                        }
-                        this.scopes.add(scope);
-                    } else {
-                        this.usages = usages;
-                    }
-                }
-
-                if (this.scopes != null && this.usages != null) {
-                    throw new IllegalArgumentException(
-                            "Can't mix TLS protocol specific constraints"
-                            + " with other usage constraints");
-                }
-
-            } else {
-                this.usages = usages;
-            }
-        }
-
-        @Override
-        public boolean permits(Set<SSLScope> scopes) {
-            if (this.scopes == null || scopes == null) {
-                return true;
-            }
-
-            return Collections.disjoint(this.scopes, scopes);
+            this.usages = usages;
         }
 
         @Override
@@ -868,7 +802,6 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
                         break;
                     case "signedjar":
                         match =
-                            variant.equals(Validator.VAR_PLUGIN_CODE_SIGNING) ||
                             variant.equals(Validator.VAR_CODE_SIGNING) ||
                             variant.equals(Validator.VAR_TSA_SERVER);
                         break;
@@ -1024,62 +957,6 @@ public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints {
         }
     }
 
-    private boolean cachedCheckAlgorithm(String algorithm) {
-        Map<String, Boolean> cache;
-        if ((cache = cacheRef.get()) == null) {
-            synchronized (this) {
-                if ((cache = cacheRef.get()) == null) {
-                    cache = new ConcurrentHashMap<>();
-                    cacheRef = new SoftReference<>(cache);
-                }
-            }
-        }
-        Boolean result = cache.get(algorithm);
-        if (result != null) {
-            return result;
-        }
-        // We won't check patterns if algorithm check fails.
-        result = checkAlgorithm(disabledAlgorithms, algorithm, decomposer)
-                && checkDisabledPatterns(algorithm);
-        cache.put(algorithm, result);
-        return result;
-    }
-
-    private boolean checkDisabledPatterns(final String algorithm) {
-        return disabledPatterns == null || disabledPatterns.stream().noneMatch(
-                p -> p.matcher(algorithm).matches());
-    }
-
-    private List<Pattern> getDisabledPatterns() {
-        List<Pattern> ret = null;
-        List<String> patternStrings = new ArrayList<>(4);
-
-        for (String p : disabledAlgorithms) {
-            if (p.contains("*")) {
-                if (!p.startsWith("TLS_")) {
-                    throw new IllegalArgumentException(
-                            "Wildcard pattern must start with \"TLS_\"");
-                }
-                patternStrings.add(p);
-            }
-        }
-
-        if (!patternStrings.isEmpty()) {
-            ret = new ArrayList<>(patternStrings.size());
-
-            for (String p : patternStrings) {
-                // Exclude patterns from algorithm code flow.
-                disabledAlgorithms.remove(p);
-
-                // Ignore all regex characters but asterisk.
-                ret.add(Pattern.compile(
-                        "^\\Q" + p.replace("*", "\\E.*\\Q") + "\\E$"));
-            }
-        }
-
-        return ret;
-    }
-
     /*
      * This constraint is used for the complete disabling of the algorithm.
      */
