@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -57,7 +57,6 @@ import sun.security.action.GetPropertyAction;
  * <p>
  * @since 1.8
  */
-@SuppressWarnings("removal")
 public final class ZoneInfoFile {
 
     /**
@@ -249,6 +248,11 @@ public final class ZoneInfoFile {
                 .privilegedGetProperty("sun.timezone.ids.oldmapping", "false")
                 .toLowerCase(Locale.ROOT);
         USE_OLDMAPPING = (oldmapping.equals("yes") || oldmapping.equals("true"));
+        loadTZDB();
+    }
+
+    @SuppressWarnings("removal")
+    private static void loadTZDB() {
         AccessController.doPrivileged(new PrivilegedAction<Void>() {
             public Void run() {
                 try {
@@ -398,16 +402,16 @@ public final class ZoneInfoFile {
     // ZoneInfo starts with UTC1900
     private static final long UTC1900 = -2208988800L;
 
-    // ZoneInfo ends with   UTC2100
-    // LocalDateTime.of(2101, 1, 1, 0, 0, 0).toEpochSecond(ZoneOffset.UTC) - 1;
-    private static final long UTC2100 = 4133980799L;
+    // ZoneInfo ends with   UTC2037
+    // LocalDateTime.of(2038, 1, 1, 0, 0, 0).toEpochSecond(ZoneOffset.UTC) - 1;
+    private static final long UTC2037 = 2145916799L;
 
-    // ZoneInfo has an ending entry for 2100, this need to be offset by
+    // ZoneInfo has an ending entry for 2037, this need to be offset by
     // a "rawOffset"
-    // LocalDateTime.of(2100, 1, 1, 0, 0, 0).toEpochSecond(ZoneOffset.UTC);
-    private static final long LDT2100 = 4102444800L;
+    // LocalDateTime.of(2037, 1, 1, 0, 0, 0).toEpochSecond(ZoneOffset.UTC));
+    private static final long LDT2037 = 2114380800L;
 
-    //Current time. Used to determine future GMT offset transitions
+    //Current time. Used to determine future GMToffset transitions
     private static final long CURRT = System.currentTimeMillis()/1000;
 
     /* Get a ZoneInfo instance.
@@ -474,7 +478,7 @@ public final class ZoneInfoFile {
 
             for (; i < savingsInstantTransitions.length; i++) {
                 long trans = savingsInstantTransitions[i];
-                if (trans > UTC2100) {
+                if (trans > UTC2037) {
                     // no trans beyond LASTYEAR
                     lastyear = LASTYEAR;
                     break;
@@ -570,12 +574,16 @@ public final class ZoneInfoFile {
                     // ZoneRulesBuilder adjusts < 0 case (-1, for last, don't have
                     // "<=" case yet) to positive value if not February (it appears
                     // we don't have February cutoff in tzdata table yet)
-                    // Ideally, if JSR310 can just pass in the negative and
+                    // Ideally, if JSR310 can just pass in the nagative and
                     // we can then pass in the dom = -1, dow > 0 into ZoneInfo
                     //
                     // hacking, assume the >=24 is the result of ZRB optimization for
-                    // "last", it works for now.
-                    if (dom < 0 || dom >= 24) {
+                    // "last", it works for now. From tzdata2020d this hacking
+                    // will not work for Asia/Gaza and Asia/Hebron which follow
+                    // Palestine DST rules.
+                    if (dom < 0 || dom >= 24 &&
+                                   !(zoneId.equals("Asia/Gaza") ||
+                                     zoneId.equals("Asia/Hebron"))) {
                         params[1] = -1;
                         params[2] = toCalendarDOW[dow];
                     } else {
@@ -597,7 +605,9 @@ public final class ZoneInfoFile {
                     params[7] = 0;
                 } else {
                     // hacking: see comment above
-                    if (dom < 0 || dom >= 24) {
+                    if (dom < 0 || dom >= 24 &&
+                                   !(zoneId.equals("Asia/Gaza") ||
+                                     zoneId.equals("Asia/Hebron"))) {
                         params[6] = -1;
                         params[7] = toCalendarDOW[dow];
                     } else {
@@ -609,23 +619,40 @@ public final class ZoneInfoFile {
                 params[9] = toSTZTime[endRule.timeDefinition];
                 dstSavings = (startRule.offsetAfter - startRule.offsetBefore) * 1000;
 
-                // Note: known mismatching -> Africa/Cairo
+                // Note: known mismatching -> Asia/Amman
                 // ZoneInfo :      startDayOfWeek=5     <= Thursday
-                //                 startTime=86400000   <= 24:00
-                // This:           startDayOfWeek=6     <= Friday
-                //                 startTime=0          <= 0:00
-                if (zoneId.equals("Africa/Cairo") &&
-                        params[7] == Calendar.FRIDAY && params[8] == 0) {
-                    params[7] = Calendar.THURSDAY;
-                    params[8] = SECONDS_PER_DAY * 1000;
+                //                 startTime=86400000   <= 24 hours
+                // This:           startDayOfWeek=6
+                //                 startTime=0
+                // Similar workaround needs to be applied to Africa/Cairo and
+                // its endDayOfWeek and endTime
+                // Below is the workarounds, it probably slows down everyone a little
+                if (params[2] == 6 && params[3] == 0 &&
+                    (zoneId.equals("Asia/Amman"))) {
+                    params[2] = 5;
+                    params[3] = 86400000;
                 }
+                // Additional check for startDayOfWeek=6 and starTime=86400000
+                // is needed for Asia/Amman;
+                if (params[2] == 7 && params[3] == 0 &&
+                     (zoneId.equals("Asia/Amman"))) {
+                    params[2] = 6;        // Friday
+                    params[3] = 86400000; // 24h
+                }
+                //endDayOfWeek and endTime workaround
+                if (params[7] == 6 && params[8] == 0 &&
+                    (zoneId.equals("Africa/Cairo"))) {
+                    params[7] = 5;
+                    params[8] = 86400000;
+                }
+
             } else if (nTrans > 0) {  // only do this if there is something in table already
                 if (lastyear < LASTYEAR) {
-                    // ZoneInfo has an ending entry for 2100
+                    // ZoneInfo has an ending entry for 2037
                     //long trans = OffsetDateTime.of(LASTYEAR, 1, 1, 0, 0, 0, 0,
                     //                               ZoneOffset.ofTotalSeconds(rawOffset/1000))
                     //                           .toEpochSecond();
-                    long trans = LDT2100 - rawOffset/1000;
+                    long trans = LDT2037 - rawOffset/1000;
 
                     int offsetIndex = indexOf(offsets, 0, nOffsets, rawOffset/1000);
                     if (offsetIndex == nOffsets)
@@ -810,9 +837,7 @@ public final class ZoneInfoFile {
     private static final long DST_MASK = 0xf0L;
     private static final int  DST_NSHIFT = 4;
     private static final int  TRANSITION_NSHIFT = 12;
-    // The `last` year that transitions are accounted for. If there are
-    // rules that go beyond this LASTYEAR, the value needs to be expanded.
-    private static final int  LASTYEAR = 2100;
+    private static final int  LASTYEAR = 2037;
 
     // from: 0 for offset lookup, 1 for dstsvings lookup
     private static int indexOf(int[] offsets, int from, int nOffsets, int offset) {
@@ -869,12 +894,12 @@ public final class ZoneInfoFile {
     }
 
     // A simple/raw version of j.t.ZoneOffsetTransitionRule
-    // timeEndOfDay is included in secondOfDay as "86,400" secs.
     private static class ZoneOffsetTransitionRule {
         private final int month;
         private final byte dom;
         private final int dow;
         private final int secondOfDay;
+        private final boolean timeEndOfDay;
         private final int timeDefinition;
         private final int standardOffset;
         private final int offsetBefore;
@@ -892,7 +917,9 @@ public final class ZoneInfoFile {
             this.dom = (byte)(((data & (63 << 22)) >>> 22) - 32);
             this.dow = dowByte == 0 ? -1 : dowByte;
             this.secondOfDay = timeByte == 31 ? in.readInt() : timeByte * 3600;
+            this.timeEndOfDay = timeByte == 24;
             this.timeDefinition = (data & (3 << 12)) >>> 12;
+
             this.standardOffset = stdByte == 255 ? in.readInt() : (stdByte - 128) * 900;
             this.offsetBefore = beforeByte == 3 ? in.readInt() : standardOffset + beforeByte * 1800;
             this.offsetAfter = afterByte == 3 ? in.readInt() : standardOffset + afterByte * 1800;
@@ -911,6 +938,9 @@ public final class ZoneInfoFile {
                     epochDay = nextOrSame(epochDay, dow);
                 }
             }
+            if (timeEndOfDay) {
+                epochDay += 1;
+            }
             int difference = 0;
             switch (timeDefinition) {
                 case 0:    // UTC
