@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -63,19 +63,19 @@ static boolean SetupI18nProps(LCID lcid, char** language, char** script, char**
 static char *
 getEncodingInternal(LCID lcid)
 {
-    int codepage = 0;
+    int codepage;
     char * ret = malloc(16);
     if (ret == NULL) {
         return NULL;
     }
 
-    if (lcid == 0) { // for sun.jnu.encoding
-        codepage = GetACP();
-        _itoa_s(codepage, ret + 2, 14, 10);
-    } else if (GetLocaleInfo(lcid,
+    if (GetLocaleInfo(lcid,
                       LOCALE_IDEFAULTANSICODEPAGE,
-                      ret + 2, 14) != 0) {
-        codepage = atoi(ret + 2);
+                      ret+2, 14) == 0) {
+        codepage = 1252;
+        strcpy(ret+2, "1252");
+    } else {
+        codepage = atoi(ret+2);
     }
 
     switch (codepage) {
@@ -174,6 +174,10 @@ getJavaIDFromLangID(LANGID langID)
         return NULL;
     }
 
+    for (index = 0; index < 5; index++) {
+        elems[index] = NULL;
+    }
+
     if (SetupI18nProps(MAKELCID(langID, SORT_DEFAULT),
                    &(elems[0]), &(elems[1]), &(elems[2]), &(elems[3]), &(elems[4]))) {
 
@@ -187,15 +191,17 @@ getJavaIDFromLangID(LANGID langID)
                 strcat(ret, elems[index]);
             }
         }
-
-        for (index = 0; index < 5; index++) {
-            free(elems[index]);
-        }
     } else {
         free(ret);
         ret = NULL;
     }
 
+    for (index = 0; index < 5; index++) {
+        if (elems[index] != NULL) {
+            free(elems[index]);
+        }
+    }
+
     return ret;
 }
 
@@ -363,9 +369,6 @@ GetJavaProperties(JNIEnv* env)
         return &sprops;
     }
 
-    /* AWT properties */
-    sprops.awt_toolkit = "sun.awt.windows.WToolkit";
-
     /* tmp dir */
     {
         WCHAR tmpdir[MAX_PATH + 1];
@@ -374,17 +377,6 @@ GetJavaProperties(JNIEnv* env)
         sprops.tmp_dir = _wcsdup(tmpdir);
     }
 
-    /* Printing properties */
-    sprops.printerJob = "sun.awt.windows.WPrinterJob";
-
-    /* Java2D properties */
-    sprops.graphics_env = "sun.awt.Win32GraphicsEnvironment";
-
-    {    /* This is used only for debugging of font problems. */
-        WCHAR *path = _wgetenv(L"JAVA2D_FONTPATH");
-        sprops.font_dir = (path != NULL) ? _wcsdup(path) : NULL;
-    }
-
     /* OS properties */
     {
         char buf[100];
@@ -479,15 +471,11 @@ GetJavaProperties(JNIEnv* env)
          * Windows Server 2012          6               2  (!VER_NT_WORKSTATION)
          * Windows Server 2012 R2       6               3  (!VER_NT_WORKSTATION)
          * Windows 10                   10              0  (VER_NT_WORKSTATION)
-         * Windows 11                   10              0  (VER_NT_WORKSTATION)
-         *       where (buildNumber >= 22000)
          * Windows Server 2016          10              0  (!VER_NT_WORKSTATION)
          * Windows Server 2019          10              0  (!VER_NT_WORKSTATION)
          *       where (buildNumber > 17762)
          * Windows Server 2022          10              0  (!VER_NT_WORKSTATION)
          *       where (buildNumber > 20347)
-         * Windows Server 2025          10              0  (!VER_NT_WORKSTATION)
-         *       where (buildNumber > 26039)
          *
          * This mapping will presumably be augmented as new Windows
          * versions are released.
@@ -556,14 +544,7 @@ GetJavaProperties(JNIEnv* env)
             } else if (majorVersion == 10) {
                 if (is_workstation) {
                     switch (minorVersion) {
-                    case  0:
-                        /* Windows 11 21H2 (original release) build number is 22000 */
-                        if (buildNumber >= 22000) {
-                            sprops.os_name = "Windows 11";
-                        } else {
-                            sprops.os_name = "Windows 10";
-                        }
-                        break;
+                    case  0: sprops.os_name = "Windows 10";           break;
                     default: sprops.os_name = "Windows NT (unknown)";
                     }
                 } else {
@@ -571,10 +552,7 @@ GetJavaProperties(JNIEnv* env)
                     case  0:
                         /* Windows server 2019 GA 10/2018 build number is 17763 */
                         /* Windows server 2022 build number is 20348 */
-                        /* Windows server 2025 Preview build is 26040 */
-                        if (buildNumber > 26039) {
-                            sprops.os_name = "Windows Server 2025";
-                        } else if (buildNumber > 20347) {
+                        if (buildNumber > 20347) {
                             sprops.os_name = "Windows Server 2022";
                         } else if (buildNumber > 17676) {
                             sprops.os_name = "Windows Server 2019";
@@ -595,16 +573,15 @@ GetJavaProperties(JNIEnv* env)
         }
         sprintf(buf, "%d.%d", majorVersion, minorVersion);
         sprops.os_version = _strdup(buf);
-#if _M_AMD64
+#if defined(_M_AMD64)
         sprops.os_arch = "amd64";
-#elif _X86_
+#elif defined(_X86_)
         sprops.os_arch = "x86";
 #elif defined(_M_ARM64)
         sprops.os_arch = "aarch64";
 #else
         sprops.os_arch = "unknown";
 #endif
-        sprops.desktop = "windows";
     }
 
     /* Endianness of platform */
@@ -674,6 +651,7 @@ GetJavaProperties(JNIEnv* env)
          * (which is a Windows LCID value),
          */
         LCID userDefaultLCID = GetUserDefaultLCID();
+        LCID systemDefaultLCID = GetSystemDefaultLCID();
         LANGID userDefaultUILang = GetUserDefaultUILanguage();
         LCID userDefaultUILCID = MAKELCID(userDefaultUILang, SORTIDFROMLCID(userDefaultLCID));
 
@@ -693,12 +671,6 @@ GetJavaProperties(JNIEnv* env)
                 userDefaultUILCID = userDefaultLCID;
             }
 
-            SetupI18nProps(userDefaultUILang,
-                           &sprops.language,
-                           &sprops.script,
-                           &sprops.country,
-                           &sprops.variant,
-                           &display_encoding);
             SetupI18nProps(userDefaultLCID,
                            &sprops.format_language,
                            &sprops.format_script,
@@ -712,10 +684,7 @@ GetJavaProperties(JNIEnv* env)
                            &sprops.display_variant,
                            &display_encoding);
 
-            sprops.sun_jnu_encoding = getEncodingInternal(0);
-            if (sprops.sun_jnu_encoding == NULL) {
-                sprops.sun_jnu_encoding = "UTF-8";
-            }
+            sprops.sun_jnu_encoding = getEncodingInternal(systemDefaultLCID);
             if (LANGIDFROMLCID(userDefaultLCID) == 0x0c04 && majorVersion == 6) {
                 // MS claims "Vista has built-in support for HKSCS-2004.
                 // All of the HKSCS-2004 characters have Unicode 4.1.
@@ -745,17 +714,12 @@ GetJavaProperties(JNIEnv* env)
     }
 
     sprops.unicode_encoding = "UnicodeLittle";
-    /* User TIMEZONE */
-    {
-        /*
-         * We defer setting up timezone until it's actually necessary.
-         * Refer to TimeZone.getDefault(). However, the system
-         * property is necessary to be able to be set by the command
-         * line interface -D. Here temporarily set a null string to
-         * timezone.
-         */
-        sprops.timezone = "";
-    }
+
+    /* User TIMEZONE
+     * We defer setting up timezone until it's actually necessary.
+     * Refer to TimeZone.getDefault(). The system property
+     * is able to be set by the command line interface -Duser.timezone.
+     */
 
     /* Current directory */
     {
