@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -27,13 +27,11 @@ package com.sun.imageio.plugins.bmp;
 
 import java.awt.Point;
 import java.awt.Rectangle;
-import java.awt.Transparency;
 import java.awt.color.ColorSpace;
 import java.awt.color.ICC_ColorSpace;
 import java.awt.color.ICC_Profile;
 import java.awt.image.BufferedImage;
 import java.awt.image.ColorModel;
-import java.awt.image.ComponentColorModel;
 import java.awt.image.ComponentSampleModel;
 import java.awt.image.DataBuffer;
 import java.awt.image.DataBufferByte;
@@ -47,31 +45,28 @@ import java.awt.image.Raster;
 import java.awt.image.SampleModel;
 import java.awt.image.SinglePixelPackedSampleModel;
 import java.awt.image.WritableRaster;
-
-import javax.imageio.IIOException;
-import javax.imageio.ImageIO;
-import javax.imageio.ImageReader;
-import javax.imageio.ImageReadParam;
-import javax.imageio.ImageTypeSpecifier;
-import javax.imageio.metadata.IIOMetadata;
-import javax.imageio.spi.ImageReaderSpi;
-import javax.imageio.stream.ImageInputStream;
-import javax.imageio.event.IIOReadProgressListener;
-import javax.imageio.event.IIOReadUpdateListener;
-import javax.imageio.event.IIOReadWarningListener;
-
-import java.io.*;
-import java.nio.*;
+import java.io.ByteArrayInputStream;
+import java.io.IOException;
+import java.nio.ByteOrder;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
-import java.util.StringTokenizer;
+import javax.imageio.IIOException;
+import javax.imageio.ImageIO;
+import javax.imageio.ImageReadParam;
+import javax.imageio.ImageReader;
+import javax.imageio.ImageTypeSpecifier;
+import javax.imageio.event.IIOReadProgressListener;
+import javax.imageio.event.IIOReadUpdateListener;
+import javax.imageio.event.IIOReadWarningListener;
+import javax.imageio.metadata.IIOMetadata;
+import javax.imageio.spi.ImageReaderSpi;
+import javax.imageio.stream.ImageInputStream;
 
 import com.sun.imageio.plugins.common.I18N;
 import com.sun.imageio.plugins.common.ImageUtil;
-import com.sun.imageio.plugins.common.ReaderUtil;
 
 /** This class is the Java Image IO plugin reader for BMP images.
  *  It may subsample the image, clip the image, select sub-bands,
@@ -124,7 +119,7 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
     private long bitmapStart;
     private long compression;
     private long imageSize;
-    private byte palette[];
+    private byte[] palette;
     private int imageType;
     private int numBands;
     private boolean isBottomUp;
@@ -179,7 +174,7 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
         super(originator);
     }
 
-    /** Overrides the method defined in the superclass. */
+    @Override
     public void setInput(Object input,
                          boolean seekForwardOnly,
                          boolean ignoreMetadata) {
@@ -190,7 +185,7 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
         resetHeaderInfo();
     }
 
-    /** Overrides the method defined in the superclass. */
+    @Override
     public int getNumImages(boolean allowSearch) throws IOException {
         if (iis == null) {
             throw new IllegalStateException(I18N.getString("GetNumImages0"));
@@ -212,6 +207,7 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
         return width;
     }
 
+    @Override
     public int getHeight(int imageIndex) throws IOException {
         checkIndex(imageIndex);
         try {
@@ -618,10 +614,8 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
             height = Math.abs(height);
         }
 
-        if (metadata.compression == BI_RGB &&
-            metadata.paletteSize == 0 &&
-            metadata.bitsPerPixel >= 16) {
-            long imageDataSize = (((long)width * height * bitsPerPixel) / 8);
+        if (metadata.compression == BI_RGB) {
+            long imageDataSize = ((long)width * height * (bitsPerPixel / 8));
             if (imageDataSize > (bitmapFileSize - bitmapOffset)) {
                 throw new IIOException(I18N.getString("BMPImageReader9"));
             }
@@ -635,20 +629,25 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
 
             iis.mark();
             iis.skipBytes(profileData - size);
-            byte[] profile = ReaderUtil.
-                    staggeredReadByteStream(iis, profileSize);
+            byte[] profile = new byte[profileSize];
+            iis.readFully(profile, 0, profileSize);
             iis.reset();
 
-            if (metadata.colorSpace == PROFILE_LINKED &&
-                isLinkedProfileAllowed())
-            {
-                String path = new String(profile, "windows-1252");
+            try {
+                if (metadata.colorSpace == PROFILE_LINKED &&
+                    isLinkedProfileAllowed() &&
+                    !isUncOrDevicePath(profile))
+                {
+                    String path = new String(profile, "windows-1252");
 
-                colorSpace =
-                    new ICC_ColorSpace(ICC_Profile.getInstance(path));
-            } else if (metadata.colorSpace == PROFILE_EMBEDDED) {
-                colorSpace =
-                    new ICC_ColorSpace(ICC_Profile.getInstance(profile));
+                    colorSpace =
+                        new ICC_ColorSpace(ICC_Profile.getInstance(path));
+                } else {
+                    colorSpace =
+                        new ICC_ColorSpace(ICC_Profile.getInstance(profile));
+                }
+            } catch (Exception e) {
+                colorSpace = ColorSpace.getInstance(ColorSpace.CS_sRGB);
             }
         }
 
@@ -683,7 +682,7 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
             }
 
             // Create IndexColorModel from the palette.
-            byte r[], g[], b[];
+            byte[] r, g, b;
             if (imageType == VERSION_2_1_BIT ||
                 imageType == VERSION_2_4_BIT ||
                 imageType == VERSION_2_8_BIT) {
@@ -790,6 +789,7 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
         gotHeader = true;
     }
 
+    @Override
     public Iterator<ImageTypeSpecifier> getImageTypes(int imageIndex)
       throws IOException {
         checkIndex(imageIndex);
@@ -804,10 +804,12 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
         return list.iterator();
     }
 
+    @Override
     public ImageReadParam getDefaultReadParam() {
         return new ImageReadParam();
     }
 
+    @Override
     public IIOMetadata getImageMetadata(int imageIndex)
       throws IOException {
         checkIndex(imageIndex);
@@ -821,10 +823,12 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
         return metadata;
     }
 
+    @Override
     public IIOMetadata getStreamMetadata() throws IOException {
         return null;
     }
 
+    @Override
     public boolean isRandomAccessEasy(int imageIndex) throws IOException {
         checkIndex(imageIndex);
         try {
@@ -835,6 +839,7 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
         return metadata.compression == BI_RGB;
     }
 
+    @Override
     public BufferedImage read(int imageIndex, ImageReadParam param)
         throws IOException {
 
@@ -913,9 +918,9 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
             noTransform &=  destinationRegion.equals(raster.getBounds());
         }
 
-        byte bdata[] = null; // buffer for byte data
-        short sdata[] = null; // buffer for short data
-        int idata[] = null; // buffer for int data
+        byte[] bdata = null; // buffer for byte data
+        short[] sdata = null; // buffer for short data
+        int[] idata = null; // buffer for int data
 
         // the sampleModel can be null in case of embedded image
         if (sampleModel != null) {
@@ -1075,10 +1080,12 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
         return bi;
     }
 
+    @Override
     public boolean canReadRaster() {
         return true;
     }
 
+    @Override
     public Raster readRaster(int imageIndex,
                              ImageReadParam param) throws IOException {
         BufferedImage bi = read(imageIndex, param);
@@ -1092,6 +1099,7 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
         colorModel = originalColorModel = null;
     }
 
+    @Override
     public void reset() {
         super.reset();
         iis = null;
@@ -1396,7 +1404,7 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
         }
     }
 
-    private void read16Bit(short sdata[]) throws IOException {
+    private void read16Bit(short[] sdata) throws IOException {
         // Padding bytes at the end of each scanline
         // width * bitsPerPixel should be divisible by 32
         int padding = width * 2 % 4;
@@ -1462,7 +1470,7 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
         }
     }
 
-    private void read32Bit(int idata[]) throws IOException {
+    private void read32Bit(int[] idata) throws IOException {
         if (noTransform) {
             int j = isBottomUp ? (height -1) * width : 0;
 
@@ -1518,7 +1526,7 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
         }
     }
 
-    private void readRLE8(byte bdata[]) throws IOException {
+    private void readRLE8(byte[] bdata) throws IOException {
         // If imageSize field is not provided, calculate it.
         int imSize = (int)imageSize;
         if (imSize == 0) {
@@ -1534,8 +1542,9 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
         }
 
         // Read till we have the whole image
-        byte[] values = ReaderUtil.
-            staggeredReadByteStream(iis, imSize);
+        byte[] values = new byte[imSize];
+        int bytesRead = 0;
+        iis.readFully(values, 0, imSize);
 
         // Since data is compressed, decompress it
         decodeRLE8(imSize, padding, values, bdata);
@@ -1589,7 +1598,7 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
                             byte[] values,
                             byte[] bdata) throws IOException {
 
-        byte val[] = new byte[width];
+        byte[] val = new byte[width];
         int count = 0, l = 0;
         int value;
         boolean flag = false;
@@ -1717,8 +1726,8 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
         }
 
         // Read till we have the whole image
-        byte[] values = ReaderUtil.
-            staggeredReadByteStream(iis, imSize);
+        byte[] values = new byte[imSize];
+        iis.readFully(values, 0, imSize);
 
         // Decompress the RLE4 compressed data.
         decodeRLE4(imSize, padding, values, bdata);
@@ -1891,7 +1900,7 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
                 // Ensure to check if the source index-count, does not
                 // exceed the source image size
                 if (count < imSize) {
-                    int alternate[] = { (values[count] & 0xf0) >> 4,
+                    int[] alternate = { (values[count] & 0xf0) >> 4,
                                         values[count] & 0x0f };
                     for (int i=0; (i < value) && (l < width); i++) {
                         val[l++] = (byte)alternate[i & 1];
@@ -1951,6 +1960,7 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
         }
 
         reader.addIIOReadProgressListener(new EmbeddedProgressAdapter() {
+                @Override
                 public void imageProgress(ImageReader source,
                                           float percentageDone)
                 {
@@ -1959,6 +1969,7 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
             });
 
         reader.addIIOReadUpdateListener(new IIOReadUpdateListener() {
+                @Override
                 public void imageUpdate(ImageReader source,
                                         BufferedImage theImage,
                                         int minX, int minY,
@@ -1970,11 +1981,13 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
                                        width, height,
                                        periodX, periodY, bands);
                 }
+                @Override
                 public void passComplete(ImageReader source,
                                          BufferedImage theImage)
                 {
                     processPassComplete(theImage);
                 }
+                @Override
                 public void passStarted(ImageReader source,
                                         BufferedImage theImage,
                                         int pass,
@@ -1987,8 +2000,10 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
                                        minX, minY, periodX, periodY,
                                        bands);
                 }
+                @Override
                 public void thumbnailPassComplete(ImageReader source,
                                                   BufferedImage thumb) {}
+                @Override
                 public void thumbnailPassStarted(ImageReader source,
                                                  BufferedImage thumb,
                                                  int pass,
@@ -1996,6 +2011,7 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
                                                  int minX, int minY,
                                                  int periodX, int periodY,
                                                  int[] bands) {}
+                @Override
                 public void thumbnailUpdate(ImageReader source,
                                             BufferedImage theThumbnail,
                                             int minX, int minY,
@@ -2005,6 +2021,7 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
             });
 
         reader.addIIOReadWarningListener(new IIOReadWarningListener() {
+                @Override
                 public void warningOccurred(ImageReader source, String warning)
                 {
                     processWarningOccurred(warning);
@@ -2025,30 +2042,94 @@ public class BMPImageReader extends ImageReader implements BMPConstants {
         return bi;
     }
 
-    private class EmbeddedProgressAdapter implements IIOReadProgressListener {
+    private static class EmbeddedProgressAdapter implements IIOReadProgressListener {
+        @Override
         public void imageComplete(ImageReader src) {}
+        @Override
         public void imageProgress(ImageReader src, float percentageDone) {}
+        @Override
         public void imageStarted(ImageReader src, int imageIndex) {}
+        @Override
         public void thumbnailComplete(ImageReader src) {}
+        @Override
         public void thumbnailProgress(ImageReader src, float percentageDone) {}
+        @Override
         public void thumbnailStarted(ImageReader src, int iIdx, int tIdx) {}
+        @Override
         public void sequenceComplete(ImageReader src) {}
+        @Override
         public void sequenceStarted(ImageReader src, int minIndex) {}
+        @Override
         public void readAborted(ImageReader src) {}
     }
 
-    private static Boolean isLinkedProfileAllowed = null;
+    private static Boolean isLinkedProfileDisabled = null;
 
+    @SuppressWarnings("removal")
     private static boolean isLinkedProfileAllowed() {
-        if (isLinkedProfileAllowed == null) {
+        if (isLinkedProfileDisabled == null) {
             PrivilegedAction<Boolean> a = new PrivilegedAction<Boolean>() {
+                @Override
                 public Boolean run() {
-                    return Boolean.
-                        getBoolean("sun.imageio.bmp.enableLinkedProfiles");
+                    return Boolean.getBoolean("sun.imageio.plugins.bmp.disableLinkedProfiles");
                 }
             };
-            isLinkedProfileAllowed = AccessController.doPrivileged(a);
+            isLinkedProfileDisabled = AccessController.doPrivileged(a);
+        }
+        return !isLinkedProfileDisabled;
+    }
+
+    private static Boolean isWindowsPlatform = null;
+
+    /**
+     * Verifies whether the byte array contans a unc path.
+     * Non-UNC path examples:
+     *  c:\path\to\file  - simple notation
+     *  \\?\c:\path\to\file - long notation
+     *
+     * UNC path examples:
+     *  \\server\share - a UNC path in simple notation
+     *  \\?\UNC\server\share - a UNC path in long notation
+     *  \\.\some\device - a path to device.
+     */
+    @SuppressWarnings("removal")
+    private static boolean isUncOrDevicePath(byte[] p) {
+        if (isWindowsPlatform == null) {
+            PrivilegedAction<Boolean> a = new PrivilegedAction<Boolean>() {
+                @Override
+                public Boolean run() {
+                    String osname = System.getProperty("os.name");
+                    return (osname != null &&
+                            osname.toLowerCase().startsWith("win"));
+                }
+            };
+            isWindowsPlatform = AccessController.doPrivileged(a);
+        }
+
+        if (!isWindowsPlatform) {
+            /* no need for the check on platforms except windows */
+            return false;
+        }
+
+        /* normalize prefix of the path */
+        if (p[0] == '/') p[0] = '\\';
+        if (p[1] == '/') p[1] = '\\';
+        if (p[3] == '/') p[3] = '\\';
+
+
+        if ((p[0] == '\\') && (p[1] == '\\')) {
+            if ((p[2] == '?') && (p[3] == '\\')) {
+                // long path: whether unc or local
+                return ((p[4] == 'U' || p[4] == 'u') &&
+                        (p[5] == 'N' || p[5] == 'n') &&
+                        (p[6] == 'C' || p[6] == 'c'));
+            } else {
+                // device path or short unc notation
+                return true;
+            }
+        } else {
+            return false;
         }
-        return isLinkedProfileAllowed;
     }
 }
+
