@@ -680,16 +680,12 @@ public class ServiceRegistry {
      * @exception Throwable if an error occurs during superclass
      * finalization.
      *
-     * @deprecated The {@code finalize} method has been deprecated.
-     *     Subclasses that override {@code finalize} in order to perform cleanup
-     *     should be modified to use alternative cleanup mechanisms and
-     *     to remove the overriding {@code finalize} method.
-     *     When overriding the {@code finalize} method, its implementation must explicitly
-     *     ensure that {@code super.finalize()} is invoked as described in {@link Object#finalize}.
-     *     See the specification for {@link Object#finalize()} for further
-     *     information about migration options.
+     * @deprecated Finalization has been deprecated for removal.  See
+     * {@link java.lang.Object#finalize} for background information and details
+     * about migration options.
      */
-    @Deprecated(since="9")
+    @Deprecated(since="9", forRemoval=true)
+    @SuppressWarnings("removal")
     public void finalize() throws Throwable {
         deregisterAll();
         super.finalize();
@@ -842,7 +838,7 @@ class SubRegistry {
         accMap.clear();
     }
 
-    @SuppressWarnings("deprecation")
+    @SuppressWarnings("removal")
     public synchronized void finalize() {
         clear();
     }
