@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -89,7 +89,6 @@ public class BasicOptionPaneUI extends OptionPaneUI {
     public static final int MinimumHeight = 90;
 
     private static String newline;
-    private static int recursionCount;
 
     /**
      * {@code JOptionPane} that the receiver is providing the
@@ -374,7 +373,6 @@ public class BasicOptionPaneUI extends OptionPaneUI {
                       "OptionPane.messageAnchor", GridBagConstraints.CENTER);
         cons.insets = new Insets(0,0,3,0);
 
-        recursionCount = 0;
         addMessageComponents(body, cons, getMessage(),
                           getMaxCharactersPerLineCount(), false);
         top.add(realBody, BorderLayout.CENTER);
@@ -456,75 +454,42 @@ public class BasicOptionPaneUI extends OptionPaneUI {
             } else if ((nl = s.indexOf('\n')) >= 0) {
                 nll = 1;
             }
-            if (s.contains("<html>")) {
-                /* line break in html text is done by <br> tag
-                 * and not by /n so it's incorrect to address newline
-                 * same as non-html text.
-                 * Text between <html> </html> tags are extracted
-                 * and rendered as JLabel text
-                 */
-                int index1 = s.indexOf("<html>");
-                int index2 = s.indexOf("</html>");
-                String str = "";
-                if (index2 >= 0) {
-                    str = s.substring(index2 + "</html>".length());
-                    s = s.substring(index1, index2 + + "</html>".length());
+            if (nl >= 0) {
+                // break up newlines
+                if (nl == 0) {
+                    @SuppressWarnings("serial") // anonymous class
+                    JPanel breakPanel = new JPanel() {
+                        public Dimension getPreferredSize() {
+                            Font       f = getFont();
+
+                            if (f != null) {
+                                return new Dimension(1, f.getSize() + 2);
+                            }
+                            return new Dimension(0, 0);
+                        }
+                    };
+                    breakPanel.setName("OptionPane.break");
+                    addMessageComponents(container, cons, breakPanel, maxll,
+                                         true);
+                } else {
+                    addMessageComponents(container, cons, s.substring(0, nl),
+                                      maxll, false);
                 }
+                addMessageComponents(container, cons, s.substring(nl + nll), maxll,
+                                  false);
+
+            } else if (len > maxll) {
+                Container c = Box.createVerticalBox();
+                c.setName("OptionPane.verticalBox");
+                burstStringInto(c, s, maxll);
+                addMessageComponents(container, cons, c, maxll, true );
+
+            } else {
                 JLabel label;
-                label = new JLabel(s, JLabel.LEADING);
+                label = new JLabel( s, JLabel.LEADING );
                 label.setName("OptionPane.label");
                 configureMessageLabel(label);
                 addMessageComponents(container, cons, label, maxll, true);
-                if (!str.isEmpty()) {
-                    addMessageComponents(container, cons, str, maxll, false);
-                }
-            } else {
-                if (nl >= 0) {
-                    // break up newlines
-                    if (nl == 0) {
-                        @SuppressWarnings("serial") // anonymous class
-                                JPanel breakPanel = new JPanel() {
-                            public Dimension getPreferredSize() {
-                                Font f = getFont();
-
-                                if (f != null) {
-                                    return new Dimension(1, f.getSize() + 2);
-                                }
-                                return new Dimension(0, 0);
-                            }
-                        };
-                        breakPanel.setName("OptionPane.break");
-                        addMessageComponents(container, cons, breakPanel, maxll,
-                                true);
-                    } else {
-                        addMessageComponents(container, cons, s.substring(0, nl),
-                                maxll, false);
-                    }
-                    // Prevent recursion of more than
-                    // 200 successive newlines in a message
-                    // and indicate message is truncated via ellipsis
-                    if (recursionCount++ > 200) {
-                        recursionCount = 0;
-                        addMessageComponents(container, cons, new String("..."),
-                                maxll, false);
-                        return;
-                    }
-                    addMessageComponents(container, cons, s.substring(nl + nll), maxll,
-                            false);
-
-                } else if (len > maxll) {
-                    Container c = Box.createVerticalBox();
-                    c.setName("OptionPane.verticalBox");
-                    burstStringInto(c, s, maxll);
-                    addMessageComponents(container, cons, c, maxll, true);
-
-                } else {
-                    JLabel label;
-                    label = new JLabel(s, JLabel.LEADING);
-                    label.setName("OptionPane.label");
-                    configureMessageLabel(label);
-                    addMessageComponents(container, cons, label, maxll, true);
-                }
             }
         }
     }
@@ -999,13 +964,10 @@ public class BasicOptionPaneUI extends OptionPaneUI {
      * the look and feel for based on the value in the inputComponent.
      */
     protected void resetInputValue() {
-        if(inputComponent != null && (inputComponent instanceof JTextField)) {
-            optionPane.setInputValue(((JTextField)inputComponent).getText());
-
-        } else if(inputComponent != null &&
-                  (inputComponent instanceof JComboBox)) {
-            optionPane.setInputValue(((JComboBox)inputComponent)
-                                     .getSelectedItem());
+        if (inputComponent instanceof JTextField textField) {
+            optionPane.setInputValue(textField.getText());
+        } else if (inputComponent instanceof JComboBox<?> comboBox) {
+            optionPane.setInputValue(comboBox.getSelectedItem());
         } else if(inputComponent != null) {
             optionPane.setInputValue(((JList)inputComponent)
                                      .getSelectedValue());
