@@ -72,54 +72,16 @@ hb_blob_create (const char        *data,
                 void              *user_data,
                 hb_destroy_func_t  destroy)
 {
-  if (!length)
-  {
+  hb_blob_t *blob;
+
+  if (!length ||
+      length >= 1u << 31 ||
+      !(blob = hb_object_create<hb_blob_t> ())) {
     if (destroy)
       destroy (user_data);
     return hb_blob_get_empty ();
   }
 
-  hb_blob_t *blob = hb_blob_create_or_fail (data, length, mode,
-                                            user_data, destroy);
-  return likely (blob) ? blob : hb_blob_get_empty ();
-}
-
-/**
- * hb_blob_create_or_fail: (skip)
- * @data: Pointer to blob data.
- * @length: Length of @data in bytes.
- * @mode: Memory mode for @data.
- * @user_data: Data parameter to pass to @destroy.
- * @destroy: (nullable): Callback to call when @data is not needed anymore.
- *
- * Creates a new "blob" object wrapping @data.  The @mode parameter is used
- * to negotiate ownership and lifecycle of @data.
- *
- * Note that this function returns a freshly-allocated empty blob even if @length
- * is zero. This is in contrast to hb_blob_create(), which returns the singleton
- * empty blob (as returned by hb_blob_get_empty()) if @length is zero.
- *
- * Return value: New blob, or `NULL` if failed.  Destroy with hb_blob_destroy().
- *
- * Since: 2.8.2
- **/
-hb_blob_t *
-hb_blob_create_or_fail (const char        *data,
-                        unsigned int       length,
-                        hb_memory_mode_t   mode,
-                        void              *user_data,
-                        hb_destroy_func_t  destroy)
-{
-  hb_blob_t *blob;
-
-  if (length >= 1u << 31 ||
-      !(blob = hb_object_create<hb_blob_t> ()))
-  {
-    if (destroy)
-      destroy (user_data);
-    return nullptr;
-  }
-
   blob->data = data;
   blob->length = length;
   blob->mode = mode;
@@ -129,10 +91,9 @@ hb_blob_create_or_fail (const char        *data,
 
   if (blob->mode == HB_MEMORY_MODE_DUPLICATE) {
     blob->mode = HB_MEMORY_MODE_READONLY;
-    if (!blob->try_make_writable ())
-    {
+    if (!blob->try_make_writable ()) {
       hb_blob_destroy (blob);
-      return nullptr;
+      return hb_blob_get_empty ();
     }
   }
 
@@ -263,7 +224,9 @@ hb_blob_destroy (hb_blob_t *blob)
 {
   if (!hb_object_destroy (blob)) return;
 
-  hb_free (blob);
+  blob->fini_shallow ();
+
+  free (blob);
 }
 
 /**
@@ -276,7 +239,7 @@ hb_blob_destroy (hb_blob_t *blob)
  *
  * Attaches a user-data key/data pair to the specified blob.
  *
- * Return value: `true` if success, `false` otherwise
+ * Return value: %true if success, %false otherwise
  *
  * Since: 0.9.2
  **/
@@ -303,7 +266,7 @@ hb_blob_set_user_data (hb_blob_t          *blob,
  * Since: 0.9.2
  **/
 void *
-hb_blob_get_user_data (const hb_blob_t    *blob,
+hb_blob_get_user_data (hb_blob_t          *blob,
                        hb_user_data_key_t *key)
 {
   return hb_object_get_user_data (blob, key);
@@ -333,7 +296,7 @@ hb_blob_make_immutable (hb_blob_t *blob)
  *
  * Tests whether a blob is immutable.
  *
- * Return value: `true` if @blob is immutable, `false` otherwise
+ * Return value: %true if @blob is immutable, %false otherwise
  *
  * Since: 0.9.2
  **/
@@ -367,7 +330,7 @@ hb_blob_get_length (hb_blob_t *blob)
  *
  * Fetches the data from a blob.
  *
- * Returns: (nullable) (transfer none) (array length=length): the byte data of @blob.
+ * Returns: (transfer none) (array length=length): the byte data of @blob.
  *
  * Since: 0.9.2
  **/
@@ -392,7 +355,7 @@ hb_blob_get_data (hb_blob_t *blob, unsigned int *length)
  * fails.
  *
  * Returns: (transfer none) (array length=length): Writable blob data,
- * or `NULL` if failed.
+ * or %NULL if failed.
  *
  * Since: 0.9.2
  **/
@@ -489,18 +452,18 @@ hb_blob_t::try_make_writable ()
 
   char *new_data;
 
-  new_data = (char *) hb_malloc (this->length);
+  new_data = (char *) malloc (this->length);
   if (unlikely (!new_data))
     return false;
 
   DEBUG_MSG_FUNC (BLOB, this, "dupped successfully -> %p\n", this->data);
 
-  hb_memcpy (new_data, this->data, this->length);
+  memcpy (new_data, this->data, this->length);
   this->destroy_user_data ();
   this->mode = HB_MEMORY_MODE_WRITABLE;
   this->data = new_data;
   this->user_data = new_data;
-  this->destroy = hb_free;
+  this->destroy = free;
 
   return true;
 }
@@ -554,7 +517,7 @@ _hb_mapped_file_destroy (void *file_)
   assert (0); // If we don't have mmap we shouldn't reach here
 #endif
 
-  hb_free (file);
+  free (file);
 }
 #endif
 
@@ -565,15 +528,15 @@ _open_resource_fork (const char *file_name, hb_mapped_file_t *file)
   size_t name_len = strlen (file_name);
   size_t len = name_len + sizeof (_PATH_RSRCFORKSPEC);
 
-  char *rsrc_name = (char *) hb_malloc (len);
+  char *rsrc_name = (char *) malloc (len);
   if (unlikely (!rsrc_name)) return -1;
 
   strncpy (rsrc_name, file_name, name_len);
   strncpy (rsrc_name + name_len, _PATH_RSRCFORKSPEC,
-           sizeof (_PATH_RSRCFORKSPEC));
+           sizeof (_PATH_RSRCFORKSPEC) - 1);
 
   int fd = open (rsrc_name, O_RDONLY | O_BINARY, 0);
-  hb_free (rsrc_name);
+  free (rsrc_name);
 
   if (fd != -1)
   {
@@ -598,47 +561,18 @@ _open_resource_fork (const char *file_name, hb_mapped_file_t *file)
  * Creates a new blob containing the data from the
  * specified binary font file.
  *
- * The filename is passed directly to the system on all platforms,
- * except on Windows, where the filename is interpreted as UTF-8.
- * Only if the filename is not valid UTF-8, it will be interpreted
- * according to the system codepage.
- *
- * Returns: An #hb_blob_t pointer with the content of the file,
- * or hb_blob_get_empty() if failed.
+ * Returns: An #hb_blob_t pointer with the content of the file
  *
  * Since: 1.7.7
  **/
 hb_blob_t *
 hb_blob_create_from_file (const char *file_name)
-{
-  hb_blob_t *blob = hb_blob_create_from_file_or_fail (file_name);
-  return likely (blob) ? blob : hb_blob_get_empty ();
-}
-
-/**
- * hb_blob_create_from_file_or_fail:
- * @file_name: A filename
- *
- * Creates a new blob containing the data from the specified file.
- *
- * The filename is passed directly to the system on all platforms,
- * except on Windows, where the filename is interpreted as UTF-8.
- * Only if the filename is not valid UTF-8, it will be interpreted
- * according to the system codepage.
- *
- * Returns: An #hb_blob_t pointer with the content of the file,
- * or `NULL` if failed.
- *
- * Since: 2.8.2
- **/
-hb_blob_t *
-hb_blob_create_from_file_or_fail (const char *file_name)
 {
   /* Adopted from glib's gmappedfile.c with Matthias Clasen and
      Allison Lortie permission but changed a lot to suit our need. */
 #if defined(HAVE_MMAP) && !defined(HB_NO_MMAP)
-  hb_mapped_file_t *file = (hb_mapped_file_t *) hb_calloc (1, sizeof (hb_mapped_file_t));
-  if (unlikely (!file)) return nullptr;
+  hb_mapped_file_t *file = (hb_mapped_file_t *) calloc (1, sizeof (hb_mapped_file_t));
+  if (unlikely (!file)) return hb_blob_get_empty ();
 
   int fd = open (file_name, O_RDONLY | O_BINARY, 0);
   if (unlikely (fd == -1)) goto fail_without_close;
@@ -667,34 +601,25 @@ hb_blob_create_from_file_or_fail (const char *file_name)
 
   close (fd);
 
-  return hb_blob_create_or_fail (file->contents, file->length,
-                                 HB_MEMORY_MODE_READONLY_MAY_MAKE_WRITABLE, (void *) file,
-                                 (hb_destroy_func_t) _hb_mapped_file_destroy);
+  return hb_blob_create (file->contents, file->length,
+                         HB_MEMORY_MODE_READONLY_MAY_MAKE_WRITABLE, (void *) file,
+                         (hb_destroy_func_t) _hb_mapped_file_destroy);
 
 fail:
   close (fd);
 fail_without_close:
-  hb_free (file);
+  free (file);
 
 #elif defined(_WIN32) && !defined(HB_NO_MMAP)
-  hb_mapped_file_t *file = (hb_mapped_file_t *) hb_calloc (1, sizeof (hb_mapped_file_t));
-  if (unlikely (!file)) return nullptr;
+  hb_mapped_file_t *file = (hb_mapped_file_t *) calloc (1, sizeof (hb_mapped_file_t));
+  if (unlikely (!file)) return hb_blob_get_empty ();
 
   HANDLE fd;
-  int conversion;
   unsigned int size = strlen (file_name) + 1;
-  wchar_t * wchar_file_name = (wchar_t *) hb_malloc (sizeof (wchar_t) * size);
+  wchar_t * wchar_file_name = (wchar_t *) malloc (sizeof (wchar_t) * size);
   if (unlikely (!wchar_file_name)) goto fail_without_close;
-
-  /* Assume file name is given in UTF-8 encoding */
-  conversion = MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, file_name, -1, wchar_file_name, size);
-  if (conversion <= 0)
-  {
-    /* Conversion failed due to invalid UTF-8 characters,
-       Repeat conversion based on system code page */
-    mbstowcs(wchar_file_name, file_name, size);
-  }
-#if !WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP) && WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_APP)
+  mbstowcs (wchar_file_name, file_name, size);
+#if !WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
   {
     CREATEFILE2_EXTENDED_PARAMETERS ceparams = { 0 };
     ceparams.dwSize = sizeof(CREATEFILE2_EXTENDED_PARAMETERS);
@@ -711,11 +636,11 @@ fail_without_close:
                     OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL|FILE_FLAG_OVERLAPPED,
                     nullptr);
 #endif
-  hb_free (wchar_file_name);
+  free (wchar_file_name);
 
   if (unlikely (fd == INVALID_HANDLE_VALUE)) goto fail_without_close;
 
-#if !WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP) && WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_APP)
+#if !WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
   {
     LARGE_INTEGER length;
     GetFileSizeEx (fd, &length);
@@ -728,7 +653,7 @@ fail_without_close:
 #endif
   if (unlikely (!file->mapping)) goto fail;
 
-#if !WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP) && WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_APP)
+#if !WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
   file->contents = (char *) MapViewOfFileFromApp (file->mapping, FILE_MAP_READ, 0, 0);
 #else
   file->contents = (char *) MapViewOfFile (file->mapping, FILE_MAP_READ, 0, 0, 0);
@@ -736,22 +661,22 @@ fail_without_close:
   if (unlikely (!file->contents)) goto fail;
 
   CloseHandle (fd);
-  return hb_blob_create_or_fail (file->contents, file->length,
-                                 HB_MEMORY_MODE_READONLY_MAY_MAKE_WRITABLE, (void *) file,
-                                 (hb_destroy_func_t) _hb_mapped_file_destroy);
+  return hb_blob_create (file->contents, file->length,
+                         HB_MEMORY_MODE_READONLY_MAY_MAKE_WRITABLE, (void *) file,
+                         (hb_destroy_func_t) _hb_mapped_file_destroy);
 
 fail:
   CloseHandle (fd);
 fail_without_close:
-  hb_free (file);
+  free (file);
 
 #endif
 
   /* The following tries to read a file without knowing its size beforehand
      It's used as a fallback for systems without mmap or to read from pipes */
   unsigned long len = 0, allocated = BUFSIZ * 16;
-  char *data = (char *) hb_malloc (allocated);
-  if (unlikely (!data)) return nullptr;
+  char *data = (char *) malloc (allocated);
+  if (unlikely (!data)) return hb_blob_get_empty ();
 
   FILE *fp = fopen (file_name, "rb");
   if (unlikely (!fp)) goto fread_fail_without_close;
@@ -764,7 +689,7 @@ fail_without_close:
       /* Don't allocate and go more than ~536MB, our mmap reader still
          can cover files like that but lets limit our fallback reader */
       if (unlikely (allocated > (2 << 28))) goto fread_fail;
-      char *new_data = (char *) hb_realloc (data, allocated);
+      char *new_data = (char *) realloc (data, allocated);
       if (unlikely (!new_data)) goto fread_fail;
       data = new_data;
     }
@@ -781,13 +706,13 @@ fail_without_close:
   }
         fclose (fp);
 
-  return hb_blob_create_or_fail (data, len, HB_MEMORY_MODE_WRITABLE, data,
-                                 (hb_destroy_func_t) hb_free);
+  return hb_blob_create (data, len, HB_MEMORY_MODE_WRITABLE, data,
+                         (hb_destroy_func_t) free);
 
 fread_fail:
   fclose (fp);
 fread_fail_without_close:
-  hb_free (data);
-  return nullptr;
+  free (data);
+  return hb_blob_get_empty ();
 }
 #endif /* !HB_NO_OPEN */
