@@ -29,6 +29,11 @@
 #include "hb.hh"
 #include "hb-machinery.hh"
 
+#include <locale.h>
+
+#ifdef HB_NO_SETLOCALE
+#define setlocale(Category, Locale) "C"
+#endif
 
 /**
  * SECTION:hb-common
@@ -73,7 +78,7 @@ _hb_options_init ()
   }
 
   /* This is idempotent and threadsafe. */
-  _hb_options = u.i;
+  _hb_options.set_relaxed (u.i);
 }
 
 
@@ -82,7 +87,7 @@ _hb_options_init ()
 /**
  * hb_tag_from_string:
  * @str: (array length=len) (element-type uint8_t): String to convert
- * @len: Length of @str, or -1 if it is `NULL`-terminated
+ * @len: Length of @str, or -1 if it is %NULL-terminated
  *
  * Converts a string into an #hb_tag_t. Valid tags
  * are four characters. Shorter input strings will be
@@ -134,7 +139,7 @@ hb_tag_to_string (hb_tag_t tag, char *buf)
 
 /* hb_direction_t */
 
-static const char direction_strings[][4] = {
+const char direction_strings[][4] = {
   "ltr",
   "rtl",
   "ttb",
@@ -144,7 +149,7 @@ static const char direction_strings[][4] = {
 /**
  * hb_direction_from_string:
  * @str: (array length=len) (element-type uint8_t): String to convert
- * @len: Length of @str, or -1 if it is `NULL`-terminated
+ * @len: Length of @str, or -1 if it is %NULL-terminated
  *
  * Converts a string to an #hb_direction_t.
  *
@@ -252,14 +257,16 @@ struct hb_language_item_t {
   bool operator == (const char *s) const
   { return lang_equal (lang, s); }
 
-  hb_language_item_t & operator = (const char *s)
-  {
-    /* We can't call strdup(), because we allow custom allocators. */
+  hb_language_item_t & operator = (const char *s) {
+    /* If a custom allocated is used calling strdup() pairs
+    badly with a call to the custom free() in fini() below.
+    Therefore don't call strdup(), implement its behavior.
+    */
     size_t len = strlen(s) + 1;
-    lang = (hb_language_t) hb_malloc(len);
+    lang = (hb_language_t) malloc(len);
     if (likely (lang))
     {
-      hb_memcpy((unsigned char *) lang, s, len);
+      memcpy((unsigned char *) lang, s, len);
       for (unsigned char *p = (unsigned char *) lang; *p; p++)
         *p = canon_map[*p];
     }
@@ -267,15 +274,16 @@ struct hb_language_item_t {
     return *this;
   }
 
-  void fini () { hb_free ((void *) lang); }
+  void fini () { free ((void *) lang); }
 };
 
 
-/* Thread-safe lockfree language list */
+/* Thread-safe lock-free language list */
 
 static hb_atomic_ptr_t <hb_language_item_t> langs;
 
-static inline void
+#if HB_USE_ATEXIT
+static void
 free_langs ()
 {
 retry:
@@ -286,10 +294,11 @@ retry:
   while (first_lang) {
     hb_language_item_t *next = first_lang->next;
     first_lang->fini ();
-    hb_free (first_lang);
+    free (first_lang);
     first_lang = next;
   }
 }
+#endif
 
 static hb_language_item_t *
 lang_find_or_insert (const char *key)
@@ -302,26 +311,28 @@ retry:
       return lang;
 
   /* Not found; allocate one. */
-  hb_language_item_t *lang = (hb_language_item_t *) hb_calloc (1, sizeof (hb_language_item_t));
+  hb_language_item_t *lang = (hb_language_item_t *) calloc (1, sizeof (hb_language_item_t));
   if (unlikely (!lang))
     return nullptr;
   lang->next = first_lang;
   *lang = key;
   if (unlikely (!lang->lang))
   {
-    hb_free (lang);
+    free (lang);
     return nullptr;
   }
 
   if (unlikely (!langs.cmpexch (first_lang, lang)))
   {
     lang->fini ();
-    hb_free (lang);
+    free (lang);
     goto retry;
   }
 
+#if HB_USE_ATEXIT
   if (!first_lang)
-    hb_atexit (free_langs); /* First person registers atexit() callback. */
+    atexit (free_langs); /* First person registers atexit() callback. */
+#endif
 
   return lang;
 }
@@ -331,7 +342,7 @@ retry:
  * hb_language_from_string:
  * @str: (array length=len) (element-type uint8_t): a string representing
  *       a BCP 47 language tag
- * @len: length of the @str, or -1 if it is `NULL`-terminated.
+ * @len: length of the @str, or -1 if it is %NULL-terminated.
  *
  * Converts @str representing a BCP 47 language tag to the corresponding
  * #hb_language_t.
@@ -353,7 +364,7 @@ hb_language_from_string (const char *str, int len)
     /* NUL-terminate it. */
     char strbuf[64];
     len = hb_min (len, (int) sizeof (strbuf) - 1);
-    hb_memcpy (strbuf, str, len);
+    memcpy (strbuf, str, len);
     strbuf[len] = '\0';
     item = lang_find_or_insert (strbuf);
   }
@@ -370,7 +381,7 @@ hb_language_from_string (const char *str, int len)
  * Converts an #hb_language_t to a string.
  *
  * Return value: (transfer none):
- * A `NULL`-terminated string representing the @language. Must not be freed by
+ * A %NULL-terminated string representing the @language. Must not be freed by
  * the caller.
  *
  * Since: 0.9.2
@@ -408,45 +419,13 @@ hb_language_get_default ()
   hb_language_t language = default_language;
   if (unlikely (language == HB_LANGUAGE_INVALID))
   {
-    language = hb_language_from_string (hb_setlocale (LC_CTYPE, nullptr), -1);
+    language = hb_language_from_string (setlocale (LC_CTYPE, nullptr), -1);
     (void) default_language.cmpexch (HB_LANGUAGE_INVALID, language);
   }
 
   return language;
 }
 
-/**
- * hb_language_matches:
- * @language: The #hb_language_t to work on
- * @specific: Another #hb_language_t
- *
- * Check whether a second language tag is the same or a more
- * specific version of the provided language tag.  For example,
- * "fa_IR.utf8" is a more specific tag for "fa" or for "fa_IR".
- *
- * Return value: `true` if languages match, `false` otherwise.
- *
- * Since: 5.0.0
- **/
-hb_bool_t
-hb_language_matches (hb_language_t language,
-                     hb_language_t specific)
-{
-  if (language == specific) return true;
-  if (!language || !specific) return false;
-
-  const char *l = language->s;
-  const char *s = specific->s;
-  unsigned ll = strlen (l);
-  unsigned sl = strlen (s);
-
-  if (ll > sl)
-    return false;
-
-  return strncmp (l, s, ll) == 0 &&
-         (s[ll] == '\0' || s[ll] == '-');
-}
-
 
 /* hb_script_t */
 
@@ -504,7 +483,7 @@ hb_script_from_iso15924_tag (hb_tag_t tag)
  * hb_script_from_string:
  * @str: (array length=len) (element-type uint8_t): a string representing an
  *       ISO 15924 tag.
- * @len: length of the @str, or -1 if it is `NULL`-terminated.
+ * @len: length of the @str, or -1 if it is %NULL-terminated.
  *
  * Converts a string @str representing an ISO 15924 script tag to a
  * corresponding #hb_script_t. Shorthand for hb_tag_from_string() then
@@ -622,9 +601,6 @@ hb_script_get_horizontal_direction (hb_script_t script)
     case HB_SCRIPT_CHORASMIAN:
     case HB_SCRIPT_YEZIDI:
 
-    /* Unicode-14.0 additions */
-    case HB_SCRIPT_OLD_UYGHUR:
-
       return HB_DIRECTION_RTL;
 
 
@@ -632,7 +608,6 @@ hb_script_get_horizontal_direction (hb_script_t script)
     case HB_SCRIPT_OLD_HUNGARIAN:
     case HB_SCRIPT_OLD_ITALIC:
     case HB_SCRIPT_RUNIC:
-    case HB_SCRIPT_TIFINAGH:
 
       return HB_DIRECTION_INVALID;
   }
@@ -700,8 +675,8 @@ hb_version_string ()
  * Tests the library version against a minimum value,
  * as three integer components.
  *
- * Return value: `true` if the library is equal to or greater than
- * the test value, `false` otherwise
+ * Return value: %true if the library is equal to or greater than
+ * the test value, %false otherwise
  *
  * Since: 0.9.30
  **/
@@ -815,7 +790,7 @@ parse_tag (const char **pp, const char *end, hb_tag_t *tag)
   }
 
   const char *p = *pp;
-  while (*pp < end && (**pp != ' ' && **pp != '=' && **pp != '[' && **pp != quote))
+  while (*pp < end && (ISALNUM(**pp) || **pp == '_'))
     (*pp)++;
 
   if (p == *pp || *pp - p > 4)
@@ -888,7 +863,7 @@ parse_one_feature (const char **pp, const char *end, hb_feature_t *feature)
 /**
  * hb_feature_from_string:
  * @str: (array length=len) (element-type uint8_t): a string to parse
- * @len: length of @str, or -1 if string is `NULL` terminated
+ * @len: length of @str, or -1 if string is %NULL terminated
  * @feature: (out): the #hb_feature_t to initialize with the parsed values
  *
  * Parses a string into a #hb_feature_t.
@@ -930,7 +905,7 @@ parse_one_feature (const char **pp, const char *end, hb_feature_t *feature)
  * </informaltable>
  *
  * Return value:
- * `true` if @str is successfully parsed, `false` otherwise
+ * %true if @str is successfully parsed, %false otherwise
  *
  * Since: 0.9.5
  **/
@@ -951,7 +926,7 @@ hb_feature_from_string (const char *str, int len,
   }
 
   if (feature)
-    hb_memset (feature, 0, sizeof (*feature));
+    memset (feature, 0, sizeof (*feature));
   return false;
 }
 
@@ -961,7 +936,7 @@ hb_feature_from_string (const char *str, int len,
  * @buf: (array length=size) (out): output string
  * @size: the allocated size of @buf
  *
- * Converts a #hb_feature_t into a `NULL`-terminated string in the format
+ * Converts a #hb_feature_t into a %NULL-terminated string in the format
  * understood by hb_feature_from_string(). The client in responsible for
  * allocating big enough size for @buf, 128 bytes is more than enough.
  *
@@ -1000,7 +975,7 @@ hb_feature_to_string (hb_feature_t *feature,
   }
   assert (len < ARRAY_LENGTH (s));
   len = hb_min (len, size - 1);
-  hb_memcpy (buf, s, len);
+  memcpy (buf, s, len);
   buf[len] = '\0';
 }
 
@@ -1029,7 +1004,7 @@ parse_one_variation (const char **pp, const char *end, hb_variation_t *variation
 /**
  * hb_variation_from_string:
  * @str: (array length=len) (element-type uint8_t): a string to parse
- * @len: length of @str, or -1 if string is `NULL` terminated
+ * @len: length of @str, or -1 if string is %NULL terminated
  * @variation: (out): the #hb_variation_t to initialize with the parsed values
  *
  * Parses a string into a #hb_variation_t.
@@ -1042,7 +1017,7 @@ parse_one_variation (const char **pp, const char *end, hb_variation_t *variation
  * number. For example `wght=500`, or `slnt=-7.5`.
  *
  * Return value:
- * `true` if @str is successfully parsed, `false` otherwise
+ * %true if @str is successfully parsed, %false otherwise
  *
  * Since: 1.4.2
  */
@@ -1063,58 +1038,17 @@ hb_variation_from_string (const char *str, int len,
   }
 
   if (variation)
-    hb_memset (variation, 0, sizeof (*variation));
+    memset (variation, 0, sizeof (*variation));
   return false;
 }
 
-#ifndef HB_NO_SETLOCALE
-
-static inline void free_static_C_locale ();
-
-static struct hb_C_locale_lazy_loader_t : hb_lazy_loader_t<hb_remove_pointer<hb_locale_t>,
-                                                           hb_C_locale_lazy_loader_t>
-{
-  static hb_locale_t create ()
-  {
-    hb_locale_t l = newlocale (LC_ALL_MASK, "C", NULL);
-    if (!l)
-      return l;
-
-    hb_atexit (free_static_C_locale);
-
-    return l;
-  }
-  static void destroy (hb_locale_t l)
-  {
-    freelocale (l);
-  }
-  static hb_locale_t get_null ()
-  {
-    return (hb_locale_t) 0;
-  }
-} static_C_locale;
-
-static inline
-void free_static_C_locale ()
-{
-  static_C_locale.free_instance ();
-}
-
-static hb_locale_t
-get_C_locale ()
-{
-  return static_C_locale.get_unconst ();
-}
-
-#endif
-
 /**
  * hb_variation_to_string:
  * @variation: an #hb_variation_t to convert
- * @buf: (array length=size) (out caller-allocates): output string
+ * @buf: (array length=size) (out): output string
  * @size: the allocated size of @buf
  *
- * Converts an #hb_variation_t into a `NULL`-terminated string in the format
+ * Converts an #hb_variation_t into a %NULL-terminated string in the format
  * understood by hb_variation_from_string(). The client in responsible for
  * allocating big enough size for @buf, 128 bytes is more than enough.
  *
@@ -1133,15 +1067,11 @@ hb_variation_to_string (hb_variation_t *variation,
   while (len && s[len - 1] == ' ')
     len--;
   s[len++] = '=';
-
-  hb_locale_t oldlocale HB_UNUSED;
-  oldlocale = hb_uselocale (get_C_locale ());
   len += hb_max (0, snprintf (s + len, ARRAY_LENGTH (s) - len, "%g", (double) variation->value));
-  (void) hb_uselocale (oldlocale);
 
   assert (len < ARRAY_LENGTH (s));
   len = hb_min (len, size - 1);
-  hb_memcpy (buf, s, len);
+  memcpy (buf, s, len);
   buf[len] = '\0';
 }
 
