@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2005, 2014, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -70,12 +70,10 @@ typedef struct tagBitmapheader  {
 
 jfieldID AwtTrayIcon::idID;
 jfieldID AwtTrayIcon::actionCommandID;
-jmethodID AwtTrayIcon::updateImageID;
 
 HWND AwtTrayIcon::sm_msgWindow = NULL;
 AwtTrayIcon::TrayIconListItem* AwtTrayIcon::sm_trayIconList = NULL;
 int AwtTrayIcon::sm_instCount = 0;
-bool AwtTrayIcon::m_bDPIChanged = false;
 
 /************************************************************************
  * AwtTrayIcon methods
@@ -223,18 +221,6 @@ void AwtTrayIcon::InitNID(UINT uID)
     m_nid.uVersion = NOTIFYICON_VERSION;
 }
 
-// Call updateImage() method on the peer when screen scale changes
-void AwtTrayIcon::UpdateImage()
-{
-    JNIEnv *env =(JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
-
-    jobject peer = GetPeer(env);
-    if (peer != NULL) {
-        env->CallVoidMethod(peer, updateImageID);
-        env->ExceptionClear();
-    }
-}
-
 BOOL AwtTrayIcon::SendTrayMessage(DWORD dwMessage)
 {
     return Shell_NotifyIcon(dwMessage, (PNOTIFYICONDATA)&m_nid);
@@ -262,10 +248,6 @@ LRESULT CALLBACK AwtTrayIcon::TrayWindowProc(HWND hwnd, UINT uMsg, WPARAM wParam
                 }
             }
             break;
-        case WM_DISPLAYCHANGE:
-            // Set the flag to update icon images, see WmTaskbarCreated
-            m_bDPIChanged = true;
-            break;
         default:
             if(uMsg == s_msgTaskbarCreated) {
                 if (hwnd == AwtTrayIcon::sm_msgWindow) {
@@ -492,13 +474,12 @@ MsgRouting AwtTrayIcon::WmContextMenu(UINT flags, int x, int y)
 MsgRouting AwtTrayIcon::WmTaskbarCreated() {
     TrayIconListItem* item;
     for (item = sm_trayIconList; item != NULL; item = item->m_next) {
-        if (m_bDPIChanged) {
-            // Update the icon image
-            item->m_trayIcon->UpdateImage();
+        BOOL result = item->m_trayIcon->SendTrayMessage(NIM_ADD);
+        // 6270114: Instructs the taskbar to behave according to the Shell version 5.0
+        if (result) {
+            item->m_trayIcon->SendTrayMessage(NIM_SETVERSION);
         }
-        item->m_trayIcon->AddTrayIcon();
     }
-    m_bDPIChanged = false;
     return mrDoDefault;
 }
 
@@ -744,7 +725,7 @@ void AwtTrayIcon::SetToolTip(LPCTSTR tooltip)
         _tcscpy_s(m_nid.szTip, TRAY_ICON_TOOLTIP_MAX_SIZE, tooltip);
     }
 
-    ModifyTrayIcon();
+    SendTrayMessage(NIM_MODIFY);
 }
 
 void AwtTrayIcon::_SetToolTip(void *param)
@@ -812,10 +793,10 @@ void AwtTrayIcon::_UpdateIcon(void *param)
     JNI_CHECK_PEER_GOTO(self, ret);
     trayIcon = (AwtTrayIcon *)pData;
 
-    if (jupdate == JNI_TRUE) {
-        trayIcon->ModifyTrayIcon();
-    } else {
-        trayIcon->AddTrayIcon();
+    BOOL result = trayIcon->SendTrayMessage(jupdate == JNI_TRUE ? NIM_MODIFY : NIM_ADD);
+    // 6270114: Instructs the taskbar to behave according to the Shell version 5.0
+    if (result && jupdate == JNI_FALSE) {
+        trayIcon->SendTrayMessage(NIM_SETVERSION);
     }
 ret:
     env->DeleteGlobalRef(self);
@@ -864,7 +845,7 @@ void AwtTrayIcon::DisplayMessage(LPCTSTR caption, LPCTSTR text, LPCTSTR msgType)
         _tcscpy_s(m_nid.szInfo, TRAY_ICON_BALLOON_INFO_MAX_SIZE, text);
     }
 
-    ModifyTrayIcon();
+    SendTrayMessage(NIM_MODIFY);
     m_nid.uFlags &= ~NIF_INFO;
 }
 
@@ -936,14 +917,6 @@ Java_java_awt_TrayIcon_initIDs(JNIEnv *env, jclass cls)
     DASSERT(AwtTrayIcon::actionCommandID != NULL);
     CHECK_NULL( AwtTrayIcon::actionCommandID);
 
-    jclass wPeerCls = env->FindClass("sun/awt/windows/WTrayIconPeer");
-    DASSERT(wPeerCls != NULL);
-    CHECK_NULL(wPeerCls);
-
-    AwtTrayIcon::updateImageID = env->GetMethodID(wPeerCls, "updateImage", "()V");
-    DASSERT(AwtTrayIcon::updateImageID != NULL);
-    CHECK_NULL(AwtTrayIcon::updateImageID);
-
     CATCH_BAD_ALLOC;
 }
 
