@@ -36,14 +36,12 @@ import java.lang.ref.WeakReference;
 import java.net.Authenticator;
 import java.net.ConnectException;
 import java.net.CookieHandler;
-import java.net.ProtocolException;
 import java.net.ProxySelector;
 import java.net.http.HttpConnectTimeoutException;
 import java.net.http.HttpTimeoutException;
 import java.nio.ByteBuffer;
 import java.nio.channels.CancelledKeyException;
 import java.nio.channels.ClosedChannelException;
-import java.nio.channels.ClosedSelectorException;
 import java.nio.channels.SelectableChannel;
 import java.nio.channels.SelectionKey;
 import java.nio.channels.Selector;
@@ -58,26 +56,19 @@ import java.time.temporal.ChronoUnit;
 import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.Iterator;
-import java.util.LinkedList;
 import java.util.List;
 import java.util.Objects;
 import java.util.Optional;
 import java.util.Set;
 import java.util.TreeSet;
 import java.util.concurrent.CompletableFuture;
-import java.util.concurrent.ConcurrentSkipListSet;
-import java.util.concurrent.CompletionException;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Executor;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
-import java.util.concurrent.RejectedExecutionException;
 import java.util.concurrent.ThreadFactory;
-import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLong;
-import java.util.concurrent.atomic.AtomicReference;
-import java.util.function.BiConsumer;
 import java.util.function.BooleanSupplier;
 import java.util.stream.Stream;
 import java.net.http.HttpClient;
@@ -86,12 +77,9 @@ import java.net.http.HttpResponse;
 import java.net.http.HttpResponse.BodyHandler;
 import java.net.http.HttpResponse.PushPromiseHandler;
 import java.net.http.WebSocket;
-
 import jdk.internal.net.http.common.BufferSupplier;
-import jdk.internal.net.http.common.HttpBodySubscriberWrapper;
 import jdk.internal.net.http.common.Log;
 import jdk.internal.net.http.common.Logger;
-import jdk.internal.net.http.common.MinimalFuture;
 import jdk.internal.net.http.common.Pair;
 import jdk.internal.net.http.common.Utils;
 import jdk.internal.net.http.common.OperationTrackers.Trackable;
@@ -112,7 +100,6 @@ final class HttpClientImpl extends HttpClient implements Trackable {
     final Logger debugelapsed = Utils.getDebugLogger(this::dbgString, DEBUGELAPSED);
     final Logger debugtimeout = Utils.getDebugLogger(this::dbgString, DEBUGTIMEOUT);
     static final AtomicLong CLIENT_IDS = new AtomicLong();
-    private final AtomicLong CONNECTION_IDS = new AtomicLong();
 
     // Define the default factory as a static inner class
     // that embeds all the necessary logic to avoid
@@ -148,42 +135,27 @@ final class HttpClientImpl extends HttpClient implements Trackable {
      * is the SelectorManager thread. If the current thread is not
      * the selector manager thread the given task is executed inline.
      */
-    static final class DelegatingExecutor implements Executor {
+    final static class DelegatingExecutor implements Executor {
         private final BooleanSupplier isInSelectorThread;
         private final Executor delegate;
-        private final BiConsumer<Runnable, Throwable> errorHandler;
-        DelegatingExecutor(BooleanSupplier isInSelectorThread,
-                           Executor delegate,
-                           BiConsumer<Runnable, Throwable> errorHandler) {
+        DelegatingExecutor(BooleanSupplier isInSelectorThread, Executor delegate) {
             this.isInSelectorThread = isInSelectorThread;
             this.delegate = delegate;
-            this.errorHandler = errorHandler;
         }
 
         Executor delegate() {
             return delegate;
         }
 
-
-
         @Override
         public void execute(Runnable command) {
             if (isInSelectorThread.getAsBoolean()) {
-                ensureExecutedAsync(command);
+                delegate.execute(command);
             } else {
                 command.run();
             }
         }
 
-        public void ensureExecutedAsync(Runnable command) {
-            try {
-                delegate.execute(command);
-            } catch (Throwable t) {
-                errorHandler.accept(command, t);
-                ASYNC_POOL.execute(command);
-            }
-        }
-
         @SuppressWarnings("removal")
         private void shutdown() {
             if (delegate instanceof ExecutorService service) {
@@ -195,131 +167,7 @@ final class HttpClientImpl extends HttpClient implements Trackable {
                         new RuntimePermission("modifyThread"));
             }
         }
-    }
 
-    // We maintain a list of pending requests that will be aborted if ever
-    // the selector manager thread exists abnormally.
-    // The request, its id, and its completable future, are stored in a record-like
-    // PendingRequest object added to the pending requests set (pendingRequests).
-    //
-    // When the request's cf completes, either normally or abnormally, a dependent action
-    // will remove the PendingRequest object from the pending requests set.
-    // If the SelectorManager threads exits abnormally, all pending requests in the
-    // pending requests set will be completed exceptionally, in the ASYNC_POOL.
-    //
-    // HttpClientImpl::registerPending(id, req, cf, client) is called from sendAsync
-    // to register the pending request in the pending requests set before returning
-    // the completable future to the caller.
-    //
-    // HttpClientImpl::abortPendingRequests(client, Throwable reason) is called from
-    // the SelectorManager when a throwable is caught just before exiting.
-    //
-    // A dependent action is registered with the pending request's cf to make sure
-    // that the pending request will be removed from the pending requests set if,
-    // or after, the cf is completed.
-    //
-    private final Set<PendingRequest> pendingRequests;
-    // an id to ensure total order of pending request objects
-    private final AtomicLong pendingRequestId = new AtomicLong();
-    private static final class PendingRequest implements Comparable<PendingRequest> {
-        final long id;
-        final HttpRequest request;
-        final CompletableFuture<?> cf;
-        final HttpClientImpl client;
-        final MultiExchange<?> mex;
-        Object ref;
-        private PendingRequest(long id,
-                               HttpRequest request,
-                               CompletableFuture<?> cf,
-                               MultiExchange<?> mex,
-                               HttpClientImpl client) {
-            this.id = id;
-            this.request = request;
-            this.cf = cf;
-            this.mex = mex;
-            this.client = client;
-        }
-
-        public void abort(Throwable t) {
-            try {
-                if (client.isSelectorThread()) {
-                    var done = cf.exceptionally((e) -> null);
-                    ASYNC_POOL.execute(() -> completeExceptionally(t));
-                    // special case for when this method is called in the SelectorManager thread:
-                    // we want to wait until all futures are completed before proceeding to
-                    // shutdown. This ensures that the caller receive the actual `reason`
-                    // and not something like "HTTP/2 client closed"...
-                    done.join();
-                } else {
-                    cf.completeExceptionally(t);
-                }
-            } finally {
-                mex.cancel(Utils.getIOException(t));
-            }
-        }
-
-        private void completeExceptionally(Throwable t) {
-            if (client.debug.on()) {
-                client.debug.log("aborting %s with %s", this, t);
-            }
-            try { cf.completeExceptionally(t); } catch (Throwable e) {
-                client.debug.log("Failed to complete cf for [%s]: %s", this, e);
-            }
-        }
-
-
-        @Override
-        public int compareTo(PendingRequest o) {
-            if (o == null) return 1;
-            return Long.compare(id, o.id);
-        }
-
-        public String toString() {
-            return id + ": " + request.toString();
-        }
-    }
-
-    static <T> CompletableFuture<T> registerPending(PendingRequest pending, CompletableFuture<T> res) {
-        // shortcut if cf is already completed: no need to go through the trouble of
-        //    registering it
-        if (pending.cf.isDone()) return res;
-
-        var client = pending.client;
-        var cf = pending.cf;
-        var id = pending.id;
-        boolean added = client.pendingRequests.add(pending);
-        // this may immediately remove `pending` from the set is the cf is already completed
-        var ref = res.whenComplete((r,t) -> client.pendingRequests.remove(pending));
-        pending.ref = ref;
-        assert added : "request %d was already added".formatted(id);
-        // should not happen, unless the selector manager has already
-        // exited abnormally
-        if (client.selmgr.isClosed()) {
-            pending.abort(client.selmgr.selectorClosedException());
-        }
-        return ref;
-    }
-
-    static void abortPendingRequests(HttpClientImpl client, Throwable reason) {
-        reason = Utils.getCompletionCause(reason);
-        if (client.debug.on()) {
-            var msg = reason instanceof RejectedExecutionException
-                    ? reason.getClass() : reason;
-            client.debug.log("aborting pending requests due to: %s", msg);
-        }
-        closeSubscribers(client, reason);
-        var pendingRequests = client.pendingRequests;
-        while (!pendingRequests.isEmpty()) {
-            var pendings = pendingRequests.iterator();
-            while (pendings.hasNext()) {
-                var pending = pendings.next();
-                try {
-                    pending.abort(reason);
-                } finally {
-                    pendings.remove();
-                }
-            }
-        }
     }
 
     private final CookieHandler cookieHandler;
@@ -357,12 +205,6 @@ final class HttpClientImpl extends HttpClient implements Trackable {
     // nature of the API, we also need to wait until all pending operations
     // have completed.
     private final WeakReference<HttpClientFacade> facadeRef;
-    private final WeakReference<HttpClientImpl> implRef;
-
-    private final ConcurrentSkipListSet<PlainHttpConnection> openedConnections
-            = new ConcurrentSkipListSet<>(HttpConnection.COMPARE_BY_ID);
-    private final ConcurrentSkipListSet<HttpBodySubscriberWrapper<?>> subscribers
-            = new ConcurrentSkipListSet<>(HttpBodySubscriberWrapper.COMPARE_BY_ID);
 
     // This counter keeps track of the number of operations pending
     // on the HttpClient. The SelectorManager thread will wait
@@ -395,13 +237,8 @@ final class HttpClientImpl extends HttpClient implements Trackable {
     // the response has been fully received or the web socket is closed.
     private final AtomicLong pendingOperationCount = new AtomicLong();
     private final AtomicLong pendingWebSocketCount = new AtomicLong();
-    private final AtomicLong pendingHttpOperationsCount = new AtomicLong();
     private final AtomicLong pendingHttpRequestCount = new AtomicLong();
     private final AtomicLong pendingHttp2StreamCount = new AtomicLong();
-    private final AtomicLong pendingTCPConnectionCount = new AtomicLong();
-    private final AtomicLong pendingSubscribersCount = new AtomicLong();
-    private final AtomicBoolean isAlive = new AtomicBoolean();
-    private final AtomicBoolean isStarted = new AtomicBoolean();
 
     /** A Set of, deadline first, ordered timeout events. */
     private final TreeSet<TimeoutEvent> timeouts;
@@ -455,11 +292,8 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         } else {
             isDefaultExecutor = false;
         }
-        pendingRequests = new ConcurrentSkipListSet<>();
-        delegatingExecutor = new DelegatingExecutor(this::isSelectorThread, ex,
-                this::onSubmitFailure);
+        delegatingExecutor = new DelegatingExecutor(this::isSelectorThread, ex);
         facadeRef = new WeakReference<>(facadeFactory.createFacade(this));
-        implRef = new WeakReference<>(this);
         client2 = new Http2ClientImpl(this);
         cookieHandler = builder.cookieHandler;
         connectTimeout = builder.connectTimeout;
@@ -497,24 +331,8 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         assert facadeRef.get() != null;
     }
 
-    // called when the facade is GC'ed.
-    // Just wakes up the selector to cleanup...
-    void facadeCleanup() {
-        SelectorManager selmgr = this.selmgr;
-        if (selmgr != null) selmgr.wakeupSelector();
-    }
-
-    void onSubmitFailure(Runnable command, Throwable failure) {
-        selmgr.abort(failure);
-    }
-
     private void start() {
-        try {
-            selmgr.start();
-        } catch (Throwable t) {
-            isStarted.set(true);
-            throw t;
-        }
+        selmgr.start();
     }
 
     // Called from the SelectorManager thread, just before exiting.
@@ -526,74 +344,10 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         connections.stop();
         // Clears HTTP/2 cache and close its connections.
         client2.stop();
-        // make sure all subscribers are completed
-        closeSubscribers();
-        // close TCP connection if any are still opened
-        openedConnections.forEach(this::closeConnection);
         // shutdown the executor if needed
         if (isDefaultExecutor) delegatingExecutor.shutdown();
     }
 
-    private void closeSubscribers() {
-        if (subscribers.isEmpty()) return;
-        IOException io = selmgr.selectorClosedException();
-        closeSubscribers(this, io);
-    }
-
-    private static void closeSubscribers(HttpClientImpl client, Throwable t) {
-        client.subscribers.forEach(s -> s.onError(t));
-    }
-
-    /**
-     * Adds the given subscriber to the subscribers list, or call
-     * its {@linkplain HttpBodySubscriberWrapper#onError onError}
-     * method if the client is shutting down.
-     * @param subscriber the subscriber
-     * @return true if the subscriber was added to the list.
-     */
-    public boolean registerSubscriber(HttpBodySubscriberWrapper<?> subscriber) {
-        if (!selmgr.isClosed()) {
-            synchronized (selmgr) {
-                if (!selmgr.isClosed()) {
-                    if (subscribers.add(subscriber)) {
-                        long count = pendingSubscribersCount.incrementAndGet();
-                        if (debug.on()) {
-                            debug.log("body subscriber registered: " + count);
-                        }
-                        return true;
-                    }
-                    return false;
-                }
-            }
-        }
-        subscriber.onError(selmgr.selectorClosedException());
-        return false;
-    }
-
-    /**
-     * Remove the given subscriber from the subscribers list.
-     * @param subscriber the subscriber
-     * @return true if the subscriber was found and removed from the list.
-     */
-    public boolean unregisterSubscriber(HttpBodySubscriberWrapper<?> subscriber) {
-        if (subscribers.remove(subscriber)) {
-            long count = pendingSubscribersCount.decrementAndGet();
-            if (debug.on()) {
-                debug.log("body subscriber unregistered: " + count);
-            }
-            return true;
-        }
-        return false;
-    }
-
-    private void closeConnection(HttpConnection conn) {
-        try { conn.close(); } catch (Throwable e) {
-            if (Log.channel()) {
-                Log.logChannel("Failed to close connection: " + e);
-            }
-        }
-    }
-
     private static SSLParameters getDefaultParams(SSLContext ctx) {
         SSLParameters params = ctx.getDefaultSSLParameters();
         return params;
@@ -611,49 +365,19 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         return facadeRef.get();
     }
 
-    public long newConnectionId() {
-        return CONNECTION_IDS.incrementAndGet();
-    }
-
-    // Increments the pendingTCPConnectionCount
-    public void connectionOpened(PlainHttpConnection plainHttpConnection) {
-        if (openedConnections.add(plainHttpConnection)) {
-            pendingTCPConnectionCount.incrementAndGet();
-        }
-    }
-
-    // Decrements the pendingTCPConnectionCount
-    public void connectionClosed(PlainHttpConnection plainHttpConnection) {
-        if (openedConnections.remove(plainHttpConnection)) {
-            pendingTCPConnectionCount.decrementAndGet();
-        }
-    }
-
-    // Increments the pendingHttpRequestCount and pendingOperationCount.
-    final long requestReference() {
-        pendingHttpRequestCount.incrementAndGet();
-        return reference();
-    }
-
-    // Decrements the pendingHttpRequestCount and pendingOperationCount.
-    final long requestUnreference() {
-        pendingHttpRequestCount.decrementAndGet();
-        return unreference();
-    }
-
-    // Increments the pendingHttpOperationsCount and pendingOperationCount.
+    // Increments the pendingOperationCount.
     final long reference() {
-        pendingHttpOperationsCount.incrementAndGet();
+        pendingHttpRequestCount.incrementAndGet();
         return pendingOperationCount.incrementAndGet();
     }
 
-    // Decrements the pendingHttpOperationsCount and pendingOperationCount.
+    // Decrements the pendingOperationCount.
     final long unreference() {
         final long count = pendingOperationCount.decrementAndGet();
-        final long httpCount = pendingHttpOperationsCount.decrementAndGet();
+        final long httpCount = pendingHttpRequestCount.decrementAndGet();
         final long http2Count = pendingHttp2StreamCount.get();
         final long webSocketCount = pendingWebSocketCount.get();
-        if (count == 0 && facadeRef.refersTo(null)) {
+        if (count == 0 && facade() == null) {
             selmgr.wakeupSelector();
         }
         assert httpCount >= 0 : "count of HTTP/1.1 operations < 0";
@@ -663,19 +387,19 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         return count;
     }
 
-    // Increments the pendingHttp2StreamCount and pendingOperationCount.
+    // Increments the pendingOperationCount.
     final long streamReference() {
         pendingHttp2StreamCount.incrementAndGet();
         return pendingOperationCount.incrementAndGet();
     }
 
-    // Decrements the pendingHttp2StreamCount and pendingOperationCount.
+    // Decrements the pendingOperationCount.
     final long streamUnreference() {
         final long count = pendingOperationCount.decrementAndGet();
         final long http2Count = pendingHttp2StreamCount.decrementAndGet();
-        final long httpCount = pendingHttpOperationsCount.get();
+        final long httpCount = pendingHttpRequestCount.get();
         final long webSocketCount = pendingWebSocketCount.get();
-        if (count == 0 && facadeRef.refersTo(null)) {
+        if (count == 0 && facade() == null) {
             selmgr.wakeupSelector();
         }
         assert httpCount >= 0 : "count of HTTP/1.1 operations < 0";
@@ -685,19 +409,19 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         return count;
     }
 
-    // Increments the pendingWebSocketCount and pendingOperationCount.
+    // Increments the pendingOperationCount.
     final long webSocketOpen() {
         pendingWebSocketCount.incrementAndGet();
         return pendingOperationCount.incrementAndGet();
     }
 
-    // Decrements the pendingWebSocketCount and pendingOperationCount.
+    // Decrements the pendingOperationCount.
     final long webSocketClose() {
         final long count = pendingOperationCount.decrementAndGet();
         final long webSocketCount = pendingWebSocketCount.decrementAndGet();
-        final long httpCount = pendingHttpOperationsCount.get();
+        final long httpCount = pendingHttpRequestCount.get();
         final long http2Count = pendingHttp2StreamCount.get();
-        if (count == 0 && facadeRef.refersTo(null)) {
+        if (count == 0 && facade() == null) {
             selmgr.wakeupSelector();
         }
         assert httpCount >= 0 : "count of HTTP/1.1 operations < 0";
@@ -708,67 +432,35 @@ final class HttpClientImpl extends HttpClient implements Trackable {
     }
 
     // Returns the pendingOperationCount.
-    // Incremented with any operation, whether it's HTTP/1.1, HTTP/2, or WebSocket
     final long referenceCount() {
         return pendingOperationCount.get();
     }
 
-    // Trackers are used in test to verify that an instance of
-    // HttpClient has shutdown correctly, and that all operations
-    // have terminated.
-    static final class HttpClientTracker implements Tracker {
-        final AtomicLong requestCount;
+    final static class HttpClientTracker implements Tracker {
         final AtomicLong httpCount;
         final AtomicLong http2Count;
         final AtomicLong websocketCount;
         final AtomicLong operationsCount;
-        final AtomicLong connnectionsCount;
-        final AtomicLong subscribersCount;
         final Reference<?> reference;
-        final Reference<?> implRef;
-        final AtomicBoolean isAlive;
-        final AtomicBoolean isStarted;
         final String name;
-        HttpClientTracker(AtomicLong request,
-                          AtomicLong http,
+        HttpClientTracker(AtomicLong http,
                           AtomicLong http2,
                           AtomicLong ws,
                           AtomicLong ops,
-                          AtomicLong conns,
-                          AtomicLong subscribers,
                           Reference<?> ref,
-                          Reference<?> implRef,
-                          AtomicBoolean isAlive,
-                          AtomicBoolean isStarted,
                           String name) {
-            this.requestCount = request;
             this.httpCount = http;
             this.http2Count = http2;
             this.websocketCount = ws;
             this.operationsCount = ops;
-            this.connnectionsCount = conns;
-            this.subscribersCount = subscribers;
             this.reference = ref;
-            this.implRef = implRef;
-            this.isAlive = isAlive;
-            this.isStarted = isStarted;
             this.name = name;
         }
         @Override
-        public long getOutstandingSubscribers() {
-            return subscribersCount.get();
-        }
-        @Override
         public long getOutstandingOperations() {
             return operationsCount.get();
         }
         @Override
-        public long getOutstandingHttpRequests() {
-            return requestCount.get();
-        }
-        @Override
-        public long getOutstandingTcpConnections() { return connnectionsCount.get();}
-        @Override
         public long getOutstandingHttpOperations() {
             return httpCount.get();
         }
@@ -780,14 +472,8 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         }
         @Override
         public boolean isFacadeReferenced() {
-            return !reference.refersTo(null);
+            return reference.get() != null;
         }
-        public boolean isImplementationReferenced() {
-            return !implRef.refersTo(null);
-        }
-        // The selector is considered alive if it's not yet started
-        @Override
-        public boolean isSelectorAlive() { return isAlive.get() || !isStarted.get(); }
         @Override
         public String getName() {
             return name;
@@ -795,25 +481,19 @@ final class HttpClientImpl extends HttpClient implements Trackable {
     }
 
     public Tracker getOperationsTracker() {
-        return new HttpClientTracker(
-                pendingHttpRequestCount,
-                pendingHttpOperationsCount,
+        return new HttpClientTracker(pendingHttpRequestCount,
                 pendingHttp2StreamCount,
                 pendingWebSocketCount,
                 pendingOperationCount,
-                pendingTCPConnectionCount,
-                pendingSubscribersCount,
                 facadeRef,
-                implRef,
-                isAlive,
-                isStarted,
                 dbgTag);
     }
 
     // Called by the SelectorManager thread to figure out whether it's time
     // to terminate.
-    boolean isReferenced() {
-        return !facadeRef.refersTo(null) || referenceCount() > 0;
+    final boolean isReferenced() {
+        HttpClient facade = facade();
+        return facade != null || referenceCount() > 0;
     }
 
     /**
@@ -843,14 +523,6 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         return Thread.currentThread() == selmgr;
     }
 
-    boolean isSelectorClosed() {
-        return selmgr.isClosed();
-    }
-
-    IOException selectorClosedException() {
-        return selmgr.selectorClosedException();
-    }
-
     Http2ClientImpl client2() {
         return client2;
     }
@@ -878,9 +550,8 @@ final class HttpClientImpl extends HttpClient implements Trackable {
             cf = sendAsync(req, responseHandler, null, null);
             return cf.get();
         } catch (InterruptedException ie) {
-            if (cf != null) {
+            if (cf != null )
                 cf.cancel(true);
-            }
             throw ie;
         } catch (ExecutionException e) {
             final Throwable throwable = e.getCause();
@@ -909,10 +580,6 @@ final class HttpClientImpl extends HttpClient implements Trackable {
                 // any other SSLException is wrapped in a plain
                 // SSLException
                 throw new SSLException(msg, throwable);
-            } else if (throwable instanceof ProtocolException) {
-                ProtocolException pe = new ProtocolException(msg);
-                pe.initCause(throwable);
-                throw pe;
             } else if (throwable instanceof IOException) {
                 throw new IOException(msg, throwable);
             } else {
@@ -948,12 +615,6 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         Objects.requireNonNull(userRequest);
         Objects.requireNonNull(responseHandler);
 
-        // should not happen, unless the selector manager has
-        // exited abnormally
-        if (selmgr.isClosed()) {
-            return MinimalFuture.failedFuture(selmgr.selectorClosedException());
-        }
-
         AccessControlContext acc = null;
         if (System.getSecurityManager() != null)
             acc = AccessController.getContext();
@@ -963,9 +624,8 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         if (requestImpl.method().equals("CONNECT"))
             throw new IllegalArgumentException("Unsupported method CONNECT");
 
-        long id = pendingRequestId.incrementAndGet();
         long start = DEBUGELAPSED ? System.nanoTime() : 0;
-        requestReference();
+        reference();
         try {
             if (debugelapsed.on())
                 debugelapsed.log("ClientImpl (async) send %s", userRequest);
@@ -988,38 +648,29 @@ final class HttpClientImpl extends HttpClient implements Trackable {
                                                             responseHandler,
                                                             pushPromiseHandler,
                                                             acc);
-            CompletableFuture<HttpResponse<T>> mexCf = mex.responseAsync(executor);
-            CompletableFuture<HttpResponse<T>> res = mexCf.whenComplete((b,t) -> requestUnreference());
+            CompletableFuture<HttpResponse<T>> res =
+                    mex.responseAsync(executor).whenComplete((b,t) -> unreference());
             if (DEBUGELAPSED) {
                 res = res.whenComplete(
                         (b,t) -> debugCompleted("ClientImpl (async)", start, userRequest));
             }
 
-            // The mexCf is the Cf we need to abort if the SelectorManager thread
-            // is aborted.
-            PendingRequest pending = new PendingRequest(id, requestImpl, mexCf, mex, this);
-            res = registerPending(pending, res);
-
+            // makes sure that any dependent actions happen in the CF default
+            // executor. This is only needed for sendAsync(...), when
+            // exchangeExecutor is non-null.
             if (exchangeExecutor != null) {
-                // makes sure that any dependent actions happen in the CF default
-                // executor. This is only needed for sendAsync(...), when
-                // exchangeExecutor is non-null.
-                return res.isDone() ? res
-                        : res.whenCompleteAsync((r, t) -> { /* do nothing */}, ASYNC_POOL);
-            } else {
-                // make a defensive copy that can be safely canceled
-                // by the caller
-                return res.isDone() ? res : res.copy();
+                res = res.whenCompleteAsync((r, t) -> { /* do nothing */}, ASYNC_POOL);
             }
-        } catch (Throwable t) {
-            requestUnreference();
+            return res;
+        } catch(Throwable t) {
+            unreference();
             debugCompleted("ClientImpl (async)", start, userRequest);
             throw t;
         }
     }
 
     // Main loop for this client's selector
-    private static final class SelectorManager extends Thread {
+    private final static class SelectorManager extends Thread {
 
         // For testing purposes we have an internal System property that
         // can control the frequency at which the selector manager will wake
@@ -1052,9 +703,8 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         private final List<AsyncTriggerEvent> deregistrations;
         private final Logger debug;
         private final Logger debugtimeout;
-        private final HttpClientImpl owner;
-        private final ConnectionPool pool;
-        private final AtomicReference<Throwable> errorRef = new AtomicReference<>();
+        HttpClientImpl owner;
+        ConnectionPool pool;
 
         SelectorManager(HttpClientImpl ref) throws IOException {
             super(null, null,
@@ -1069,22 +719,13 @@ final class HttpClientImpl extends HttpClient implements Trackable {
             selector = Selector.open();
         }
 
-        IOException selectorClosedException() {
-            var io = new IOException("selector manager closed");
-            var cause = errorRef.get();
-            if (cause != null) {
-                io.initCause(cause);
-            }
-            return io;
-        }
-
         void eventUpdated(AsyncEvent e) throws ClosedChannelException {
             if (Thread.currentThread() == this) {
                 SelectionKey key = e.channel().keyFor(selector);
                 if (key != null && key.isValid()) {
                     SelectorAttachment sa = (SelectorAttachment) key.attachment();
                     sa.register(e);
-                } else if (e.interestOps() != 0) {
+                } else if (e.interestOps() != 0){
                     // We don't care about paused events.
                     // These are actually handled by
                     // SelectorAttachment::resetInterestOps later on.
@@ -1100,98 +741,43 @@ final class HttpClientImpl extends HttpClient implements Trackable {
 
         // This returns immediately. So caller not allowed to send/receive
         // on connection.
-        void register(AsyncEvent e) {
-            var closed = this.closed;
-            if (!closed) {
-                synchronized (this) {
-                    closed = this.closed;
-                    if (!closed) {
-                        registrations.add(e);
-                    }
-                }
-            }
-            if (closed) {
-                e.abort(selectorClosedException());
-            } else {
-                selector.wakeup();
+        synchronized void register(AsyncEvent e) {
+            registrations.add(e);
+            selector.wakeup();
+        }
+
+        synchronized void cancel(SocketChannel e) {
+            SelectionKey key = e.keyFor(selector);
+            if (key != null) {
+                key.cancel();
             }
+            selector.wakeup();
         }
 
         void wakeupSelector() {
             selector.wakeup();
         }
 
-        void abort(Throwable t) {
-            boolean closed = this.closed;
-            errorRef.compareAndSet(null, t);
-            if (debug.on()) {
-                debug.log("aborting selector manager(closed=%s): " + t, closed);
-            }
-            t = errorRef.get();
-            boolean inSelectorThread = owner.isSelectorThread();
-            if (!inSelectorThread) {
-                // abort anything pending, then close
-                abortPendingRequests(owner, t);
-            }
-            Set<SelectionKey> keys = new HashSet<>();
-            Set<AsyncEvent> toAbort = new HashSet<>();
-            synchronized (this) {
-                if (closed = this.closed) return;
-                this.closed = true;
-                try {
-                    keys.addAll(selector.keys());
-                } catch (ClosedSelectorException ce) {
-                    // OK - nothing to do...
-                }
-                toAbort.addAll(this.registrations);
-                toAbort.addAll(this.deregistrations);
-                this.registrations.clear();
-                this.deregistrations.clear();
-            }
-            // double check after closing
-            abortPendingRequests(owner, t);
-
-            IOException io = toAbort.isEmpty()
-                    ? null : selectorClosedException();
-            for (AsyncEvent e : toAbort) {
-                try {
-                    e.abort(io);
-                } catch (Throwable x) {
-                    debug.log("Failed to abort event: " + x);
-                }
-            }
-            if (!inSelectorThread) selector.wakeup();
-        }
-
-        // Only called by the selector manager thread
-        private void shutdown() {
+        synchronized void shutdown() {
+            Log.logTrace("{0}: shutting down", getName());
+            if (debug.on()) debug.log("SelectorManager shutting down");
+            closed = true;
             try {
-                synchronized (this) {
-                    Log.logTrace("{0}: shutting down", getName());
-                    if (debug.on()) debug.log("SelectorManager shutting down");
-                    closed = true;
-                    selector.close();
-                }
+                selector.close();
             } catch (IOException ignored) {
             } finally {
                 owner.stop();
             }
         }
 
-        boolean isClosed() {
-            return closed;
-        }
-
         @Override
         public void run() {
             List<Pair<AsyncEvent,IOException>> errorList = new ArrayList<>();
             List<AsyncEvent> readyList = new ArrayList<>();
             List<Runnable> resetList = new ArrayList<>();
-            owner.isAlive.set(true);   // goes back to false when run exits
-            owner.isStarted.set(true); // never goes back to false
             try {
                 if (Log.channel()) Log.logChannel(getName() + ": starting");
-                while (!Thread.currentThread().isInterrupted() && !closed) {
+                while (!Thread.currentThread().isInterrupted()) {
                     synchronized (this) {
                         assert errorList.isEmpty();
                         assert readyList.isEmpty();
@@ -1224,7 +810,7 @@ final class HttpClientImpl extends HttpClient implements Trackable {
                                 // may throw IOE if channel closed: that's OK
                                 sa.register(event);
                                 if (!chan.isOpen()) {
-                                    throw new ClosedChannelException();
+                                    throw new IOException("Channel closed");
                                 }
                             } catch (IOException e) {
                                 Log.logTrace("{0}: {1}", getName(), e);
@@ -1343,8 +929,7 @@ final class HttpClientImpl extends HttpClient implements Trackable {
                     selector.selectedKeys().clear();
 
                     // handle selected events
-                    IOException ioe = closed ? selectorClosedException() : null;
-                    readyList.forEach((e) -> handleEvent(e, ioe));
+                    readyList.forEach((e) -> handleEvent(e, null));
                     readyList.clear();
 
                     // handle errors (closed channels etc...)
@@ -1357,26 +942,19 @@ final class HttpClientImpl extends HttpClient implements Trackable {
 
                 }
             } catch (Throwable e) {
-                errorRef.compareAndSet(null, e);
                 if (!closed) {
-                    closed = true; // set closed early so that new requests are rejected
                     // This terminates thread. So, better just print stack trace
                     String err = Utils.stackTrace(e);
                     Log.logError("{0}: {1}: {2}", getName(),
                             "HttpClientImpl shutting down due to fatal error", err);
                 }
-                abortPendingRequests(owner, selectorClosedException());
                 if (debug.on()) debug.log("shutting down", e);
                 if (Utils.ASSERTIONSENABLED && !debug.on()) {
                     e.printStackTrace(System.err); // always print the stack
                 }
             } finally {
                 if (Log.channel()) Log.logChannel(getName() + ": stopping");
-                try {
-                    shutdown();
-                } finally {
-                    owner.isAlive.set(false);
-                }
+                shutdown();
             }
         }
 
@@ -1393,10 +971,7 @@ final class HttpClientImpl extends HttpClient implements Trackable {
 
         /** Handles the given event. The given ioe may be null. */
         void handleEvent(AsyncEvent event, IOException ioe) {
-            if (ioe == null && closed) {
-                ioe = selectorClosedException();
-            }
-            if (ioe != null) {
+            if (closed || ioe != null) {
                 event.abort(ioe);
             } else {
                 event.handle();
@@ -1432,7 +1007,7 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         private final SelectableChannel chan;
         private final Selector selector;
         private final Set<AsyncEvent> pending;
-        private static final Logger debug =
+        private final static Logger debug =
                 Utils.getDebugLogger("SelectorAttachment"::toString, Utils.DEBUG);
         private int interestOps;
 
@@ -1629,7 +1204,7 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         filters.addFilter(f);
     }
 
-    final LinkedList<HeaderFilter> filterChain() {
+    final List<HeaderFilter> filterChain() {
         return filters.getFilterChain();
     }
 
