@@ -86,7 +86,7 @@ public class KeyTab implements KeyTabConstants {
     /**
      * Constructs a KeyTab object.
      *
-     * If there is any I/O error or format error during the loading, the
+     * If there is any I/O error or format errot during the loading, the
      * isValid flag is set to false, and all half-read entries are dismissed.
      * @param filename path name for the keytab file, must not be null
      */
@@ -101,9 +101,15 @@ public class KeyTab implements KeyTabConstants {
         } catch (FileNotFoundException e) {
             entries.clear();
             isMissing = true;
+            if (DEBUG) {
+                System.out.println("Cannot load keytab " + tabName + ": " + e);
+            }
         } catch (Exception ioe) {
             entries.clear();
             isValid = false;
+            if (DEBUG) {
+                System.out.println("Cannot load keytab " + tabName + ": " + ioe);
+            }
         }
     }
 
