@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2021, Oracle and/or its affiliates. All rights reserved.
  */
 /*
  * Licensed to the Apache Software Foundation (ASF) under one or more
@@ -20,6 +20,7 @@
 
 package com.sun.org.apache.xalan.internal.xsltc.trax;
 
+import com.sun.org.apache.xalan.internal.utils.XMLSecurityManager;
 import com.sun.org.apache.xalan.internal.xsltc.DOM;
 import com.sun.org.apache.xalan.internal.xsltc.DOMCache;
 import com.sun.org.apache.xalan.internal.xsltc.StripFilter;
@@ -88,7 +89,6 @@ import jdk.xml.internal.JdkXmlFeatures;
 import jdk.xml.internal.JdkXmlUtils;
 import jdk.xml.internal.JdkProperty.ImplPropMap;
 import jdk.xml.internal.JdkProperty.State;
-import jdk.xml.internal.XMLSecurityManager;
 import jdk.xml.internal.SecuritySupport;
 import jdk.xml.internal.TransformErrorListener;
 import org.xml.sax.ContentHandler;
@@ -101,7 +101,7 @@ import org.xml.sax.ext.LexicalHandler;
  * @author Morten Jorgensen
  * @author G. Todd Miller
  * @author Santiago Pericas-Geertsen
- * @LastModified: Jan 2022
+ * @LastModified: Sept 2021
  */
 public final class TransformerImpl extends Transformer
     implements DOMCache
@@ -288,8 +288,10 @@ public final class TransformerImpl extends Transformer
             _translet.setMessageHandler(new MessageHandler(_errorListener));
         }
         _properties = createOutputProperties(outputProperties);
+        String isStandalone = SecuritySupport.getJAXPSystemProperty(
+                String.class, SP_XSLTC_IS_STANDALONE, "no");
         _xsltcIsStandalone = new JdkProperty<>(ImplPropMap.XSLTCISSTANDALONE,
-                String.class, "no", State.DEFAULT);
+                isStandalone, State.DEFAULT);
         _propertiesClone = (Properties) _properties.clone();
         _indentNumber = indentNumber;
         _tfactory = tfactory;
