@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2016, the original author(s).
+ * Copyright (c) 2002-2016, the original author or authors.
  *
  * This software is distributable under the BSD license. See the terms of the
  * BSD license in the documentation provided with this software.
@@ -17,7 +17,8 @@ public final class NativeSignalHandler implements SignalHandler {
 
     public static final NativeSignalHandler SIG_IGN = new NativeSignalHandler();
 
-    private NativeSignalHandler() {}
+    private NativeSignalHandler() {
+    }
 
     public void handle(Signal signal) {
         throw new UnsupportedOperationException();
