@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2007, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -39,6 +39,8 @@
  * If the ei field is non-zero, then one of the possible
  * co-located events has been posted and the other fields describe
  * the event's location.
+ *
+ * See comment above deferEventReport() for an explanation of co-located events.
  */
 typedef struct CoLocatedEventInfo_ {
     EventIndex ei;
@@ -63,28 +65,31 @@ typedef struct CoLocatedEventInfo_ {
  */
 typedef struct ThreadNode {
     jthread thread;
-    unsigned int toBeResumed : 1;
-    unsigned int pendingInterrupt : 1;
-    unsigned int isDebugThread : 1;
-    unsigned int suspendOnStart : 1;
-    unsigned int isStarted : 1;
+    unsigned int toBeResumed : 1;      /* true if this thread was successfully suspended. */
+    unsigned int pendingInterrupt : 1; /* true if thread is interrupted while handling an event. */
+    unsigned int isDebugThread : 1;    /* true if this is one of our debug agent threads. */
+    unsigned int suspendOnStart : 1;   /* true for new threads if we are currently in a VM.suspend(). */
+    unsigned int isStarted : 1;        /* THREAD_START or VIRTUAL_THREAD_SCHEDULED event received. */
     unsigned int popFrameEvent : 1;
     unsigned int popFrameProceed : 1;
     unsigned int popFrameThread : 1;
-    unsigned int handlingAppResume : 1;
-    EventIndex current_ei;
-    jobject pendingStop;
+    EventIndex current_ei; /* Used to determine if we are currently handling an event on this thread. */
+    jobject pendingStop;   /* Object we are throwing to stop the thread (ThreadReferenceImpl.stop). */
     jint suspendCount;
     jint resumeFrameDepth; /* !=0 => This thread is in a call to Thread.resume() */
     jvmtiEventMode instructionStepMode;
     StepRequest currentStep;
     InvokeRequest currentInvoke;
-    struct bag *eventBag;
-    CoLocatedEventInfo cleInfo;
+    struct bag *eventBag;       /* Accumulation of JDWP events to be sent as a reply. */
+    CoLocatedEventInfo cleInfo; /* See comment above deferEventReport() for an explanation. */
     struct ThreadNode *next;
     struct ThreadNode *prev;
-    jlong frameGeneration;
-    struct ThreadList *list;  /* Tells us what list this thread is in */
+    jlong frameGeneration;    /* used to generate a unique frameID. Incremented whenever existing frameID
+                                 needs to be invalidated, such as when the thread is resumed. */
+    struct ThreadList *list;  /* Tells us what list this thread is in. */
+#ifdef DEBUG_THREADNAME
+    char name[256];
+#endif
 } ThreadNode;
 
 static jint suspendAllCount;
@@ -138,6 +143,11 @@ typedef struct {
 
 static DeferredEventModeList deferredEventModes;
 
+#ifdef DEBUG
+static void dumpThreadList(ThreadList *list);
+static void dumpThread(ThreadNode *node);
+#endif
+
 static jint
 getStackDepth(jthread thread)
 {
@@ -169,8 +179,8 @@ setThreadLocalStorage(jthread thread, ThreadNode *node)
 
     error = JVMTI_FUNC_PTR(gdata->jvmti,SetThreadLocalStorage)
             (gdata->jvmti, thread, (void*)node);
-    if ( error == JVMTI_ERROR_THREAD_NOT_ALIVE ) {
-        /* Just return, thread hasn't started yet */
+    if ( error == JVMTI_ERROR_THREAD_NOT_ALIVE && node == NULL) {
+        /* Just return. This can happen when clearing the TLS. */
         return;
     } else if ( error != JVMTI_ERROR_NONE ) {
         /* The jthread object must be valid, so this must be a fatal error */
@@ -221,7 +231,10 @@ nonTlsSearch(JNIEnv *env, ThreadList *list, jthread thread)
 /*
  * These functions maintain the linked list of currently running threads.
  * All assume that the threadLock is held before calling.
- * If list==NULL, search both lists.
+ */
+
+/*
+ * Search for a thread on the list. If list==NULL, search all lists.
  */
 static ThreadNode *
 findThread(ThreadList *list, jthread thread)
@@ -231,24 +244,47 @@ findThread(ThreadList *list, jthread thread)
     /* Get thread local storage for quick thread -> node access */
     node = getThreadLocalStorage(thread);
 
-    /* In some rare cases we might get NULL, so we check the list manually for
-     *   any threads that we could match.
-     */
     if ( node == NULL ) {
-        JNIEnv *env;
-
-        env = getEnv();
-        if ( list != NULL ) {
-            node = nonTlsSearch(env, list, thread);
-        } else {
-            node = nonTlsSearch(env, &runningThreads, thread);
-            if ( node == NULL ) {
-                node = nonTlsSearch(env, &otherThreads, thread);
-            }
+        /*
+         * If the thread was not yet started when the ThreadNode was created, then it
+         * got added to the otherThreads list and its thread local storage was not set.
+         * Search for it in the otherThreads list.
+         */
+        if ( list == NULL || list == &otherThreads ) {
+            node = nonTlsSearch(getEnv(), &otherThreads, thread);
         }
-        if ( node != NULL ) {
-            /* Here we make another attempt to set TLS, it's ok if this fails */
-            setThreadLocalStorage(thread, (void*)node);
+        /*
+         * Normally we can assume that a thread with no TLS will never be in the runningThreads
+         * list. This is because we always set the TLS when adding to runningThreads.
+         * However, when a thread exits, its TLS is automatically cleared. Normally this
+         * is not a problem because the debug agent will first get a THREAD_END event,
+         * and that will cause the thread to be removed from runningThreads, thus we
+         * avoid this situation of having a thread in runningThreads, but with no TLS.
+         *
+         * However... there is one exception to this. While handling VM_DEATH, the first thing
+         * the debug agent does is clear all the callbacks. This means we will no longer
+         * get THREAD_END events as threads exit. This means we might find threads on
+         * runningThreads with no TLS during VM_DEATH. Essentially the THREAD_END that
+         * would normally have resulted in removing the thread from runningThreads is
+         * missed, so the thread remains on runningThreads.
+         *
+         * The end result of all this is that if the TLS lookup failed, we still need to check
+         * if the thread is on runningThreads, but only if JVMTI callbacks have been cleared.
+         * Otherwise the thread should not be on the runningThreads.
+         */
+        if ( !gdata->jvmtiCallBacksCleared ) {
+            /* The thread better not be on runningThreads if the TLS lookup failed. */
+            JDI_ASSERT(!nonTlsSearch(getEnv(), &runningThreads, thread));
+        } else {
+            /*
+             * Search the runningThreads list. The TLS lookup may have failed because the
+             * thread has terminated, but we never got the THREAD_END event.
+             */
+            if ( node == NULL ) {
+                if ( list == NULL || list == &runningThreads ) {
+                    node = nonTlsSearch(getEnv(), &runningThreads, thread);
+                }
+            }
         }
     }
 
@@ -350,11 +386,31 @@ insertThread(JNIEnv *env, ThreadList *list, jthread thread)
         node->eventBag = eventBag;
         addNode(list, node);
 
+#ifdef DEBUG_THREADNAME
+        {
+            /* Set the thread name */
+            jvmtiThreadInfo info;
+            jvmtiError error;
+
+            memset(&info, 0, sizeof(info));
+            error = JVMTI_FUNC_PTR(gdata->jvmti,GetThreadInfo)
+                    (gdata->jvmti, node->thread, &info);
+            if (info.name != NULL) {
+                strncpy(node->name, info.name, sizeof(node->name) - 1);
+                jvmtiDeallocate(info.name);
+            }
+        }
+#endif
+
         /* Set thread local storage for quick thread -> node access.
-         *   Some threads may not be in a state that allows setting of TLS,
-         *   which is ok, see findThread, it deals with threads without TLS set.
+         *   Threads that are not yet started do not allow setting of TLS. These
+         *   threads go on the otherThreads list and have their TLS set
+         *   when moved to the runningThreads list. findThread() knows to look
+         *   on otherThreads when the TLS lookup fails.
          */
-        setThreadLocalStorage(node->thread, (void*)node);
+        if (list != &otherThreads) {
+            setThreadLocalStorage(node->thread, (void*)node);
+        }
     }
 
     return node;
@@ -623,10 +679,7 @@ pendingAppResume(jboolean includeSuspended)
                 if (error != JVMTI_ERROR_NONE) {
                     EXIT_ERROR(error, "getting thread state");
                 }
-                /* !node->handlingAppResume && resumeFrameDepth > 0
-                 * means the thread has entered Thread.resume() */
-                if (!(state & JVMTI_THREAD_STATE_SUSPENDED) &&
-                    !node->handlingAppResume) {
+                if (!(state & JVMTI_THREAD_STATE_SUSPENDED)) {
                     return JNI_TRUE;
                 }
             }
@@ -652,6 +705,10 @@ notifyAppResumeComplete(void)
     }
 }
 
+/*
+ * Event handler for FRAME_POP and EXCEPTION_CATCH when in Thread.resume()
+ * so we can detect its completion.
+ */
 static void
 handleAppResumeCompletion(JNIEnv *env, EventInfo *evinfo,
                           HandlerNode *handlerNode,
@@ -695,11 +752,6 @@ blockOnDebuggerSuspend(jthread thread)
     }
 }
 
-/*
- * The caller is expected to hold threadLock and handlerLock.
- * eventHandler_createInternalThreadOnly() can deadlock because of
- * wrong lock ordering if the caller does not hold handlerLock.
- */
 static void
 trackAppResume(jthread thread)
 {
@@ -741,25 +793,35 @@ trackAppResume(jthread thread)
     }
 }
 
+/* Global breakpoint handler for Thread.resume() */
 static void
 handleAppResumeBreakpoint(JNIEnv *env, EventInfo *evinfo,
                           HandlerNode *handlerNode,
                           struct bag *eventBag)
 {
     jthread resumer = evinfo->thread;
+    jthread resumee = getResumee(resumer);
 
     debugMonitorEnter(threadLock);
+    if (resumee != NULL) {
+        /*
+         * Hold up any attempt to resume as long as the debugger
+         * has suspended the resumee.
+         */
+        blockOnDebuggerSuspend(resumee);
+    }
 
-    /*
-     * Actual handling has to be deferred. We cannot block right here if the
-     * target of the resume call is suspended by the debugger since we are
-     * holding handlerLock which must not be released. See doPendingTasks().
-     */
     if (resumer != NULL) {
-        ThreadNode* node = findThread(&runningThreads, resumer);
-        if (node != NULL) {
-            node->handlingAppResume = JNI_TRUE;
-        }
+        /*
+         * Track the resuming thread by marking it as being within
+         * a resume and by setting up for notification on
+         * a frame pop or exception. We won't allow the debugger
+         * to suspend threads while any thread is within a
+         * call to resume. This (along with the block above)
+         * ensures that when the debugger
+         * suspends a thread it will remain suspended.
+         */
+        trackAppResume(resumer);
     }
 
     debugMonitorExit(threadLock);
@@ -836,6 +898,7 @@ threadControl_onHook(void)
                  */
                 node->isStarted = JNI_TRUE;
             }
+            jvmtiDeallocate(threads);
         }
 
     } END_WITH_LOCAL_REFS(env)
@@ -902,7 +965,7 @@ deferredSuspendThreadByNode(ThreadNode *node)
          * happens when suspendOnStart is set to true.
          */
         if (error != JVMTI_ERROR_NONE) {
-          node->suspendCount--;
+            node->suspendCount--;
         }
     }
 
@@ -1055,6 +1118,9 @@ commonSuspend(JNIEnv *env, jthread thread, jboolean deferred)
      * to a separate list of threads so that we'll resume it later.
      */
     node = findThread(&runningThreads, thread);
+#if 0
+    tty_message("commonSuspend: node(%p) suspendCount(%d) %s", node, node->suspendCount, node->name);
+#endif
     if (node == NULL) {
         node = insertThread(env, &otherThreads, thread);
     }
@@ -1362,7 +1428,6 @@ commonSuspendList(JNIEnv *env, jint initCount, jthread *initList)
     return error;
 }
 
-
 static jvmtiError
 commonResume(jthread thread)
 {
@@ -1374,6 +1439,9 @@ commonResume(jthread thread)
      * not, check the auxiliary list used by threadControl_suspendThread.
      */
     node = findThread(NULL, thread);
+#if 0
+    tty_message("commonResume: node(%p) suspendCount(%d) %s", node, node->suspendCount, node->name);
+#endif
 
     /*
      * If the node is in neither list, the debugger never suspended
@@ -1495,6 +1563,9 @@ threadControl_suspendAll(void)
 {
     jvmtiError error;
     JNIEnv    *env;
+#if 0
+    tty_message("threadControl_suspendAll: suspendAllCount(%d)", suspendAllCount);
+#endif
 
     env = getEnv();
 
@@ -1515,22 +1586,9 @@ threadControl_suspendAll(void)
             error = AGENT_ERROR_OUT_OF_MEMORY;
             goto err;
         }
-        if (canSuspendResumeThreadLists()) {
-            error = commonSuspendList(env, count, threads);
-            if (error != JVMTI_ERROR_NONE) {
-                goto err;
-            }
-        } else {
-
-            int i;
-
-            for (i = 0; i < count; i++) {
-                error = commonSuspend(env, threads[i], JNI_FALSE);
-
-                if (error != JVMTI_ERROR_NONE) {
-                    goto err;
-                }
-            }
+        error = commonSuspendList(env, count, threads);
+        if (error != JVMTI_ERROR_NONE) {
+            goto err;
         }
 
         /*
@@ -1546,10 +1604,17 @@ threadControl_suspendAll(void)
         }
 
         if (error == JVMTI_ERROR_NONE) {
+            /*
+             * Pin all objects to prevent objects from being
+             * garbage collected while the VM is suspended.
+             */
+            commonRef_pinAll();
+
             suspendAllCount++;
         }
 
-    err: ;
+    err:
+        jvmtiDeallocate(threads);
 
     } END_WITH_LOCAL_REFS(env)
 
@@ -1574,6 +1639,9 @@ threadControl_resumeAll(void)
 {
     jvmtiError error;
     JNIEnv    *env;
+#if 0
+    tty_message("threadControl_resumeAll: suspendAllCount(%d)", suspendAllCount);
+#endif
 
     env = getEnv();
 
@@ -1588,12 +1656,7 @@ threadControl_resumeAll(void)
      * no need to get the whole thread list from JVMTI (unlike
      * suspendAll).
      */
-    if (canSuspendResumeThreadLists()) {
-        error = commonResumeList(env);
-    } else {
-        error = enumerateOverThreadList(env, &runningThreads,
-                                        resumeHelper, NULL);
-    }
+    error = commonResumeList(env);
     if ((error == JVMTI_ERROR_NONE) && (otherThreads.first != NULL)) {
         error = enumerateOverThreadList(env, &otherThreads,
                                         resumeHelper, NULL);
@@ -1601,6 +1664,11 @@ threadControl_resumeAll(void)
     }
 
     if (suspendAllCount > 0) {
+        /*
+         * Unpin all objects.
+         */
+        commonRef_unpinAll();
+
         suspendAllCount--;
     }
 
@@ -2031,13 +2099,15 @@ checkForPopFrameEvents(JNIEnv *env, EventIndex ei, jthread thread)
 }
 
 struct bag *
-threadControl_onEventHandlerEntry(jbyte sessionID, EventIndex ei, jthread thread, jobject currentException)
+threadControl_onEventHandlerEntry(jbyte sessionID, EventInfo *evinfo, jobject currentException)
 {
     ThreadNode *node;
     JNIEnv     *env;
     struct bag *eventBag;
     jthread     threadToSuspend;
     jboolean    consumed;
+    EventIndex  ei = evinfo->ei;
+    jthread     thread = evinfo->thread;
 
     env             = getEnv();
     threadToSuspend = NULL;
@@ -2068,6 +2138,8 @@ threadControl_onEventHandlerEntry(jbyte sessionID, EventIndex ei, jthread thread
     node = findThread(&otherThreads, thread);
     if (node != NULL) {
         moveNode(&otherThreads, &runningThreads, node);
+        /* Now that we know the thread has started, we can set its TLS.*/
+        setThreadLocalStorage(thread, (void*)node);
     } else {
         /*
          * Get a thread node for the reporting thread. For thread start
@@ -2107,59 +2179,6 @@ threadControl_onEventHandlerEntry(jbyte sessionID, EventIndex ei, jthread thread
 static void
 doPendingTasks(JNIEnv *env, ThreadNode *node)
 {
-    /* Deferred breakpoint handling for Thread.resume() */
-    if (node->handlingAppResume) {
-        jthread resumer = node->thread;
-        jthread resumee = getResumee(resumer);
-
-        if (resumer != NULL) {
-            /*
-             * trackAppResume indirectly aquires handlerLock. For proper lock
-             * ordering handlerLock has to be acquired before threadLock.
-             */
-            debugMonitorExit(threadLock);
-            eventHandler_lock();
-            debugMonitorEnter(threadLock);
-
-            /*
-             * Track the resuming thread by marking it as being within
-             * a resume and by setting up for notification on
-             * a frame pop or exception. We won't allow the debugger
-             * to suspend threads while any thread is within a
-             * call to resume. This (along with the block below)
-             * ensures that when the debugger
-             * suspends a thread it will remain suspended.
-             */
-            trackAppResume(resumer);
-
-            /*
-             * handlerLock is not needed anymore. We must release it before calling
-             * blockOnDebuggerSuspend() because it is required for resumes done by
-             * the debugger. If resumee is currently suspended by the debugger, then
-             * blockOnDebuggerSuspend() will block until a debugger resume is done.
-             * If it blocks while holding the handlerLock, then the resume will deadlock.
-             */
-            eventHandler_unlock();
-        }
-
-        if (resumee != NULL) {
-            /*
-             * Hold up any attempt to resume as long as the debugger
-             * has suspended the resumee.
-             */
-            blockOnDebuggerSuspend(resumee);
-        }
-
-        node->handlingAppResume = JNI_FALSE;
-
-        /*
-         * The blocks exit condition: resumee's suspendCount == 0.
-         *
-         * Debugger suspends are blocked if any thread is executing
-         * Thread.resume(), i.e. !handlingAppResume && frameDepth > 0.
-         */
-    }
-
     /*
      * Take care of any pending interrupts/stops, and clear out
      * info on pending interrupts/stops.
@@ -2460,8 +2479,6 @@ threadControl_reset(void)
     /* Everything should have been resumed */
     JDI_ASSERT(otherThreads.first == NULL);
 
-    /* Threads could be waiting in blockOnDebuggerSuspend */
-    debugMonitorNotifyAll(threadLock);
     debugMonitorExit(threadLock);
     eventHandler_unlock();
 }
@@ -2519,7 +2536,8 @@ threadControl_setEventMode(jvmtiEventMode mode, EventIndex ei, jthread thread)
  * Returns the current thread, if the thread has generated at least
  * one event, and has not generated a thread end event.
  */
-jthread threadControl_currentThread(void)
+jthread
+threadControl_currentThread(void)
 {
     jthread thread;
 
@@ -2554,3 +2572,40 @@ threadControl_getFrameGeneration(jthread thread)
 
     return frameGeneration;
 }
+
+/***** debugging *****/
+
+#ifdef DEBUG
+
+void
+threadControl_dumpAllThreads()
+{
+    tty_message("Dumping runningThreads:\n");
+    dumpThreadList(&runningThreads);
+    tty_message("Dumping otherThreads:\n");
+    dumpThreadList(&otherThreads);
+}
+
+static void
+dumpThreadList(ThreadList *list)
+{
+    ThreadNode *node;
+    for (node = list->first; node != NULL; node = node->next) {
+        if (!node->isDebugThread) {
+            dumpThread(node);
+        }
+    }
+}
+
+static void
+dumpThread(ThreadNode *node) {
+    tty_message("  Thread: node = %p, jthread = %p", node, node->thread);
+#ifdef DEBUG_THREADNAME
+    tty_message("\tname: %s", node->name);
+#endif
+    // More fields can be printed here when needed. The amount of output is intentionlly
+    // kept small so it doesn't generate too much output.
+    tty_message("\tsuspendCount: %d", node->suspendCount);
+}
+
+#endif /* DEBUG */
