@@ -62,11 +62,6 @@ import java.util.stream.Collectors;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipException;
 import java.util.zip.ZipFile;
-import java.time.LocalDateTime;
-import java.time.ZonedDateTime;
-import java.time.ZoneOffset;
-import java.time.format.DateTimeFormatter;
-import java.time.format.DateTimeParseException;
 
 import jdk.internal.jmod.JmodFile;
 import jdk.internal.jmod.JmodFile.Section;
@@ -165,13 +160,8 @@ public class JmodTask {
         boolean dryrun;
         List<PathMatcher> excludes;
         Path extractDir;
-        LocalDateTime date;
     }
 
-    // Valid --date range
-    static final ZonedDateTime DATE_MIN = ZonedDateTime.parse("1980-01-01T00:00:02Z");
-    static final ZonedDateTime DATE_MAX = ZonedDateTime.parse("2099-12-31T23:59:59Z");
-
     public int run(String[] args) {
 
         try {
@@ -437,7 +427,7 @@ public class JmodTask {
         Path target = options.jmodFile;
         Path tempTarget = jmodTempFilePath(target);
         try {
-            try (JmodOutputStream jos = JmodOutputStream.newOutputStream(tempTarget, options.date)) {
+            try (JmodOutputStream jos = JmodOutputStream.newOutputStream(tempTarget)) {
                 jmod.write(jos);
             }
             Files.move(tempTarget, target);
@@ -911,7 +901,7 @@ public class JmodTask {
             // filter modules resolved from the system module finder
             this.modules = config.modules().stream()
                 .map(ResolvedModule::name)
-                .filter(mn -> roots.contains(mn) && !system.find(mn).isPresent())
+                .filter(mn -> roots.contains(mn) && system.find(mn).isEmpty())
                 .collect(Collectors.toSet());
 
             this.hashesBuilder = new ModuleHashesBuilder(config, modules);
@@ -994,11 +984,7 @@ public class JmodTask {
                         if (e.getName().equals(MODULE_INFO)) {
                             // what about module-info.class in versioned entries?
                             ZipEntry ze = new ZipEntry(e.getName());
-                            if (options.date != null) {
-                                ze.setTimeLocal(options.date);
-                            } else {
-                                ze.setTime(System.currentTimeMillis());
-                            }
+                            ze.setTime(System.currentTimeMillis());
                             jos.putNextEntry(ze);
                             recordHashes(in, jos, moduleHashes);
                             jos.closeEntry();
@@ -1026,7 +1012,7 @@ public class JmodTask {
         {
 
             try (JmodFile jf = new JmodFile(target);
-                 JmodOutputStream jos = JmodOutputStream.newOutputStream(tempTarget, options.date))
+                 JmodOutputStream jos = JmodOutputStream.newOutputStream(tempTarget))
             {
                 jf.stream().forEach(e -> {
                     try (InputStream in = jf.getInputStream(e.section(), e.name())) {
@@ -1066,7 +1052,7 @@ public class JmodTask {
      * Specific subclasses should do whatever validation is required on the
      * individual path elements, if any.
      */
-    static abstract class AbstractPathConverter implements ValueConverter<List<Path>> {
+    abstract static class AbstractPathConverter implements ValueConverter<List<Path>> {
         @Override
         public List<Path> convert(String value) {
             List<Path> paths = new ArrayList<>();
@@ -1161,26 +1147,6 @@ public class JmodTask {
         @Override public String valuePattern() { return "module-version"; }
     }
 
-    static class DateConverter implements ValueConverter<LocalDateTime> {
-        @Override
-        public LocalDateTime convert(String value) {
-            try {
-                ZonedDateTime date = ZonedDateTime.parse(value, DateTimeFormatter.ISO_ZONED_DATE_TIME)
-                                                          .withZoneSameInstant(ZoneOffset.UTC);
-                if (date.isBefore(DATE_MIN) || date.isAfter(DATE_MAX)) {
-                    throw new CommandException("err.date.out.of.range", value);
-                }
-                return date.toLocalDateTime();
-            } catch (DateTimeParseException x) {
-                throw new CommandException("err.invalid.date", value, x.getMessage());
-            }
-        }
-
-        @Override public Class<LocalDateTime> valueType() { return LocalDateTime.class; }
-
-        @Override public String valuePattern() { return "date"; }
-    }
-
     static class WarnIfResolvedReasonConverter
         implements ValueConverter<ModuleResolution>
     {
@@ -1416,11 +1382,6 @@ public class JmodTask {
         OptionSpec<Void> version
                 = parser.accepts("version", getMessage("main.opt.version"));
 
-        OptionSpec<LocalDateTime> date
-                = parser.accepts("date", getMessage("main.opt.date"))
-                        .withRequiredArg()
-                        .withValuesConvertedBy(new DateConverter());
-
         NonOptionArgumentSpec<String> nonOptions
                 = parser.nonOptions();
 
@@ -1464,8 +1425,6 @@ public class JmodTask {
                 options.manPages = getLastElement(opts.valuesOf(manPages));
             if (opts.has(legalNotices))
                 options.legalNotices = getLastElement(opts.valuesOf(legalNotices));
-            if (opts.has(date))
-                options.date = opts.valueOf(date);
             if (opts.has(modulePath)) {
                 Path[] dirs = getLastElement(opts.valuesOf(modulePath)).toArray(new Path[0]);
                 options.moduleFinder = ModulePath.of(Runtime.version(), true, dirs);
