@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -24,14 +24,12 @@
 package compiler.lib.ir_framework;
 
 import compiler.lib.ir_framework.driver.*;
-import compiler.lib.ir_framework.driver.irmatching.IRMatcher;
-import compiler.lib.ir_framework.driver.irmatching.IRViolationException;
 import compiler.lib.ir_framework.shared.*;
 import compiler.lib.ir_framework.test.*;
 import jdk.test.lib.Platform;
 import jdk.test.lib.Utils;
 import jdk.test.lib.helpers.ClassFileInstaller;
-import jdk.test.whitebox.WhiteBox;
+import sun.hotspot.WhiteBox;
 
 import java.io.PrintWriter;
 import java.io.StringWriter;
@@ -110,6 +108,7 @@ public class TestFramework {
                     "Trace",
                     "Print",
                     "Verify",
+                    "TLAB",
                     "UseNewCode",
                     "Xmn",
                     "Xms",
@@ -604,11 +603,11 @@ public class TestFramework {
                 // Print stack trace otherwise
                 StringWriter errors = new StringWriter();
                 e.printStackTrace(new PrintWriter(errors));
-                builder.append(errors);
+                builder.append(errors.toString());
             }
             builder.append(System.lineSeparator());
         }
-        System.err.println(builder);
+        System.err.println(builder.toString());
         if (!VERBOSE && !REPORT_STDOUT && !TESTLIST && !EXCLUDELIST) {
             // Provide a hint to the user how to get additional output/debugging information.
             System.err.println(RERUN_HINT);
