@@ -27,12 +27,12 @@
  * @test
  * @key cgroups
  * @summary Test JVM's awareness of pids controller
- * @requires container.support
+ * @requires docker.support
  * @library /test/lib
  * @modules java.base/jdk.internal.misc
  *          java.management
- * @build jdk.test.whitebox.WhiteBox PrintContainerInfo
- * @run driver jdk.test.lib.helpers.ClassFileInstaller -jar whitebox.jar jdk.test.whitebox.WhiteBox
+ * @build sun.hotspot.WhiteBox PrintContainerInfo
+ * @run driver jdk.test.lib.helpers.ClassFileInstaller -jar whitebox.jar sun.hotspot.WhiteBox
  * @run driver TestPids
  */
 import java.util.List;
@@ -40,17 +40,11 @@ import jdk.test.lib.containers.docker.Common;
 import jdk.test.lib.containers.docker.DockerRunOptions;
 import jdk.test.lib.containers.docker.DockerTestUtils;
 import jdk.test.lib.Asserts;
-import jdk.test.lib.Container;
 import jdk.test.lib.Platform;
 import jdk.test.lib.Utils;
 
 public class TestPids {
     private static final String imageName = Common.imageName("pids");
-    private static final boolean IS_PODMAN = Container.ENGINE_COMMAND.contains("podman");
-    private static final int UNLIMITED_PIDS_PODMAN = 0;
-    private static final int UNLIMITED_PIDS_DOCKER = -1;
-
-    static final String warning_kernel_no_pids_support = "WARNING: Your kernel does not support pids limit capabilities";
 
     public static void main(String[] args) throws Exception {
         if (!DockerTestUtils.canTestDocker()) {
@@ -89,7 +83,7 @@ public class TestPids {
         boolean lineMarkerFound = false;
 
         for (String line : lines) {
-            if (line.contains(warning_kernel_no_pids_support)) {
+            if (line.contains("WARNING: Your kernel does not support pids limit capabilities")) {
                 System.out.println("Docker pids limitation seems not to work, avoiding check");
                 return;
             }
@@ -99,17 +93,6 @@ public class TestPids {
                 String[] parts = line.split(":");
                 System.out.println("DEBUG: line = " + line);
                 System.out.println("DEBUG: parts.length = " + parts.length);
-                if (expectedValue.equals("any_integer")) {
-                    Asserts.assertEquals(parts.length, 2);
-                    String ivalue = parts[1].replaceAll("\\s","");
-                    try {
-                        int ai = Integer.parseInt(ivalue);
-                        System.out.println("Found " + lineMarker + " with value: " + ai + ". PASS.");
-                    } catch (NumberFormatException ex) {
-                        throw new RuntimeException("Could not convert " + ivalue + " to an integer, log line was " + line);
-                    }
-                    break;
-                }
 
                 Asserts.assertEquals(parts.length, 2);
                 String actual = parts[1].replaceAll("\\s","");
@@ -143,8 +126,7 @@ public class TestPids {
 
         DockerRunOptions opts = commonOpts();
         if (value.equals("Unlimited")) {
-            int unlimited = IS_PODMAN ? UNLIMITED_PIDS_PODMAN : UNLIMITED_PIDS_DOCKER;
-            opts.addDockerOpts("--pids-limit=" + unlimited);
+            opts.addDockerOpts("--pids-limit=-1");
         } else {
             opts.addDockerOpts("--pids-limit="+value);
         }
@@ -155,8 +137,6 @@ public class TestPids {
         } else {
             checkResult(lines, "Maximum number of tasks is: ", value);
         }
-        // current number of tasks value is hard to predict, so better expect no value
-        checkResult(lines, "Current number of tasks is: ", "any_integer");
     }
 
 }
