@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2004, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -45,18 +45,16 @@ static jlong timeout = 0;
 #define PATH_TO_NEW_BYTECODE "pathToNewByteCode"
 
 static jint testStep;
+static int redefineNumber;
+static jint newClassSize;
+static unsigned char* newClassBytes;
 static jthread testedThread;
 static jclass testClass;
+char chbuffer[255];
 
-// callbackException (isCatch == false) and callbackExceptionCatch (isCatch == true) handler
-void handleException(bool isCatch,
-    jvmtiEnv* jvmti_env, JNIEnv* jni_env, jthread thread,
-    jmethodID method, jlocation location,
-    jobject exception);
-
-const char* getThreadName(jvmtiEnv* jvmti_env, JNIEnv* jni_env, jthread thread);
+const char* getThreadName(JNIEnv* jni_env, jthread thread);
 const char* getClassName(jvmtiEnv *jvmti_env, JNIEnv* jni_env, jobject object);
-int readNewBytecode(jvmtiEnv* jvmti, jint *newClassSize, unsigned char* *newClassBytes);
+int readNewBytecode(jvmtiEnv* jvmti);
 int getLocalVariableValue(jvmtiEnv *jvmti_env, jthread thread, jmethodID method);
 
 /* ============================================================================= */
@@ -101,19 +99,17 @@ void disableEvent(jvmtiEnv *jvmti_env, jvmtiEvent event, jthread thread) {
 void redefineClass(jvmtiEnv *jvmti_env, jclass klass) {
 
     jvmtiClassDefinition classDef;
+
     char *className;
-    jint newClassSize;
-    unsigned char* newClassBytes;
 
     if (!NSK_JVMTI_VERIFY(jvmti_env->GetClassSignature(klass, &className, NULL))) {
         nsk_jvmti_setFailStatus();
         return;
     }
 
-    if (!NSK_VERIFY(readNewBytecode(jvmti_env, &newClassSize, &newClassBytes))) {
+    if (!NSK_VERIFY(readNewBytecode(jvmti_env))) {
         NSK_COMPLAIN0("TEST FAILED: new bytecode could not be read\n");
         nsk_jvmti_setFailStatus();
-        jvmti_env->Deallocate((unsigned char*)className);
         return;
     }
 
@@ -125,14 +121,12 @@ void redefineClass(jvmtiEnv *jvmti_env, jclass klass) {
     if (!NSK_JVMTI_VERIFY(jvmti_env->RedefineClasses(1, &classDef))) {
         NSK_COMPLAIN1("TEST FAILED: while redefining class %s\n", className);
         nsk_jvmti_setFailStatus();
+        return;
     }
 
     if (!NSK_JVMTI_VERIFY(jvmti_env->Deallocate((unsigned char*)className))) {
         nsk_jvmti_setFailStatus();
     }
-    if (!NSK_JVMTI_VERIFY(jvmti_env->Deallocate(newClassBytes))) {
-        nsk_jvmti_setFailStatus();
-    }
 
 }
 
@@ -142,6 +136,7 @@ void redefineClass(jvmtiEnv *jvmti_env, jclass klass) {
 static void JNICALL
 agentProc(jvmtiEnv* jvmti, JNIEnv* agentJNI, void* arg) {
 
+    redefineNumber = 1;
     jni = agentJNI;
 
     NSK_DISPLAY0("Waiting for debuggee to become ready\n");
@@ -149,7 +144,7 @@ agentProc(jvmtiEnv* jvmti, JNIEnv* agentJNI, void* arg) {
         return;
 
     testStep = 1;
-    NSK_DISPLAY0(">>>> Debugge started, waiting for class loading \n");
+    NSK_DISPLAY0("\n\n>>>> Debugge started, waiting for class loading \n");
     if (!nsk_jvmti_resumeSync())
         return;
 
@@ -188,20 +183,27 @@ callbackClassLoad(jvmtiEnv *jvmti_env, JNIEnv* jni_env, jthread thread,
                         jclass klass) {
 
     char *className;
+    char *generic;
 
-    if (!NSK_JVMTI_VERIFY(jvmti_env->GetClassSignature(klass, &className, NULL))) {
+    if (!NSK_JVMTI_VERIFY(jvmti_env->GetClassSignature(klass, &className, &generic))) {
         nsk_jvmti_setFailStatus();
         return;
     }
 
     if (strcmp(className, EXPECTED_CLASS_SIGN) == 0) {
-        NSK_DISPLAY1(">>>> Class loaded: %s, activating breakpoint\n", className);
+        NSK_DISPLAY1("\n\n>>>> Class loaded: %s", className);
+        NSK_DISPLAY0(", activating breakpoint\n");
         setBreakPoint(jvmti_env, jni_env, klass);
     }
 
     if (!NSK_JVMTI_VERIFY(jvmti_env->Deallocate((unsigned char*)className))) {
         nsk_jvmti_setFailStatus();
     }
+
+    if (generic != NULL)
+        if (!NSK_JVMTI_VERIFY(jvmti_env->Deallocate((unsigned char*)generic))) {
+            nsk_jvmti_setFailStatus();
+        }
 }
 
 /* ============================================================================= */
@@ -214,7 +216,7 @@ JNIEXPORT void JNICALL
 callbackBreakpoint(jvmtiEnv *jvmti_env, JNIEnv* jni_env, jthread thread,
                          jmethodID method, jlocation location) {
 
-    NSK_DISPLAY0(">>>>Breakpoint fired, enabling SINGLE_STEP\n");
+    NSK_DISPLAY0("\n\n>>>>Breakpoint fired, enabling SINGLE_STEP\n");
     enableEvent(jvmti_env, JVMTI_EVENT_SINGLE_STEP, thread);
 }
 
@@ -264,7 +266,7 @@ callbackSingleStep(jvmtiEnv *jvmti_env, JNIEnv* jni_env, jthread thread,
 
             case 2:
 
-                NSK_DISPLAY1(">>>> Checking if redefined method is not obsolete\n", testStep);
+                NSK_DISPLAY1("\n\n>>>> Checking if redefined method is not obsolete\n", testStep);
 
                 if (!NSK_JVMTI_VERIFY(jvmti->IsMethodObsolete(method, &is_obsolete))) {
                     NSK_COMPLAIN0("TEST FAILED: unable to check method to be obsolete\n");
@@ -281,7 +283,7 @@ callbackSingleStep(jvmtiEnv *jvmti_env, JNIEnv* jni_env, jthread thread,
 
             case 3:
 
-                NSK_DISPLAY1(">>>> Popping the currently executing frame\n", testStep);
+                NSK_DISPLAY1("\n\n>>>> Popping the currently executing frame\n", testStep);
                 testStep++;
                 setCurrentStep(jni_env, testStep);
 
@@ -290,7 +292,7 @@ callbackSingleStep(jvmtiEnv *jvmti_env, JNIEnv* jni_env, jthread thread,
             case 5:
 
                 if (value < 10) {
-                    NSK_DISPLAY1(">>>> Disabling single step\n", testStep);
+                    NSK_DISPLAY1("\n\n>>>> Disabling single step\n", testStep);
                     disableEvent(jvmti_env, JVMTI_EVENT_SINGLE_STEP, thread);
                     setCurrentStep(jni_env, testStep);
                 }
@@ -300,7 +302,7 @@ callbackSingleStep(jvmtiEnv *jvmti_env, JNIEnv* jni_env, jthread thread,
         }
 
         if (!NSK_JVMTI_VERIFY(jvmti_env->Deallocate((unsigned char*) declaringClassName))) {
-            NSK_COMPLAIN0("TEST FAILED: unable to deallocate memory pointed to declaringClassName\n\n");
+            NSK_COMPLAIN0("TEST FAILED: unable to deallocate memory pointed to method name\n\n");
         }
 
     }
@@ -322,7 +324,24 @@ callbackException(jvmtiEnv *jvmti_env, JNIEnv* jni_env, jthread thread,
                         jmethodID method, jlocation location, jobject exception,
                         jmethodID catch_method, jlocation catch_location) {
 
-    handleException(false, jvmti_env, jni_env, thread, method, location, exception);
+    const char *className;
+
+    className = getClassName(jvmti_env, jni_env, exception);
+
+    if (strcmp(EXPECTED_CLASS_SIGN, className) == 0) {
+        jclass klass;
+
+        NSK_DISPLAY2("\n\n>>>> Exception %s in thread - %s\n",
+                        className, getThreadName(jni_env, thread));
+
+        testStep++;
+        if (!NSK_JNI_VERIFY(jni_env, (klass = jni_env->GetObjectClass(exception)) != NULL)) {
+            nsk_jvmti_setFailStatus();
+            return;
+        }
+
+        redefineClass(jvmti_env, klass);
+    }
 }
 
 /* ============================================================================= */
@@ -336,27 +355,15 @@ callbackExceptionCatch(jvmtiEnv *jvmti_env, JNIEnv* jni_env, jthread thread,
                         jmethodID method, jlocation location,
                         jobject exception) {
 
-    handleException(true, jvmti_env, jni_env, thread, method, location, exception);
-}
+    const char *className;
 
-/* ============================================================================= */
+    className = getClassName(jvmti_env, jni_env, exception);
 
-void handleException(bool isCatch,
-                     jvmtiEnv* jvmti_env, JNIEnv* jni_env, jthread thread,
-                     jmethodID method, jlocation location,
-                     jobject exception) {
-    const char* className = getClassName(jvmti_env, jni_env, exception);
-
-    if (className != NULL && strcmp(EXPECTED_CLASS_SIGN, className) == 0) {
+    if (strcmp(EXPECTED_CLASS_SIGN, className) == 0) {
         jclass klass;
 
-        const char* threadName = getThreadName(jvmti_env, jni_env, thread);
-        NSK_DISPLAY3(">>>> %s %s in thread - %s\n", isCatch ? "Caught exception" : "Exception",
-            className, threadName != NULL ? threadName : "NULL");
-        jvmti->Deallocate((unsigned char*)className);
-        if (threadName != NULL) {
-            jvmti->Deallocate((unsigned char*)threadName);
-        }
+        NSK_DISPLAY2("\n\n>>>> Caught exception %s in thread - %s\n",
+                        className, getThreadName(jni_env, thread));
 
         testStep++;
         if (!NSK_JNI_VERIFY(jni_env, (klass = jni_env->GetObjectClass(exception)) != NULL)) {
@@ -366,11 +373,11 @@ void handleException(bool isCatch,
 
         redefineClass(jvmti_env, klass);
     }
-
 }
 
+/* ============================================================================= */
 
-int readNewBytecode(jvmtiEnv* jvmti, jint *newClassSize, unsigned char* *newClassBytes) {
+int readNewBytecode(jvmtiEnv* jvmti) {
 
     char filename[256];
     FILE *bytecode;
@@ -383,7 +390,7 @@ int readNewBytecode(jvmtiEnv* jvmti, jint *newClassSize, unsigned char* *newClas
         return NSK_FALSE;
     }
 
-    snprintf(filename, sizeof(filename), "%s/%s/%s.class",
+    sprintf(filename,"%s/%s/%s.class",
                 pathToByteCode, "newclass", EXPECTED_CLASS_NAME);
 
     NSK_DISPLAY1("\treading new bytecode for the tested class\n\tfile name: %s\n",
@@ -396,19 +403,17 @@ int readNewBytecode(jvmtiEnv* jvmti, jint *newClassSize, unsigned char* *newClas
     }
 
     fseek(bytecode, 0, SEEK_END);
-    *newClassSize = ftell(bytecode);
+    newClassSize = ftell(bytecode);
     rewind(bytecode);
 
-    if (!NSK_JVMTI_VERIFY(jvmti->Allocate(*newClassSize, newClassBytes))) {
+    if (!NSK_JVMTI_VERIFY(jvmti->Allocate(newClassSize, &newClassBytes))) {
         NSK_COMPLAIN0("buffer couldn't be allocated\n");
         return NSK_FALSE;
     }
-    read_bytes = (jint) fread(*newClassBytes, 1, *newClassSize, bytecode);
+    read_bytes = (jint) fread(newClassBytes, 1, newClassSize, bytecode);
     fclose(bytecode);
-    if (read_bytes != *newClassSize) {
+    if (read_bytes != newClassSize) {
         NSK_COMPLAIN0("TEST FAILED: error reading file\n");
-        jvmti->Deallocate(*newClassBytes);
-        *newClassBytes = NULL;
         return NSK_FALSE;
     }
 
@@ -417,41 +422,34 @@ int readNewBytecode(jvmtiEnv* jvmti, jint *newClassSize, unsigned char* *newClas
 
 /* ============================================================================= */
 
-const char* getThreadName(jvmtiEnv* jvmti_env, JNIEnv* jni_env, jthread thread) {
+const char* getThreadName(JNIEnv* jni_env, jthread thread) {
     jmethodID methodID;
     jclass klass;
     jstring jthreadName;
-    jsize jthreadNameLen;
-    unsigned char *result = NULL;
     const char *threadName;
 
+    strcpy(chbuffer, "");
+
     if (!NSK_JNI_VERIFY(jni_env, (klass = jni_env->GetObjectClass(thread)) != NULL)) {
         nsk_jvmti_setFailStatus();
-        return NULL;
+        return chbuffer;
     }
 
     if (!NSK_JNI_VERIFY(jni_env, (methodID =
             jni_env->GetMethodID(klass, "getName", "()Ljava/lang/String;")) != NULL)) {
         nsk_jvmti_setFailStatus();
-        return NULL;
+        return chbuffer;
     }
 
     jthreadName = (jstring) jni_env->CallObjectMethod(thread, methodID);
 
-    jthreadNameLen = jni_env->GetStringUTFLength(jthreadName);
-
-    if (!NSK_JVMTI_VERIFY(jvmti_env->Allocate(jthreadNameLen + 1, &result))) {
-        NSK_COMPLAIN0("buffer couldn't be allocated\n");
-        return NULL;
-    }
-
     threadName = jni_env->GetStringUTFChars(jthreadName, 0);
 
-    memcpy(result, threadName, jthreadNameLen + 1);
+    strcpy(chbuffer, threadName);
 
     jni_env->ReleaseStringUTFChars(jthreadName, threadName);
 
-    return (char*)result;
+    return chbuffer;
 }
 
 /* ============================================================================= */
@@ -459,19 +457,33 @@ const char* getThreadName(jvmtiEnv* jvmti_env, JNIEnv* jni_env, jthread thread)
 const char* getClassName(jvmtiEnv *jvmti_env, JNIEnv* jni_env, jobject object) {
 
     char *className;
+    char *generic;
     jclass klass;
 
+    strcpy(chbuffer, "");
+
     if (!NSK_JNI_VERIFY(jni_env, (klass = jni_env->GetObjectClass(object)) != NULL)) {
         nsk_jvmti_setFailStatus();
-        return NULL;
+        return chbuffer;
     }
 
-    if (!NSK_JVMTI_VERIFY(jvmti_env->GetClassSignature(klass, &className, NULL))) {
+    if (!NSK_JVMTI_VERIFY(jvmti_env->GetClassSignature(klass, &className, &generic))) {
         nsk_jvmti_setFailStatus();
-        return NULL;
+        return chbuffer;
     }
 
-    return className;
+    strcpy(chbuffer, className);
+
+    if (!NSK_JVMTI_VERIFY(jvmti_env->Deallocate((unsigned char*)className))) {
+        nsk_jvmti_setFailStatus();
+    }
+
+    if (generic != NULL)
+        if (!NSK_JVMTI_VERIFY(jvmti_env->Deallocate((unsigned char*)generic))) {
+            nsk_jvmti_setFailStatus();
+        }
+
+    return chbuffer;
 }
 
 /* ============================================================================= */
@@ -534,10 +546,6 @@ Java_nsk_jvmti_scenarios_hotswap_HS201_hs201t002_setThread(JNIEnv *env,
     if (!NSK_JNI_VERIFY(env, (testedThread = env->NewGlobalRef(thread)) != NULL))
         nsk_jvmti_setFailStatus();
 
-    enableEvent(jvmti, JVMTI_EVENT_CLASS_LOAD, testedThread);
-    enableEvent(jvmti, JVMTI_EVENT_BREAKPOINT, testedThread);
-    enableEvent(jvmti, JVMTI_EVENT_EXCEPTION, testedThread);
-    enableEvent(jvmti, JVMTI_EVENT_EXCEPTION_CATCH, testedThread);
 }
 
 /* ============================================================================= */
@@ -657,6 +665,11 @@ jint Agent_Initialize(JavaVM *jvm, char *options, void *reserved) {
 
     NSK_DISPLAY0("Enable events\n");
 
+    enableEvent(jvmti, JVMTI_EVENT_CLASS_LOAD, testedThread);
+    enableEvent(jvmti, JVMTI_EVENT_BREAKPOINT, testedThread);
+    enableEvent(jvmti, JVMTI_EVENT_EXCEPTION, testedThread);
+    enableEvent(jvmti, JVMTI_EVENT_EXCEPTION_CATCH, testedThread);
+
     if (!NSK_VERIFY(nsk_jvmti_setAgentProc(agentProc, NULL)))
         return JNI_ERR;
 
