@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -48,6 +48,7 @@ import java.util.Map;
 import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.CopyOnWriteArrayList;
+import java.util.concurrent.ExecutorService;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 import java.util.stream.Stream;
@@ -196,7 +197,7 @@ public interface HttpServerAdapters {
     /**
      * A version agnostic adapter class for HTTP Server Exchange.
      */
-    public static abstract class HttpTestExchange {
+    public static abstract class HttpTestExchange implements AutoCloseable {
         public abstract Version getServerVersion();
         public abstract Version getExchangeVersion();
         public abstract InputStream   getRequestBody();
@@ -207,8 +208,6 @@ public interface HttpServerAdapters {
         public abstract URI getRequestURI();
         public abstract String getRequestMethod();
         public abstract void close();
-        public abstract InetSocketAddress getRemoteAddress();
-        public abstract InetSocketAddress getLocalAddress();
         public void serverPush(URI uri, HttpHeaders headers, byte[] body) {
             ByteArrayInputStream bais = new ByteArrayInputStream(body);
             serverPush(uri, headers, bais);
@@ -266,15 +265,6 @@ public interface HttpServerAdapters {
             }
             @Override
             public void close() { exchange.close(); }
-
-            @Override
-            public InetSocketAddress getRemoteAddress() {
-                return exchange.getRemoteAddress();
-            }
-            @Override
-            public InetSocketAddress getLocalAddress() {
-                return exchange.getLocalAddress();
-            }
             @Override
             public URI getRequestURI() { return exchange.getRequestURI(); }
             @Override
@@ -330,16 +320,6 @@ public interface HttpServerAdapters {
             }
             @Override
             public void close() { exchange.close();}
-
-            @Override
-            public InetSocketAddress getRemoteAddress() {
-                return exchange.getRemoteAddress();
-            }
-            @Override
-            public InetSocketAddress getLocalAddress() {
-                return exchange.getLocalAddress();
-            }
-
             @Override
             public URI getRequestURI() { return exchange.getRequestURI(); }
             @Override
@@ -526,13 +506,23 @@ public interface HttpServerAdapters {
             return new Http1TestServer(server);
         }
 
+        public static HttpTestServer of(HttpServer server, ExecutorService executor) {
+            return new Http1TestServer(server, executor);
+        }
+
         public static HttpTestServer of(Http2TestServer server) {
             return new Http2TestServerImpl(server);
         }
 
         private static class Http1TestServer extends  HttpTestServer {
             private final HttpServer impl;
+            private final ExecutorService executor;
             Http1TestServer(HttpServer server) {
+                this(server, null);
+            }
+            Http1TestServer(HttpServer server, ExecutorService executor) {
+                if (executor != null) server.setExecutor(executor);
+                this.executor = executor;
                 this.impl = server;
             }
             @Override
@@ -543,7 +533,13 @@ public interface HttpServerAdapters {
             @Override
             public void stop() {
                 System.out.println("Http1TestServer: stop");
-                impl.stop(0);
+                try {
+                    impl.stop(0);
+                } finally {
+                    if (executor != null) {
+                        executor.shutdownNow();
+                    }
+                }
             }
             @Override
             public HttpTestContext addHandler(HttpTestHandler handler, String path) {
