@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -32,6 +32,7 @@ import java.net.InetAddress;
 import java.net.InetSocketAddress;
 import java.net.URI;
 import java.net.http.HttpClient;
+import java.net.http.HttpClient.Builder;
 import java.net.http.HttpRequest;
 import java.net.http.HttpRequest.BodyPublishers;
 import java.net.http.HttpResponse;
@@ -43,7 +44,11 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.CopyOnWriteArrayList;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Executors;
 import java.util.concurrent.Flow;
+import java.util.concurrent.ThreadFactory;
+import java.util.concurrent.atomic.AtomicInteger;
 import java.util.function.Function;
 import java.util.function.Supplier;
 import java.util.stream.Collectors;
@@ -71,6 +76,7 @@ import static org.testng.Assert.assertTrue;
  * @summary Basic tests for line adapter subscribers as created by
  *          the BodyHandlers returned by BodyHandler::fromLineSubscriber
  *          and BodyHandler::asLines
+ * @bug 8256459
  * @modules java.base/sun.net.www.http
  *          java.net.http/jdk.internal.net.http.common
  *          java.net.http/jdk.internal.net.http.frame
@@ -80,7 +86,7 @@ import static org.testng.Assert.assertTrue;
  * @library /test/lib http2/server
  * @build Http2TestServer LineBodyHandlerTest HttpServerAdapters
  * @build jdk.test.lib.net.SimpleSSLContext
- * @run testng/othervm LineBodyHandlerTest
+ * @run testng/othervm -XX:+UnlockDiagnosticVMOptions -XX:DiagnoseSyncOnValueBasedClasses=1 LineBodyHandlerTest
  */
 
 public class LineBodyHandlerTest implements HttpServerAdapters {
@@ -182,11 +188,17 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
         }
     }
 
+    HttpClient newClient() {
+        return HttpClient.newBuilder()
+                .sslContext(sslContext)
+                .proxy(Builder.NO_PROXY)
+                .build();
+    }
+
     @Test(dataProvider = "uris")
     void testStringWithFinisher(String url) {
         String body = "May the luck of the Irish be with you!";
-        HttpClient client = HttpClient.newBuilder().sslContext(sslContext)
-                .build();
+        HttpClient client = newClient();
         HttpRequest request = HttpRequest.newBuilder(URI.create(url))
                 .POST(BodyPublishers.ofString(body))
                 .build();
@@ -207,8 +219,7 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
     @Test(dataProvider = "uris")
     void testAsStream(String url) {
         String body = "May the luck of the Irish be with you!";
-        HttpClient client = HttpClient.newBuilder().sslContext(sslContext)
-                .build();
+        HttpClient client = newClient();
         HttpRequest request = HttpRequest.newBuilder(URI.create(url))
                 .POST(BodyPublishers.ofString(body))
                 .build();
@@ -230,7 +241,8 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
     @Test(dataProvider = "uris")
     void testStringWithFinisher2(String url) {
         String body = "May the luck\r\n\r\n of the Irish be with you!";
-        HttpClient client = HttpClient.newBuilder().sslContext(sslContext).build();
+        HttpClient client = newClient();
+
         HttpRequest request = HttpRequest.newBuilder(URI.create(url))
                 .POST(BodyPublishers.ofString(body))
                 .build();
@@ -251,7 +263,7 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
     @Test(dataProvider = "uris")
     void testAsStreamWithCRLF(String url) {
         String body = "May the luck\r\n\r\n of the Irish be with you!";
-        HttpClient client = HttpClient.newBuilder().sslContext(sslContext).build();
+        HttpClient client = newClient();
         HttpRequest request = HttpRequest.newBuilder(URI.create(url))
                 .POST(BodyPublishers.ofString(body))
                 .build();
@@ -275,7 +287,7 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
     @Test(dataProvider = "uris")
     void testStringWithFinisherBlocking(String url) throws Exception {
         String body = "May the luck of the Irish be with you!";
-        HttpClient client = HttpClient.newBuilder().sslContext(sslContext).build();
+        HttpClient client = newClient();
         HttpRequest request = HttpRequest.newBuilder(URI.create(url))
                 .POST(BodyPublishers.ofString(body)).build();
 
@@ -292,7 +304,7 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
     @Test(dataProvider = "uris")
     void testStringWithoutFinisherBlocking(String url) throws Exception {
         String body = "May the luck of the Irish be with you!";
-        HttpClient client = HttpClient.newBuilder().sslContext(sslContext).build();
+        HttpClient client = newClient();
         HttpRequest request = HttpRequest.newBuilder(URI.create(url))
                 .POST(BodyPublishers.ofString(body)).build();
 
@@ -311,7 +323,7 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
     @Test(dataProvider = "uris")
     void testAsStreamWithMixedCRLF(String url) {
         String body = "May\r\n the wind\r\n always be\rat your back.\r\r";
-        HttpClient client = HttpClient.newBuilder().sslContext(sslContext).build();
+        HttpClient client = newClient();
         HttpRequest request = HttpRequest.newBuilder(URI.create(url))
                 .POST(BodyPublishers.ofString(body))
                 .build();
@@ -338,7 +350,7 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
     @Test(dataProvider = "uris")
     void testAsStreamWithMixedCRLF_UTF8(String url) {
         String body = "May\r\n the wind\r\n always be\rat your back.\r\r";
-        HttpClient client = HttpClient.newBuilder().sslContext(sslContext).build();
+        HttpClient client = newClient();
         HttpRequest request = HttpRequest.newBuilder(URI.create(url))
                 .header("Content-type", "text/text; charset=UTF-8")
                 .POST(BodyPublishers.ofString(body, UTF_8)).build();
@@ -364,7 +376,7 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
     @Test(dataProvider = "uris")
     void testAsStreamWithMixedCRLF_UTF16(String url) {
         String body = "May\r\n the wind\r\n always be\rat your back.\r\r";
-        HttpClient client = HttpClient.newBuilder().sslContext(sslContext).build();
+        HttpClient client = newClient();
         HttpRequest request = HttpRequest.newBuilder(URI.create(url))
                 .header("Content-type", "text/text; charset=UTF-16")
                 .POST(BodyPublishers.ofString(body, UTF_16)).build();
@@ -391,7 +403,7 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
     @Test(dataProvider = "uris")
     void testObjectWithFinisher(String url) {
         String body = "May\r\n the wind\r\n always be\rat your back.";
-        HttpClient client = HttpClient.newBuilder().sslContext(sslContext).build();
+        HttpClient client = newClient();
         HttpRequest request = HttpRequest.newBuilder(URI.create(url))
                 .POST(BodyPublishers.ofString(body))
                 .build();
@@ -416,7 +428,7 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
     @Test(dataProvider = "uris")
     void testObjectWithFinisher_UTF16(String url) {
         String body = "May\r\n the wind\r\n always be\rat your back.\r\r";
-        HttpClient client = HttpClient.newBuilder().sslContext(sslContext).build();
+        HttpClient client = newClient();
         HttpRequest request = HttpRequest.newBuilder(URI.create(url))
                 .header("Content-type", "text/text; charset=UTF-16")
                 .POST(BodyPublishers.ofString(body, UTF_16)).build();
@@ -442,8 +454,7 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
     @Test(dataProvider = "uris")
     void testObjectWithoutFinisher(String url) {
         String body = "May\r\n the wind\r\n always be\rat your back.";
-        HttpClient client = HttpClient.newBuilder().sslContext(sslContext)
-                .build();
+        HttpClient client = newClient();
         HttpRequest request = HttpRequest.newBuilder(URI.create(url))
                 .POST(BodyPublishers.ofString(body))
                 .build();
@@ -469,8 +480,7 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
     @Test(dataProvider = "uris")
     void testObjectWithFinisherBlocking(String url) throws Exception {
         String body = "May\r\n the wind\r\n always be\nat your back.";
-        HttpClient client = HttpClient.newBuilder().sslContext(sslContext)
-                .build();
+        HttpClient client = newClient();
         HttpRequest request = HttpRequest.newBuilder(URI.create(url))
                 .POST(BodyPublishers.ofString(body))
                 .build();
@@ -494,8 +504,7 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
     @Test(dataProvider = "uris")
     void testObjectWithoutFinisherBlocking(String url) throws Exception {
         String body = "May\r\n the wind\r\n always be\nat your back.";
-        HttpClient client = HttpClient.newBuilder().sslContext(sslContext)
-                .build();
+        HttpClient client = newClient();
         HttpRequest request = HttpRequest.newBuilder(URI.create(url))
                 .POST(BodyPublishers.ofString(body))
                 .build();
@@ -529,8 +538,7 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
 
     @Test(dataProvider = "uris")
     void testBigTextFromLineSubscriber(String url) {
-        HttpClient client = HttpClient.newBuilder().sslContext(sslContext)
-                .build();
+        HttpClient client = newClient();
         String bigtext = bigtext();
         HttpRequest request = HttpRequest.newBuilder(URI.create(url))
                 .POST(BodyPublishers.ofString(bigtext))
@@ -551,8 +559,7 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
 
     @Test(dataProvider = "uris")
     void testBigTextAsStream(String url) {
-        HttpClient client = HttpClient.newBuilder().sslContext(sslContext)
-                .build();
+        HttpClient client = newClient();
         String bigtext = bigtext();
         HttpRequest request = HttpRequest.newBuilder(URI.create(url))
                 .POST(BodyPublishers.ofString(bigtext))
@@ -640,6 +647,19 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
         }
     }
 
+    private static ExecutorService executorFor(String serverThreadName) {
+        ThreadFactory factory = new ThreadFactory() {
+            final AtomicInteger counter = new AtomicInteger();
+            @Override
+            public Thread newThread(Runnable r) {
+                Thread thread = new Thread(r);
+                thread.setName(serverThreadName + "#" + counter.incrementAndGet());
+                return thread;
+            }
+        };
+        return Executors.newCachedThreadPool(factory);
+    }
+
     @BeforeTest
     public void setup() throws Exception {
         sslContext = new SimpleSSLContext().get();
@@ -647,13 +667,15 @@ public class LineBodyHandlerTest implements HttpServerAdapters {
             throw new AssertionError("Unexpected null sslContext");
 
         InetSocketAddress sa = new InetSocketAddress(InetAddress.getLoopbackAddress(), 0);
-        httpTestServer = HttpTestServer.of(HttpServer.create(sa, 0));
+        httpTestServer = HttpTestServer.of(HttpServer.create(sa, 0),
+                executorFor("HTTP/1.1 Server Thread"));
         httpTestServer.addHandler(new HttpTestEchoHandler(), "/http1/echo");
         httpURI = "http://" + httpTestServer.serverAuthority() + "/http1/echo";
 
         HttpsServer httpsServer = HttpsServer.create(sa, 0);
         httpsServer.setHttpsConfigurator(new HttpsConfigurator(sslContext));
-        httpsTestServer = HttpTestServer.of(httpsServer);
+        httpsTestServer = HttpTestServer.of(httpsServer,
+                executorFor("HTTPS/1.1 Server Thread"));
         httpsTestServer.addHandler(new HttpTestEchoHandler(),"/https1/echo");
         httpsURI = "https://" + httpsTestServer.serverAuthority() + "/https1/echo";
 
