@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -84,11 +84,10 @@ public class SimpleOCSPServer {
     private boolean logEnabled = false;
     private ExecutorService threadPool;
     private volatile boolean started = false;
-    private CountDownLatch serverReady = new CountDownLatch(1);
+    private volatile boolean serverReady = false;
     private volatile boolean receivedShutdown = false;
     private volatile boolean acceptConnections = true;
     private volatile long delayMsec = 0;
-    private boolean omitContentLength = false;
 
     // Fields used in the generation of responses
     private long nextUpdateInterval = -1;
@@ -217,15 +216,14 @@ public class SimpleOCSPServer {
                             listenPort), 128);
                     log("Listening on " + servSocket.getLocalSocketAddress());
 
+                    // Singal ready
+                    serverReady = true;
+
                     // Update the listenPort with the new port number.  If
                     // the server is restarted, it will bind to the same
                     // port rather than picking a new one.
                     listenPort = servSocket.getLocalPort();
 
-                    // Decrement the latch, allowing any waiting entities
-                    // to proceed with their requests.
-                    serverReady.countDown();
-
                     // Main dispatch loop
                     while (!receivedShutdown) {
                         try {
@@ -259,7 +257,7 @@ public class SimpleOCSPServer {
                     // Reset state variables so the server can be restarted
                     receivedShutdown = false;
                     started = false;
-                    serverReady = new CountDownLatch(1);
+                    serverReady = false;
                 }
             }
         });
@@ -288,18 +286,10 @@ public class SimpleOCSPServer {
     public synchronized void stop() {
         if (started) {
             receivedShutdown = true;
-            started = false;
             log("Received shutdown notification");
         }
     }
 
-    public synchronized void shutdownNow() {
-        stop();
-        if (threadPool != null) {
-            threadPool.shutdownNow();
-        }
-    }
-
     /**
      * Print {@code SimpleOCSPServer} operating parameters.
      *
@@ -329,13 +319,12 @@ public class SimpleOCSPServer {
      * @return the hexdump of the byte array
      */
     private static String dumpHexBytes(byte[] data) {
-        return dumpHexBytes(data, data.length, 16, "\n", " ");
+        return dumpHexBytes(data, 16, "\n", " ");
     }
 
     /**
      *
-     * @param data the array of bytes to dump to stdout
-     * @param dataLen the length of the data to be displayed
+     * @param data the array of bytes to dump to stdout.
      * @param itemsPerLine the number of bytes to display per line
      * if the {@code lineDelim} character is blank then all bytes will be
      * printed on a single line.
@@ -344,11 +333,11 @@ public class SimpleOCSPServer {
      *
      * @return The hexdump of the byte array
      */
-    private static String dumpHexBytes(byte[] data, int dataLen,
-            int itemsPerLine, String lineDelim, String itemDelim) {
+    private static String dumpHexBytes(byte[] data, int itemsPerLine,
+            String lineDelim, String itemDelim) {
         StringBuilder sb = new StringBuilder();
         if (data != null) {
-            for (int i = 0; i < dataLen; i++) {
+            for (int i = 0; i < data.length; i++) {
                 if (i % itemsPerLine == 0 && i != 0) {
                     sb.append(lineDelim);
                 }
@@ -498,7 +487,6 @@ public class SimpleOCSPServer {
             throws NoSuchAlgorithmException {
         if (!started) {
             sigAlgId = AlgorithmId.get(algName);
-            log("Signature algorithm set to " + sigAlgId.getName());
         }
     }
 
@@ -509,7 +497,7 @@ public class SimpleOCSPServer {
      * server has not yet been bound to a port.
      */
     public int getPort() {
-        if (serverReady.getCount() == 0) {
+        if (serverReady) {
             InetSocketAddress inetSock =
                     (InetSocketAddress)servSocket.getLocalSocketAddress();
             return inetSock.getPort();
@@ -519,21 +507,12 @@ public class SimpleOCSPServer {
     }
 
     /**
-     * Allow SimpleOCSPServer consumers to wait for the server to be in
-     * the ready state before sending requests.
+     * Use to check if OCSP server is ready to accept connection.
      *
-     * @param timeout the length of time to wait for the server to be ready
-     * @param unit the unit of time applied to the timeout parameter
-     *
-     * @return true if the server enters the ready state, false if the
-     *      timeout period elapses while the caller is waiting for the server
-     *      to become ready.
-     *
-     * @throws InterruptedException if the current thread is interrupted.
+     * @return true if server ready, false otherwise
      */
-    public boolean awaitServerReady(long timeout, TimeUnit unit)
-            throws InterruptedException {
-        return serverReady.await(timeout, unit);
+    public boolean isServerReady() {
+        return serverReady;
     }
 
     /**
@@ -552,21 +531,6 @@ public class SimpleOCSPServer {
         }
     }
 
-    /**
-     * Setting to control whether HTTP responses have the Content-Length
-     * field asserted or not.
-     *
-     * @param isDisabled true if the Content-Length field should not be
-     *        asserted, false otherwise.
-     */
-    public void setDisableContentLength(boolean isDisabled) {
-        if (!started) {
-            omitContentLength = isDisabled;
-            log("Response Content-Length field " +
-                    (isDisabled ? "disabled" : "enabled"));
-        }
-    }
-
     /**
      * Log a message to stdout.
      *
@@ -574,8 +538,8 @@ public class SimpleOCSPServer {
      */
     private synchronized void log(String message) {
         if (logEnabled || debug != null) {
-            System.out.println("[" + Thread.currentThread().getName() + "][" +
-                    System.currentTimeMillis() + "]: " + message);
+            System.out.println("[" + Thread.currentThread().getName() + "]: " +
+                    message);
         }
     }
 
@@ -585,7 +549,7 @@ public class SimpleOCSPServer {
      * @param message the message to log
      */
     private static synchronized void err(String message) {
-        System.err.println("[" + Thread.currentThread().getName() + "]: " +
+        System.out.println("[" + Thread.currentThread().getName() + "]: " +
                 message);
     }
 
@@ -704,9 +668,6 @@ public class SimpleOCSPServer {
      * responses.
      */
     private class OcspHandler implements Runnable {
-        private final boolean USE_GET =
-            !System.getProperty("com.sun.security.ocsp.useget", "").equals("false");
-
         private final Socket sock;
         InetSocketAddress peerSockAddr;
 
@@ -729,7 +690,6 @@ public class SimpleOCSPServer {
             // wait out the delay here before any other processing.
             try {
                 if (delayMsec > 0) {
-                    log("Delaying response for " + delayMsec + " milliseconds.");
                     Thread.sleep(delayMsec);
                 }
             } catch (InterruptedException ie) {
@@ -742,10 +702,6 @@ public class SimpleOCSPServer {
                     OutputStream out = ocspSocket.getOutputStream()) {
                 peerSockAddr =
                         (InetSocketAddress)ocspSocket.getRemoteSocketAddress();
-
-                // Read in the first line which will be the request line.
-                // This will be tokenized so we know if we are dealing with
-                // a GET or POST.
                 String[] headerTokens = readLine(in).split(" ");
                 LocalOcspRequest ocspReq = null;
                 LocalOcspResponse ocspResp = null;
@@ -754,12 +710,12 @@ public class SimpleOCSPServer {
                     if (headerTokens[0] != null) {
                         log("Received incoming HTTP " + headerTokens[0] +
                                 " from " + peerSockAddr);
-                        switch (headerTokens[0].toUpperCase()) {
+                        switch (headerTokens[0]) {
                             case "POST":
                                 ocspReq = parseHttpOcspPost(in);
                                 break;
                             case "GET":
-                                ocspReq = parseHttpOcspGet(headerTokens, in);
+                                ocspReq = parseHttpOcspGet(headerTokens);
                                 break;
                             default:
                                 respStat = ResponseStatus.MALFORMED_REQUEST;
@@ -793,9 +749,6 @@ public class SimpleOCSPServer {
                     ocspResp = new LocalOcspResponse(respStat);
                 }
                 sendResponse(out, ocspResp);
-                out.flush();
-
-                log("Closing " + ocspSocket);
             } catch (IOException | CertificateException exc) {
                 err(exc);
             }
@@ -823,11 +776,8 @@ public class SimpleOCSPServer {
 
             sb.append("HTTP/1.0 200 OK\r\n");
             sb.append("Content-Type: application/ocsp-response\r\n");
-            if (!omitContentLength) {
-                sb.append("Content-Length: ").append(respBytes.length).
-                        append("\r\n");
-            }
-            sb.append("\r\n");
+            sb.append("Content-Length: ").append(respBytes.length);
+            sb.append("\r\n\r\n");
 
             out.write(sb.toString().getBytes("UTF-8"));
             out.write(respBytes);
@@ -880,12 +830,6 @@ public class SimpleOCSPServer {
             // Okay, make sure we got what we needed from the header, then
             // read the remaining OCSP Request bytes
             if (properContentType && length >= 0) {
-                if (USE_GET && length <= 255) {
-                    // Received a small POST request. Check that our client code properly
-                    // handled the relevant flag. We expect small GET requests, unless
-                    // explicitly disabled.
-                    throw new IOException("Should have received small GET, not POST.");
-                }
                 byte[] ocspBytes = new byte[length];
                 inStream.read(ocspBytes);
                 return new LocalOcspRequest(ocspBytes);
@@ -899,8 +843,6 @@ public class SimpleOCSPServer {
          *
          * @param headerTokens the individual String tokens from the first
          * line of the HTTP GET.
-         * @param inStream the input stream from the socket bound to this
-         * {@code OcspHandler}.
          *
          * @return the OCSP Request as a {@code LocalOcspRequest}
          *
@@ -909,33 +851,8 @@ public class SimpleOCSPServer {
          * @throws CertificateException if one or more of the certificates in
          * the OCSP request cannot be read/parsed.
          */
-        private LocalOcspRequest parseHttpOcspGet(String[] headerTokens,
-                InputStream inStream) throws IOException, CertificateException {
-            // Display the whole request
-            StringBuilder sb = new StringBuilder("OCSP GET REQUEST\n");
-            for (String hTok : headerTokens) {
-                sb.append(hTok).append("\n");
-            }
-            log(sb.toString());
-
-            // Before we process the remainder of the GET URL, we should drain
-            // the InputStream of any other header data.  We (for now) won't
-            // use it, but will display the contents if logging is enabled.
-            boolean endOfHeader = false;
-            while (!endOfHeader) {
-                String[] lineTokens = readLine(inStream).split(":", 2);
-                // We expect to see a type and value pair delimited by a colon.
-                if (lineTokens[0].isEmpty()) {
-                    endOfHeader = true;
-                } else if (lineTokens.length == 2) {
-                    log(String.format("ReqHdr: %s: %s", lineTokens[0].trim(),
-                            lineTokens[1].trim()));
-                } else {
-                    // A colon wasn't found and token 0 should be the whole line
-                    log("ReqHdr: " + lineTokens[0].trim());
-                }
-            }
-
+        private LocalOcspRequest parseHttpOcspGet(String[] headerTokens)
+                throws IOException, CertificateException {
             // We have already established headerTokens[0] to be "GET".
             // We should have the URL-encoded base64 representation of the
             // OCSP request in headerTokens[1].  We need to strip any leading
@@ -1010,10 +927,6 @@ public class SimpleOCSPServer {
                 CertificateException {
             Objects.requireNonNull(requestBytes, "Received null input");
 
-            // Display the DER encoding before parsing
-            log("Local OCSP Request Constructor, parsing bytes:\n" +
-                    dumpHexBytes(requestBytes));
-
             DerInputStream dis = new DerInputStream(requestBytes);
 
             // Parse the top-level structure, it should have no more than
@@ -1260,14 +1173,10 @@ public class SimpleOCSPServer {
                 sb.append("CertId, Algorithm = ");
                 sb.append(cid.getHashAlgorithm()).append("\n");
                 sb.append("\tIssuer Name Hash: ");
-                byte[] cidHashBuf = cid.getIssuerNameHash();
-                sb.append(dumpHexBytes(cidHashBuf, cidHashBuf.length,
-                        256, "", ""));
+                sb.append(dumpHexBytes(cid.getIssuerNameHash(), 256, "", ""));
                 sb.append("\n");
                 sb.append("\tIssuer Key Hash: ");
-                cidHashBuf = cid.getIssuerKeyHash();
-                sb.append(dumpHexBytes(cidHashBuf, cidHashBuf.length,
-                        256, "", ""));
+                sb.append(dumpHexBytes(cid.getIssuerKeyHash(), 256, "", ""));
                 sb.append("\n");
                 sb.append("\tSerial Number: ").append(cid.getSerialNumber());
                 if (!extensions.isEmpty()) {
@@ -1607,14 +1516,10 @@ public class SimpleOCSPServer {
                 sb.append("CertId, Algorithm = ");
                 sb.append(certId.getHashAlgorithm()).append("\n");
                 sb.append("\tIssuer Name Hash: ");
-                byte[] cidHashBuf = certId.getIssuerNameHash();
-                sb.append(dumpHexBytes(cidHashBuf, cidHashBuf.length,
-                        256, "", ""));
+                sb.append(dumpHexBytes(certId.getIssuerNameHash(), 256, "", ""));
                 sb.append("\n");
                 sb.append("\tIssuer Key Hash: ");
-                cidHashBuf = certId.getIssuerKeyHash();
-                sb.append(dumpHexBytes(cidHashBuf, cidHashBuf.length,
-                        256, "", ""));
+                sb.append(dumpHexBytes(certId.getIssuerKeyHash(), 256, "", ""));
                 sb.append("\n");
                 sb.append("\tSerial Number: ").append(certId.getSerialNumber());
                 sb.append("\n");
