@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -26,15 +26,13 @@
  * @bug 4035266 4052418 4068133 4068137 4068139 4086052 4095322 4097779
  *      4097920 4098467 4111338 4113835 4117554 4143071 4146175 4152117
  *      4152416 4153072 4158381 4214367 4217703 4638433 8264765
- * @run junit/timeout=2000 BreakIteratorTest
+ * @library /java/text/testlib
+ * @run main/timeout=2000 BreakIteratorTest
  * @summary test BreakIterator
  */
 
 /*
- * This file is available under and governed by the GNU General Public
- * License version 2 only, as published by the Free Software Foundation.
- * However, the following notice accompanied the original version of this
- * file and, per its terms, should not be removed:
+ *
  *
  * (C) Copyright Taligent, Inc. 1996, 1997 - All Rights Reserved
  * (C) Copyright IBM Corp. 1996 - 1998 - All Rights Reserved
@@ -74,16 +72,24 @@ import java.util.Vector;
 import java.util.Enumeration;
 import java.io.*;
 
-import org.junit.jupiter.api.Test;
+public class BreakIteratorTest extends IntlTest
+{
+    private BreakIterator characterBreak;
+    private BreakIterator wordBreak;
+    private BreakIterator lineBreak;
+    private BreakIterator sentenceBreak;
 
-import static org.junit.jupiter.api.Assertions.fail;
-
-public class BreakIteratorTest {
-    private final BreakIterator characterBreak = BreakIterator.getCharacterInstance();
-    private final BreakIterator wordBreak = BreakIterator.getWordInstance();
-    private final BreakIterator lineBreak = BreakIterator.getLineInstance();
-    private final BreakIterator sentenceBreak = BreakIterator.getSentenceInstance();
+    public static void main(String[] args) throws Exception {
+        new BreakIteratorTest().run(args);
+    }
 
+    public BreakIteratorTest()
+    {
+        characterBreak = BreakIterator.getCharacterInstance();
+        wordBreak = BreakIterator.getWordInstance();
+        lineBreak = BreakIterator.getLineInstance();
+        sentenceBreak = BreakIterator.getSentenceInstance();
+    }
 
     //=========================================================================
     // general test subroutines
@@ -103,12 +109,15 @@ public class BreakIteratorTest {
         Vector nextResults = testFirstAndNext(bi, text);
         Vector previousResults = testLastAndPrevious(bi, text);
 
-        System.out.println("comparing forward and backward...");
+        logln("comparing forward and backward...");
+        int errs = getErrorCount();
         compareFragmentLists("forward iteration", "backward iteration", nextResults,
                         previousResults);
-        System.out.println("comparing expected and actual...");
-        compareFragmentLists("expected result", "actual result", expectedResult,
-                        nextResults);
+        if (getErrorCount() == errs) {
+            logln("comparing expected and actual...");
+            compareFragmentLists("expected result", "actual result", expectedResult,
+                            nextResults);
+        }
 
         int[] boundaries = new int[expectedResult.size() + 3];
         boundaries[0] = BreakIterator.DONE;
@@ -131,19 +140,19 @@ public class BreakIteratorTest {
         Vector<String> result = new Vector<String>();
 
         if (p != 0)
-            fail("first() returned " + p + " instead of 0");
+            errln("first() returned " + p + " instead of 0");
         while (p != BreakIterator.DONE) {
             p = bi.next();
             if (p != BreakIterator.DONE) {
                 if (p <= lastP)
-                    fail("next() failed to move forward: next() on position "
+                    errln("next() failed to move forward: next() on position "
                                     + lastP + " yielded " + p);
 
                 result.addElement(text.substring(lastP, p));
             }
             else {
                 if (lastP != text.length())
-                    fail("next() returned DONE prematurely: offset was "
+                    errln("next() returned DONE prematurely: offset was "
                                     + lastP + " instead of " + text.length());
             }
             lastP = p;
@@ -157,19 +166,19 @@ public class BreakIteratorTest {
         Vector<String> result = new Vector<String>();
 
         if (p != text.length())
-            fail("last() returned " + p + " instead of " + text.length());
+            errln("last() returned " + p + " instead of " + text.length());
         while (p != BreakIterator.DONE) {
             p = bi.previous();
             if (p != BreakIterator.DONE) {
                 if (p >= lastP)
-                    fail("previous() failed to move backward: previous() on position "
+                    errln("previous() failed to move backward: previous() on position "
                                     + lastP + " yielded " + p);
 
                 result.insertElementAt(text.substring(p, lastP), 0);
             }
             else {
                 if (lastP != 0)
-                    fail("previous() returned DONE prematurely: offset was "
+                    errln("previous() returned DONE prematurely: offset was "
                                     + lastP + " instead of 0");
             }
             lastP = p;
@@ -212,28 +221,27 @@ public class BreakIteratorTest {
                         ++tempP2;
                     }
                 }
-                System.out.println("*** " + f1Name + " has:");
+                logln("*** " + f1Name + " has:");
                 while (p1 <= tempP1 && p1 < f1.size()) {
                     s1 = (String)f1.elementAt(p1);
                     t1 += s1.length();
                     debugLogln(" *** >" + s1 + "<");
                     ++p1;
                 }
-                System.out.println("***** " + f2Name + " has:");
+                logln("***** " + f2Name + " has:");
                 while (p2 <= tempP2 && p2 < f2.size()) {
                     s2 = (String)f2.elementAt(p2);
                     t2 += s2.length();
                     debugLogln(" ***** >" + s2 + "<");
                     ++p2;
                 }
-                fail("Discrepancy between " + f1Name + " and " + f2Name
-                        + "\n---\n" + f1 +"\n---\n" + f2);
+                errln("Discrepancy between " + f1Name + " and " + f2Name + "\n---\n" + f1 +"\n---\n" + f2);
             }
         }
     }
 
     private void testFollowing(BreakIterator bi, String text, int[] boundaries) {
-        System.out.println("testFollowing():");
+        logln("testFollowing():");
         int p = 2;
         int i = 0;
         try {
@@ -242,59 +250,59 @@ public class BreakIteratorTest {
                     ++p;
 
                 int b = bi.following(i);
-                System.out.println("bi.following(" + i + ") -> " + b);
+                logln("bi.following(" + i + ") -> " + b);
                 if (b != boundaries[p])
-                    fail("Wrong result from following() for " + i + ": expected " + boundaries[p]
+                    errln("Wrong result from following() for " + i + ": expected " + boundaries[p]
                           + ", got " + b);
             }
         } catch (IllegalArgumentException illargExp) {
-            fail("IllegalArgumentException caught from following() for offset: " + i);
+            errln("IllegalArgumentException caught from following() for offset: " + i);
         }
     }
 
     private void testPreceding(BreakIterator bi, String text, int[] boundaries) {
-        System.out.println("testPreceding():");
+        logln("testPreceding():");
         int p = 0;
         int i = 0;
         try {
             for (i = 0; i <= text.length(); i++) {  // change to <= when new BI code goes in
                 int b = bi.preceding(i);
-                System.out.println("bi.preceding(" + i + ") -> " + b);
+                logln("bi.preceding(" + i + ") -> " + b);
                 if (b != boundaries[p])
-                    fail("Wrong result from preceding() for " + i + ": expected " + boundaries[p]
+                    errln("Wrong result from preceding() for " + i + ": expected " + boundaries[p]
                           + ", got " + b);
 
                 if (i == boundaries[p + 1])
                     ++p;
             }
         } catch (IllegalArgumentException illargExp) {
-            fail("IllegalArgumentException caught from preceding() for offset: " + i);
+            errln("IllegalArgumentException caught from preceding() for offset: " + i);
         }
     }
 
     private void testIsBoundary(BreakIterator bi, String text, int[] boundaries) {
-        System.out.println("testIsBoundary():");
+        logln("testIsBoundary():");
         int p = 1;
         boolean isB;
         for (int i = 0; i <= text.length(); i++) {  // change to <= when new BI code goes in
             isB = bi.isBoundary(i);
-            System.out.println("bi.isBoundary(" + i + ") -> " + isB);
+            logln("bi.isBoundary(" + i + ") -> " + isB);
 
             if (i == boundaries[p]) {
                 if (!isB)
-                    fail("Wrong result from isBoundary() for " + i + ": expected true, got false");
+                    errln("Wrong result from isBoundary() for " + i + ": expected true, got false");
                 ++p;
             }
             else {
                 if (isB)
-                    fail("Wrong result from isBoundary() for " + i + ": expected false, got true");
+                    errln("Wrong result from isBoundary() for " + i + ": expected false, got true");
             }
         }
     }
 
     private void doMultipleSelectionTest(BreakIterator iterator, String testText)
     {
-        System.out.println("Multiple selection test...");
+        logln("Multiple selection test...");
         BreakIterator testIterator = (BreakIterator)iterator.clone();
         int offset = iterator.first();
         int testOffset;
@@ -303,9 +311,9 @@ public class BreakIteratorTest {
         do {
             testOffset = testIterator.first();
             testOffset = testIterator.next(count);
-            System.out.println("next(" + count + ") -> " + testOffset);
+            logln("next(" + count + ") -> " + testOffset);
             if (offset != testOffset)
-                fail("next(n) and next() not returning consistent results: for step " + count + ", next(n) returned " + testOffset + " and next() had " + offset);
+                errln("next(n) and next() not returning consistent results: for step " + count + ", next(n) returned " + testOffset + " and next() had " + offset);
 
             if (offset != BreakIterator.DONE) {
                 count++;
@@ -320,9 +328,9 @@ public class BreakIteratorTest {
         do {
             testOffset = testIterator.last();
             testOffset = testIterator.next(count);
-            System.out.println("next(" + count + ") -> " + testOffset);
+            logln("next(" + count + ") -> " + testOffset);
             if (offset != testOffset)
-                fail("next(n) and next() not returning consistent results: for step " + count + ", next(n) returned " + testOffset + " and next() had " + offset);
+                errln("next(n) and next() not returning consistent results: for step " + count + ", next(n) returned " + testOffset + " and next() had " + offset);
 
             if (offset != BreakIterator.DONE) {
                 count--;
@@ -368,7 +376,7 @@ public class BreakIteratorTest {
                             seen2 = true;
                     }
                     if (!seen2) {
-                        fail("No break between U+" + Integer.toHexString((int)(work.charAt(1)))
+                        errln("No break between U+" + Integer.toHexString((int)(work.charAt(1)))
                                     + " and U+" + Integer.toHexString((int)(work.charAt(2))));
                         errorCount++;
                         if (errorCount >= 75)
@@ -392,7 +400,7 @@ public class BreakIteratorTest {
                 tb.setText(work.toString());
                 for (int k = tb.first(); k != BreakIterator.DONE; k = tb.next())
                     if (k == 2) {
-                        fail("Break between CR and LF in string U+" + Integer.toHexString(
+                        errln("Break between CR and LF in string U+" + Integer.toHexString(
                                 (int)(work.charAt(0))) + ", U+d U+a U+" + Integer.toHexString(
                                 (int)(work.charAt(3))));
                         errorCount++;
@@ -430,7 +438,7 @@ public class BreakIteratorTest {
                 tb.setText(work.toString());
                 for (int k = tb.first(); k != BreakIterator.DONE; k = tb.next())
                     if (k == 2) {
-                        fail("Break between U+" + Integer.toHexString((int)(work.charAt(1)))
+                        errln("Break between U+" + Integer.toHexString((int)(work.charAt(1)))
                                 + " and U+" + Integer.toHexString((int)(work.charAt(2))));
                         errorCount++;
                         if (errorCount >= 75)
@@ -455,14 +463,13 @@ public class BreakIteratorTest {
                 out.append(temp);
             }
         }
-        System.out.println(out.toString());
+        logln(out.toString());
     }
 
     //=========================================================================
     // tests
     //=========================================================================
 
-    @Test
     public void TestWordBreak() {
 
         Vector<String> wordSelectionData = new Vector<String>();
@@ -546,7 +553,6 @@ public class BreakIteratorTest {
         generalIteratorTest(wordBreak, wordSelectionData);
     }
 
-    @Test
     public void TestBug4097779() {
         Vector<String> wordSelectionData = new Vector<String>();
 
@@ -556,7 +562,6 @@ public class BreakIteratorTest {
         generalIteratorTest(wordBreak, wordSelectionData);
     }
 
-    @Test
     public void TestBug4098467Words() {
         Vector<String> wordSelectionData = new Vector<String>();
 
@@ -586,7 +591,6 @@ public class BreakIteratorTest {
         generalIteratorTest(wordBreak, wordSelectionData);
     }
 
-    @Test
     public void TestBug4117554Words() {
         Vector<String> wordSelectionData = new Vector<String>();
 
@@ -599,7 +603,6 @@ public class BreakIteratorTest {
         generalIteratorTest(wordBreak, wordSelectionData);
     }
 
-    @Test
     public void TestSentenceBreak() {
         Vector<String> sentenceSelectionData = new Vector<String>();
 
@@ -635,7 +638,6 @@ public class BreakIteratorTest {
         generalIteratorTest(sentenceBreak, sentenceSelectionData);
     }
 
-    @Test
     public void TestBug4113835() {
         Vector<String> sentenceSelectionData = new Vector<String>();
 
@@ -645,7 +647,6 @@ public class BreakIteratorTest {
         generalIteratorTest(sentenceBreak, sentenceSelectionData);
     }
 
-    @Test
     public void TestBug4111338() {
         Vector<String> sentenceSelectionData = new Vector<String>();
 
@@ -665,7 +666,6 @@ public class BreakIteratorTest {
         generalIteratorTest(sentenceBreak, sentenceSelectionData);
     }
 
-    @Test
     public void TestBug4117554Sentences() {
         Vector<String> sentenceSelectionData = new Vector<String>();
 
@@ -688,7 +688,6 @@ public class BreakIteratorTest {
         generalIteratorTest(sentenceBreak, sentenceSelectionData);
     }
 
-    @Test
     public void TestBug4158381() {
         Vector<String> sentenceSelectionData = new Vector<String>();
 
@@ -706,7 +705,6 @@ public class BreakIteratorTest {
         generalIteratorTest(sentenceBreak, sentenceSelectionData);
     }
 
-    @Test
     public void TestBug4143071() {
         Vector<String> sentenceSelectionData = new Vector<String>();
 
@@ -718,7 +716,6 @@ public class BreakIteratorTest {
         generalIteratorTest(sentenceBreak, sentenceSelectionData);
     }
 
-    @Test
     public void TestBug4152416() {
         Vector<String> sentenceSelectionData = new Vector<String>();
 
@@ -731,7 +728,6 @@ public class BreakIteratorTest {
         generalIteratorTest(sentenceBreak, sentenceSelectionData);
     }
 
-    @Test
     public void TestBug4152117() {
         Vector<String> sentenceSelectionData = new Vector<String>();
 
@@ -750,7 +746,6 @@ public class BreakIteratorTest {
         generalIteratorTest(sentenceBreak, sentenceSelectionData);
     }
 
-    @Test
     public void TestBug8264765() {
         Vector<String> sentenceSelectionData = new Vector<String>();
 
@@ -762,7 +757,6 @@ public class BreakIteratorTest {
         generalIteratorTest(sentenceBreak, sentenceSelectionData);
     }
 
-    @Test
     public void TestLineBreak() {
         Vector<String> lineSelectionData = new Vector<String>();
 
@@ -799,7 +793,6 @@ public class BreakIteratorTest {
         generalIteratorTest(lineBreak, lineSelectionData);
     }
 
-    @Test
     public void TestBug4068133() {
         Vector<String> lineSelectionData = new Vector<String>();
 
@@ -816,7 +809,6 @@ public class BreakIteratorTest {
         generalIteratorTest(lineBreak, lineSelectionData);
     }
 
-    @Test
     public void TestBug4086052() {
         Vector<String> lineSelectionData = new Vector<String>();
 
@@ -826,7 +818,6 @@ public class BreakIteratorTest {
         generalIteratorTest(lineBreak, lineSelectionData);
     }
 
-    @Test
     public void TestBug4097920() {
         Vector<String> lineSelectionData = new Vector<String>();
 
@@ -839,7 +830,6 @@ public class BreakIteratorTest {
         generalIteratorTest(lineBreak, lineSelectionData);
     }
     /*
-    @Test
     public void TestBug4035266() {
         Vector<String> lineSelectionData = new Vector<String>();
 
@@ -853,7 +843,6 @@ public class BreakIteratorTest {
         generalIteratorTest(lineBreak, lineSelectionData);
     }
     */
-    @Test
     public void TestBug4098467Lines() {
         Vector<String> lineSelectionData = new Vector<String>();
 
@@ -879,14 +868,13 @@ public class BreakIteratorTest {
         lineSelectionData.addElement("\u110c\u1161\u11bc\u1105\u1169\u1100\u116d\u1112\u116c");
 
         if (Locale.getDefault().getLanguage().equals("th")) {
-            System.out.println("This test is skipped in th locale.");
+            logln("This test is skipped in th locale.");
             return;
         }
 
         generalIteratorTest(lineBreak, lineSelectionData);
     }
 
-    @Test
     public void TestBug4117554Lines() {
         Vector<String> lineSelectionData = new Vector<String>();
 
@@ -898,10 +886,9 @@ public class BreakIteratorTest {
         generalIteratorTest(lineBreak, lineSelectionData);
     }
 
-    @Test
     public void TestBug4217703() {
         if (Locale.getDefault().getLanguage().equals("th")) {
-            System.out.println("This test is skipped in th locale.");
+            logln("This test is skipped in th locale.");
             return;
         }
 
@@ -928,7 +915,6 @@ public class BreakIteratorTest {
     private static final String circumflexA = "a\u0302";
     private static final String tildeE = "e\u0303";
 
-    @Test
     public void TestCharacterBreak() {
         Vector<String> characterSelectionData = new Vector<String>();
 
@@ -960,7 +946,6 @@ public class BreakIteratorTest {
         generalIteratorTest(characterBreak, characterSelectionData);
     }
 
-    @Test
     public void TestBug4098467Characters() {
         Vector<String> characterSelectionData = new Vector<String>();
 
@@ -1001,7 +986,6 @@ public class BreakIteratorTest {
         generalIteratorTest(characterBreak, characterSelectionData);
     }
 
-    @Test
     public void TestBug4153072() {
         BreakIterator iter = BreakIterator.getWordInstance();
         String str = "...Hello, World!...";
@@ -1015,18 +999,17 @@ public class BreakIteratorTest {
             try {
                 dummy = iter.isBoundary(index);
                 if (index < begin)
-                    fail("Didn't get exception with offset = " + index +
+                    errln("Didn't get exception with offset = " + index +
                                     " and begin index = " + begin);
             }
             catch (IllegalArgumentException e) {
                 if (index >= begin)
-                    fail("Got exception with offset = " + index +
+                    errln("Got exception with offset = " + index +
                                     " and begin index = " + begin);
             }
         }
     }
 
-    @Test
     public void TestBug4146175Sentences() {
         Vector<String> sentenceSelectionData = new Vector<String>();
 
@@ -1042,10 +1025,9 @@ public class BreakIteratorTest {
         generalIteratorTest(sentenceBreak, sentenceSelectionData);
     }
 
-    @Test
     public void TestBug4146175Lines() {
         if (Locale.getDefault().getLanguage().equals("th")) {
-            System.out.println("This test is skipped in th locale.");
+            logln("This test is skipped in th locale.");
             return;
         }
 
@@ -1058,10 +1040,9 @@ public class BreakIteratorTest {
         generalIteratorTest(lineBreak, lineSelectionData);
     }
 
-    @Test
     public void TestBug4214367() {
         if (Locale.getDefault().getLanguage().equals("th")) {
-            System.out.println("This test is skipped in th locale.");
+            logln("This test is skipped in th locale.");
             return;
         }
 
@@ -1083,18 +1064,16 @@ public class BreakIteratorTest {
         + "\u2001\u2002\u200c\u200d\u200e\u200f\u2010\u2011\u2012\u2028\u2029\u202a\u203e\u203f"
         + "\u2040\u20dd\u20de\u20df\u20e0\u2160\u2161\u2162\u2163\u2164";
 
-    @Test
     public void TestSentenceInvariants()
     {
         BreakIterator e = BreakIterator.getSentenceInstance();
         doOtherInvariantTest(e, cannedTestChars + ".,\u3001\u3002\u3041\u3042\u3043\ufeff");
     }
 
-    @Test
     public void TestWordInvariants()
     {
         if (Locale.getDefault().getLanguage().equals("th")) {
-            System.out.println("This test is skipped in th locale.");
+            logln("This test is skipped in th locale.");
             return;
         }
 
@@ -1105,11 +1084,10 @@ public class BreakIteratorTest {
             + "\u30a3\u4e00\u4e01\u4e02");
     }
 
-    @Test
     public void TestLineInvariants()
     {
         if (Locale.getDefault().getLanguage().equals("th")) {
-            System.out.println("This test is skipped in th locale.");
+            logln("This test is skipped in th locale.");
             return;
         }
 
@@ -1158,7 +1136,7 @@ public class BreakIteratorTest {
                                                                  work.charAt(l) == '\ufeff')) {
                                 continue;
                             }
-                            fail("Got break between U+" + Integer.toHexString((int)
+                            errln("Got break between U+" + Integer.toHexString((int)
                                     (work.charAt(l - 1))) + " and U+" + Integer.toHexString(
                                     (int)(work.charAt(l))));
                             errorCount++;
@@ -1211,7 +1189,7 @@ public class BreakIteratorTest {
                         if (l == 2)
                             saw2 = true;
                     if (!saw2) {
-                        fail("Didn't get break between U+" + Integer.toHexString((int)
+                        errln("Didn't get break between U+" + Integer.toHexString((int)
                                     (work.charAt(1))) + " and U+" + Integer.toHexString(
                                     (int)(work.charAt(2))));
                         errorCount++;
@@ -1224,7 +1202,6 @@ public class BreakIteratorTest {
         */
     }
 
-    @Test
     public void TestCharacterInvariants()
     {
         BreakIterator e = BreakIterator.getCharacterInstance();
@@ -1234,7 +1211,6 @@ public class BreakIteratorTest {
             + "\u11a9\u11aa");
     }
 
-    @Test
     public void TestEmptyString()
     {
         String text = "";
@@ -1244,13 +1220,12 @@ public class BreakIteratorTest {
         generalIteratorTest(lineBreak, x);
     }
 
-    @Test
     public void TestGetAvailableLocales()
     {
         Locale[] locList = BreakIterator.getAvailableLocales();
 
         if (locList.length == 0)
-            fail("getAvailableLocales() returned an empty list!");
+            errln("getAvailableLocales() returned an empty list!");
         // I have no idea how to test this function...
     }
 
@@ -1258,7 +1233,6 @@ public class BreakIteratorTest {
     /**
      * Bug 4095322
      */
-    @Test
     public void TestJapaneseLineBreak()
     {
         StringBuffer testString = new StringBuffer("\u4e00x\u4e8c");
@@ -1307,18 +1281,18 @@ public class BreakIteratorTest {
             iter.setText(testString.toString());
             int j = iter.first();
             if (j != 0) {
-                fail("ja line break failure: failed to start at 0 and bounced at " + j);
+                errln("ja line break failure: failed to start at 0 and bounced at " + j);
             }
             j = iter.next();
             if (j != 1) {
-                fail("ja line break failure: failed to stop before '"
+                errln("ja line break failure: failed to stop before '"
                         + precedingChars.charAt(i) + "' (\\u"
                         + Integer.toString(precedingChars.charAt(i), 16)
                         + ") at 1 and bounded at " + j);
             }
             j = iter.next();
             if (j != 3) {
-                fail("ja line break failure: failed to skip position after '"
+                errln("ja line break failure: failed to skip position after '"
                         + precedingChars.charAt(i) + "' (\\u"
                         + Integer.toString(precedingChars.charAt(i), 16)
                         + ") at 3 and bounded at " + j);
@@ -1330,18 +1304,18 @@ public class BreakIteratorTest {
             iter.setText(testString.toString());
             int j = iter.first();
             if (j != 0) {
-                fail("ja line break failure: failed to start at 0 and bounded at " + j);
+                errln("ja line break failure: failed to start at 0 and bounded at " + j);
             }
             j = iter.next();
             if (j != 2) {
-                fail("ja line break failure: failed to skip position before '"
+                errln("ja line break failure: failed to skip position before '"
                         + followingChars.charAt(i) + "' (\\u"
                         + Integer.toString(followingChars.charAt(i), 16)
                         + ") at 2 and bounded at " + j);
             }
             j = iter.next();
             if (j != 3) {
-                fail("ja line break failure: failed to stop after '"
+                errln("ja line break failure: failed to stop after '"
                         + followingChars.charAt(i) + "' (\\u"
                         + Integer.toString(followingChars.charAt(i), 16)
                         + ") at 3 and bounded at " + j);
@@ -1352,7 +1326,6 @@ public class BreakIteratorTest {
     /**
      * Bug 4638433
      */
-    @Test
     public void TestLineBreakBasedOnUnicode3_0_0()
     {
         BreakIterator iter;
@@ -1366,7 +1339,7 @@ public class BreakIteratorTest {
         i = iter.first();
         i = iter.next();
         if (i != 5) {
-            fail("Word break failure: failed to stop at 5 and bounded at " + i);
+            errln("Word break failure: failed to stop at 5 and bounded at " + i);
         }
 
 
@@ -1379,7 +1352,7 @@ public class BreakIteratorTest {
         i = iter.first();
         i = iter.next();
         if (i != 3) {
-            fail("Line break failure: failed to skip before \\u301F(Pe) at 3 and bounded at " + i);
+            errln("Line break failure: failed to skip before \\u301F(Pe) at 3 and bounded at " + i);
         }
 
         /* Mongolian <Letter A(Lo)><Todo Soft Hyphen(Pd)><Letter E(Lo)>
@@ -1389,7 +1362,7 @@ public class BreakIteratorTest {
         i = iter.first();
         i = iter.next();
         if (i != 2) {
-            fail("Mongolian line break failure: failed to skip position before \\u1806(Pd) at 2 and bounded at " + i);
+            errln("Mongolian line break failure: failed to skip position before \\u1806(Pd) at 2 and bounded at " + i);
         }
 
         /* Khmer <ZERO(Nd)><Currency Symbol(Sc)><ONE(Nd)> which have
@@ -1399,11 +1372,11 @@ public class BreakIteratorTest {
         i = iter.first();
         i = iter.next();
         if (i != 1) {
-            fail("Khmer line break failure: failed to stop before \\u17DB(Sc) at 1 and bounded at " + i);
+            errln("Khmer line break failure: failed to stop before \\u17DB(Sc) at 1 and bounded at " + i);
         }
         i = iter.next();
         if (i != 3) {
-            fail("Khmer line break failure: failed to skip position after \\u17DB(Sc) at 3 and bounded at " + i);
+            errln("Khmer line break failure: failed to skip position after \\u17DB(Sc) at 3 and bounded at " + i);
         }
 
         /* Ogham <Letter UR(Lo)><Space Mark(Zs)><Letter OR(Lo)> which have
@@ -1413,7 +1386,7 @@ public class BreakIteratorTest {
         i = iter.first();
         i = iter.next();
         if (i != 2) {
-            fail("Ogham line break failure: failed to skip postion before \\u1680(Zs) at 2 and bounded at " + i);
+            errln("Ogham line break failure: failed to skip postion before \\u1680(Zs) at 2 and bounded at " + i);
         }
 
 
@@ -1430,18 +1403,17 @@ public class BreakIteratorTest {
         i = iter.first();
         i = iter.next();
         if (i != 1) {
-            fail("Thai line break failure: failed to stop before \\u201C(Pi) at 1 and bounded at " + i);
+            errln("Thai line break failure: failed to stop before \\u201C(Pi) at 1 and bounded at " + i);
         }
         i = iter.next();
         if (i != 4) {
-            fail("Thai line break failure: failed to stop after \\u201D(Pf) at 4 and bounded at " + i);
+            errln("Thai line break failure: failed to stop after \\u201D(Pf) at 4 and bounded at " + i);
         }
     }
 
     /**
      * Bug 4068137
      */
-    @Test
     public void TestEndBehavior()
     {
         String testString = "boo.";
@@ -1449,11 +1421,11 @@ public class BreakIteratorTest {
         wb.setText(testString);
 
         if (wb.first() != 0)
-            fail("Didn't get break at beginning of string.");
+            errln("Didn't get break at beginning of string.");
         if (wb.next() != 3)
-            fail("Didn't get break before period in \"boo.\"");
+            errln("Didn't get break before period in \"boo.\"");
         if (wb.current() != 4 && wb.next() != 4)
-            fail("Didn't get break at end of string.");
+            errln("Didn't get break at end of string.");
     }
 
     // [serialization test has been removed pursuant to bug #4152965]
@@ -1461,7 +1433,6 @@ public class BreakIteratorTest {
     /**
      * Bug 4450804
      */
-    @Test
     public void TestLineBreakContractions() {
         Vector<String> expected = new Vector<String>();
 
