@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2025, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -30,19 +30,18 @@ import jdk.jfr.consumer.RecordedEvent;
 import jdk.test.lib.Asserts;
 import jdk.test.lib.jfr.EventNames;
 import jdk.test.lib.jfr.Events;
-import jdk.test.whitebox.WhiteBox;
-import jdk.test.whitebox.code.BlobType;
+import sun.hotspot.WhiteBox;
+import sun.hotspot.code.BlobType;
 
 /**
  * @test TestCodeCacheFull
  * @requires vm.hasJFR
- * @requires vm.opt.UseCodeCacheFlushing == null | vm.opt.UseCodeCacheFlushing == true
  *
  * @library /test/lib
  * @modules jdk.jfr
  *          jdk.management.jfr
- * @build jdk.test.whitebox.WhiteBox
- * @run driver jdk.test.lib.helpers.ClassFileInstaller jdk.test.whitebox.WhiteBox
+ * @build sun.hotspot.WhiteBox
+ * @run driver jdk.test.lib.helpers.ClassFileInstaller sun.hotspot.WhiteBox
  *
  * @run main/othervm -Xbootclasspath/a:.
  *     -XX:+UnlockDiagnosticVMOptions -XX:+WhiteBoxAPI
@@ -80,7 +79,6 @@ public class TestCodeCacheFull {
         Events.assertField(event, "startAddress").notEqual(0L);
         Events.assertField(event, "commitedTopAddress").notEqual(0L);
         Events.assertField(event, "reservedTopAddress").notEqual(0L);
-        Events.assertField(event, "codeCacheMaxCapacity").notEqual(0L);
     }
 
     private static BlobType blobTypeFromName(String codeBlobTypeName) throws Exception {
