@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -27,13 +27,10 @@ import java.io.BufferedInputStream;
 import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.File;
-import java.nio.charset.Charset;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.Paths;
 import java.nio.file.StandardOpenOption;
-import java.time.Instant;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
@@ -46,8 +43,6 @@ import java.util.concurrent.TimeUnit;
 import java.util.function.Supplier;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
-import java.util.stream.Collectors;
-import java.util.stream.Stream;
 
 import jdk.test.whitebox.code.Compiler;
 import jdk.test.whitebox.cpuinfo.CPUInfo;
@@ -82,10 +77,6 @@ public class VMProps implements Callable<Map<String, String>> {
             }
             map.put(key, value);
         }
-
-        public void putAll(Map<String, String> map) {
-            map.entrySet().forEach(e -> put(e.getKey(), () -> e.getValue()));
-        }
     }
 
     /**
@@ -96,7 +87,6 @@ public class VMProps implements Callable<Map<String, String>> {
      */
     @Override
     public Map<String, String> call() {
-        log("Entering call()");
         SafeMap map = new SafeMap();
         map.put("vm.flavor", this::vmFlavor);
         map.put("vm.compMode", this::vmCompMode);
@@ -105,7 +95,6 @@ public class VMProps implements Callable<Map<String, String>> {
         map.put("vm.simpleArch", this::vmArch);
         map.put("vm.debug", this::vmDebug);
         map.put("vm.jvmci", this::vmJvmci);
-        map.put("vm.jvmci.enabled", this::vmJvmciEnabled);
         map.put("vm.emulatedClient", this::vmEmulatedClient);
         // vm.hasSA is "true" if the VM contains the serviceability agent
         // and jhsdb.
@@ -113,7 +102,6 @@ public class VMProps implements Callable<Map<String, String>> {
         // vm.hasJFR is "true" if JFR is included in the build of the VM and
         // so tests can be executed.
         map.put("vm.hasJFR", this::vmHasJFR);
-        map.put("vm.hasDTrace", this::vmHasDTrace);
         map.put("vm.jvmti", this::vmHasJVMTI);
         map.put("vm.cpu.features", this::cpuFeatures);
         map.put("vm.pageSize", this::vmPageSize);
@@ -127,17 +115,16 @@ public class VMProps implements Callable<Map<String, String>> {
         map.put("vm.graal.enabled", this::isGraalEnabled);
         map.put("vm.compiler1.enabled", this::isCompiler1Enabled);
         map.put("vm.compiler2.enabled", this::isCompiler2Enabled);
-        map.put("container.support", this::containerSupport);
+        map.put("docker.support", this::dockerSupport);
         map.put("vm.musl", this::isMusl);
         map.put("release.implementor", this::implementor);
         map.put("jdk.containerized", this::jdkContainerized);
         map.put("vm.flagless", this::isFlagless);
-        map.putAll(xOptFlags()); // -Xmx4g -> @requires vm.opt.x.Xmx == "4g" )
         vmGC(map); // vm.gc.X = true/false
+        vmGCforCDS(map); // may set vm.gc
         vmOptFinalFlags(map);
 
         dump(map.map);
-        log("Leaving call()");
         return map.map;
     }
 
@@ -268,20 +255,6 @@ public class VMProps implements Callable<Map<String, String>> {
         return "true";
     }
 
-
-    /**
-     * @return true if JVMCI is enabled
-     */
-    protected String vmJvmciEnabled() {
-        // builds with jvmci have this flag
-        if ("false".equals(vmJvmci())) {
-            return "false";
-        }
-
-        return "" + Compiler.isJVMCIEnabled();
-    }
-
-
     /**
      * @return true if VM runs in emulated-client mode and false otherwise.
      */
@@ -319,6 +292,34 @@ public class VMProps implements Callable<Map<String, String>> {
         }
     }
 
+    /**
+     * "jtreg -vmoptions:-Dtest.cds.runtime.options=..." can be used to specify
+     * the GC type to be used when running with a CDS archive. Set "vm.gc" accordingly,
+     * so that tests that need to explicitly choose the GC type can be excluded
+     * with "@requires vm.gc == null".
+     *
+     * @param map - property-value pairs
+     */
+    protected void vmGCforCDS(SafeMap map) {
+        if (!GC.isSelectedErgonomically()) {
+            // The GC has been explicitly specified on the command line, so
+            // jtreg will set the "vm.gc" property. Let's not interfere with it.
+            return;
+        }
+
+        String GC_PREFIX  = "-XX:+Use";
+        String GC_SUFFIX  = "GC";
+        String jtropts = System.getProperty("test.cds.runtime.options");
+        if (jtropts != null) {
+            for (String opt : jtropts.split(",")) {
+                if (opt.startsWith(GC_PREFIX) && opt.endsWith(GC_SUFFIX)) {
+                    String gc = opt.substring(GC_PREFIX.length(), opt.length() - GC_SUFFIX.length());
+                    map.put("vm.gc", () -> gc);
+                }
+            }
+        }
+    }
+
     /**
      * Selected final flag.
      *
@@ -339,7 +340,6 @@ public class VMProps implements Callable<Map<String, String>> {
         vmOptFinalFlag(map, "ClassUnloading");
         vmOptFinalFlag(map, "ClassUnloadingWithConcurrentMark");
         vmOptFinalFlag(map, "UseCompressedOops");
-        vmOptFinalFlag(map, "UseLargePages");
         vmOptFinalFlag(map, "UseVectorizedMismatchIntrinsic");
         vmOptFinalFlag(map, "EnableJVMCI");
         vmOptFinalFlag(map, "EliminateAllocations");
@@ -368,13 +368,6 @@ public class VMProps implements Callable<Map<String, String>> {
         return "" + WB.isJVMTIIncluded();
     }
 
-    /**
-     * @return "true" if the VM is compiled with DTrace
-     */
-    protected String vmHasDTrace() {
-        return "" + WB.isDTraceIncluded();
-    }
-
     /**
      * @return true if compiler in use supports RTM and false otherwise.
      */
@@ -455,16 +448,14 @@ public class VMProps implements Callable<Map<String, String>> {
     }
 
    /**
-     * A simple check for container support
+     * A simple check for docker support
      *
-     * @return true if container is supported in a given environment
+     * @return true if docker is supported in a given environment
      */
-    protected String containerSupport() {
-        log("Entering containerSupport()");
-
+    protected String dockerSupport() {
         boolean isSupported = false;
         if (Platform.isLinux()) {
-           // currently container testing is only supported for Linux,
+           // currently docker testing is only supported for Linux,
            // on certain platforms
 
            String arch = System.getProperty("os.arch");
@@ -480,69 +471,23 @@ public class VMProps implements Callable<Map<String, String>> {
            }
         }
 
-        log("containerSupport(): platform check: isSupported = " + isSupported);
-
         if (isSupported) {
            try {
-              isSupported = checkProgramSupport("checkContainerSupport()", Container.ENGINE_COMMAND);
+              isSupported = checkDockerSupport();
            } catch (Exception e) {
               isSupported = false;
            }
          }
 
-        log("containerSupport(): returning isSupported = " + isSupported);
         return "" + isSupported;
     }
 
-    // Configures process builder to redirect process stdout and stderr to a file.
-    // Returns file names for stdout and stderr.
-    private Map<String, String> redirectOutputToLogFile(String msg, ProcessBuilder pb, String fileNameBase) {
-        Map<String, String> result = new HashMap<>();
-        String timeStamp = Instant.now().toString().replace(":", "-").replace(".", "-");
-
-        String stdoutFileName = String.format("./%s-stdout--%s.log", fileNameBase, timeStamp);
-        pb.redirectOutput(new File(stdoutFileName));
-        log(msg + ": child process stdout redirected to " + stdoutFileName);
-        result.put("stdout", stdoutFileName);
-
-        String stderrFileName = String.format("./%s-stderr--%s.log", fileNameBase, timeStamp);
-        pb.redirectError(new File(stderrFileName));
-        log(msg + ": child process stderr redirected to " + stderrFileName);
-        result.put("stderr", stderrFileName);
-
-        return result;
-    }
-
-    private void printLogfileContent(Map<String, String> logFileNames) {
-        logFileNames.entrySet().stream()
-            .forEach(entry ->
-                {
-                    log("------------- " + entry.getKey());
-                    try {
-                        Files.lines(Path.of(entry.getValue()))
-                            .forEach(line -> log(line));
-                    } catch (IOException ie) {
-                        log("Exception while reading file: " + ie);
-                    }
-                    log("-------------");
-                });
-    }
-    private boolean checkProgramSupport(String logString, String cmd) throws IOException, InterruptedException {
-        log(logString + ": entering");
-        ProcessBuilder pb = new ProcessBuilder("which", cmd);
-        Map<String, String> logFileNames =
-            redirectOutputToLogFile(logString + ": which " + cmd,
-                                                      pb, "which-cmd");
+    private boolean checkDockerSupport() throws IOException, InterruptedException {
+        ProcessBuilder pb = new ProcessBuilder(Container.ENGINE_COMMAND, "ps");
         Process p = pb.start();
         p.waitFor(10, TimeUnit.SECONDS);
-        int exitValue = p.exitValue();
 
-        log(String.format("%s: exitValue = %s, pid = %s", logString, exitValue, p.pid()));
-        if (exitValue != 0) {
-            printLogfileContent(logFileNames);
-        }
-
-        return (exitValue == 0);
+        return (p.exitValue() == 0);
     }
 
     /**
@@ -579,18 +524,19 @@ public class VMProps implements Callable<Map<String, String>> {
      * Checks if we are in <i>almost</i> out-of-box configuration, i.e. the flags
      * which JVM is started with don't affect its behavior "significantly".
      * {@code TEST_VM_FLAGLESS} enviroment variable can be used to force this
-     * method to return true or false and allow or reject any flags.
+     * method to return true and allow any flags.
      *
      * @return true if there are no JVM flags
      */
     private String isFlagless() {
         boolean result = true;
-        String flagless = System.getenv("TEST_VM_FLAGLESS");
-        if (flagless != null) {
-            return "" + "true".equalsIgnoreCase(flagless);
+        if (System.getenv("TEST_VM_FLAGLESS") != null) {
+            return "" + result;
         }
 
-        List<String> allFlags = allFlags().toList();
+        List<String> allFlags = new ArrayList<String>();
+        Collections.addAll(allFlags, System.getProperty("test.vm.opts", "").trim().split("\\s+"));
+        Collections.addAll(allFlags, System.getProperty("test.java.opts", "").trim().split("\\s+"));
 
         // check -XX flags
         var ignoredXXFlags = Set.of(
@@ -616,19 +562,15 @@ public class VMProps implements Callable<Map<String, String>> {
         // check -X flags
         var ignoredXFlags = Set.of(
                 // default, yet still seen to be explicitly set
-                "mixed",
-                // -XmxmNNNm added by run-test framework for non-hotspot tests
-                "mx"
+                "mixed"
         );
         result &= allFlags.stream()
                           .filter(s -> s.startsWith("-X") && !s.startsWith("-XX:"))
                           // map to names:
-                          // remove -X
-                          .map(s -> s.substring(2))
-                          // remove :.* from flags with values
-                          .map(s -> s.contains(":") ? s.substring(0, s.indexOf(':')) : s)
-                          // remove size like 4G, 768m which might be set for non-hotspot tests
-                          .map(s -> s.replaceAll("(\\d+)[mMgGkK]", ""))
+                              // remove -X
+                              .map(s -> s.substring(2))
+                              // remove :.* from flags with values
+                              .map(s -> s.contains(":") ? s.substring(0, s.indexOf(':')) : s)
                           // skip known-to-be-there flags
                           .filter(s -> !ignoredXFlags.contains(s))
                           .findAny()
@@ -637,35 +579,6 @@ public class VMProps implements Callable<Map<String, String>> {
         return "" + result;
     }
 
-    private Stream<String> allFlags() {
-        return Stream.of((System.getProperty("test.vm.opts", "") + " " + System.getProperty("test.java.opts", "")).trim().split("\\s+"));
-    }
-
-    /**
-     * Parses extra options, options that start with -X excluding the
-     * bare -X option (as it is not considered an extra option).
-     * Ignores extra options not starting with -X
-     *
-     * This could be improved to handle extra options not starting
-     * with -X as well as "standard" options.
-     */
-    private Map<String, String> xOptFlags() {
-        return allFlags()
-            .filter(s -> s.startsWith("-X") && !s.startsWith("-XX:") && !s.equals("-X"))
-            .map(s -> s.replaceFirst("-", ""))
-            .map(flag -> {
-                String[] split = flag.split("[:0123456789]", 2);
-                return split.length == 2 ? new String[] {split[0], flag.substring(split[0].length(), flag.length() - split[1].length()), split[1]}
-                    : split;
-            })
-            .collect(Collectors.toMap(a -> "vm.opt.x." + a[0],
-                                      a -> (a.length == 1)
-                                      ? "true" // -Xnoclassgc
-                                      : (a[1].equals(":")
-                                         ? a[2]            // ["-XshowSettings", ":", "system"]
-                                         : a[1] + a[2]))); // ["-Xmx", "4", "g"]
-    }
-
     /**
      * Dumps the map to the file if the file name is given as the property.
      * This functionality could be helpful to know context in the real
@@ -689,40 +602,6 @@ public class VMProps implements Callable<Map<String, String>> {
         }
     }
 
-    /**
-     * Log diagnostic message.
-     *
-     * @param msg
-     */
-    protected static void log(String msg) {
-        // Always log to a file.
-        logToFile(msg);
-
-        // Also log to stderr; guarded by property to avoid excessive verbosity.
-        // By jtreg design stderr produced here will be visible
-        // in the output of a parent process. Note: stdout should not be used
-        // for logging as jtreg parses that output directly and only echoes it
-        // in the event of a failure.
-        if (Boolean.getBoolean("jtreg.log.vmprops")) {
-            System.err.println("VMProps: " + msg);
-        }
-    }
-
-    /**
-     * Log diagnostic message to a file.
-     *
-     * @param msg
-     */
-    protected static void logToFile(String msg) {
-        String fileName = "./vmprops.log";
-        try {
-            Files.writeString(Paths.get(fileName), msg + "\n", Charset.forName("ISO-8859-1"),
-                    StandardOpenOption.APPEND, StandardOpenOption.CREATE);
-        } catch (IOException e) {
-            throw new RuntimeException("Failed to log into '" + fileName + "'", e);
-        }
-    }
-
     /**
      * This method is for the testing purpose only.
      *
