@@ -25,7 +25,7 @@
  * SealedCompilationTests
  *
  * @test
- * @bug 8246353
+ * @bug 8246353 8273257
  * @summary Negative compilation tests, and positive compilation (smoke) tests for sealed classes
  * @library /lib/combo /tools/lib
  * @modules
@@ -727,6 +727,18 @@ public class SealedCompilationTests extends CompilationTestCase {
                    """);
     }
 
+    public void testNonSealedErroneousSuperInterface() {
+        assertFail("compiler.err.cant.resolve",
+                   d -> {
+                       if (diags.keys().size() != 1) {
+                           fail("Unexpected errors: " + diags.toString());
+                       }
+                   },
+                   """
+                   non-sealed class C implements Undefined {}
+                   """);
+    }
+
     public void testIllFormedNonSealed() {
         for (String s : List.of(
             """
