@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,6 +23,7 @@
 
 package jdk.test.lib.security;
 
+import jdk.test.lib.Asserts;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.Node;
@@ -48,14 +49,15 @@ import javax.xml.transform.stream.StreamResult;
 import javax.xml.xpath.XPath;
 import javax.xml.xpath.XPathConstants;
 import javax.xml.xpath.XPathFactory;
+import java.io.File;
 import java.io.StringReader;
 import java.io.StringWriter;
 import java.net.URI;
+import java.nio.file.Files;
+import java.nio.file.Path;
 import java.security.*;
 import java.security.cert.X509Certificate;
-import java.security.interfaces.EdECPrivateKey;
 import java.security.interfaces.RSAKey;
-import java.security.spec.NamedParameterSpec;
 import java.security.spec.PSSParameterSpec;
 import java.util.*;
 
@@ -66,6 +68,35 @@ public class XMLUtils {
     private static final XMLSignatureFactory FAC =
             XMLSignatureFactory.getInstance("DOM");
 
+    //////////// MAIN as TEST ////////////
+    public static void main(String[] args) throws Exception {
+        var x = "<a><b>c</b>x</a>";
+        var p = Files.write(Path.of("x.xml"), List.of(x));
+        var b = Path.of("").toUri().toString();
+        var d = string2doc(x);
+        // keytool -keystore ks -keyalg ec -storepass changeit -genkeypair -alias a -dname CN=a
+        var pass = "changeit".toCharArray();
+        var ks = KeyStore.getInstance(new File("ks"), pass);
+        var c = (X509Certificate) ks.getCertificate("a");
+        var pr = (PrivateKey) ks.getKey("a", pass);
+        var pu = c.getPublicKey();
+        var s0 = signer(pr); // No KeyInfo
+        var s1 = signer(pr, pu); // KeyInfo is PublicKey
+        var s2 = signer(pr, c); // KeyInfo is X509Data
+        var s3 = signer(ks, "a", pass); // KeyInfo is KeyName
+        var v1 = validator(); // knows nothing
+        var v2 = validator(ks); // knows KeyName
+        Asserts.assertTrue(v1.validate(s0.sign(d), pu)); // need PublicKey
+        Asserts.assertTrue(v1.validate(s1.sign(d))); // can read KeyInfo
+        Asserts.assertTrue(v1.validate(s2.sign(d))); // can read KeyInfo
+        Asserts.assertTrue(v2.validate(s3.sign(d))); // can read KeyInfo
+        Asserts.assertTrue(v2.secureValidation(false).validate(s3.sign(p.toUri()))); // can read KeyInfo
+        Asserts.assertTrue(v2.secureValidation(false).baseURI(b).validate(
+                s3.sign(p.getParent().toUri(), p.getFileName().toUri()))); // can read KeyInfo
+        Asserts.assertTrue(v1.validate(s1.sign("text"))); // plain text
+        Asserts.assertTrue(v1.validate(s1.sign("binary".getBytes()))); // raw data
+    }
+
     //////////// CONVERT ////////////
 
     // Converts a Document object to string
@@ -187,20 +218,38 @@ public class XMLUtils {
 
     public static class Signer {
 
-        final PrivateKey privateKey;  // signer key, never null
-
+        PrivateKey privateKey;  // signer key, never null
         X509Certificate cert;   // certificate, optional
         PublicKey publicKey;    // public key, optional
         String keyName;         // alias, optional
 
-        String sm = null;       // default determined by privateKey
-        SignatureMethodParameterSpec smSpec = null;
-        String dm = DigestMethod.SHA256;
-        String cm = CanonicalizationMethod.EXCLUSIVE;
-        String tr = Transform.ENVELOPED;
+        SignatureMethod sm;     // default determined by privateKey
+        DigestMethod dm;        // default SHA-256
+        CanonicalizationMethod cm;  // default EXCLUSIVE
+        Transform tr;           // default ENVELOPED
 
-        public Signer(PrivateKey privateKey) {
-            this.privateKey = Objects.requireNonNull(privateKey);
+        public Signer(PrivateKey privateKey) throws Exception {
+            this.privateKey = privateKey;
+            dm(DigestMethod.SHA256);
+            tr(Transform.ENVELOPED);
+            cm(CanonicalizationMethod.EXCLUSIVE);
+            String alg = privateKey.getAlgorithm();
+            if (alg.equals("RSASSA-PSS")) {
+                PSSParameterSpec pspec
+                        = (PSSParameterSpec) ((RSAKey) privateKey).getParams();
+                if (pspec != null) {
+                    sm(SignatureMethod.RSA_PSS, new RSAPSSParameterSpec(pspec));
+                } else {
+                    sm(SignatureMethod.RSA_PSS);
+                }
+            } else {
+                sm(switch (privateKey.getAlgorithm()) {
+                    case "RSA" -> SignatureMethod.RSA_SHA256;
+                    case "DSA" -> SignatureMethod.DSA_SHA256;
+                    case "EC" -> SignatureMethod.ECDSA_SHA256;
+                    default -> throw new InvalidKeyException();
+                });
+            }
         }
 
         // Change KeyInfo source
@@ -222,29 +271,47 @@ public class XMLUtils {
 
         // Change various methods
 
-        public Signer tr(String transform) {
-            tr = Objects.requireNonNull(transform);
-            return this;
-        }
-
-        public Signer dm(String method) {
-            dm = Objects.requireNonNull(method);
-            return this;
-        }
-
-        public Signer cm(String method) {
-            cm = Objects.requireNonNull(method);
-            return this;
-        }
-
-        public Signer sm(String method, SignatureMethodParameterSpec spec) {
-            sm = method;
-            smSpec = spec;
+        public Signer tr(String transform) throws Exception {
+            TransformParameterSpec params = null;
+            switch (transform) {
+                case Transform.XPATH:
+                    params = new XPathFilterParameterSpec("//.");
+                    break;
+                case Transform.XPATH2:
+                    params = new XPathFilter2ParameterSpec(
+                            Collections.singletonList(new XPathType("//.",
+                                    XPathType.Filter.INTERSECT)));
+                    break;
+            }
+            tr = FAC.newTransform(transform, params);
             return this;
         }
 
         public Signer sm(String method) throws Exception {
-            return sm(method, null);
+            sm = FAC.newSignatureMethod(method, null);
+            return this;
+        }
+
+        public Signer dm(String method) throws Exception {
+            dm = FAC.newDigestMethod(method, null);
+            return this;
+        }
+
+        public Signer cm(String method) throws Exception {
+            cm = FAC.newCanonicalizationMethod(method, (C14NMethodParameterSpec) null);
+            return this;
+        }
+
+        public Signer sm(String method, SignatureMethodParameterSpec spec)
+                throws Exception {
+            sm = FAC.newSignatureMethod(method, spec);
+            return this;
+        }
+
+        public Signer dm(String method, DigestMethodParameterSpec spec)
+                throws Exception {
+            dm = FAC.newDigestMethod(method, spec);
+            return this;
         }
 
         // Signs different sources
@@ -280,14 +347,14 @@ public class XMLUtils {
         }
 
         // Signs a document in enveloping mode
-        public Document signEnveloping(Document document, String id, String ref) throws Exception {
+        public Document signEnveloping(Document document) throws Exception {
             Document newDocument = DocumentBuilderFactory.newInstance()
                     .newDocumentBuilder().newDocument();
             FAC.newXMLSignature(
-                    buildSignedInfo(FAC.newReference(ref, FAC.newDigestMethod(dm, null))),
+                    buildSignedInfo(FAC.newReference("#object", dm)),
                     buildKeyInfo(),
                     List.of(FAC.newXMLObject(List.of(new DOMStructure(document.getDocumentElement())),
-                            id, null, null)),
+                            "object", null, null)),
                     null,
                     null)
                     .sign(new DOMSignContext(privateKey, newDocument));
@@ -299,7 +366,7 @@ public class XMLUtils {
             Document newDocument = DocumentBuilderFactory.newInstance()
                     .newDocumentBuilder().newDocument();
             FAC.newXMLSignature(
-                    buildSignedInfo(FAC.newReference("#object", FAC.newDigestMethod(dm, null), List.of
+                    buildSignedInfo(FAC.newReference("#object", dm, List.of
                             (FAC.newTransform(Transform.BASE64,
                                     (TransformParameterSpec) null)), null, null)),
                     buildKeyInfo(),
@@ -317,11 +384,9 @@ public class XMLUtils {
             Document newDocument = DocumentBuilderFactory.newInstance()
                     .newDocumentBuilder().newDocument();
             FAC.newXMLSignature(
-                    buildSignedInfo(
-                            FAC.newReference("#object", FAC.newDigestMethod(dm, null))),
+                    buildSignedInfo(FAC.newReference("#object", dm)),
                     buildKeyInfo(),
-                    List.of(FAC.newXMLObject(
-                            List.of(new DOMStructure(newDocument.createTextNode(str))),
+                    List.of(FAC.newXMLObject(List.of(new DOMStructure(newDocument.createTextNode(str))),
                             "object", null, null)),
                     null,
                     null)
@@ -330,61 +395,22 @@ public class XMLUtils {
         }
 
         // Builds a SignedInfo for a string reference
-        private SignedInfo buildSignedInfo(String ref) throws Exception {
-            return buildSignedInfo(FAC.newReference(
+        private SignedInfo buildSignedInfo(String ref) {
+            return FAC.newSignedInfo(
+                    cm,
+                    sm,
+                    List.of(FAC.newReference(
                             ref,
-                            FAC.newDigestMethod(dm, null),
-                            List.of(FAC.newTransform(tr, switch (tr) {
-                                case Transform.XPATH ->
-                                    new XPathFilterParameterSpec("//.");
-                                case Transform.XPATH2 -> new XPathFilter2ParameterSpec(
-                                            Collections.singletonList(new XPathType("//.",
-                                                    XPathType.Filter.INTERSECT)));
-                                default -> null;
-                            })),
-                            null, null));
+                            dm,
+                            List.of(tr),
+                            null, null)));
         }
 
         // Builds a SignedInfo for a Reference
-        private SignedInfo buildSignedInfo(Reference ref) throws Exception {
-            SignatureMethod signatureMethod;
-            if (sm == null) {
-                String alg = privateKey.getAlgorithm().toUpperCase(Locale.ROOT);
-                if (alg.equals("RSASSA-PSS")) {
-                    PSSParameterSpec pspec
-                            = (PSSParameterSpec) ((RSAKey) privateKey).getParams();
-                    if (pspec != null) {
-                        signatureMethod = FAC.newSignatureMethod(
-                                SignatureMethod.RSA_PSS, new RSAPSSParameterSpec(pspec));
-                    } else {
-                        signatureMethod = FAC.newSignatureMethod(SignatureMethod.RSA_PSS, null);
-                    }
-                } else {
-                    signatureMethod = FAC.newSignatureMethod(switch (alg) {
-                        case "RSA" -> SignatureMethod.RSA_SHA256;
-                        case "DSA" -> SignatureMethod.DSA_SHA256;
-                        case "EC" -> SignatureMethod.ECDSA_SHA256;
-                        case "ED25519" -> "http://www.w3.org/2021/04/xmldsig-more#eddsa-ed25519";
-                        case "ED448" -> "http://www.w3.org/2021/04/xmldsig-more#eddsa-ed448";
-                        case "EDDSA" -> {
-                            if (privateKey instanceof EdECPrivateKey edsk) {
-                                yield edsk.getParams().getName()
-                                        .equals(NamedParameterSpec.ED25519.getName())
-                                        ? "http://www.w3.org/2021/04/xmldsig-more#eddsa-ed25519"
-                                        : "http://www.w3.org/2021/04/xmldsig-more#eddsa-ed448";
-                            } else {
-                                throw new InvalidKeyException();
-                            }
-                        }
-                        default -> throw new InvalidKeyException();
-                    }, null);
-                }
-            } else {
-                signatureMethod = FAC.newSignatureMethod(sm, smSpec);
-            }
+        private SignedInfo buildSignedInfo(Reference ref) {
             return FAC.newSignedInfo(
-                    FAC.newCanonicalizationMethod(cm, (C14NMethodParameterSpec) null),
-                    signatureMethod,
+                    cm,
+                    sm,
                     List.of(ref));
         }
 
@@ -448,7 +474,7 @@ public class XMLUtils {
         // If key is not null, any key from the signature will be ignored
         public boolean validate(Document document, PublicKey key)
                 throws Exception {
-            NodeList nodeList = document.getElementsByTagNameNS(XMLSignature.XMLNS, "Signature");
+            NodeList nodeList = document.getElementsByTagName("Signature");
             if (nodeList.getLength() == 1) {
                 Node signatureNode = nodeList.item(0);
                 if (signatureNode != null) {
@@ -490,9 +516,7 @@ public class XMLUtils {
                                             AlgorithmMethod method,
                                             XMLCryptoContext context)
                     throws KeySelectorException {
-                if (keyInfo == null) {
-                    throw new IllegalArgumentException("Null KeyInfo object!");
-                }
+                Objects.requireNonNull(keyInfo, "Null KeyInfo object!");
 
                 for (XMLStructure xmlStructure : keyInfo.getContent()) {
                     PublicKey pk;
